@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015 Canonical Ltd
+ * Copyright (C) 2015-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -21,12 +21,15 @@ package asserts_test
 
 import (
 	"fmt"
+	"regexp"
 
 	. "gopkg.in/check.v1"
 	"gopkg.in/yaml.v2"
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/snap"
+
+	"github.com/snapcore/snapd/testutil"
 )
 
 var (
@@ -34,7 +37,9 @@ var (
 	_ = Suite(&plugSlotRulesSuite{})
 )
 
-type attrConstraintsSuite struct{}
+type attrConstraintsSuite struct {
+	testutil.BaseTest
+}
 
 func attrs(yml string) map[string]interface{} {
 	var attrs map[string]interface{}
@@ -58,6 +63,15 @@ func attrs(yml string) map[string]interface{} {
 	return info.Plugs["plug"].Attrs
 }
 
+func (s *attrConstraintsSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+}
+
+func (s *attrConstraintsSuite) TearDownTest(c *C) {
+	s.BaseTest.TearDownTest(c)
+}
+
 func (s *attrConstraintsSuite) TestSimple(c *C) {
 	m, err := asserts.ParseHeaders([]byte(`attrs:
   foo: FOO
@@ -71,20 +85,20 @@ func (s *attrConstraintsSuite) TestSimple(c *C) {
 		"foo": "FOO",
 		"bar": "BAR",
 		"baz": "BAZ",
-	})
+	}, nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(map[string]interface{}{
 		"foo": "FOO",
 		"bar": "BAZ",
 		"baz": "BAZ",
-	})
+	}, nil)
 	c.Check(err, ErrorMatches, `attribute "bar" value "BAZ" does not match \^\(BAR\)\$`)
 
 	err = cstrs.Check(map[string]interface{}{
 		"foo": "FOO",
 		"baz": "BAZ",
-	})
+	}, nil)
 	c.Check(err, ErrorMatches, `attribute "bar" has constraints but is unset`)
 }
 
@@ -98,27 +112,27 @@ func (s *attrConstraintsSuite) TestSimpleAnchorsVsRegexpAlt(c *C) {
 
 	err = cstrs.Check(map[string]interface{}{
 		"bar": "BAR",
-	})
+	}, nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(map[string]interface{}{
 		"bar": "BARR",
-	})
+	}, nil)
 	c.Check(err, ErrorMatches, `attribute "bar" value "BARR" does not match \^\(BAR|BAZ\)\$`)
 
 	err = cstrs.Check(map[string]interface{}{
 		"bar": "BBAZ",
-	})
+	}, nil)
 	c.Check(err, ErrorMatches, `attribute "bar" value "BAZZ" does not match \^\(BAR|BAZ\)\$`)
 
 	err = cstrs.Check(map[string]interface{}{
 		"bar": "BABAZ",
-	})
+	}, nil)
 	c.Check(err, ErrorMatches, `attribute "bar" value "BABAZ" does not match \^\(BAR|BAZ\)\$`)
 
 	err = cstrs.Check(map[string]interface{}{
 		"bar": "BARAZ",
-	})
+	}, nil)
 	c.Check(err, ErrorMatches, `attribute "bar" value "BARAZ" does not match \^\(BAR|BAZ\)\$`)
 }
 
@@ -140,14 +154,14 @@ bar:
   bar2: BAR2
   bar3: BAR3
 baz: BAZ
-`))
+`), nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(attrs(`
 foo: FOO
 bar: BAZ
 baz: BAZ
-`))
+`), nil)
 	c.Check(err, ErrorMatches, `attribute "bar" must be a map`)
 
 	err = cstrs.Check(attrs(`
@@ -157,7 +171,7 @@ bar:
   bar2: BAR22
   bar3: BAR3
 baz: BAZ
-`))
+`), nil)
 	c.Check(err, ErrorMatches, `attribute "bar\.bar2" value "BAR22" does not match \^\(BAR2\)\$`)
 
 	err = cstrs.Check(attrs(`
@@ -168,7 +182,7 @@ bar:
     bar22: true
   bar3: BAR3
 baz: BAZ
-`))
+`), nil)
 	c.Check(err, ErrorMatches, `attribute "bar\.bar2" must be a scalar or list`)
 }
 
@@ -189,21 +203,21 @@ func (s *attrConstraintsSuite) TestAlternative(c *C) {
 		"foo": "FOO",
 		"bar": "BAR",
 		"baz": "BAZ",
-	})
+	}, nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(map[string]interface{}{
 		"foo": "FOO",
 		"bar": "BAZ",
 		"baz": "BAZ",
-	})
+	}, nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(map[string]interface{}{
 		"foo": "FOO",
 		"bar": "BARR",
 		"baz": "BAR",
-	})
+	}, nil)
 	c.Check(err, ErrorMatches, `no alternative matches: attribute "bar" value "BARR" does not match \^\(BAR\)\$`)
 }
 
@@ -225,7 +239,7 @@ foo: FOO
 bar:
   bar1: BAR1
   bar2: BAR2
-`))
+`), nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(attrs(`
@@ -233,7 +247,7 @@ foo: FOO
 bar:
   bar1: BAR1
   bar2: BAR22
-`))
+`), nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(attrs(`
@@ -241,7 +255,7 @@ foo: FOO
 bar:
   bar1: BAR1
   bar2: BAR3
-`))
+`), nil)
 	c.Check(err, ErrorMatches, `no alternative for attribute "bar\.bar2" matches: attribute "bar\.bar2" value "BAR3" does not match \^\(BAR2\)\$`)
 }
 
@@ -257,13 +271,13 @@ func (s *attrConstraintsSuite) TestOtherScalars(c *C) {
 	err = cstrs.Check(attrs(`
 foo: 1
 bar: true
-`))
+`), nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(map[string]interface{}{
 		"foo": int64(1),
 		"bar": true,
-	})
+	}, nil)
 	c.Check(err, IsNil)
 }
 
@@ -288,6 +302,21 @@ func (s *attrConstraintsSuite) TestCompileErrors(c *C) {
 
 	_, err = asserts.CompileAttributeConstraints([]interface{}{"FOO"})
 	c.Check(err, ErrorMatches, `first level of non alternative constraints must be a set of key-value contraints`)
+
+	wrongDollarConstraints := []string{
+		"$",
+		"$FOO(a)",
+		"$SLOT",
+		"$SLOT()",
+	}
+
+	for _, wrong := range wrongDollarConstraints {
+		_, err := asserts.CompileAttributeConstraints(map[string]interface{}{
+			"foo": wrong,
+		})
+		c.Check(err, ErrorMatches, fmt.Sprintf(`cannot compile "foo" constraint "%s": not a valid \$SLOT\(\)/\$PLUG\(\) constraint`, regexp.QuoteMeta(wrong)))
+
+	}
 }
 
 func (s *attrConstraintsSuite) TestMatchingListsSimple(c *C) {
@@ -300,15 +329,107 @@ func (s *attrConstraintsSuite) TestMatchingListsSimple(c *C) {
 
 	err = cstrs.Check(attrs(`
 foo: ["/foo/x", "/foo/y"]
-`))
+`), nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(attrs(`
 foo: ["/foo/x", "/foo"]
-`))
+`), nil)
 	c.Check(err, ErrorMatches, `attribute "foo\.1" value "/foo" does not match \^\(/foo/\.\*\)\$`)
 }
 
+func (s *attrConstraintsSuite) TestMissingCheck(c *C) {
+	m, err := asserts.ParseHeaders([]byte(`attrs:
+  foo: $MISSING`))
+	c.Assert(err, IsNil)
+
+	cstrs, err := asserts.CompileAttributeConstraints(m["attrs"].(map[string]interface{}))
+	c.Assert(err, IsNil)
+	c.Check(asserts.RuleFeature(cstrs, "dollar-attr-constraints"), Equals, true)
+
+	err = cstrs.Check(attrs(`
+bar: baz
+`), nil)
+	c.Check(err, IsNil)
+
+	err = cstrs.Check(attrs(`
+foo: ["x"]
+`), nil)
+	c.Check(err, ErrorMatches, `attribute "foo" is constrained to be missing but is set`)
+}
+
+type testEvalAttr struct {
+	comp func(side string, arg string) (interface{}, error)
+}
+
+func (ca testEvalAttr) SlotAttr(arg string) (interface{}, error) {
+	return ca.comp("slot", arg)
+}
+
+func (ca testEvalAttr) PlugAttr(arg string) (interface{}, error) {
+	return ca.comp("plug", arg)
+}
+
+func (s *attrConstraintsSuite) TestEvalCheck(c *C) {
+	m, err := asserts.ParseHeaders([]byte(`attrs:
+  foo: $SLOT(foo)
+  bar: $PLUG(bar.baz)`))
+	c.Assert(err, IsNil)
+
+	cstrs, err := asserts.CompileAttributeConstraints(m["attrs"].(map[string]interface{}))
+	c.Assert(err, IsNil)
+	c.Check(asserts.RuleFeature(cstrs, "dollar-attr-constraints"), Equals, true)
+
+	err = cstrs.Check(attrs(`
+foo: foo
+bar: bar
+`), nil)
+	c.Check(err, ErrorMatches, `attribute "(foo|bar)" cannot be matched without context`)
+
+	calls := make(map[[2]string]bool)
+	comp1 := func(op string, arg string) (interface{}, error) {
+		calls[[2]string{op, arg}] = true
+		return arg, nil
+	}
+
+	err = cstrs.Check(attrs(`
+foo: foo
+bar: bar.baz
+`), testEvalAttr{comp1})
+	c.Check(err, IsNil)
+
+	c.Check(calls, DeepEquals, map[[2]string]bool{
+		{"slot", "foo"}:     true,
+		{"plug", "bar.baz"}: true,
+	})
+
+	comp2 := func(op string, arg string) (interface{}, error) {
+		if op == "plug" {
+			return nil, fmt.Errorf("boom")
+		}
+		return arg, nil
+	}
+
+	err = cstrs.Check(attrs(`
+foo: foo
+bar: bar.baz
+`), testEvalAttr{comp2})
+	c.Check(err, ErrorMatches, `attribute "bar" constraint \$PLUG\(bar\.baz\) cannot be evaluated: boom`)
+
+	comp3 := func(op string, arg string) (interface{}, error) {
+		if op == "slot" {
+			return "other-value", nil
+		}
+		return arg, nil
+	}
+
+	err = cstrs.Check(attrs(`
+foo: foo
+bar: bar.baz
+`), testEvalAttr{comp3})
+	c.Check(err, ErrorMatches, `attribute "foo" does not match \$SLOT\(foo\): foo != other-value`)
+}
+
 func (s *attrConstraintsSuite) TestMatchingListsMap(c *C) {
 	m, err := asserts.ParseHeaders([]byte(`attrs:
   foo:
@@ -320,21 +441,21 @@ func (s *attrConstraintsSuite) TestMatchingListsMap(c *C) {
 
 	err = cstrs.Check(attrs(`
 foo: [{p: "/foo/x"}, {p: "/foo/y"}]
-`))
+`), nil)
 	c.Check(err, IsNil)
 
 	err = cstrs.Check(attrs(`
 foo: [{p: "zzz"}, {p: "/foo/y"}]
-`))
+`), nil)
 	c.Check(err, ErrorMatches, `attribute "foo\.0\.p" value "zzz" does not match \^\(/foo/\.\*\)\$`)
 }
 
 func (s *attrConstraintsSuite) TestAlwaysMatchAttributeConstraints(c *C) {
-	c.Check(asserts.AlwaysMatchAttributes.Check(nil), IsNil)
+	c.Check(asserts.AlwaysMatchAttributes.Check(nil, nil), IsNil)
 }
 
 func (s *attrConstraintsSuite) TestNeverMatchAttributeConstraints(c *C) {
-	c.Check(asserts.NeverMatchAttributes.Check(nil), NotNil)
+	c.Check(asserts.NeverMatchAttributes.Check(nil, nil), NotNil)
 }
 
 type plugSlotRulesSuite struct{}
@@ -342,10 +463,10 @@ type plugSlotRulesSuite struct{}
 func checkAttrs(c *C, attrs *asserts.AttributeConstraints, witness, expected string) {
 	c.Check(attrs.Check(map[string]interface{}{
 		witness: "XYZ",
-	}), ErrorMatches, fmt.Sprintf(`attribute "%s".*does not match.*`, witness))
+	}, nil), ErrorMatches, fmt.Sprintf(`attribute "%s".*does not match.*`, witness))
 	c.Check(attrs.Check(map[string]interface{}{
 		witness: expected,
-	}), IsNil)
+	}, nil), IsNil)
 }
 
 func checkBoolPlugConnConstraints(c *C, cstrs []*asserts.PlugConnectionConstraints, always bool) {
@@ -778,6 +899,50 @@ func (s *plugSlotRulesSuite) TestCompilePlugRuleErrors(c *C) {
 	}
 }
 
+func (s *plugSlotRulesSuite) TestPlugRuleFeatures(c *C) {
+	combos := []struct {
+		subrule         string
+		attrConstraints []string
+	}{
+		{"allow-installation", []string{"plug-attributes"}},
+		{"deny-installation", []string{"plug-attributes"}},
+		{"allow-connection", []string{"plug-attributes", "slot-attributes"}},
+		{"deny-connection", []string{"plug-attributes", "slot-attributes"}},
+		{"allow-auto-connection", []string{"plug-attributes", "slot-attributes"}},
+		{"deny-auto-connection", []string{"plug-attributes", "slot-attributes"}},
+	}
+
+	for _, combo := range combos {
+		for _, attrConstr := range combo.attrConstraints {
+			attrConstraintMap := map[string]interface{}{
+				"a":     "ATTR",
+				"other": []interface{}{"x", "y"},
+			}
+			ruleMap := map[string]interface{}{
+				combo.subrule: map[string]interface{}{
+					attrConstr: attrConstraintMap,
+				},
+			}
+
+			rule, err := asserts.CompilePlugRule("iface", ruleMap)
+			c.Assert(err, IsNil)
+			c.Check(asserts.RuleFeature(rule, "dollar-attr-constraints"), Equals, false, Commentf("%v", ruleMap))
+
+			attrConstraintMap["a"] = "$MISSING"
+			rule, err = asserts.CompilePlugRule("iface", ruleMap)
+			c.Assert(err, IsNil)
+			c.Check(asserts.RuleFeature(rule, "dollar-attr-constraints"), Equals, true, Commentf("%v", ruleMap))
+
+			// covers also alternation
+			attrConstraintMap["a"] = []interface{}{"$SLOT(a)"}
+			rule, err = asserts.CompilePlugRule("iface", ruleMap)
+			c.Assert(err, IsNil)
+			c.Check(asserts.RuleFeature(rule, "dollar-attr-constraints"), Equals, true, Commentf("%v", ruleMap))
+
+		}
+	}
+}
+
 func (s *plugSlotRulesSuite) TestCompileSlotRuleAllAllowDenyStanzas(c *C) {
 	m, err := asserts.ParseHeaders([]byte(`iface:
   allow-installation:
@@ -1171,3 +1336,40 @@ func (s *plugSlotRulesSuite) TestCompileSlotRuleErrors(c *C) {
 		c.Check(err, ErrorMatches, t.err, Commentf(t.stanza))
 	}
 }
+
+func (s *plugSlotRulesSuite) TestSlotRuleFeatures(c *C) {
+	combos := []struct {
+		subrule         string
+		attrConstraints []string
+	}{
+		{"allow-installation", []string{"slot-attributes"}},
+		{"deny-installation", []string{"slot-attributes"}},
+		{"allow-connection", []string{"plug-attributes", "slot-attributes"}},
+		{"deny-connection", []string{"plug-attributes", "slot-attributes"}},
+		{"allow-auto-connection", []string{"plug-attributes", "slot-attributes"}},
+		{"deny-auto-connection", []string{"plug-attributes", "slot-attributes"}},
+	}
+
+	for _, combo := range combos {
+		for _, attrConstr := range combo.attrConstraints {
+			attrConstraintMap := map[string]interface{}{
+				"a": "ATTR",
+			}
+			ruleMap := map[string]interface{}{
+				combo.subrule: map[string]interface{}{
+					attrConstr: attrConstraintMap,
+				},
+			}
+
+			rule, err := asserts.CompileSlotRule("iface", ruleMap)
+			c.Assert(err, IsNil)
+			c.Check(asserts.RuleFeature(rule, "dollar-attr-constraints"), Equals, false, Commentf("%v", ruleMap))
+
+			attrConstraintMap["a"] = "$PLUG(a)"
+			rule, err = asserts.CompileSlotRule("iface", ruleMap)
+			c.Assert(err, IsNil)
+			c.Check(asserts.RuleFeature(rule, "dollar-attr-constraints"), Equals, true, Commentf("%v", ruleMap))
+
+		}
+	}
+}
