@@ -41,6 +41,7 @@ type SnapDeclaration struct {
 	plugRules      map[string]*PlugRule
 	slotRules      map[string]*SlotRule
 	autoAliases    []string
+	aliases        map[string]string
 	timestamp      time.Time
 }
 
@@ -85,10 +86,16 @@ func (snapdcl *SnapDeclaration) SlotRule(interfaceName string) *SlotRule {
 }
 
 // AutoAliases returns the optional auto-aliases granted to this snap.
+// XXX: deprecated, will go away
 func (snapdcl *SnapDeclaration) AutoAliases() []string {
 	return snapdcl.autoAliases
 }
 
+// Aliases returns the optional explicit aliases granted to this snap.
+func (snapdcl *SnapDeclaration) Aliases() map[string]string {
+	return snapdcl.aliases
+}
+
 // Implement further consistency checks.
 func (snapdcl *SnapDeclaration) checkConsistency(db RODatabase, acck *AccountKey) error {
 	if !db.IsTrustedAccount(snapdcl.AuthorityID()) {
@@ -97,7 +104,7 @@ func (snapdcl *SnapDeclaration) checkConsistency(db RODatabase, acck *AccountKey
 	_, err := db.Find(AccountType, map[string]string{
 		"account-id": snapdcl.PublisherID(),
 	})
-	if err == ErrNotFound {
+	if IsNotFound(err) {
 		return fmt.Errorf("snap-declaration assertion for %q (id %q) does not have a matching account assertion for the publisher %q", snapdcl.SnapName(), snapdcl.SnapID(), snapdcl.PublisherID())
 	}
 	if err != nil {
@@ -176,7 +183,52 @@ func snapDeclarationFormatAnalyze(headers map[string]interface{}, body []byte) (
 	return formatnum, nil
 }
 
-var validAlias = regexp.MustCompile("^[a-zA-Z0-9][-_.a-zA-Z0-9]*$")
+var (
+	validAlias   = regexp.MustCompile("^[a-zA-Z0-9][-_.a-zA-Z0-9]*$")
+	validAppName = regexp.MustCompile("^[a-zA-Z0-9](?:-?[a-zA-Z0-9])*$")
+)
+
+func checkAliases(headers map[string]interface{}) (map[string]string, error) {
+	value, ok := headers["aliases"]
+	if !ok {
+		return nil, nil
+	}
+	aliasList, ok := value.([]interface{})
+	if !ok {
+		return nil, fmt.Errorf(`"aliases" header must be a list of alias maps`)
+	}
+	if len(aliasList) == 0 {
+		return nil, nil
+	}
+
+	aliasMap := make(map[string]string, len(aliasList))
+	for i, item := range aliasList {
+		aliasItem, ok := item.(map[string]interface{})
+		if !ok {
+			return nil, fmt.Errorf(`"aliases" header must be a list of alias maps`)
+		}
+
+		what := fmt.Sprintf(`in "aliases" item %d`, i+1)
+		name, err := checkStringMatchesWhat(aliasItem, "name", what, validAlias)
+		if err != nil {
+			return nil, err
+		}
+
+		what = fmt.Sprintf(`for alias %q`, name)
+		target, err := checkStringMatchesWhat(aliasItem, "target", what, validAppName)
+		if err != nil {
+			return nil, err
+		}
+
+		if _, ok := aliasMap[name]; ok {
+			return nil, fmt.Errorf(`duplicated definition in "aliases" for alias %q`, name)
+		}
+
+		aliasMap[name] = target
+	}
+
+	return aliasMap, nil
+}
 
 func assembleSnapDeclaration(assert assertionBase) (Assertion, error) {
 	_, err := checkExistsString(assert.headers, "snap-name")
@@ -231,17 +283,24 @@ func assembleSnapDeclaration(assert assertionBase) (Assertion, error) {
 		}
 	}
 
+	// XXX: depracated, will go away later
 	autoAliases, err := checkStringListMatches(assert.headers, "auto-aliases", validAlias)
 	if err != nil {
 		return nil, err
 	}
 
+	aliases, err := checkAliases(assert.headers)
+	if err != nil {
+		return nil, err
+	}
+
 	return &SnapDeclaration{
 		assertionBase:  assert,
 		refreshControl: refControl,
 		plugRules:      plugRules,
 		slotRules:      slotRules,
 		autoAliases:    autoAliases,
+		aliases:        aliases,
 		timestamp:      timestamp,
 	}, nil
 }
@@ -377,7 +436,7 @@ func (snaprev *SnapRevision) checkConsistency(db RODatabase, acck *AccountKey) e
 	_, err := db.Find(AccountType, map[string]string{
 		"account-id": snaprev.DeveloperID(),
 	})
-	if err == ErrNotFound {
+	if IsNotFound(err) {
 		return fmt.Errorf("snap-revision assertion for snap id %q does not have a matching account assertion for the developer %q", snaprev.SnapID(), snaprev.DeveloperID())
 	}
 	if err != nil {
@@ -388,7 +447,7 @@ func (snaprev *SnapRevision) checkConsistency(db RODatabase, acck *AccountKey) e
 		"series":  release.Series,
 		"snap-id": snaprev.SnapID(),
 	})
-	if err == ErrNotFound {
+	if IsNotFound(err) {
 		return fmt.Errorf("snap-revision assertion for snap id %q does not have a matching snap-declaration assertion", snaprev.SnapID())
 	}
 	if err != nil {
@@ -498,7 +557,7 @@ func (validation *Validation) checkConsistency(db RODatabase, acck *AccountKey)
 		"series":  validation.Series(),
 		"snap-id": validation.ApprovedSnapID(),
 	})
-	if err == ErrNotFound {
+	if IsNotFound(err) {
 		return fmt.Errorf("validation assertion by snap-id %q does not have a matching snap-declaration assertion for approved-snap-id %q", validation.SnapID(), validation.ApprovedSnapID())
 	}
 	if err != nil {
@@ -508,7 +567,7 @@ func (validation *Validation) checkConsistency(db RODatabase, acck *AccountKey)
 		"series":  validation.Series(),
 		"snap-id": validation.SnapID(),
 	})
-	if err == ErrNotFound {
+	if IsNotFound(err) {
 		return fmt.Errorf("validation assertion by snap-id %q does not have a matching snap-declaration assertion", validation.SnapID())
 	}
 	if err != nil {
@@ -698,3 +757,175 @@ func InitBuiltinBaseDeclaration(headers []byte) error {
 	builtinBaseDeclaration = a.(*BaseDeclaration)
 	return nil
 }
+
+type dateRange struct {
+	Since time.Time
+	Until time.Time
+}
+
+// SnapDeveloper holds a snap-developer assertion, defining the developers who
+// can collaborate on a snap while it's owned by a specific publisher.
+//
+// The primary key (snap-id, publisher-id) allows a snap to have many
+// snap-developer assertions, e.g. to allow a future publisher's collaborations
+// to be defined before the snap is transferred. However only the
+// snap-developer for the current publisher (the snap-declaration publisher-id)
+// is relevant to a device.
+type SnapDeveloper struct {
+	assertionBase
+	developerRanges map[string][]*dateRange
+}
+
+// SnapID returns the snap id of the snap.
+func (snapdev *SnapDeveloper) SnapID() string {
+	return snapdev.HeaderString("snap-id")
+}
+
+// PublisherID returns the publisher's account id.
+func (snapdev *SnapDeveloper) PublisherID() string {
+	return snapdev.HeaderString("publisher-id")
+}
+
+func (snapdev *SnapDeveloper) checkConsistency(db RODatabase, acck *AccountKey) error {
+	// Check authority is the publisher or trusted.
+	authorityID := snapdev.AuthorityID()
+	publisherID := snapdev.PublisherID()
+	if !db.IsTrustedAccount(authorityID) && (publisherID != authorityID) {
+		return fmt.Errorf("snap-developer must be signed by the publisher or a trusted authority but got authority %q and publisher %q", authorityID, publisherID)
+	}
+
+	// Check snap-declaration for the snap-id exists for the series.
+	// Note: the current publisher is irrelevant here because this assertion
+	// may be for a future publisher.
+	_, err := db.Find(SnapDeclarationType, map[string]string{
+		// XXX: mediate getting current series through some context object? this gets the job done for now
+		"series":  release.Series,
+		"snap-id": snapdev.SnapID(),
+	})
+	if err != nil {
+		if IsNotFound(err) {
+			return fmt.Errorf("snap-developer assertion for snap id %q does not have a matching snap-declaration assertion", snapdev.SnapID())
+		}
+		return err
+	}
+
+	// check there's an account for the publisher-id
+	_, err = db.Find(AccountType, map[string]string{"account-id": publisherID})
+	if err != nil {
+		if IsNotFound(err) {
+			return fmt.Errorf("snap-developer assertion for snap-id %q does not have a matching account assertion for the publisher %q", snapdev.SnapID(), publisherID)
+		}
+		return err
+	}
+
+	// check there's an account for each developer
+	for developerID := range snapdev.developerRanges {
+		if developerID == publisherID {
+			continue
+		}
+		_, err = db.Find(AccountType, map[string]string{"account-id": developerID})
+		if err != nil {
+			if IsNotFound(err) {
+				return fmt.Errorf("snap-developer assertion for snap-id %q does not have a matching account assertion for the developer %q", snapdev.SnapID(), developerID)
+			}
+			return err
+		}
+	}
+
+	return nil
+}
+
+// sanity
+var _ consistencyChecker = (*SnapDeveloper)(nil)
+
+// Prerequisites returns references to this snap-developer's prerequisite assertions.
+func (snapdev *SnapDeveloper) Prerequisites() []*Ref {
+	// Capacity for the snap-declaration, the publisher and all developers.
+	refs := make([]*Ref, 0, 2+len(snapdev.developerRanges))
+
+	// snap-declaration
+	// XXX: mediate getting current series through some context object? this gets the job done for now
+	refs = append(refs, &Ref{SnapDeclarationType, []string{release.Series, snapdev.SnapID()}})
+
+	// the publisher and developers
+	publisherID := snapdev.PublisherID()
+	refs = append(refs, &Ref{AccountType, []string{publisherID}})
+	for developerID := range snapdev.developerRanges {
+		if developerID != publisherID {
+			refs = append(refs, &Ref{AccountType, []string{developerID}})
+		}
+	}
+
+	return refs
+}
+
+func assembleSnapDeveloper(assert assertionBase) (Assertion, error) {
+	developerRanges, err := checkDevelopers(assert.headers)
+	if err != nil {
+		return nil, err
+	}
+
+	return &SnapDeveloper{
+		assertionBase:   assert,
+		developerRanges: developerRanges,
+	}, nil
+}
+
+func checkDevelopers(headers map[string]interface{}) (map[string][]*dateRange, error) {
+	value, ok := headers["developers"]
+	if !ok {
+		return nil, nil
+	}
+	developers, ok := value.([]interface{})
+	if !ok {
+		return nil, fmt.Errorf(`"developers" must be a list of developer maps`)
+	}
+	if len(developers) == 0 {
+		return nil, nil
+	}
+
+	// Used to check for a developer with revoking and non-revoking items.
+	// No entry means developer not yet seen, false means seen but not revoked,
+	// true means seen and revoked.
+	revocationStatus := map[string]bool{}
+
+	developerRanges := make(map[string][]*dateRange)
+	for i, item := range developers {
+		developer, ok := item.(map[string]interface{})
+		if !ok {
+			return nil, fmt.Errorf(`"developers" must be a list of developer maps`)
+		}
+
+		what := fmt.Sprintf(`in "developers" item %d`, i+1)
+		accountID, err := checkStringMatchesWhat(developer, "developer-id", what, validAccountID)
+		if err != nil {
+			return nil, err
+		}
+
+		what = fmt.Sprintf(`in "developers" item %d for developer %q`, i+1, accountID)
+		since, err := checkRFC3339DateWhat(developer, "since", what)
+		if err != nil {
+			return nil, err
+		}
+		until, err := checkRFC3339DateWithDefaultWhat(developer, "until", what, time.Time{})
+		if err != nil {
+			return nil, err
+		}
+		if !until.IsZero() && since.After(until) {
+			return nil, fmt.Errorf(`"since" %s must be less than or equal to "until"`, what)
+		}
+
+		// Track/check for revocation conflicts.
+		revoked := since.Equal(until)
+		previouslyRevoked, ok := revocationStatus[accountID]
+		if !ok {
+			revocationStatus[accountID] = revoked
+		} else if previouslyRevoked || revoked {
+			return nil, fmt.Errorf(`revocation for developer %q must be standalone but found other "developers" items`, accountID)
+		}
+
+		developerRanges[accountID] = append(developerRanges[accountID], &dateRange{since, until})
+	}
+
+	return developerRanges, nil
+}
