@@ -33,6 +33,7 @@ import (
 	"time"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/jsonutil"
 )
 
 func unixDialer(socketPath string) func(string, string) (net.Conn, error) {
@@ -58,6 +59,11 @@ type Config struct {
 	// Authorization header from reading the auth.json data.
 	DisableAuth bool
 
+	// Interactive controls whether the client runs in interactive mode.
+	// At present, this only affects whether interactive polkit
+	// authorisation is requested.
+	Interactive bool
+
 	// Socket is the path to the unix socket to use
 	Socket string
 }
@@ -68,6 +74,7 @@ type Client struct {
 	doer    doer
 
 	disableAuth bool
+	interactive bool
 }
 
 // New returns a new instance of Client
@@ -87,6 +94,7 @@ func New(config *Config) *Client {
 				Transport: &http.Transport{Dial: unixDialer(config.Socket)},
 			},
 			disableAuth: config.DisableAuth,
+			interactive: config.Interactive,
 		}
 	}
 
@@ -98,9 +106,22 @@ func New(config *Config) *Client {
 		baseURL:     *baseURL,
 		doer:        &http.Client{},
 		disableAuth: config.DisableAuth,
+		interactive: config.Interactive,
 	}
 }
 
+func (client *Client) WhoAmI() (string, error) {
+	user, err := readAuthData()
+	if os.IsNotExist(err) {
+		return "", nil
+	}
+	if err != nil {
+		return "", err
+	}
+
+	return user.Email, nil
+}
+
 func (client *Client) setAuthorization(req *http.Request) error {
 	user, err := readAuthData()
 	if os.IsNotExist(err) {
@@ -137,6 +158,10 @@ func (e ConnectionError) Error() string {
 	return fmt.Sprintf("cannot communicate with server: %v", e.error)
 }
 
+// AllowInteractionHeader is the HTTP request header used to indicate
+// that the client is willing to allow interaction.
+const AllowInteractionHeader = "X-Allow-Interaction"
+
 // raw performs a request and returns the resulting http.Response and
 // error you usually only need to call this directly if you expect the
 // response to not be JSON, otherwise you'd call Do(...) instead.
@@ -162,6 +187,10 @@ func (client *Client) raw(method, urlpath string, query url.Values, headers map[
 		}
 	}
 
+	if client.interactive {
+		req.Header.Set(AllowInteractionHeader, "true")
+	}
+
 	rsp, err := client.doer.Do(req)
 	if err != nil {
 		return nil, ConnectionError{err}
@@ -244,7 +273,7 @@ func (client *Client) doSync(method, path string, query url.Values, headers map[
 	}
 
 	if v != nil {
-		if err := json.Unmarshal(rsp.Result, v); err != nil {
+		if err := jsonutil.DecodeWithNumber(bytes.NewReader(rsp.Result), v); err != nil {
 			return nil, fmt.Errorf("cannot unmarshal: %v", err)
 		}
 	}
@@ -264,7 +293,7 @@ func (client *Client) doAsync(method, path string, query url.Values, headers map
 	if rsp.Type != "async" {
 		return "", fmt.Errorf("expected async response for %q on %q, got %q", method, path, rsp.Type)
 	}
-	if rsp.StatusCode != http.StatusAccepted {
+	if rsp.StatusCode != 202 {
 		return "", fmt.Errorf("operation not accepted")
 	}
 	if rsp.Change == "" {
@@ -315,8 +344,9 @@ type response struct {
 
 // Error is the real value of response.Result when an error occurs.
 type Error struct {
-	Kind    string `json:"kind"`
-	Message string `json:"message"`
+	Kind    string      `json:"kind"`
+	Value   interface{} `json:"value"`
+	Message string      `json:"message"`
 
 	StatusCode int
 }
@@ -332,12 +362,21 @@ const (
 	ErrorKindTermsNotAccepted  = "terms-not-accepted"
 	ErrorKindNoPaymentMethods  = "no-payment-methods"
 	ErrorKindPaymentDeclined   = "payment-declined"
+	ErrorKindPasswordPolicy    = "password-policy"
 
-	ErrorKindSnapAlreadyInstalled = "snap-already-installed"
-	ErrorKindSnapNotInstalled     = "snap-not-installed"
-	ErrorKindNoUpdateAvailable    = "snap-no-update-available"
+	ErrorKindSnapAlreadyInstalled   = "snap-already-installed"
+	ErrorKindSnapNotInstalled       = "snap-not-installed"
+	ErrorKindSnapNotFound           = "snap-not-found"
+	ErrorKindSnapLocal              = "snap-local"
+	ErrorKindSnapNeedsDevMode       = "snap-needs-devmode"
+	ErrorKindSnapNeedsClassic       = "snap-needs-classic"
+	ErrorKindSnapNeedsClassicSystem = "snap-needs-classic-system"
+	ErrorKindNoUpdateAvailable      = "snap-no-update-available"
 
 	ErrorKindNotSnap = "snap-not-a-snap"
+
+	ErrorKindNetworkTimeout      = "network-timeout"
+	ErrorKindInterfacesUnchanged = "interfaces-unchanged"
 )
 
 // IsTwoFactorError returns whether the given error is due to problems
@@ -351,12 +390,33 @@ func IsTwoFactorError(err error) bool {
 	return e.Kind == ErrorKindTwoFactorFailed || e.Kind == ErrorKindTwoFactorRequired
 }
 
+// IsInterfacesUnchangedError returns whether the given error means the requested
+// change to interfaces was not made, because there was nothing to do.
+func IsInterfacesUnchangedError(err error) bool {
+	e, ok := err.(*Error)
+	if !ok || e == nil {
+		return false
+	}
+	return e.Kind == ErrorKindInterfacesUnchanged
+}
+
 // OSRelease contains information about the system extracted from /etc/os-release.
 type OSRelease struct {
 	ID        string `json:"id"`
 	VersionID string `json:"version-id,omitempty"`
 }
 
+// RefreshInfo contains information about refreshes.
+type RefreshInfo struct {
+	// Timer contains the refresh.timer setting.
+	Timer string `json:"timer,omitempty"`
+	// Schedule contains the legacy refresh.schedule setting.
+	Schedule string `json:"schedule,omitempty"`
+	Last     string `json:"last,omitempty"`
+	Hold     string `json:"hold,omitempty"`
+	Next     string `json:"next,omitempty"`
+}
+
 // SysInfo holds system information
 type SysInfo struct {
 	Series    string    `json:"series,omitempty"`
@@ -366,6 +426,9 @@ type SysInfo struct {
 	Managed   bool      `json:"managed"`
 
 	KernelVersion string `json:"kernel-version,omitempty"`
+
+	Refresh     RefreshInfo `json:"refresh,omitempty"`
+	Confinement string      `json:"confinement"`
 }
 
 func (rsp *response) err() error {
