@@ -28,7 +28,6 @@ import (
 	"mime"
 	"mime/multipart"
 	"path/filepath"
-	"strconv"
 
 	"gopkg.in/check.v1"
 
@@ -47,6 +46,7 @@ var ops = []struct {
 	{(*client.Client).Revert, "revert"},
 	{(*client.Client).Enable, "enable"},
 	{(*client.Client).Disable, "disable"},
+	{(*client.Client).Switch, "switch"},
 }
 
 var multiOps = []struct {
@@ -192,7 +192,6 @@ func (cs *clientSuite) TestClientOpInstallPath(c *check.C) {
 
 	c.Assert(string(body), check.Matches, "(?s).*\r\nsnap-data\r\n.*")
 	c.Assert(string(body), check.Matches, "(?s).*Content-Disposition: form-data; name=\"action\"\r\n\r\ninstall\r\n.*")
-	c.Assert(string(body), check.Matches, "(?s).*Content-Disposition: form-data; name=\"dangerous\"\r\n\r\nfalse\r\n.*")
 
 	c.Check(cs.req.Method, check.Equals, "POST")
 	c.Check(cs.req.URL.Path, check.Equals, fmt.Sprintf("/v2/snaps"))
@@ -260,30 +259,45 @@ func (cs *clientSuite) TestClientOpTryMode(c *check.C) {
 	snapdir := filepath.Join(c.MkDir(), "/some/path")
 
 	for _, opts := range []*client.SnapOptions{
-		{DevMode: false, JailMode: false},
-		{DevMode: false, JailMode: true},
-		{DevMode: true, JailMode: true},
-		{DevMode: true, JailMode: false},
+		{Classic: false, DevMode: false, JailMode: false},
+		{Classic: false, DevMode: false, JailMode: true},
+		{Classic: false, DevMode: true, JailMode: true},
+		{Classic: false, DevMode: true, JailMode: false},
+		{Classic: true, DevMode: false, JailMode: false},
+		{Classic: true, DevMode: false, JailMode: true},
+		{Classic: true, DevMode: true, JailMode: true},
+		{Classic: true, DevMode: true, JailMode: false},
 	} {
+		comment := check.Commentf("when Classic:%t DevMode:%t JailMode:%t", opts.Classic, opts.DevMode, opts.JailMode)
 		id, err := cs.cli.Try(snapdir, opts)
 		c.Assert(err, check.IsNil)
 
 		// ensure we send the right form-data
 		_, params, err := mime.ParseMediaType(cs.req.Header.Get("Content-Type"))
-		c.Assert(err, check.IsNil)
+		c.Assert(err, check.IsNil, comment)
 		mr := multipart.NewReader(cs.req.Body, params["boundary"])
 		formData := formToMap(c, mr)
-		c.Check(formData, check.DeepEquals, map[string]string{
-			"action":    "try",
-			"snap-path": snapdir,
-			"devmode":   strconv.FormatBool(opts.DevMode),
-			"jailmode":  strconv.FormatBool(opts.JailMode),
-		})
+		c.Check(formData["action"], check.Equals, "try", comment)
+		c.Check(formData["snap-path"], check.Equals, snapdir, comment)
+		expectedLength := 2
+		if opts.Classic {
+			c.Check(formData["classic"], check.Equals, "true", comment)
+			expectedLength++
+		}
+		if opts.DevMode {
+			c.Check(formData["devmode"], check.Equals, "true", comment)
+			expectedLength++
+		}
+		if opts.JailMode {
+			c.Check(formData["jailmode"], check.Equals, "true", comment)
+			expectedLength++
+		}
+		c.Check(len(formData), check.Equals, expectedLength)
 
-		c.Check(cs.req.Method, check.Equals, "POST")
-		c.Check(cs.req.URL.Path, check.Equals, fmt.Sprintf("/v2/snaps"))
-		c.Assert(cs.req.Header.Get("Content-Type"), check.Matches, "multipart/form-data; boundary=.*")
-		c.Check(id, check.Equals, "66b3")
+		c.Check(cs.req.Method, check.Equals, "POST", comment)
+		c.Check(cs.req.URL.Path, check.Equals, fmt.Sprintf("/v2/snaps"), comment)
+		c.Assert(cs.req.Header.Get("Content-Type"), check.Matches, "multipart/form-data; boundary=.*", comment)
+		c.Check(id, check.Equals, "66b3", comment)
 	}
 }
 
