@@ -56,6 +56,7 @@ void sc_init_apparmor_support(struct sc_apparmor *apparmor)
 		case ENOENT:
 			debug
 			    ("apparmor is available but the interface but the interface is not available");
+			break;
 		case EPERM:
 			// NOTE: fall-through
 		case EACCES:
@@ -73,7 +74,7 @@ void sc_init_apparmor_support(struct sc_apparmor *apparmor)
 	// Use aa_getcon() to check the label of the current process and
 	// confinement type. Note that the returned label must be released with
 	// free() but the mode is a constant string that must not be freed.
-	char *label __attribute__ ((cleanup(sc_cleanup_string))) = NULL;
+	char *label SC_CLEANUP(sc_cleanup_string) = NULL;
 	char *mode = NULL;
 	if (aa_getcon(&label, &mode) < 0) {
 		die("cannot query current apparmor profile");
@@ -109,6 +110,9 @@ void
 sc_maybe_aa_change_onexec(struct sc_apparmor *apparmor, const char *profile)
 {
 #ifdef HAVE_APPARMOR
+	if (apparmor->mode == SC_AA_NOT_APPLICABLE) {
+		return;
+	}
 	debug("requesting changing of apparmor profile on next exec to %s",
 	      profile);
 	if (aa_change_onexec(profile) < 0) {
@@ -124,6 +128,9 @@ sc_maybe_aa_change_hat(struct sc_apparmor *apparmor,
 		       const char *subprofile, unsigned long magic_token)
 {
 #ifdef HAVE_APPARMOR
+	if (apparmor->mode == SC_AA_NOT_APPLICABLE) {
+		return;
+	}
 	if (apparmor->is_confined) {
 		debug("changing apparmor hat to %s", subprofile);
 		if (aa_change_hat(subprofile, magic_token) < 0) {
