@@ -31,7 +31,7 @@
  *
  * This function should be called before sc_initialize_ns_groups().
  **/
-void sc_reassociate_with_pid1_mount_ns();
+void sc_reassociate_with_pid1_mount_ns(void);
 
 /**
  * Initialize namespace sharing.
@@ -44,10 +44,9 @@ void sc_reassociate_with_pid1_mount_ns();
  * used for storing preserved namespaces as bind-mounted files from the nsfs
  * filesystem (namespace filesystem).
  *
- * This function acquires a flock(2)-based lock to ensure that no other instance
- * of snap-confine attempts to do this concurrently. If a process dies for any
- * reason then the lock is released and other instances of snap-confine can
- * complete the initialization.
+ * This function should be called with a global lock (see sc_lock_global) held
+ * to ensure that no other instance of snap-confine attempts to do this
+ * concurrently.
  *
  * This function inspects /proc/self/mountinfo to determine if the directory
  * where namespaces are kept (/run/snapd/ns) is correctly prepared as described
@@ -55,7 +54,7 @@ void sc_reassociate_with_pid1_mount_ns();
  *
  * For more details see namespaces(7).
  **/
-void sc_initialize_ns_groups();
+void sc_initialize_ns_groups(void);
 
 /**
  * Data required to manage namespaces amongst a group of processes.
@@ -69,13 +68,18 @@ enum {
 /**
  * Open a namespace group.
  *
- * This will open and keep file descriptors for /run/snapd/ns/ as well as for
- * /run/snapd/ns/${group_name}.lock. The lock file is created if necessary but
- * is not locked until sc_lock_ns_mutex() is called.
+ * This will open and keep file descriptors for /run/snapd/ns/.
  *
  * If the flags argument is SC_NS_FAIL_GRACEFULLY then the function returns
  * NULL if the /run/snapd/ns directory doesn't exist. In all other cases it
  * calls die() and exits the process.
+ *
+ * The following methods should be called only while holding a lock protecting
+ * that specific snap namespace:
+ * - sc_create_or_join_ns_group()
+ * - sc_should_populate_ns_group()
+ * - sc_preserve_populated_ns_group()
+ * - sc_discard_preserved_ns_group()
  */
 struct sc_ns_group *sc_open_ns_group(const char *group_name,
 				     const unsigned flags);
@@ -87,30 +91,6 @@ struct sc_ns_group *sc_open_ns_group(const char *group_name,
  */
 void sc_close_ns_group(struct sc_ns_group *group);
 
-/**
- * Acquire exclusive lock to the namespace group.
- *
- * This will attempt to acquire an flock-based exclusive lock on the file
- * descriptor associated with /run/snapd/ns/${group_name}.lock. If the process
- * is killed while the lock is held the lock is automatically released by the
- * kernel.
- *
- * The following methods should be called only while holding the lock:
- * - sc_create_or_join_ns_group()
- * - sc_should_populate_ns_group()
- * - sc_preserve_populated_ns_group()
- * - sc_discard_preserved_ns_group()
- **/
-void sc_lock_ns_mutex(struct sc_ns_group *group);
-
-/**
- * Release lock to the namespace group.
- *
- * This will attempt to release a flock-based lock on the file descriptor
- * associated with /run/snapd/ns/${group_name}.lock.
- **/
-void sc_unlock_ns_mutex(struct sc_ns_group *group);
-
 /**
  * Join the mount namespace associated with this group if one exists.
  *
@@ -125,10 +105,13 @@ void sc_unlock_ns_mutex(struct sc_ns_group *group);
  * the parent process unshares the mount namespace and sets a flag so that
  * sc_should_populate_ns_group() returns true.
  *
- * @returns true if the mount namespace needs to be populated
+ * @returns 0 on success and EAGAIN if the namespace was stale and needs
+ * to be re-made.
  **/
-void sc_create_or_join_ns_group(struct sc_ns_group *group,
-				struct sc_apparmor *apparmor);
+int sc_create_or_join_ns_group(struct sc_ns_group *group,
+			       struct sc_apparmor *apparmor,
+			       const char *base_snap_name,
+			       const char *snap_name);
 
 /**
  * Check if the namespace needs to be populated.
