@@ -46,6 +46,7 @@ func init() {
 		return &cmdBuy{}
 	}, map[string]string{}, []argDesc{{
 		name: "<snap>",
+		// TRANSLATORS: This should probably not start with a lowercase letter.
 		desc: i18n.G("Snap name"),
 	}})
 }
@@ -94,11 +95,13 @@ func buySnap(snapName string) error {
 		if e, ok := err.(*client.Error); ok {
 			switch e.Kind {
 			case client.ErrorKindNoPaymentMethods:
-				return fmt.Errorf(i18n.G(`You do not have a payment method associated with your account, visit https://my.ubuntu.com/payment/edit to add one.
-Once completed, return here and run 'snap buy %s' again.`), snap.Name)
+				return fmt.Errorf(i18n.G(`You need to have a payment method associated with your account in order to buy a snap, please visit https://my.ubuntu.com/payment/edit to add one.
+
+Once you’ve added your payment details, you just need to run 'snap buy %s' again.`), snap.Name)
 			case client.ErrorKindTermsNotAccepted:
-				return fmt.Errorf(i18n.G(`Please visit https://my.ubuntu.com/payment/edit to agree to the latest terms and conditions.
-Once completed, return here and run 'snap buy %s' again.`), snap.Name)
+				return fmt.Errorf(i18n.G(`In order to buy %q, you need to agree to the latest terms and conditions. Please visit https://my.ubuntu.com/payment/edit to do this.
+
+Once completed, return here and run 'snap buy %s' again.`), snap.Name, snap.Name)
 			}
 		}
 		return err
