@@ -22,6 +22,7 @@ package main
 import (
 	"fmt"
 
+	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/i18n"
 
 	"github.com/jessevdk/go-flags"
@@ -29,8 +30,8 @@ import (
 
 type cmdDisconnect struct {
 	Positionals struct {
-		Offer SnapAndName `required:"true"`
-		Use   SnapAndName
+		Offer disconnectSlotOrPlugSpec `required:"true"`
+		Use   disconnectSlotSpec
 	} `positional-args:"true"`
 }
 
@@ -53,7 +54,9 @@ func init() {
 	addCommand("disconnect", shortDisconnectHelp, longDisconnectHelp, func() flags.Commander {
 		return &cmdDisconnect{}
 	}, nil, []argDesc{
+		// TRANSLATORS: This needs to be wrapped in <>s.
 		{name: i18n.G("<snap>:<plug>")},
+		// TRANSLATORS: This needs to be wrapped in <>s.
 		{name: i18n.G("<snap>:<slot>")},
 	})
 }
@@ -63,19 +66,27 @@ func (x *cmdDisconnect) Execute(args []string) error {
 		return ErrExtraArgs
 	}
 
+	offer := x.Positionals.Offer.SnapAndName
+	use := x.Positionals.Use.SnapAndName
+
 	// snap disconnect <snap>:<slot>
 	// snap disconnect <snap>
-	if x.Positionals.Use.Snap == "" && x.Positionals.Use.Name == "" {
+	if use.Snap == "" && use.Name == "" {
 		// Swap Offer and Use around
-		x.Positionals.Offer, x.Positionals.Use = x.Positionals.Use, x.Positionals.Offer
+		offer, use = use, offer
 	}
-	if x.Positionals.Use.Name == "" {
-		return fmt.Errorf("please provide the plug or slot name to disconnect from snap %q", x.Positionals.Use.Snap)
+	if use.Name == "" {
+		return fmt.Errorf("please provide the plug or slot name to disconnect from snap %q", use.Snap)
 	}
 
 	cli := Client()
-	id, err := cli.Disconnect(x.Positionals.Offer.Snap, x.Positionals.Offer.Name, x.Positionals.Use.Snap, x.Positionals.Use.Name)
+	id, err := cli.Disconnect(offer.Snap, offer.Name, use.Snap, use.Name)
 	if err != nil {
+		if client.IsInterfacesUnchangedError(err) {
+			fmt.Fprintf(Stdout, i18n.G("No connections to disconnect"))
+			fmt.Fprintf(Stdout, "\n")
+			return nil
+		}
 		return err
 	}
 
