@@ -20,9 +20,9 @@
 package main
 
 import (
-	"github.com/jessevdk/go-flags"
+	"fmt"
 
-	"github.com/ubuntu-core/snappy/overlord"
+	"github.com/jessevdk/go-flags"
 )
 
 type cmdInternalFirstBoot struct{}
@@ -32,10 +32,18 @@ func init() {
 		"internal",
 		"internal", func() flags.Commander {
 			return &cmdInternalFirstBoot{}
-		})
+		}, nil, nil)
 	cmd.hidden = true
 }
 
+// WARNING: do not remove this command, older systems may still have
+//          a systemd snapd.firstboot.service job in /etc/systemd/system
+//          that we did not cleanup. so we need this dummy command or
+//          those units will start failing.
 func (x *cmdInternalFirstBoot) Execute(args []string) error {
-	return overlord.FirstBoot()
+	if len(args) > 0 {
+		return ErrExtraArgs
+	}
+	fmt.Fprintf(Stderr, "firstboot command is deprecated\n")
+	return nil
 }
