@@ -22,52 +22,16 @@ package main_test
 import (
 	"io/ioutil"
 	"net/http"
+	"os"
 
+	"github.com/jessevdk/go-flags"
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/client"
 	. "github.com/snapcore/snapd/cmd/snap"
 )
 
-func (s *SnapSuite) TestInterfacesHelp(c *C) {
-	msg := `Usage:
-  snap.test [OPTIONS] interfaces [interfaces-OPTIONS] [<snap>:<slot or plug>]
-
-The interfaces command lists interfaces available in the system.
-
-By default all slots and plugs, used and offered by all snaps, are displayed.
-
-$ snap interfaces <snap>:<slot or plug>
-
-Lists only the specified slot or plug.
-
-$ snap interfaces <snap>
-
-Lists the slots offered and plugs used by the specified snap.
-
-$ snap interfaces -i=<interface> [<snap>]
-
-Filters the complete output so only plugs and/or slots matching the provided
-details are listed.
-
-Application Options:
-      --version                    Print the version and exit
-
-Help Options:
-  -h, --help                       Show this help message
-
-[interfaces command options]
-      -i=                          Constrain listing to specific interfaces
-
-[interfaces command arguments]
-  <snap>:<slot or plug>:           Constrain listing to a specific snap or snap:name
-`
-	rest, err := Parser().ParseArgs([]string{"interfaces", "--help"})
-	c.Assert(err.Error(), Equals, msg)
-	c.Assert(rest, DeepEquals, []string{})
-}
-
-func (s *SnapSuite) TestInterfacesZeroSlotsOnePlug(c *C) {
+func (s *SnapSuite) TestConnectionsZeroSlotsOnePlug(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -76,7 +40,7 @@ func (s *SnapSuite) TestInterfacesZeroSlotsOnePlug(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Plugs: []client.Plug{
 					{
 						Snap: "keyboard-lights",
@@ -96,7 +60,7 @@ func (s *SnapSuite) TestInterfacesZeroSlotsOnePlug(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesZeroPlugsOneSlot(c *C) {
+func (s *SnapSuite) TestConnectionsZeroPlugsOneSlot(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -105,7 +69,7 @@ func (s *SnapSuite) TestInterfacesZeroPlugsOneSlot(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "canonical-pi2",
@@ -127,7 +91,7 @@ func (s *SnapSuite) TestInterfacesZeroPlugsOneSlot(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesOneSlotOnePlug(c *C) {
+func (s *SnapSuite) TestConnectionsOneSlotOnePlug(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -136,7 +100,7 @@ func (s *SnapSuite) TestInterfacesOneSlotOnePlug(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "canonical-pi2",
@@ -194,7 +158,7 @@ func (s *SnapSuite) TestInterfacesOneSlotOnePlug(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesTwoPlugs(c *C) {
+func (s *SnapSuite) TestConnectionsTwoPlugs(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -203,7 +167,7 @@ func (s *SnapSuite) TestInterfacesTwoPlugs(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "canonical-pi2",
@@ -235,7 +199,7 @@ func (s *SnapSuite) TestInterfacesTwoPlugs(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesPlugsWithCommonName(c *C) {
+func (s *SnapSuite) TestConnectionsPlugsWithCommonName(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -244,7 +208,7 @@ func (s *SnapSuite) TestInterfacesPlugsWithCommonName(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "canonical-pi2",
@@ -302,7 +266,7 @@ func (s *SnapSuite) TestInterfacesPlugsWithCommonName(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesOsSnapSlots(c *C) {
+func (s *SnapSuite) TestConnectionsOsSnapSlots(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -311,7 +275,7 @@ func (s *SnapSuite) TestInterfacesOsSnapSlots(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "core",
@@ -369,7 +333,7 @@ func (s *SnapSuite) TestInterfacesOsSnapSlots(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesTwoSlotsAndFiltering(c *C) {
+func (s *SnapSuite) TestConnectionsTwoSlotsAndFiltering(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -378,7 +342,7 @@ func (s *SnapSuite) TestInterfacesTwoSlotsAndFiltering(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "canonical-pi2",
@@ -418,7 +382,7 @@ func (s *SnapSuite) TestInterfacesTwoSlotsAndFiltering(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesOfSpecificSnap(c *C) {
+func (s *SnapSuite) TestConnectionsOfSpecificSnap(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -427,7 +391,7 @@ func (s *SnapSuite) TestInterfacesOfSpecificSnap(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "cheese",
@@ -462,7 +426,7 @@ func (s *SnapSuite) TestInterfacesOfSpecificSnap(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesOfSpecificSnapAndSlot(c *C) {
+func (s *SnapSuite) TestConnectionsOfSpecificSnapAndSlot(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -471,7 +435,7 @@ func (s *SnapSuite) TestInterfacesOfSpecificSnapAndSlot(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type": "sync",
-			"result": client.Interfaces{
+			"result": client.Connections{
 				Slots: []client.Slot{
 					{
 						Snap:      "cheese",
@@ -505,7 +469,7 @@ func (s *SnapSuite) TestInterfacesOfSpecificSnapAndSlot(c *C) {
 	c.Assert(s.Stderr(), Equals, "")
 }
 
-func (s *SnapSuite) TestInterfacesNothingAtAll(c *C) {
+func (s *SnapSuite) TestConnectionsNothingAtAll(c *C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, Equals, "GET")
 		c.Check(r.URL.Path, Equals, "/v2/interfaces")
@@ -514,7 +478,7 @@ func (s *SnapSuite) TestInterfacesNothingAtAll(c *C) {
 		c.Check(body, DeepEquals, []byte{})
 		EncodeResponseBody(c, w, map[string]interface{}{
 			"type":   "sync",
-			"result": client.Interfaces{},
+			"result": client.Connections{},
 		})
 	})
 	rest, err := Parser().ParseArgs([]string{"interfaces"})
@@ -525,3 +489,86 @@ func (s *SnapSuite) TestInterfacesNothingAtAll(c *C) {
 	c.Assert(s.Stdout(), Equals, "")
 	c.Assert(s.Stderr(), Equals, "")
 }
+
+func (s *SnapSuite) TestConnectionsOfSpecificType(c *C) {
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		c.Check(r.Method, Equals, "GET")
+		c.Check(r.URL.Path, Equals, "/v2/interfaces")
+		body, err := ioutil.ReadAll(r.Body)
+		c.Check(err, IsNil)
+		c.Check(body, DeepEquals, []byte{})
+		EncodeResponseBody(c, w, map[string]interface{}{
+			"type": "sync",
+			"result": client.Connections{
+				Slots: []client.Slot{
+					{
+						Snap:      "cheese",
+						Name:      "photo-trigger",
+						Interface: "bool-file",
+						Label:     "Photo trigger",
+					},
+					{
+						Snap:      "wake-up-alarm",
+						Name:      "toggle",
+						Interface: "bool-file",
+						Label:     "Alarm toggle",
+					},
+					{
+						Snap:      "wake-up-alarm",
+						Name:      "snooze",
+						Interface: "bool-file",
+						Label:     "Alarm snooze",
+					},
+				},
+			},
+		})
+	})
+	rest, err := Parser().ParseArgs([]string{"interfaces", "-i", "bool-file"})
+	c.Assert(err, IsNil)
+	c.Assert(rest, DeepEquals, []string{})
+	expectedStdout := "" +
+		"Slot                  Plug\n" +
+		"cheese:photo-trigger  -\n" +
+		"wake-up-alarm:toggle  -\n" +
+		"wake-up-alarm:snooze  -\n"
+	c.Assert(s.Stdout(), Equals, expectedStdout)
+	c.Assert(s.Stderr(), Equals, "")
+}
+
+func (s *SnapSuite) TestConnectionsCompletion(c *C) {
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		switch r.URL.Path {
+		case "/v2/interfaces":
+			c.Assert(r.Method, Equals, "GET")
+			EncodeResponseBody(c, w, map[string]interface{}{
+				"type":   "sync",
+				"result": fortestingConnectionList,
+			})
+		default:
+			c.Fatalf("unexpected path %q", r.URL.Path)
+		}
+	})
+	os.Setenv("GO_FLAGS_COMPLETION", "verbose")
+	defer os.Unsetenv("GO_FLAGS_COMPLETION")
+
+	expected := []flags.Completion{}
+	parser := Parser()
+	parser.CompletionHandler = func(obtained []flags.Completion) {
+		c.Check(obtained, DeepEquals, expected)
+	}
+
+	expected = []flags.Completion{{Item: "canonical-pi2:"}, {Item: "core:"}, {Item: "keyboard-lights:"}, {Item: "paste-daemon:"}, {Item: "potato:"}, {Item: "wake-up-alarm:"}}
+	_, err := parser.ParseArgs([]string{"interfaces", ""})
+	c.Assert(err, IsNil)
+
+	expected = []flags.Completion{{Item: "paste-daemon:network-listening", Description: "plug"}}
+	_, err = parser.ParseArgs([]string{"interfaces", "pa"})
+	c.Assert(err, IsNil)
+
+	expected = []flags.Completion{{Item: "wake-up-alarm:toggle", Description: "slot"}}
+	_, err = parser.ParseArgs([]string{"interfaces", "wa"})
+	c.Assert(err, IsNil)
+
+	c.Assert(s.Stdout(), Equals, "")
+	c.Assert(s.Stderr(), Equals, "")
+}
