@@ -56,7 +56,7 @@ func (s *SnapSuite) TestList(c *check.C) {
 			c.Check(r.Method, check.Equals, "GET")
 			c.Check(r.URL.Path, check.Equals, "/v2/snaps")
 			c.Check(r.URL.RawQuery, check.Equals, "")
-			fmt.Fprintln(w, `{"type": "sync", "result": [{"name": "foo", "status": "active", "version": "4.2", "developer": "bar", "revision":17}]}`)
+			fmt.Fprintln(w, `{"type": "sync", "result": [{"name": "foo", "status": "active", "version": "4.2", "developer": "bar", "revision":17, "tracking-channel": "potatoes"}]}`)
 		default:
 			c.Fatalf("expected to get 1 requests, now on %d", n+1)
 		}
@@ -66,8 +66,8 @@ func (s *SnapSuite) TestList(c *check.C) {
 	rest, err := snap.Parser().ParseArgs([]string{"list"})
 	c.Assert(err, check.IsNil)
 	c.Assert(rest, check.DeepEquals, []string{})
-	c.Check(s.Stdout(), check.Matches, `Name +Version +Rev +Developer +Notes
-foo +4.2 +17 +bar +-
+	c.Check(s.Stdout(), check.Matches, `Name +Version +Rev +Tracking +Developer +Notes
+foo +4.2 +17 +potatoes +bar +-
 `)
 	c.Check(s.Stderr(), check.Equals, "")
 }
@@ -80,7 +80,7 @@ func (s *SnapSuite) TestListAll(c *check.C) {
 			c.Check(r.Method, check.Equals, "GET")
 			c.Check(r.URL.Path, check.Equals, "/v2/snaps")
 			c.Check(r.URL.RawQuery, check.Equals, "select=all")
-			fmt.Fprintln(w, `{"type": "sync", "result": [{"name": "foo", "status": "active", "version": "4.2", "developer": "bar", "revision":17}]}`)
+			fmt.Fprintln(w, `{"type": "sync", "result": [{"name": "foo", "status": "active", "version": "4.2", "developer": "bar", "revision":17, "tracking-channel": "stable"}]}`)
 		default:
 			c.Fatalf("expected to get 1 requests, now on %d", n+1)
 		}
@@ -90,8 +90,8 @@ func (s *SnapSuite) TestListAll(c *check.C) {
 	rest, err := snap.Parser().ParseArgs([]string{"list", "--all"})
 	c.Assert(err, check.IsNil)
 	c.Assert(rest, check.DeepEquals, []string{})
-	c.Check(s.Stdout(), check.Matches, `Name +Version +Rev +Developer +Notes
-foo +4.2 +17 +bar +-
+	c.Check(s.Stdout(), check.Matches, `Name +Version +Rev +Tracking +Developer +Notes
+foo +4.2 +17 +stable +bar +-
 `)
 	c.Check(s.Stderr(), check.Equals, "")
 }
@@ -142,7 +142,8 @@ func (s *SnapSuite) TestListWithNoMatchingQuery(c *check.C) {
 		case 0:
 			c.Check(r.Method, check.Equals, "GET")
 			c.Check(r.URL.Path, check.Equals, "/v2/snaps")
-			fmt.Fprintln(w, `{"type": "sync", "result": [{"name": "foo", "status": "active", "version": "4.2", "developer": "bar", "revision":17}]}`)
+			c.Check(r.URL.Query().Get("snaps"), check.Equals, "quux")
+			fmt.Fprintln(w, `{"type": "sync", "result": []}`)
 		default:
 			c.Fatalf("expected to get 1 requests, now on %d", n+1)
 		}
@@ -160,8 +161,8 @@ func (s *SnapSuite) TestListWithQuery(c *check.C) {
 		case 0:
 			c.Check(r.Method, check.Equals, "GET")
 			c.Check(r.URL.Path, check.Equals, "/v2/snaps")
-			c.Check(r.URL.Query(), check.HasLen, 0)
-			fmt.Fprintln(w, `{"type": "sync", "result": [{"name": "foo", "status": "active", "version": "4.2", "developer": "bar", "revision":17}]}`)
+			c.Check(r.URL.Query().Get("snaps"), check.Equals, "foo")
+			fmt.Fprintln(w, `{"type": "sync", "result": [{"name": "foo", "status": "active", "version": "4.2", "developer": "bar", "revision":17, "tracking-channel": "1.10/stable/fix1234"}]}`)
 		default:
 			c.Fatalf("expected to get 1 requests, now on %d", n+1)
 		}
@@ -171,8 +172,8 @@ func (s *SnapSuite) TestListWithQuery(c *check.C) {
 	rest, err := snap.Parser().ParseArgs([]string{"list", "foo"})
 	c.Assert(err, check.IsNil)
 	c.Assert(rest, check.DeepEquals, []string{})
-	c.Check(s.Stdout(), check.Matches, `Name +Version +Rev +Developer +Notes
-foo +4.2 +17 +bar +-
+	c.Check(s.Stdout(), check.Matches, `Name +Version +Rev +Tracking +Developer +Notes
+foo +4.2 +17 +1\.10/stable/… +bar +-
 `)
 	c.Check(s.Stderr(), check.Equals, "")
 	// ensure that the fake server api was actually hit
@@ -201,10 +202,49 @@ func (s *SnapSuite) TestListWithNotes(c *check.C) {
 	rest, err := snap.Parser().ParseArgs([]string{"list"})
 	c.Assert(err, check.IsNil)
 	c.Assert(rest, check.DeepEquals, []string{})
-	c.Check(s.Stdout(), check.Matches, `(?ms)^Name +Version +Rev +Developer +Notes$`)
-	c.Check(s.Stdout(), check.Matches, `(?ms).*^foo +4.2 +17 +bar +try$`)
+	c.Check(s.Stdout(), check.Matches, `(?ms)^Name +Version +Rev +Tracking +Developer +Notes$`)
+	c.Check(s.Stdout(), check.Matches, `(?ms).*^foo +4.2 +17 +- +bar +try$`)
 	c.Check(s.Stdout(), check.Matches, `(?ms).*^dm1 +.* +devmode$`)
 	c.Check(s.Stdout(), check.Matches, `(?ms).*^dm2 +.* +devmode$`)
 	c.Check(s.Stdout(), check.Matches, `(?ms).*^cf1 +.* +jailmode$`)
 	c.Check(s.Stderr(), check.Equals, "")
 }
+
+func (s *SnapSuite) TestFormatChannel(c *check.C) {
+	type tableT struct {
+		channel  string
+		expected string
+	}
+	for _, t := range []tableT{
+		{"", "-"},
+		{"stable", "stable"},
+		{"edge", "edge"},
+		{"foo/stable", "foo"},
+		{"foo/edge", "foo/edge"},
+		{"foo", "foo"},
+		{"foo/stable/bar", "foo/stable/…"},
+		{"foo/edge/bar", "foo/edge/…"},
+		{"stable/bar", "stable/…"},
+		{"edge/bar", "edge/…"},
+	} {
+		c.Check(snap.FormatChannel(t.channel), check.Equals, t.expected, check.Commentf(t.channel))
+	}
+
+	// and some SISO tests just to check it doesn't panic nor return empty string
+	// (the former would break scripts)
+	for _, ch := range []string{
+		"",
+		"\x00",
+		"/",
+		"//",
+		"///",
+		"////",
+		"a/",
+		"/b",
+		"a//b",
+		"/stable",
+		"/edge",
+	} {
+		c.Check(snap.FormatChannel(ch), check.Not(check.Equals), "", check.Commentf(ch))
+	}
+}
