@@ -26,7 +26,6 @@ import (
 	"mime/multipart"
 	"net/http"
 	"net/http/httptest"
-	"os"
 	"path/filepath"
 	"regexp"
 	"time"
@@ -35,6 +34,9 @@ import (
 
 	"github.com/snapcore/snapd/client"
 	snap "github.com/snapcore/snapd/cmd/snap"
+	"github.com/snapcore/snapd/progress"
+	"github.com/snapcore/snapd/progress/progresstest"
+	"github.com/snapcore/snapd/testutil"
 )
 
 type snapOpTestServer struct {
@@ -53,7 +55,7 @@ func (t *snapOpTestServer) handle(w http.ResponseWriter, r *http.Request) {
 	case 0:
 		t.checker(r)
 		t.c.Check(r.Method, check.Equals, "POST")
-		w.WriteHeader(http.StatusAccepted)
+		w.WriteHeader(202)
 		fmt.Fprintln(w, `{"type":"async", "change": "42", "status-code": 202}`)
 	case 1:
 		t.c.Check(r.Method, check.Equals, "GET")
@@ -103,6 +105,8 @@ func (s *SnapOpSuite) TearDownTest(c *check.C) {
 }
 
 func (s *SnapOpSuite) TestWait(c *check.C) {
+	meter := &progresstest.Meter{}
+	defer progress.MockMeter(meter)()
 	restore := snap.MockMaxGoneTime(time.Millisecond)
 	defer restore()
 
@@ -111,27 +115,16 @@ func (s *SnapOpSuite) TestWait(c *check.C) {
 	snap.ClientConfig.BaseURL = server.URL
 	server.Close()
 
-	d := c.MkDir()
-	oldStdout := os.Stdout
-	stdout, err := ioutil.TempFile(d, "stdout")
-	c.Assert(err, check.IsNil)
-	defer func() {
-		os.Stdout = oldStdout
-		stdout.Close()
-		os.Remove(stdout.Name())
-	}()
-	os.Stdout = stdout
-
 	cli := snap.Client()
 	chg, err := snap.Wait(cli, "x")
 	c.Assert(chg, check.IsNil)
 	c.Assert(err, check.NotNil)
-	buf, err := ioutil.ReadFile(stdout.Name())
-	c.Assert(err, check.IsNil)
-	c.Check(string(buf), check.Matches, "(?ms).*Waiting for server to restart.*")
+	c.Check(meter.Labels, testutil.Contains, "Waiting for server to restart")
 }
 
 func (s *SnapOpSuite) TestWaitRecovers(c *check.C) {
+	meter := &progresstest.Meter{}
+	defer progress.MockMeter(meter)()
 	restore := snap.MockMaxGoneTime(time.Millisecond)
 	defer restore()
 
@@ -144,27 +137,14 @@ func (s *SnapOpSuite) TestWaitRecovers(c *check.C) {
 		fmt.Fprintln(w, `{"type": "sync", "result": {"ready": true, "status": "Done"}}`)
 	})
 
-	d := c.MkDir()
-	oldStdout := os.Stdout
-	stdout, err := ioutil.TempFile(d, "stdout")
-	c.Assert(err, check.IsNil)
-	defer func() {
-		os.Stdout = oldStdout
-		stdout.Close()
-		os.Remove(stdout.Name())
-	}()
-	os.Stdout = stdout
-
 	cli := snap.Client()
 	chg, err := snap.Wait(cli, "x")
 	// we got the change
 	c.Assert(chg, check.NotNil)
 	c.Assert(err, check.IsNil)
-	buf, err := ioutil.ReadFile(stdout.Name())
-	c.Assert(err, check.IsNil)
 
 	// but only after recovering
-	c.Check(string(buf), check.Matches, "(?ms).*Waiting for server to restart.*")
+	c.Check(meter.Labels, testutil.Contains, "Waiting for server to restart")
 }
 
 func (s *SnapOpSuite) TestInstall(c *check.C) {
@@ -172,16 +152,57 @@ func (s *SnapOpSuite) TestInstall(c *check.C) {
 		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
 		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
 			"action":  "install",
-			"channel": "chan",
+			"channel": "candidate",
 		})
-		s.srv.channel = "chan"
+		s.srv.channel = "candidate"
 	}
 
 	s.RedirectClientToTestServer(s.srv.handle)
-	rest, err := snap.Parser().ParseArgs([]string{"install", "--channel", "chan", "foo"})
+	rest, err := snap.Parser().ParseArgs([]string{"install", "--channel", "candidate", "foo"})
 	c.Assert(err, check.IsNil)
 	c.Assert(rest, check.DeepEquals, []string{})
-	c.Check(s.Stdout(), check.Matches, `(?sm).*foo \(chan\) 1.0 from 'bar' installed`)
+	c.Check(s.Stdout(), check.Matches, `(?sm).*foo \(candidate\) 1.0 from 'bar' installed`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(s.srv.n, check.Equals, s.srv.total)
+}
+
+func (s *SnapOpSuite) TestInstallFromTrack(c *check.C) {
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action":  "install",
+			"channel": "3.4/stable",
+		})
+		s.srv.channel = "3.4/stable"
+	}
+
+	s.RedirectClientToTestServer(s.srv.handle)
+	// snap install --channel=3.4 means 3.4/stable, this is what we test here
+	rest, err := snap.Parser().ParseArgs([]string{"install", "--channel", "3.4", "foo"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Matches, `(?sm).*foo \(3.4/stable\) 1.0 from 'bar' installed`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(s.srv.n, check.Equals, s.srv.total)
+}
+
+func (s *SnapOpSuite) TestInstallFromBranch(c *check.C) {
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action":  "install",
+			"channel": "3.4/hotfix-1",
+		})
+		s.srv.channel = "3.4/hotfix-1"
+	}
+
+	s.RedirectClientToTestServer(s.srv.handle)
+	rest, err := snap.Parser().ParseArgs([]string{"install", "--channel", "3.4/hotfix-1", "foo"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Matches, `(?sm).*foo \(3.4/hotfix-1\) 1.0 from 'bar' installed`)
 	c.Check(s.Stderr(), check.Equals, "")
 	// ensure that the fake server api was actually hit
 	c.Check(s.srv.n, check.Equals, s.srv.total)
@@ -193,16 +214,16 @@ func (s *SnapOpSuite) TestInstallDevMode(c *check.C) {
 		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
 			"action":  "install",
 			"devmode": true,
-			"channel": "chan",
+			"channel": "beta",
 		})
-		s.srv.channel = "chan"
+		s.srv.channel = "beta"
 	}
 
 	s.RedirectClientToTestServer(s.srv.handle)
-	rest, err := snap.Parser().ParseArgs([]string{"install", "--channel", "chan", "--devmode", "foo"})
+	rest, err := snap.Parser().ParseArgs([]string{"install", "--channel", "beta", "--devmode", "foo"})
 	c.Assert(err, check.IsNil)
 	c.Assert(rest, check.DeepEquals, []string{})
-	c.Check(s.Stdout(), check.Matches, `(?sm).*foo \(chan\) 1.0 from 'bar' installed`)
+	c.Check(s.Stdout(), check.Matches, `(?sm).*foo \(beta\) 1.0 from 'bar' installed`)
 	c.Check(s.Stderr(), check.Equals, "")
 	// ensure that the fake server api was actually hit
 	c.Check(s.srv.n, check.Equals, s.srv.total)
@@ -227,6 +248,25 @@ func (s *SnapOpSuite) TestInstallClassic(c *check.C) {
 	c.Check(s.srv.n, check.Equals, s.srv.total)
 }
 
+func (s *SnapOpSuite) TestInstallUnaliased(c *check.C) {
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action":    "install",
+			"unaliased": true,
+		})
+	}
+
+	s.RedirectClientToTestServer(s.srv.handle)
+	rest, err := snap.Parser().ParseArgs([]string{"install", "--unaliased", "foo"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Matches, `(?sm).*foo 1.0 from 'bar' installed`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(s.srv.n, check.Equals, s.srv.total)
+}
+
 func testForm(r *http.Request, c *check.C) *multipart.Form {
 	contentType := r.Header.Get("Content-Type")
 	mediaType, params, err := mime.ParseMediaType(contentType)
@@ -383,6 +423,29 @@ func (s *SnapOpSuite) TestInstallPathDangerous(c *check.C) {
 	c.Check(s.srv.n, check.Equals, s.srv.total)
 }
 
+func (s *SnapOpSuite) TestRevertRunthrough(c *check.C) {
+	s.srv.total = 4
+	s.srv.channel = "potato"
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action": "revert",
+		})
+	}
+
+	s.RedirectClientToTestServer(s.srv.handle)
+	rest, err := snap.Parser().ParseArgs([]string{"revert", "foo"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	// tracking channel is "" in the test server
+	c.Check(s.Stdout(), check.Equals, `foo reverted to 1.0
+Snap foo is no longer tracking .
+`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(s.srv.n, check.Equals, s.srv.total)
+}
+
 func (s *SnapOpSuite) runRevertTest(c *check.C, opts *client.SnapOptions) {
 	modes := []struct {
 		enabled bool
@@ -446,6 +509,12 @@ func (s *SnapOpSuite) TestRevertClassic(c *check.C) {
 	s.runRevertTest(c, &client.SnapOptions{Classic: true})
 }
 
+func (s *SnapOpSuite) TestRevertMissingName(c *check.C) {
+	_, err := snap.Parser().ParseArgs([]string{"revert"})
+	c.Assert(err, check.NotNil)
+	c.Assert(err, check.ErrorMatches, "the required argument `<snap>` was not provided")
+}
+
 func (s *SnapSuite) TestRefreshList(c *check.C) {
 	n := 0
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
@@ -472,6 +541,95 @@ foo +4.2update1 +17 +bar +-.*
 	c.Check(n, check.Equals, 1)
 }
 
+func (s *SnapSuite) TestRefreshLegacyTime(c *check.C) {
+	n := 0
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		switch n {
+		case 0:
+			c.Check(r.Method, check.Equals, "GET")
+			c.Check(r.URL.Path, check.Equals, "/v2/system-info")
+			fmt.Fprintln(w, `{"type": "sync", "status-code": 200, "result": {"refresh": {"schedule": "00:00-04:59/5:00-10:59/11:00-16:59/17:00-23:59", "last": "2017-04-25T17:35:00+0200", "next": "2017-04-26T00:58:00+0200"}}}`)
+		default:
+			c.Fatalf("expected to get 1 requests, now on %d", n+1)
+		}
+
+		n++
+	})
+	rest, err := snap.Parser().ParseArgs([]string{"refresh", "--time"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Equals, `schedule: 00:00-04:59/5:00-10:59/11:00-16:59/17:00-23:59
+last: 2017-04-25T17:35:00+0200
+next: 2017-04-26T00:58:00+0200
+`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(n, check.Equals, 1)
+}
+
+func (s *SnapSuite) TestRefreshTimer(c *check.C) {
+	n := 0
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		switch n {
+		case 0:
+			c.Check(r.Method, check.Equals, "GET")
+			c.Check(r.URL.Path, check.Equals, "/v2/system-info")
+			fmt.Fprintln(w, `{"type": "sync", "status-code": 200, "result": {"refresh": {"timer": "0:00-24:00/4", "last": "2017-04-25T17:35:00+0200", "next": "2017-04-26T00:58:00+0200"}}}`)
+		default:
+			c.Fatalf("expected to get 1 requests, now on %d", n+1)
+		}
+
+		n++
+	})
+	rest, err := snap.Parser().ParseArgs([]string{"refresh", "--time"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Equals, `timer: 0:00-24:00/4
+last: 2017-04-25T17:35:00+0200
+next: 2017-04-26T00:58:00+0200
+`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(n, check.Equals, 1)
+}
+
+func (s *SnapSuite) TestRefreshHold(c *check.C) {
+	n := 0
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		switch n {
+		case 0:
+			c.Check(r.Method, check.Equals, "GET")
+			c.Check(r.URL.Path, check.Equals, "/v2/system-info")
+			fmt.Fprintln(w, `{"type": "sync", "status-code": 200, "result": {"refresh": {"timer": "0:00-24:00/4", "last": "2017-04-25T17:35:00+0200", "next": "2017-04-26T00:58:00+0200", "hold": "2017-04-28T00:00:00+0200"}}}`)
+		default:
+			c.Fatalf("expected to get 1 requests, now on %d", n+1)
+		}
+
+		n++
+	})
+	rest, err := snap.Parser().ParseArgs([]string{"refresh", "--time"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Equals, `timer: 0:00-24:00/4
+last: 2017-04-25T17:35:00+0200
+hold: 2017-04-28T00:00:00+0200
+next: 2017-04-26T00:58:00+0200
+`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(n, check.Equals, 1)
+}
+
+func (s *SnapSuite) TestRefreshNoTimerNoSchedule(c *check.C) {
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		c.Check(r.Method, check.Equals, "GET")
+		c.Check(r.URL.Path, check.Equals, "/v2/system-info")
+		fmt.Fprintln(w, `{"type": "sync", "status-code": 200, "result": {"refresh": {"last": "2017-04-25T17:35:00+0200", "next": "2017-04-26T00:58:00+0200"}}}`)
+	})
+	_, err := snap.Parser().ParseArgs([]string{"refresh", "--time"})
+	c.Assert(err, check.ErrorMatches, `internal error: both refresh.timer and refresh.schedule are empty`)
+}
+
 func (s *SnapSuite) TestRefreshListErr(c *check.C) {
 	s.RedirectClientToTestServer(nil)
 	_, err := snap.Parser().ParseArgs([]string{"refresh", "--list", "--beta"})
@@ -601,6 +759,20 @@ func (s *SnapOpSuite) TestRefreshAllModeFlags(c *check.C) {
 	c.Assert(err, check.ErrorMatches, `a single snap name is needed to specify mode or channel flags`)
 }
 
+func (s *SnapOpSuite) TestRefreshOneAmend(c *check.C) {
+	s.RedirectClientToTestServer(s.srv.handle)
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.Method, check.Equals, "POST")
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/one")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action": "refresh",
+			"amend":  true,
+		})
+	}
+	_, err := snap.Parser().ParseArgs([]string{"refresh", "--amend", "one"})
+	c.Assert(err, check.IsNil)
+}
+
 func (s *SnapOpSuite) runTryTest(c *check.C, opts *client.SnapOptions) {
 	// pass relative path to cmd
 	tryDir := "some-dir"
@@ -676,7 +848,7 @@ func (s *SnapOpSuite) TestTryClassic(c *check.C) {
 func (s *SnapOpSuite) TestTryNoSnapDirErrors(c *check.C) {
 	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
 		c.Check(r.Method, check.Equals, "POST")
-		w.WriteHeader(http.StatusAccepted)
+		w.WriteHeader(202)
 		fmt.Fprintln(w, `
 {
   "type": "error",
@@ -685,9 +857,7 @@ func (s *SnapOpSuite) TestTryNoSnapDirErrors(c *check.C) {
     "kind":"snap-not-a-snap"
   },
   "status-code": 400
-}
-`)
-
+}`)
 	})
 
 	cmd := []string{"try", "/"}
@@ -803,7 +973,7 @@ func (s *SnapOpSuite) TestRemoveMany(c *check.C) {
 			})
 
 			c.Check(r.Method, check.Equals, "POST")
-			w.WriteHeader(http.StatusAccepted)
+			w.WriteHeader(202)
 			fmt.Fprintln(w, `{"type":"async", "change": "42", "status-code": 202}`)
 		case 1:
 			c.Check(r.Method, check.Equals, "GET")
@@ -855,7 +1025,7 @@ func (s *SnapOpSuite) TestInstallMany(c *check.C) {
 			})
 
 			c.Check(r.Method, check.Equals, "POST")
-			w.WriteHeader(http.StatusAccepted)
+			w.WriteHeader(202)
 			fmt.Fprintln(w, `{"type":"async", "change": "42", "status-code": 202}`)
 		case 1:
 			c.Check(r.Method, check.Equals, "GET")
@@ -904,8 +1074,8 @@ func (s *SnapOpSuite) TestNoWait(c *check.C) {
 		{"try", "--no-wait", "."},
 	}
 
+	s.RedirectClientToTestServer(s.srv.handle)
 	for _, cmd := range cmds {
-		s.RedirectClientToTestServer(s.srv.handle)
 		rest, err := snap.Parser().ParseArgs(cmd)
 		c.Assert(err, check.IsNil, check.Commentf("%v", cmd))
 		c.Assert(rest, check.DeepEquals, []string{})
@@ -917,3 +1087,55 @@ func (s *SnapOpSuite) TestNoWait(c *check.C) {
 		s.stdout.Reset()
 	}
 }
+
+func (s *SnapOpSuite) TestSwitchHappy(c *check.C) {
+	s.srv.total = 3
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action":  "switch",
+			"channel": "beta",
+		})
+	}
+
+	s.RedirectClientToTestServer(s.srv.handle)
+	rest, err := snap.Parser().ParseArgs([]string{"switch", "--beta", "foo"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Matches, `(?sm).*"foo" switched to the "beta" channel`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(s.srv.n, check.Equals, s.srv.total)
+}
+
+func (s *SnapOpSuite) TestSwitchUnhappy(c *check.C) {
+	_, err := snap.Parser().ParseArgs([]string{"switch"})
+	c.Assert(err, check.ErrorMatches, "the required argument `<snap>` was not provided")
+}
+
+func (s *SnapOpSuite) TestSwitchAlsoUnhappy(c *check.C) {
+	_, err := snap.Parser().ParseArgs([]string{"switch", "foo"})
+	c.Assert(err, check.ErrorMatches, `missing --channel=<channel-name> parameter`)
+}
+
+func (s *SnapOpSuite) TestSnapOpNetworkTimeoutError(c *check.C) {
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		c.Check(r.Method, check.Equals, "POST")
+		w.WriteHeader(202)
+		w.Write([]byte(`
+{
+  "type": "error",
+  "result": {
+    "message":"Get https://api.snapcraft.io/api/v1/snaps/details/hello?channel=stable&fields=anon_download_url%2Carchitecture%2Cchannel%2Cdownload_sha3_384%2Csummary%2Cdescription%2Cdeltas%2Cbinary_filesize%2Cdownload_url%2Cepoch%2Cicon_url%2Clast_updated%2Cpackage_name%2Cprices%2Cpublisher%2Cratings_average%2Crevision%2Cscreenshot_urls%2Csnap_id%2Clicense%2Cbase%2Csupport_url%2Ccontact%2Ctitle%2Ccontent%2Cversion%2Corigin%2Cdeveloper_id%2Cprivate%2Cconfinement%2Cchannel_maps_list: net/http: request canceled while waiting for connection (Client.Timeout exceeded while awaiting headers)",
+    "kind":"network-timeout"
+  },
+  "status-code": 400
+}
+`))
+
+	})
+
+	cmd := []string{"install", "hello"}
+	_, err := snap.Parser().ParseArgs(cmd)
+	c.Assert(err, check.ErrorMatches, `unable to contact snap store`)
+}
