@@ -23,6 +23,8 @@ import (
 	"os/user"
 	"time"
 
+	"github.com/jessevdk/go-flags"
+
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/store"
 )
@@ -31,10 +33,15 @@ var RunMain = run
 
 var (
 	CreateUserDataDirs = createUserDataDirs
-	SnapRunApp         = snapRunApp
-	SnapRunHook        = snapRunHook
 	Wait               = wait
 	ResolveApp         = resolveApp
+	IsReexeced         = isReexeced
+	MaybePrintServices = maybePrintServices
+	MaybePrintCommands = maybePrintCommands
+	SortByPath         = sortByPath
+	AdviseCommand      = adviseCommand
+	Antialias          = antialias
+	FormatChannel      = fmtChannel
 )
 
 func MockPollTime(d time.Duration) (restore func()) {
@@ -77,6 +84,14 @@ func MockStoreNew(f func(*store.Config, auth.AuthContext) *store.Store) (restore
 	}
 }
 
+func MockGetEnv(f func(name string) string) (restore func()) {
+	osGetenvOrig := osGetenv
+	osGetenv = f
+	return func() {
+		osGetenv = osGetenvOrig
+	}
+}
+
 func MockMountInfoPath(newMountInfoPath string) (restore func()) {
 	mountInfoPathOrig := mountInfoPath
 	mountInfoPath = newMountInfoPath
@@ -85,20 +100,48 @@ func MockMountInfoPath(newMountInfoPath string) (restore func()) {
 	}
 }
 
+func MockOsReadlink(f func(string) (string, error)) (restore func()) {
+	osReadlinkOrig := osReadlink
+	osReadlink = f
+	return func() {
+		osReadlink = osReadlinkOrig
+	}
+}
+
 var AutoImportCandidates = autoImportCandidates
 
-func AliasInfoLess(snapName1, alias1, app1, snapName2, alias2, app2 string) bool {
+func AliasInfoLess(snapName1, alias1, cmd1, snapName2, alias2, cmd2 string) bool {
 	x := aliasInfos{
 		&aliasInfo{
-			Snap:  snapName1,
-			Alias: alias1,
-			App:   app1,
+			Snap:    snapName1,
+			Alias:   alias1,
+			Command: cmd1,
 		},
 		&aliasInfo{
-			Snap:  snapName2,
-			Alias: alias2,
-			App:   app2,
+			Snap:    snapName2,
+			Alias:   alias2,
+			Command: cmd2,
 		},
 	}
 	return x.Less(0, 1)
 }
+
+func AssertTypeNameCompletion(match string) []flags.Completion {
+	return assertTypeName("").Complete(match)
+}
+
+func MockIsTerminal(t bool) (restore func()) {
+	oldIsTerminal := isTerminal
+	isTerminal = func() bool { return t }
+	return func() {
+		isTerminal = oldIsTerminal
+	}
+}
+
+func MockTimeNow(newTimeNow func() time.Time) (restore func()) {
+	oldTimeNow := timeNow
+	timeNow = newTimeNow
+	return func() {
+		timeNow = oldTimeNow
+	}
+}
