@@ -23,6 +23,7 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
+	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
 	"os"
@@ -35,7 +36,9 @@ import (
 
 	"github.com/snapcore/snapd/cmd"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/osutil"
+	snapdsnap "github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/testutil"
 
 	snap "github.com/snapcore/snapd/cmd/snap"
@@ -60,6 +63,8 @@ func (s *BaseSnapSuite) readPassword(fd int) ([]byte, error) {
 
 func (s *BaseSnapSuite) SetUpTest(c *C) {
 	s.BaseTest.SetUpTest(c)
+	dirs.SetRootDir(c.MkDir())
+
 	s.stdin = bytes.NewBuffer(nil)
 	s.stdout = bytes.NewBuffer(nil)
 	s.stderr = bytes.NewBuffer(nil)
@@ -71,6 +76,18 @@ func (s *BaseSnapSuite) SetUpTest(c *C) {
 	snap.ReadPassword = s.readPassword
 	s.AuthFile = filepath.Join(c.MkDir(), "json")
 	os.Setenv(TestAuthFileEnvKey, s.AuthFile)
+
+	snapdsnap.MockSanitizePlugsSlots(func(snapInfo *snapdsnap.Info) {})
+
+	err := os.MkdirAll(filepath.Dir(dirs.SnapSystemKeyFile), 0755)
+	c.Assert(err, IsNil)
+	err = interfaces.WriteSystemKey()
+	c.Assert(err, IsNil)
+	interfaces.MockSystemKey(`
+{
+"build-id": "7a94e9736c091b3984bd63f5aebfc883c4d859e0",
+"apparmor-features": ["caps", "dbus"]
+}`)
 }
 
 func (s *BaseSnapSuite) TearDownTest(c *C) {
@@ -82,6 +99,7 @@ func (s *BaseSnapSuite) TearDownTest(c *C) {
 	c.Assert(s.AuthFile == "", Equals, false)
 	err := os.Unsetenv(TestAuthFileEnvKey)
 	c.Assert(err, IsNil)
+	dirs.SetRootDir("/")
 	s.BaseTest.TearDownTest(c)
 }
 
@@ -127,6 +145,7 @@ var _ = Suite(&SnapSuite{})
 func DecodedRequestBody(c *C, r *http.Request) map[string]interface{} {
 	var body map[string]interface{}
 	decoder := json.NewDecoder(r.Body)
+	decoder.UseNumber()
 	err := decoder.Decode(&body)
 	c.Assert(err, IsNil)
 	return body
@@ -151,7 +170,22 @@ func mockVersion(v string) (restore func()) {
 	return func() { cmd.Version = old }
 }
 
-const TestAuthFileEnvKey = "SNAPPY_STORE_AUTH_DATA_FILENAME"
+func mockSnapConfine(libExecDir string) func() {
+	snapConfine := filepath.Join(libExecDir, "snap-confine")
+	if err := os.MkdirAll(libExecDir, 0755); err != nil {
+		panic(err)
+	}
+	if err := ioutil.WriteFile(snapConfine, nil, 0644); err != nil {
+		panic(err)
+	}
+	return func() {
+		if err := os.Remove(snapConfine); err != nil {
+			panic(err)
+		}
+	}
+}
+
+const TestAuthFileEnvKey = "SNAPD_AUTH_DATA_FILENAME"
 const TestAuthFileContents = `{"id":123,"email":"hello@mail.com","macaroon":"MDAxM2xvY2F0aW9uIHNuYXBkCjAwMTJpZGVudGlmaWVyIDQzCjAwMmZzaWduYXR1cmUg5RfMua72uYop4t3cPOBmGUuaoRmoDH1HV62nMJq7eqAK"}`
 
 func (s *SnapSuite) TestErrorResult(c *C) {
@@ -224,9 +258,6 @@ func (s *SnapSuite) TestUnknownCommand(c *C) {
 }
 
 func (s *SnapSuite) TestResolveApp(c *C) {
-	dirs.SetRootDir(c.MkDir())
-	defer dirs.SetRootDir("/")
-
 	err := os.MkdirAll(dirs.SnapBinariesDir, 0755)
 	c.Assert(err, IsNil)
 
