@@ -24,9 +24,14 @@ import (
 	"os"
 	"os/signal"
 	"syscall"
+	"time"
 
-	"github.com/ubuntu-core/snappy/daemon"
-	"github.com/ubuntu-core/snappy/logger"
+	"github.com/snapcore/snapd/cmd"
+	"github.com/snapcore/snapd/daemon"
+	"github.com/snapcore/snapd/errtracker"
+	"github.com/snapcore/snapd/httputil"
+	"github.com/snapcore/snapd/logger"
+	"github.com/snapcore/snapd/systemd"
 )
 
 func init() {
@@ -34,9 +39,13 @@ func init() {
 	if err != nil {
 		fmt.Fprintf(os.Stderr, "WARNING: failed to activate logging: %s\n", err)
 	}
+	// set here to avoid accidental submits in e.g. unit tests
+	errtracker.CrashDbURLBase = "https://daisy.ubuntu.com/"
+	errtracker.SnapdVersion = cmd.Version
 }
 
 func main() {
+	cmd.ExecInCoreSnap()
 	if err := run(); err != nil {
 		fmt.Fprintf(os.Stderr, "error: %v\n", err)
 		os.Exit(1)
@@ -44,19 +53,27 @@ func main() {
 }
 
 func run() error {
+	t0 := time.Now().Truncate(time.Millisecond)
+	httputil.SetUserAgentFromVersion(cmd.Version)
+
+	ch := make(chan os.Signal, 2)
+	signal.Notify(ch, syscall.SIGINT, syscall.SIGTERM)
+
 	d, err := daemon.New()
 	if err != nil {
 		return err
 	}
-
 	if err := d.Init(); err != nil {
 		return err
 	}
+	d.Version = cmd.Version
 
 	d.Start()
 
-	ch := make(chan os.Signal)
-	signal.Notify(ch, syscall.SIGINT, syscall.SIGTERM)
+	// notify systemd that we are ready
+	systemd.SdNotify("READY=1")
+	logger.Debugf("activation done in %v", time.Now().Truncate(time.Millisecond).Sub(t0))
+
 	select {
 	case sig := <-ch:
 		logger.Noticef("Exiting on %s signal.\n", sig)
@@ -64,5 +81,6 @@ func run() error {
 		// something called Stop()
 	}
 
+	systemd.SdNotify("STOPPING=1")
 	return d.Stop()
 }
