@@ -33,8 +33,7 @@ func (s *apiSuite) mockSnap(c *C, yamlText string) *snap.Info {
 		panic("call s.daemon(c) in your test first")
 	}
 
-	snapInfo := snaptest.MockSnap(c, yamlText, "", &snap.SideInfo{Revision: snap.R(1)})
-	snap.AddImplicitSlots(snapInfo)
+	snapInfo := snaptest.MockSnap(c, yamlText, &snap.SideInfo{Revision: snap.R(1)})
 
 	st := s.d.overlord.State()
 
@@ -121,7 +120,5 @@ name: alias-snap
 version: 1
 apps:
  app:
-  aliases: [alias1]
  app2:
-  aliases: [alias2]
 `
