@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2016 Canonical Ltd
+ * Copyright (C) 2014-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -34,7 +34,6 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/asserts/assertstest"
-	"github.com/snapcore/snapd/asserts/sysdb"
 	"github.com/snapcore/snapd/boot/boottest"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/image"
@@ -48,16 +47,33 @@ import (
 	"github.com/snapcore/snapd/testutil"
 )
 
+type emptyStore struct{}
+
+func (s *emptyStore) SnapInfo(spec store.SnapSpec, user *auth.UserState) (*snap.Info, error) {
+	return nil, fmt.Errorf("cannot find snap")
+}
+
+func (s *emptyStore) Download(ctx context.Context, name, targetFn string, downloadInfo *snap.DownloadInfo, pbar progress.Meter, user *auth.UserState) error {
+	return fmt.Errorf("cannot download")
+}
+
+func (s *emptyStore) Assertion(assertType *asserts.AssertionType, primaryKey []string, user *auth.UserState) (asserts.Assertion, error) {
+	return nil, &asserts.NotFoundError{Type: assertType}
+}
+
 func Test(t *testing.T) { TestingT(t) }
 
 type imageSuite struct {
+	testutil.BaseTest
 	root       string
 	bootloader *boottest.MockBootloader
 
 	stdout *bytes.Buffer
+	stderr *bytes.Buffer
 
 	downloadedSnaps map[string]string
 	storeSnapInfo   map[string]*snap.Info
+	tsto            *image.ToolingStore
 
 	storeSigning *assertstest.StoreStack
 	brandSigning *assertstest.SigningDB
@@ -72,14 +88,18 @@ func (s *imageSuite) SetUpTest(c *C) {
 	s.bootloader = boottest.NewMockBootloader("grub", c.MkDir())
 	partition.ForceBootloader(s.bootloader)
 
-	s.stdout = bytes.NewBuffer(nil)
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+
+	s.stdout = &bytes.Buffer{}
 	image.Stdout = s.stdout
+	s.stderr = &bytes.Buffer{}
+	image.Stderr = s.stderr
 	s.downloadedSnaps = make(map[string]string)
 	s.storeSnapInfo = make(map[string]*snap.Info)
+	s.tsto = image.MockToolingStore(s)
 
-	rootPrivKey, _ := assertstest.GenerateKey(1024)
-	storePrivKey, _ := assertstest.GenerateKey(752)
-	s.storeSigning = assertstest.NewStoreStack("can0nical", rootPrivKey, storePrivKey)
+	s.storeSigning = assertstest.NewStoreStack("canonical", nil)
 
 	brandPrivKey, _ := assertstest.GenerateKey(752)
 	s.brandSigning = assertstest.NewSigningDB("my-brand", brandPrivKey)
@@ -106,15 +126,20 @@ func (s *imageSuite) SetUpTest(c *C) {
 	}, nil, "")
 	c.Assert(err, IsNil)
 	s.model = model.(*asserts.Model)
+
+	otherAcct := assertstest.NewAccount(s.storeSigning, "other", map[string]interface{}{
+		"account-id": "other",
+	}, "")
+	s.storeSigning.Add(otherAcct)
 }
 
-func (s *imageSuite) addSystemSnapAssertions(c *C, snapName string) {
+func (s *imageSuite) addSystemSnapAssertions(c *C, snapName string, publisher string) {
 	snapID := snapName + "-Id"
 	decl, err := s.storeSigning.Sign(asserts.SnapDeclarationType, map[string]interface{}{
 		"series":       "16",
 		"snap-id":      snapID,
 		"snap-name":    snapName,
-		"publisher-id": "can0nical",
+		"publisher-id": publisher,
 		"timestamp":    time.Now().UTC().Format(time.RFC3339),
 	}, nil, "")
 	c.Assert(err, IsNil)
@@ -129,7 +154,7 @@ func (s *imageSuite) addSystemSnapAssertions(c *C, snapName string) {
 		"snap-size":     fmt.Sprintf("%d", snapSize),
 		"snap-id":       snapID,
 		"snap-revision": s.storeSnapInfo[snapName].Revision.String(),
-		"developer-id":  "can0nical",
+		"developer-id":  publisher,
 		"timestamp":     time.Now().UTC().Format(time.RFC3339),
 	}, nil, "")
 	c.Assert(err, IsNil)
@@ -138,8 +163,10 @@ func (s *imageSuite) addSystemSnapAssertions(c *C, snapName string) {
 }
 
 func (s *imageSuite) TearDownTest(c *C) {
+	s.BaseTest.TearDownTest(c)
 	partition.ForceBootloader(nil)
 	image.Stdout = os.Stdout
+	image.Stderr = os.Stderr
 }
 
 // interface for the store
@@ -271,7 +298,7 @@ func (s *imageSuite) TestHappyDecodeModelAssertion(c *C) {
 }
 
 func (s *imageSuite) TestMissingGadgetUnpackDir(c *C) {
-	err := image.DownloadUnpackGadget(s, s.model, &image.Options{}, nil)
+	err := image.DownloadUnpackGadget(s.tsto, s.model, &image.Options{}, nil)
 	c.Assert(err, ErrorMatches, `cannot create gadget unpack dir "": mkdir : no such file or directory`)
 }
 
@@ -297,10 +324,10 @@ func (s *imageSuite) TestDownloadUnpackGadget(c *C) {
 	opts := &image.Options{
 		GadgetUnpackDir: gadgetUnpackDir,
 	}
-	local, err := image.LocalSnaps(opts)
+	local, err := image.LocalSnaps(s.tsto, opts)
 	c.Assert(err, IsNil)
 
-	err = image.DownloadUnpackGadget(s, s.model, opts, local)
+	err = image.DownloadUnpackGadget(s.tsto, s.model, opts, local)
 	c.Assert(err, IsNil)
 
 	// verify the right data got unpacked
@@ -309,13 +336,11 @@ func (s *imageSuite) TestDownloadUnpackGadget(c *C) {
 		{files[0][0], files[0][1]},
 	} {
 		fn := filepath.Join(gadgetUnpackDir, t.file)
-		content, err := ioutil.ReadFile(fn)
-		c.Assert(err, IsNil)
-		c.Check(content, DeepEquals, []byte(t.content))
+		c.Check(fn, testutil.FileEquals, t.content)
 	}
 }
 
-func (s *imageSuite) setupSnaps(c *C, gadgetUnpackDir string) {
+func (s *imageSuite) setupSnaps(c *C, gadgetUnpackDir string, publishers map[string]string) {
 	err := os.MkdirAll(gadgetUnpackDir, 0755)
 	c.Assert(err, IsNil)
 	err = ioutil.WriteFile(filepath.Join(gadgetUnpackDir, "grub.conf"), nil, 0644)
@@ -323,31 +348,38 @@ func (s *imageSuite) setupSnaps(c *C, gadgetUnpackDir string) {
 
 	s.downloadedSnaps["pc"] = snaptest.MakeTestSnapWithFiles(c, packageGadget, [][]string{{"grub.cfg", "I'm a grub.cfg"}})
 	s.storeSnapInfo["pc"] = infoFromSnapYaml(c, packageGadget, snap.R(1))
-	s.addSystemSnapAssertions(c, "pc")
+	s.addSystemSnapAssertions(c, "pc", publishers["pc"])
 
 	s.downloadedSnaps["pc-kernel"] = snaptest.MakeTestSnapWithFiles(c, packageKernel, nil)
 	s.storeSnapInfo["pc-kernel"] = infoFromSnapYaml(c, packageKernel, snap.R(2))
-	s.addSystemSnapAssertions(c, "pc-kernel")
+	s.addSystemSnapAssertions(c, "pc-kernel", publishers["pc-kernel"])
 
 	s.downloadedSnaps["core"] = snaptest.MakeTestSnapWithFiles(c, packageCore, nil)
 	s.storeSnapInfo["core"] = infoFromSnapYaml(c, packageCore, snap.R(3))
-	s.addSystemSnapAssertions(c, "core")
+	s.addSystemSnapAssertions(c, "core", "canonical")
 
 	s.downloadedSnaps["required-snap1"] = snaptest.MakeTestSnapWithFiles(c, requiredSnap1, nil)
 	s.storeSnapInfo["required-snap1"] = infoFromSnapYaml(c, requiredSnap1, snap.R(3))
-	s.addSystemSnapAssertions(c, "required-snap1")
+	s.storeSnapInfo["required-snap1"].Contact = "foo@example.com"
+	s.addSystemSnapAssertions(c, "required-snap1", "other")
 }
 
 func (s *imageSuite) TestBootstrapToRootDir(c *C) {
-	restore := sysdb.InjectTrusted(s.storeSigning.Trusted)
+	restore := image.MockTrusted(s.storeSigning.Trusted)
 	defer restore()
 
 	rootdir := filepath.Join(c.MkDir(), "imageroot")
+	seeddir := filepath.Join(rootdir, "var/lib/snapd/seed")
+	seedsnapsdir := filepath.Join(seeddir, "snaps")
+	seedassertsdir := filepath.Join(seeddir, "assertions")
 
 	// FIXME: bootstrapToRootDir needs an unpacked gadget yaml
 	gadgetUnpackDir := filepath.Join(c.MkDir(), "gadget")
 
-	s.setupSnaps(c, gadgetUnpackDir)
+	s.setupSnaps(c, gadgetUnpackDir, map[string]string{
+		"pc":        "canonical",
+		"pc-kernel": "canonical",
+	})
 
 	// mock the mount cmds (for the extract kernel assets stuff)
 	c1 := testutil.MockCommand(c, "mount", "")
@@ -359,14 +391,14 @@ func (s *imageSuite) TestBootstrapToRootDir(c *C) {
 		RootDir:         rootdir,
 		GadgetUnpackDir: gadgetUnpackDir,
 	}
-	local, err := image.LocalSnaps(opts)
+	local, err := image.LocalSnaps(s.tsto, opts)
 	c.Assert(err, IsNil)
 
-	err = image.BootstrapToRootDir(s, s.model, opts, local)
+	err = image.BootstrapToRootDir(s.tsto, s.model, opts, local)
 	c.Assert(err, IsNil)
 
 	// check seed yaml
-	seed, err := snap.ReadSeedYaml(filepath.Join(rootdir, "var/lib/snapd/seed/seed.yaml"))
+	seed, err := snap.ReadSeedYaml(filepath.Join(seeddir, "seed.yaml"))
 	c.Assert(err, IsNil)
 
 	c.Check(seed.Snaps, HasLen, 4)
@@ -375,7 +407,7 @@ func (s *imageSuite) TestBootstrapToRootDir(c *C) {
 	for i, name := range []string{"core", "pc-kernel", "pc"} {
 		info := s.storeSnapInfo[name]
 		fn := filepath.Base(info.MountFile())
-		p := filepath.Join(rootdir, "var/lib/snapd/seed/snaps", fn)
+		p := filepath.Join(seedsnapsdir, fn)
 		c.Check(osutil.FileExists(p), Equals, true)
 
 		c.Check(seed.Snaps[i], DeepEquals, &snap.SeedSnap{
@@ -384,6 +416,8 @@ func (s *imageSuite) TestBootstrapToRootDir(c *C) {
 			File:   fn,
 		})
 	}
+	c.Check(seed.Snaps[3].Name, Equals, "required-snap1")
+	c.Check(seed.Snaps[3].Contact, Equals, "foo@example.com")
 
 	storeAccountKey := s.storeSigning.StoreAccountKey("")
 	brandPubKey, err := s.brandSigning.PublicKey("")
@@ -391,15 +425,12 @@ func (s *imageSuite) TestBootstrapToRootDir(c *C) {
 
 	// check the assertions are in place
 	for _, fn := range []string{"model", brandPubKey.ID() + ".account-key", "my-brand.account", storeAccountKey.PublicKeyID() + ".account-key"} {
-		p := filepath.Join(rootdir, "var/lib/snapd/seed/assertions", fn)
+		p := filepath.Join(seedassertsdir, fn)
 		c.Check(osutil.FileExists(p), Equals, true)
 	}
 
-	b, err := ioutil.ReadFile(filepath.Join(rootdir, "var/lib/snapd/seed/assertions", "model"))
-	c.Assert(err, IsNil)
-	c.Check(b, DeepEquals, asserts.Encode(s.model))
-
-	b, err = ioutil.ReadFile(filepath.Join(rootdir, "var/lib/snapd/seed/assertions", "my-brand.account"))
+	c.Check(filepath.Join(seedassertsdir, "model"), testutil.FileEquals, asserts.Encode(s.model))
+	b, err := ioutil.ReadFile(filepath.Join(seedassertsdir, "my-brand.account"))
 	c.Assert(err, IsNil)
 	a, err := asserts.Decode(b)
 	c.Assert(err, IsNil)
@@ -408,7 +439,7 @@ func (s *imageSuite) TestBootstrapToRootDir(c *C) {
 
 	// check the snap assertions are also in place
 	for _, snapId := range []string{"pc-Id", "pc-kernel-Id", "core-Id"} {
-		p := filepath.Join(rootdir, "var/lib/snapd/seed/assertions", fmt.Sprintf("16,%s.snap-declaration", snapId))
+		p := filepath.Join(seedassertsdir, fmt.Sprintf("16,%s.snap-declaration", snapId))
 		c.Check(osutil.FileExists(p), Equals, true)
 	}
 
@@ -417,18 +448,24 @@ func (s *imageSuite) TestBootstrapToRootDir(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(m["snap_kernel"], Equals, "pc-kernel_2.snap")
 	c.Check(m["snap_core"], Equals, "core_3.snap")
+
+	c.Check(s.stderr.String(), Equals, "")
 }
 
-func (s *imageSuite) TestBootstrapToRootDirLocalCore(c *C) {
-	restore := sysdb.InjectTrusted(s.storeSigning.Trusted)
+func (s *imageSuite) TestBootstrapToRootDirLocalCoreBrandKernel(c *C) {
+	restore := image.MockTrusted(s.storeSigning.Trusted)
 	defer restore()
 
 	rootdir := filepath.Join(c.MkDir(), "imageroot")
+	seedassertsdir := filepath.Join(rootdir, "var/lib/snapd/seed/assertions")
 
 	// FIXME: bootstrapToRootDir needs an unpacked gadget yaml
 	gadgetUnpackDir := filepath.Join(c.MkDir(), "gadget")
 
-	s.setupSnaps(c, gadgetUnpackDir)
+	s.setupSnaps(c, gadgetUnpackDir, map[string]string{
+		"pc":        "canonical",
+		"pc-kernel": "my-brand",
+	})
 
 	// mock the mount cmds (for the extract kernel assets stuff)
 	c1 := testutil.MockCommand(c, "mount", "")
@@ -444,10 +481,11 @@ func (s *imageSuite) TestBootstrapToRootDirLocalCore(c *C) {
 		RootDir:         rootdir,
 		GadgetUnpackDir: gadgetUnpackDir,
 	}
-	local, err := image.LocalSnaps(opts)
+	emptyToolingStore := image.MockToolingStore(&emptyStore{})
+	local, err := image.LocalSnaps(emptyToolingStore, opts)
 	c.Assert(err, IsNil)
 
-	err = image.BootstrapToRootDir(s, s.model, opts, local)
+	err = image.BootstrapToRootDir(s.tsto, s.model, opts, local)
 	c.Assert(err, IsNil)
 
 	// check seed yaml
@@ -503,22 +541,19 @@ func (s *imageSuite) TestBootstrapToRootDirLocalCore(c *C) {
 
 	// check the assertions are in place
 	for _, fn := range []string{"model", brandPubKey.ID() + ".account-key", "my-brand.account", storeAccountKey.PublicKeyID() + ".account-key"} {
-		p := filepath.Join(rootdir, "var/lib/snapd/seed/assertions", fn)
+		p := filepath.Join(seedassertsdir, fn)
 		c.Check(osutil.FileExists(p), Equals, true)
 	}
 
-	b, err := ioutil.ReadFile(filepath.Join(rootdir, "var/lib/snapd/seed/assertions", "model"))
-	c.Assert(err, IsNil)
-	c.Check(b, DeepEquals, asserts.Encode(s.model))
-
-	b, err = ioutil.ReadFile(filepath.Join(rootdir, "var/lib/snapd/seed/assertions", "my-brand.account"))
+	c.Check(filepath.Join(seedassertsdir, "model"), testutil.FileEquals, asserts.Encode(s.model))
+	b, err := ioutil.ReadFile(filepath.Join(seedassertsdir, "my-brand.account"))
 	c.Assert(err, IsNil)
 	a, err := asserts.Decode(b)
 	c.Assert(err, IsNil)
 	c.Check(a.Type(), Equals, asserts.AccountType)
 	c.Check(a.HeaderString("account-id"), Equals, "my-brand")
 
-	decls, err := filepath.Glob(filepath.Join(rootdir, "var/lib/snapd/seed/assertions", "*.snap-declaration"))
+	decls, err := filepath.Glob(filepath.Join(seedassertsdir, "*.snap-declaration"))
 	c.Assert(err, IsNil)
 	// nothing for core
 	c.Check(decls, HasLen, 2)
@@ -530,12 +565,11 @@ func (s *imageSuite) TestBootstrapToRootDirLocalCore(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(m["snap_core"], Equals, "core_x1.snap")
 
-	// check that cloud-init is setup correctly
-	c.Check(osutil.FileExists(filepath.Join(rootdir, "etc/cloud/cloud-init.disabled")), Equals, true)
+	c.Check(s.stderr.String(), Equals, "WARNING: \"core\", \"required-snap1\" were installed from local snaps disconnected from a store and cannot be refreshed subsequently!\n")
 }
 
 func (s *imageSuite) TestBootstrapToRootDirDevmodeSnap(c *C) {
-	restore := sysdb.InjectTrusted(s.storeSigning.Trusted)
+	restore := image.MockTrusted(s.storeSigning.Trusted)
 	defer restore()
 
 	rootdir := filepath.Join(c.MkDir(), "imageroot")
@@ -548,7 +582,10 @@ func (s *imageSuite) TestBootstrapToRootDirDevmodeSnap(c *C) {
 	err = ioutil.WriteFile(filepath.Join(gadgetUnpackDir, "grub.conf"), nil, 0644)
 	c.Assert(err, IsNil)
 
-	s.setupSnaps(c, gadgetUnpackDir)
+	s.setupSnaps(c, gadgetUnpackDir, map[string]string{
+		"pc":        "canonical",
+		"pc-kernel": "canonical",
+	})
 
 	s.downloadedSnaps["devmode-snap"] = snaptest.MakeTestSnapWithFiles(c, devmodeSnap, nil)
 	s.storeSnapInfo["devmode-snap"] = infoFromSnapYaml(c, devmodeSnap, snap.R(0))
@@ -565,10 +602,10 @@ func (s *imageSuite) TestBootstrapToRootDirDevmodeSnap(c *C) {
 		RootDir:         rootdir,
 		GadgetUnpackDir: gadgetUnpackDir,
 	}
-	local, err := image.LocalSnaps(opts)
+	local, err := image.LocalSnaps(s.tsto, opts)
 	c.Assert(err, IsNil)
 
-	err = image.BootstrapToRootDir(s, s.model, opts, local)
+	err = image.BootstrapToRootDir(s.tsto, s.model, opts, local)
 	c.Assert(err, IsNil)
 
 	// check seed yaml
@@ -598,6 +635,37 @@ func (s *imageSuite) TestBootstrapToRootDirDevmodeSnap(c *C) {
 	})
 }
 
+func (s *imageSuite) TestBootstrapToRootDirKernelPublisherMismatch(c *C) {
+	restore := image.MockTrusted(s.storeSigning.Trusted)
+	defer restore()
+
+	rootdir := filepath.Join(c.MkDir(), "imageroot")
+
+	// FIXME: bootstrapToRootDir needs an unpacked gadget yaml
+	gadgetUnpackDir := filepath.Join(c.MkDir(), "gadget")
+
+	s.setupSnaps(c, gadgetUnpackDir, map[string]string{
+		"pc":        "canonical",
+		"pc-kernel": "other",
+	})
+
+	// mock the mount cmds (for the extract kernel assets stuff)
+	c1 := testutil.MockCommand(c, "mount", "")
+	defer c1.Restore()
+	c2 := testutil.MockCommand(c, "umount", "")
+	defer c2.Restore()
+
+	opts := &image.Options{
+		RootDir:         rootdir,
+		GadgetUnpackDir: gadgetUnpackDir,
+	}
+	local, err := image.LocalSnaps(s.tsto, opts)
+	c.Assert(err, IsNil)
+
+	err = image.BootstrapToRootDir(s.tsto, s.model, opts, local)
+	c.Assert(err, ErrorMatches, `cannot use kernel "pc-kernel" published by "other" for model by "my-brand"`)
+}
+
 func (s *imageSuite) TestInstallCloudConfigNoConfig(c *C) {
 	targetDir := c.MkDir()
 	emptyGadgetDir := c.MkDir()
@@ -605,7 +673,7 @@ func (s *imageSuite) TestInstallCloudConfigNoConfig(c *C) {
 	dirs.SetRootDir(targetDir)
 	err := image.InstallCloudConfig(emptyGadgetDir)
 	c.Assert(err, IsNil)
-	c.Check(osutil.FileExists(filepath.Join(targetDir, "etc/cloud/cloud-init.disabled")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(targetDir, "etc/cloud")), Equals, true)
 }
 
 func (s *imageSuite) TestInstallCloudConfigWithCloudConfig(c *C) {
@@ -619,7 +687,159 @@ func (s *imageSuite) TestInstallCloudConfigWithCloudConfig(c *C) {
 	dirs.SetRootDir(targetDir)
 	err = image.InstallCloudConfig(gadgetDir)
 	c.Assert(err, IsNil)
-	content, err := ioutil.ReadFile(filepath.Join(targetDir, "etc/cloud/cloud.cfg"))
+	c.Check(filepath.Join(targetDir, "etc/cloud/cloud.cfg"), testutil.FileEquals, canary)
+}
+
+func (s *imageSuite) TestNewToolingStoreWithAuth(c *C) {
+	tmpdir := c.MkDir()
+	authFn := filepath.Join(tmpdir, "auth.json")
+	err := ioutil.WriteFile(authFn, []byte(`{
+"macaroon": "MACAROON",
+"discharges": ["DISCHARGE"]
+}`), 0600)
 	c.Assert(err, IsNil)
-	c.Check(content, DeepEquals, canary)
+
+	os.Setenv("UBUNTU_STORE_AUTH_DATA_FILENAME", authFn)
+	defer os.Unsetenv("UBUNTU_STORE_AUTH_DATA_FILENAME")
+
+	tsto, err := image.NewToolingStore()
+	c.Assert(err, IsNil)
+	user := tsto.User()
+	c.Check(user.StoreMacaroon, Equals, "MACAROON")
+	c.Check(user.StoreDischarges, DeepEquals, []string{"DISCHARGE"})
+}
+
+func (s *imageSuite) TestNewToolingStoreWithAuthFromSnapcraftLoginFile(c *C) {
+	tmpdir := c.MkDir()
+	authFn := filepath.Join(tmpdir, "auth.json")
+	err := ioutil.WriteFile(authFn, []byte(`[login.ubuntu.com]
+macaroon = MACAROON
+unbound_discharge = DISCHARGE
+
+`), 0600)
+	c.Assert(err, IsNil)
+
+	os.Setenv("UBUNTU_STORE_AUTH_DATA_FILENAME", authFn)
+	defer os.Unsetenv("UBUNTU_STORE_AUTH_DATA_FILENAME")
+
+	tsto, err := image.NewToolingStore()
+	c.Assert(err, IsNil)
+	user := tsto.User()
+	c.Check(user.StoreMacaroon, Equals, "MACAROON")
+	c.Check(user.StoreDischarges, DeepEquals, []string{"DISCHARGE"})
+}
+
+func (s *imageSuite) TestBootstrapToRootDirLocalSnapsWithStoreAsserts(c *C) {
+	restore := image.MockTrusted(s.storeSigning.Trusted)
+	defer restore()
+
+	rootdir := filepath.Join(c.MkDir(), "imageroot")
+	assertsdir := filepath.Join(rootdir, "var/lib/snapd/seed/assertions")
+
+	// FIXME: bootstrapToRootDir needs an unpacked gadget yaml
+	gadgetUnpackDir := filepath.Join(c.MkDir(), "gadget")
+
+	s.setupSnaps(c, gadgetUnpackDir, map[string]string{
+		"pc":        "canonical",
+		"pc-kernel": "my-brand",
+	})
+
+	// mock the mount cmds (for the extract kernel assets stuff)
+	c1 := testutil.MockCommand(c, "mount", "")
+	defer c1.Restore()
+	c2 := testutil.MockCommand(c, "umount", "")
+	defer c2.Restore()
+
+	opts := &image.Options{
+		Snaps: []string{
+			s.downloadedSnaps["core"],
+			s.downloadedSnaps["required-snap1"],
+		},
+		RootDir:         rootdir,
+		GadgetUnpackDir: gadgetUnpackDir,
+	}
+	local, err := image.LocalSnaps(s.tsto, opts)
+	c.Assert(err, IsNil)
+
+	err = image.BootstrapToRootDir(s.tsto, s.model, opts, local)
+	c.Assert(err, IsNil)
+
+	// check seed yaml
+	seed, err := snap.ReadSeedYaml(filepath.Join(rootdir, "var/lib/snapd/seed/seed.yaml"))
+	c.Assert(err, IsNil)
+
+	c.Check(seed.Snaps, HasLen, 4)
+
+	// check the files are in place
+	for i, name := range []string{"core_3.snap", "pc-kernel", "pc", "required-snap1_3.snap"} {
+		info := s.storeSnapInfo[name]
+		if info == nil {
+			switch name {
+			case "core_3.snap":
+				info = &snap.Info{
+					SideInfo: snap.SideInfo{
+						RealName: "core",
+						SnapID:   "core-Id",
+						Revision: snap.R(3),
+					},
+				}
+			case "required-snap1_3.snap":
+				info = &snap.Info{
+					SideInfo: snap.SideInfo{
+						RealName: "required-snap1",
+						SnapID:   "required-snap1-Id",
+						Revision: snap.R(3),
+					},
+				}
+			default:
+				c.Errorf("cannot have %s", name)
+			}
+		}
+
+		fn := filepath.Base(info.MountFile())
+		p := filepath.Join(rootdir, "var/lib/snapd/seed/snaps", fn)
+		c.Check(osutil.FileExists(p), Equals, true, Commentf("cannot find %s", p))
+
+		c.Check(seed.Snaps[i], DeepEquals, &snap.SeedSnap{
+			Name:       info.Name(),
+			SnapID:     info.SnapID,
+			File:       fn,
+			Unasserted: false,
+		})
+	}
+
+	l, err := ioutil.ReadDir(filepath.Join(rootdir, "var/lib/snapd/seed/snaps"))
+	c.Assert(err, IsNil)
+	c.Check(l, HasLen, 4)
+
+	storeAccountKey := s.storeSigning.StoreAccountKey("")
+	brandPubKey, err := s.brandSigning.PublicKey("")
+	c.Assert(err, IsNil)
+
+	// check the assertions are in place
+	for _, fn := range []string{"model", brandPubKey.ID() + ".account-key", "my-brand.account", storeAccountKey.PublicKeyID() + ".account-key"} {
+		p := filepath.Join(assertsdir, fn)
+		c.Check(osutil.FileExists(p), Equals, true)
+	}
+
+	c.Check(filepath.Join(assertsdir, "model"), testutil.FileEquals, asserts.Encode(s.model))
+	b, err := ioutil.ReadFile(filepath.Join(assertsdir, "my-brand.account"))
+	c.Assert(err, IsNil)
+	a, err := asserts.Decode(b)
+	c.Assert(err, IsNil)
+	c.Check(a.Type(), Equals, asserts.AccountType)
+	c.Check(a.HeaderString("account-id"), Equals, "my-brand")
+
+	decls, err := filepath.Glob(filepath.Join(assertsdir, "*.snap-declaration"))
+	c.Assert(err, IsNil)
+	c.Check(decls, HasLen, 4)
+
+	// check the bootloader config
+	m, err := s.bootloader.GetBootVars("snap_kernel", "snap_core")
+	c.Assert(err, IsNil)
+	c.Check(m["snap_kernel"], Equals, "pc-kernel_2.snap")
+	c.Assert(err, IsNil)
+	c.Check(m["snap_core"], Equals, "core_3.snap")
+
+	c.Check(s.stderr.String(), Equals, "")
 }
