@@ -33,7 +33,8 @@ import (
 	"path/filepath"
 	"strings"
 
-	"github.com/ubuntu-core/snappy/dirs"
+	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil"
 )
 
 // LoadProfile loads an apparmor profile from the given file.
@@ -41,15 +42,7 @@ import (
 // If no such profile was previously loaded then it is simply added to the kernel.
 // If there was a profile with the same name before, that profile is replaced.
 func LoadProfile(fname string) error {
-	// Use no-expr-simplify since expr-simplify is actually slower on armhf (LP: #1383858)
-	output, err := exec.Command(
-		"apparmor_parser", "--replace", "--write-cache", "-O",
-		"no-expr-simplify", fmt.Sprintf("--cache-loc=%s", dirs.AppArmorCacheDir),
-		fname).CombinedOutput()
-	if err != nil {
-		return fmt.Errorf("cannot load apparmor profile: %s\napparmor_parser output:\n%s", err, string(output))
-	}
-	return nil
+	return loadProfile(fname, dirs.AppArmorCacheDir)
 }
 
 // UnloadProfile removes the named profile from the running kernel.
@@ -57,11 +50,30 @@ func LoadProfile(fname string) error {
 // The operation is done with: apparmor_parser --remove $name
 // The binary cache file is removed from /var/cache/apparmor
 func UnloadProfile(name string) error {
+	return unloadProfile(name, dirs.AppArmorCacheDir)
+}
+
+func loadProfile(fname, cacheDir string) error {
+	// Use no-expr-simplify since expr-simplify is actually slower on armhf (LP: #1383858)
+	args := []string{"--replace", "--write-cache", "-O", "no-expr-simplify", fmt.Sprintf("--cache-loc=%s", cacheDir)}
+	if !osutil.GetenvBool("SNAPD_DEBUG") {
+		args = append(args, "--quiet")
+	}
+	args = append(args, fname)
+
+	output, err := exec.Command("apparmor_parser", args...).CombinedOutput()
+	if err != nil {
+		return fmt.Errorf("cannot load apparmor profile: %s\napparmor_parser output:\n%s", err, string(output))
+	}
+	return nil
+}
+
+func unloadProfile(name, cacheDir string) error {
 	output, err := exec.Command("apparmor_parser", "--remove", name).CombinedOutput()
 	if err != nil {
 		return fmt.Errorf("cannot unload apparmor profile: %s\napparmor_parser output:\n%s", err, string(output))
 	}
-	err = os.Remove(filepath.Join(dirs.AppArmorCacheDir, name))
+	err = os.Remove(filepath.Join(cacheDir, name))
 	// It is not an error if the cache file wasn't there to remove.
 	if err != nil && !os.IsNotExist(err) {
 		return fmt.Errorf("cannot remove apparmor profile cache: %s", err)
