@@ -20,9 +20,34 @@
 package apparmor
 
 import (
+	"os"
+
 	"github.com/snapcore/snapd/testutil"
 )
 
+var (
+	SnapConfineFromCoreProfile = snapConfineFromCoreProfile
+)
+
+// MockIsRootWritableOverlay mocks the real implementation of osutil.IsRootWritableOverlay
+func MockIsRootWritableOverlay(new func() (string, error)) (restore func()) {
+	old := isRootWritableOverlay
+	isRootWritableOverlay = new
+	return func() {
+		isRootWritableOverlay = old
+	}
+}
+
+// MockProcSelfExe mocks the location of /proc/self/exe read by setupSnapConfineGeneratedPolicy.
+func MockProcSelfExe(symlink string) (restore func()) {
+	old := procSelfExe
+	procSelfExe = symlink
+	return func() {
+		os.Remove(procSelfExe)
+		procSelfExe = old
+	}
+}
+
 // MockProfilesPath mocks the file read by LoadedProfiles()
 func MockProfilesPath(t *testutil.BaseTest, profiles string) {
 	profilesPath = profiles
@@ -35,15 +60,20 @@ func MockProfilesPath(t *testutil.BaseTest, profiles string) {
 //
 // NOTE: The real apparmor template is long. For testing it is convenient for
 // replace it with a shorter snippet.
-func MockTemplate(fakeTemplate []byte) (restore func()) {
+func MockTemplate(fakeTemplate string) (restore func()) {
 	orig := defaultTemplate
 	defaultTemplate = fakeTemplate
 	return func() { defaultTemplate = orig }
 }
 
 // MockClassicTemplate replaces the classic apprmor template.
-func MockClassicTemplate(fakeTemplate []byte) (restore func()) {
+func MockClassicTemplate(fakeTemplate string) (restore func()) {
 	orig := classicTemplate
 	classicTemplate = fakeTemplate
 	return func() { classicTemplate = orig }
 }
+
+// SetSpecScope sets the scope of a given specification
+func SetSpecScope(spec *Specification, securityTags []string, snapName string) (restore func()) {
+	return spec.setScope(securityTags, snapName)
+}
