@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,21 +20,42 @@
 package builtin
 
 import (
-	"bytes"
+	"strings"
 
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/interfaces/apparmor"
+	"github.com/snapcore/snapd/interfaces/dbus"
+	"github.com/snapcore/snapd/interfaces/seccomp"
+	"github.com/snapcore/snapd/interfaces/udev"
+	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snap"
 )
 
-var bluezPermanentSlotAppArmor = []byte(`
-# Description: Allow operating as the bluez service. Reserved because this
-#  gives privileged access to the system.
-# Usage: reserved
+const bluezSummary = `allows operating as the bluez service`
+
+const bluezBaseDeclarationSlots = `
+  bluez:
+    allow-installation:
+      slot-snap-type:
+        - app
+        - core
+    deny-auto-connection: true
+    deny-connection:
+      on-classic: false
+`
+
+const bluezPermanentSlotAppArmor = `
+# Description: Allow operating as the bluez service. This gives privileged
+# access to the system.
 
   network bluetooth,
 
   capability net_admin,
   capability net_bind_service,
 
+  # libudev
+  network netlink raw,
+
   # File accesses
   /sys/bus/usb/drivers/btusb/     r,
   /sys/bus/usb/drivers/btusb/**   r,
@@ -53,7 +74,7 @@ var bluezPermanentSlotAppArmor = []byte(`
      path=/org/freedesktop/DBus
      interface=org.freedesktop.DBus
      member={Request,Release}Name
-     peer=(name=org.freedesktop.DBus),
+     peer=(name=org.freedesktop.DBus, label=unconfined),
 
   dbus (send)
     bus=system
@@ -71,21 +92,28 @@ var bluezPermanentSlotAppArmor = []byte(`
       bus=system
       name="org.bluez.obex",
 
-  # Allow traffic to/from our path and interface with any method
+  # Allow traffic to/from our interface with any method for unconfined clients
+  # to talk to our bluez services. For the org.bluez interface we don't specify
+  # an Object Path since according to the bluez specification these can be
+  # anything (https://git.kernel.org/pub/scm/bluetooth/bluez.git/tree/doc).
+  dbus (receive, send)
+      bus=system
+      interface=org.bluez.*
+      peer=(label=unconfined),
   dbus (receive, send)
       bus=system
       path=/org/bluez{,/**}
-      interface=org.bluez.*,
+      interface=org.freedesktop.DBus.*
+      peer=(label=unconfined),
 
-  # Allow traffic to/from org.freedesktop.DBus for bluez service
+  # Allow traffic to/from org.freedesktop.DBus for bluez service. This rule is
+  # not snap-specific and grants privileged access to the org.freedesktop.DBus
+  # on the system bus.
   dbus (receive, send)
       bus=system
       path=/
-      interface=org.freedesktop.DBus.**,
-  dbus (receive, send)
-      bus=system
-      path=/org/bluez{,/**}
-      interface=org.freedesktop.DBus.**,
+      interface=org.freedesktop.DBus.*
+      peer=(label=unconfined),
 
   # Allow access to hostname system service
   dbus (receive, send)
@@ -93,12 +121,20 @@ var bluezPermanentSlotAppArmor = []byte(`
       path=/org/freedesktop/hostname1
       interface=org.freedesktop.DBus.Properties
       peer=(label=unconfined),
-`)
+`
 
-var bluezConnectedPlugAppArmor = []byte(`
-# Description: Allow using bluez service. Reserved because this gives
-#  privileged access to the bluez service.
-# Usage: reserved
+const bluezConnectedSlotAppArmor = `
+# Allow connected clients to interact with the service
+
+# Allow all access to bluez service
+dbus (receive, send)
+    bus=system
+    peer=(label=###PLUG_SECURITY_TAGS###),
+`
+
+const bluezConnectedPlugAppArmor = `
+# Description: Allow using bluez service. This gives privileged access to the
+# bluez service.
 
 #include <abstractions/dbus-strict>
 
@@ -126,58 +162,28 @@ dbus (receive)
     path=/org/bluez{,/**}
     interface=org.freedesktop.DBus.*
     peer=(label=unconfined),
-`)
+`
 
-var bluezPermanentSlotSecComp = []byte(`
-# Description: Allow operating as the bluez service. Reserved because this
-# gives
-#  privileged access to the system.
-# Usage: reserved
+const bluezPermanentSlotSecComp = `
+# Description: Allow operating as the bluez service. This gives privileged
+# access to the system.
 accept
 accept4
 bind
-connect
-getpeername
-getsockname
-getsockopt
 listen
-recv
-recvfrom
-recvmmsg
-recvmsg
-send
-sendmmsg
-sendmsg
-sendto
-setsockopt
-shutdown
-socketpair
-socket
-`)
+# libudev
+socket AF_NETLINK - NETLINK_KOBJECT_UEVENT
+`
 
-var bluezConnectedPlugSecComp = []byte(`
-# Description: Allow using bluez service. Reserved because this gives
-#  privileged access to the bluez service.
-# Usage: reserved
-
-# Can communicate with DBus system service
-connect
-getsockname
-recv
-recvmsg
-send
-sendto
-sendmsg
-socket
-`)
-
-var bluezPermanentSlotDBus = []byte(`
+const bluezPermanentSlotDBus = `
 <policy user="root">
     <allow own="org.bluez"/>
     <allow own="org.bluez.obex"/>
     <allow send_destination="org.bluez"/>
     <allow send_destination="org.bluez.obex"/>
     <allow send_interface="org.bluez.Agent1"/>
+    <allow send_interface="org.bluez.MediaEndpoint1"/>
+    <allow send_interface="org.bluez.MediaPlayer1"/>
     <allow send_interface="org.bluez.ThermometerWatcher1"/>
     <allow send_interface="org.bluez.AlertAgent1"/>
     <allow send_interface="org.bluez.Profile1"/>
@@ -191,56 +197,76 @@ var bluezPermanentSlotDBus = []byte(`
 <policy context="default">
     <deny send_destination="org.bluez"/>
 </policy>
-`)
+`
 
-type BluezInterface struct{}
+type bluezInterface struct{}
 
-func (iface *BluezInterface) Name() string {
+func (iface *bluezInterface) Name() string {
 	return "bluez"
 }
 
-func (iface *BluezInterface) PermanentPlugSnippet(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *BluezInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		old := []byte("###SLOT_SECURITY_TAGS###")
-		new := slotAppLabelExpr(slot)
-		snippet := bytes.Replace(bluezConnectedPlugAppArmor, old, new, -1)
-		return snippet, nil
-	case interfaces.SecuritySecComp:
-		return bluezConnectedPlugSecComp, nil
+func (iface *bluezInterface) StaticInfo() interfaces.StaticInfo {
+	return interfaces.StaticInfo{
+		Summary:              bluezSummary,
+		ImplicitOnClassic:    true,
+		BaseDeclarationSlots: bluezBaseDeclarationSlots,
 	}
-	return nil, nil
 }
 
-func (iface *BluezInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		return bluezPermanentSlotAppArmor, nil
-	case interfaces.SecuritySecComp:
-		return bluezPermanentSlotSecComp, nil
-	case interfaces.SecurityDBus:
-		return bluezPermanentSlotDBus, nil
+func (iface *bluezInterface) DBusPermanentSlot(spec *dbus.Specification, slot *snap.SlotInfo) error {
+	if !release.OnClassic {
+		spec.AddSnippet(bluezPermanentSlotDBus)
 	}
-	return nil, nil
-}
-
-func (iface *BluezInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *BluezInterface) SanitizePlug(plug *interfaces.Plug) error {
 	return nil
 }
 
-func (iface *BluezInterface) SanitizeSlot(slot *interfaces.Slot) error {
+func (iface *bluezInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	old := "###SLOT_SECURITY_TAGS###"
+	var new string
+	if release.OnClassic {
+		new = "unconfined"
+	} else {
+		new = slotAppLabelExpr(slot)
+	}
+	snippet := strings.Replace(bluezConnectedPlugAppArmor, old, new, -1)
+	spec.AddSnippet(snippet)
 	return nil
 }
 
-func (iface *BluezInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
+func (iface *bluezInterface) AppArmorConnectedSlot(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	if !release.OnClassic {
+		old := "###PLUG_SECURITY_TAGS###"
+		new := plugAppLabelExpr(plug)
+		snippet := strings.Replace(bluezConnectedSlotAppArmor, old, new, -1)
+		spec.AddSnippet(snippet)
+	}
+	return nil
+}
+
+func (iface *bluezInterface) UDevConnectedPlug(spec *udev.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	spec.TagDevice(`KERNEL=="rfkill"`)
+	return nil
+}
+
+func (iface *bluezInterface) AppArmorPermanentSlot(spec *apparmor.Specification, slot *snap.SlotInfo) error {
+	if !release.OnClassic {
+		spec.AddSnippet(bluezPermanentSlotAppArmor)
+	}
+	return nil
+}
+
+func (iface *bluezInterface) SecCompPermanentSlot(spec *seccomp.Specification, slot *snap.SlotInfo) error {
+	if !release.OnClassic {
+		spec.AddSnippet(bluezPermanentSlotSecComp)
+	}
+	return nil
+}
+
+func (iface *bluezInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
 	// allow what declarations allowed
 	return true
 }
+
+func init() {
+	registerIface(&bluezInterface{})
+}
