@@ -19,9 +19,22 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const coreSupportSummary = `special permissions for the core snap`
+
+const coreSupportBaseDeclarationPlugs = `
+  core-support:
+    allow-installation:
+      plug-snap-type:
+        - core
+`
+
+const coreSupportBaseDeclarationSlots = `
+  core-support:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
 
 const coreSupportConnectedPlugAppArmor = `
 # Description: Can control all aspects of systemd via the systemctl command,
@@ -32,6 +45,8 @@ const coreSupportConnectedPlugAppArmor = `
 
 /bin/systemctl Uxr,
 
+/usr/bin/snapctl ixr,
+
 # Allow modifying rsyslog configuration for such things as remote logging. For
 # now, only allow modifying NN-snap*.conf and snap*.conf files.
 /etc/rsyslog.d/{,*}                     r,
@@ -53,10 +68,13 @@ const coreSupportConnectedPlugAppArmor = `
 
 # Allow modifying logind configuration. For now, allow reading all logind
 # configuration but only allow modifying NN-snap*.conf and snap*.conf files
-# in /etc/systemd/logind.conf.d.
-/etc/systemd/logind.conf                            r,
-/etc/systemd/logind.conf.d/{,*}                     r,
-/etc/systemd/logind.conf.d/{,[0-9][0-9]-}snap*.conf w,
+# in /etc/systemd/logind.conf.d. Also allow creating the logind.conf.d
+# directory as it may not be there for existing installs (wirtable-path
+# magic oddness).
+/etc/systemd/logind.conf                             r,
+/etc/systemd/logind.conf.d/                          rw,
+/etc/systemd/logind.conf.d/{,*}                      r,
+/etc/systemd/logind.conf.d/{,[0-9][0-9]-}snap*.conf* w,
 
 # Allow managing the hostname with a core config option
 /etc/hostname                         rw,
@@ -69,19 +87,31 @@ const coreSupportConnectedPlugAppArmor = `
 # the core snap, general mgmt of the service is handled via systemctl
 /etc/default/swapfile rw,
 
-# Allow read/write access to the pi2 boot config.txt. WARNING: improperly
-# editing this file may render the system unbootable.
+# Allow read/write access to the pi2 boot config.txt and the directory
+# so that it can dirsync it. 
+# WARNING: improperly editing this file may render the system unbootable.
+owner /boot/uboot/           r,
 owner /boot/uboot/config.txt rwk,
-owner /boot/uboot/config.txt.tmp rwk,
+owner /boot/uboot/config.txt.* rwk,
+
+# Allow read/write /etc/environment so that proxy configuration can
+# be written
+owner /etc/              r,
+owner /etc/environment   rwk,
+owner /etc/environment.* rwk,
 `
 
-// NewShutdownInterface returns a new "shutdown" interface.
-func NewCoreSupportInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "core-support",
-		// NOTE: cure-support implicitly contains the rules from network-bind.
+func init() {
+	registerIface(&commonInterface{
+		name:                 "core-support",
+		summary:              coreSupportSummary,
+		implicitOnCore:       true,
+		implicitOnClassic:    true,
+		baseDeclarationPlugs: coreSupportBaseDeclarationPlugs,
+		baseDeclarationSlots: coreSupportBaseDeclarationSlots,
+		// NOTE: core-support implicitly contains the rules from network-bind.
 		connectedPlugAppArmor: coreSupportConnectedPlugAppArmor + networkBindConnectedPlugAppArmor,
 		connectedPlugSecComp:  "" + networkBindConnectedPlugSecComp,
 		reservedForOS:         true,
-	}
+	})
 }
