@@ -26,9 +26,26 @@ import (
 	"strings"
 
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/interfaces/apparmor"
+	"github.com/snapcore/snapd/interfaces/dbus"
+	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snap"
 )
 
+const dbusSummary = `allows owning a specifc name on DBus`
+
+const dbusBaseDeclarationSlots = `
+  dbus:
+    allow-installation:
+      slot-snap-type:
+        - app
+    deny-connection:
+      slot-attributes:
+        name: .+
+    deny-auto-connection: true
+`
+
 const dbusPermanentSlotAppArmor = `
 # Description: Allow owning a name on DBus public bus
 
@@ -78,6 +95,13 @@ dbus (send)
     path=###DBUS_PATH###
     interface=org.freedesktop.DBus.Properties
     peer=(name=org.freedesktop.DBus, label=unconfined),
+
+# Allow us to introspect org.freedesktop.DBus (needed by pydbus)
+dbus (send)
+    bus=###DBUS_BUS###
+    interface=org.freedesktop.DBus.Introspectable
+    member=Introspect
+    peer=(name=org.freedesktop.DBus, label=unconfined),
 `
 
 const dbusPermanentSlotAppArmorClassic = `
@@ -114,6 +138,13 @@ const dbusPermanentSlotDBus = `
 </policy>
 `
 
+const dbusPermanentSlotSecComp = `
+# Description: Allow owning a name and listening on DBus public bus
+listen
+accept
+accept4
+`
+
 const dbusConnectedSlotAppArmor = `
 # allow snaps to introspect us. This allows clients to introspect all
 # DBus interfaces of this service (but not use them).
@@ -176,17 +207,24 @@ dbus (receive, send)
     peer=(label=###SLOT_SECURITY_TAGS###),
 `
 
-type DbusInterface struct{}
+type dbusInterface struct{}
 
-func (iface *DbusInterface) Name() string {
+func (iface *dbusInterface) Name() string {
 	return "dbus"
 }
 
+func (iface *dbusInterface) StaticInfo() interfaces.StaticInfo {
+	return interfaces.StaticInfo{
+		Summary:              dbusSummary,
+		BaseDeclarationSlots: dbusBaseDeclarationSlots,
+	}
+}
+
 // Obtain yaml-specified bus well-known name
-func (iface *DbusInterface) getAttribs(attribs map[string]interface{}) (string, string, error) {
+func (iface *dbusInterface) getAttribs(attribs interfaces.Attrer) (string, string, error) {
 	// bus attribute
-	bus, ok := attribs["bus"].(string)
-	if !ok {
+	var bus string
+	if err := attribs.Attr("bus", &bus); err != nil {
 		return "", "", fmt.Errorf("cannot find attribute 'bus'")
 	}
 
@@ -195,8 +233,8 @@ func (iface *DbusInterface) getAttribs(attribs map[string]interface{}) (string,
 	}
 
 	// name attribute
-	name, ok := attribs["name"].(string)
-	if !ok {
+	var name string
+	if err := attribs.Attr("name", &name); err != nil {
 		return "", "", fmt.Errorf("cannot find attribute 'name'")
 	}
 
@@ -231,14 +269,14 @@ func getAppArmorAbstraction(bus string) (string, error) {
 }
 
 // Calculate individual snippet policy based on bus and name
-func getAppArmorSnippet(policy []byte, bus string, name string) []byte {
-	old := []byte("###DBUS_BUS###")
-	new := []byte(bus)
-	snippet := bytes.Replace(policy, old, new, -1)
+func getAppArmorSnippet(policy string, bus string, name string) string {
+	old := "###DBUS_BUS###"
+	new := bus
+	snippet := strings.Replace(policy, old, new, -1)
 
-	old = []byte("###DBUS_NAME###")
-	new = []byte(name)
-	snippet = bytes.Replace(snippet, old, new, -1)
+	old = "###DBUS_NAME###"
+	new = name
+	snippet = strings.Replace(snippet, old, new, -1)
 
 	// convert name to AppArmor dbus path (eg 'org.foo' to '/org/foo{,/**}')
 	var pathBuf bytes.Buffer
@@ -246,9 +284,9 @@ func getAppArmorSnippet(policy []byte, bus string, name string) []byte {
 	pathBuf.WriteString(strings.Replace(name, ".", "/", -1))
 	pathBuf.WriteString(`{,/**}"`)
 
-	old = []byte("###DBUS_PATH###")
-	new = pathBuf.Bytes()
-	snippet = bytes.Replace(snippet, old, new, -1)
+	old = "###DBUS_PATH###"
+	new = pathBuf.String()
+	snippet = strings.Replace(snippet, old, new, -1)
 
 	// convert name to AppArmor dbus interface (eg, 'org.foo' to 'org.foo{,.*}')
 	var ifaceBuf bytes.Buffer
@@ -256,118 +294,108 @@ func getAppArmorSnippet(policy []byte, bus string, name string) []byte {
 	ifaceBuf.WriteString(name)
 	ifaceBuf.WriteString(`{,.*}"`)
 
-	old = []byte("###DBUS_INTERFACE###")
-	new = ifaceBuf.Bytes()
-	snippet = bytes.Replace(snippet, old, new, -1)
+	old = "###DBUS_INTERFACE###"
+	new = ifaceBuf.String()
+	snippet = strings.Replace(snippet, old, new, -1)
 
 	return snippet
 }
 
-func (iface *DbusInterface) PermanentPlugSnippet(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *DbusInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	bus, name, err := iface.getAttribs(plug.Attrs)
+func (iface *dbusInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	bus, name, err := iface.getAttribs(plug)
 	if err != nil {
-		return nil, err
+		return err
 	}
 
-	busSlot, nameSlot, err := iface.getAttribs(slot.Attrs)
+	busSlot, nameSlot, err := iface.getAttribs(slot)
 	if err != nil {
-		return nil, err
+		return err
 	}
 
 	// ensure that we only connect to slot with matching attributes
 	if bus != busSlot || name != nameSlot {
-		return nil, nil
+		return nil
 	}
 
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		// well-known DBus name-specific connected plug policy
-		snippet := getAppArmorSnippet([]byte(dbusConnectedPlugAppArmor), bus, name)
+	// well-known DBus name-specific connected plug policy
+	snippet := getAppArmorSnippet(dbusConnectedPlugAppArmor, bus, name)
 
-		// abstraction policy
-		abstraction, err := getAppArmorAbstraction(bus)
-		if err != nil {
-			return nil, err
-		}
-
-		old := []byte("###DBUS_ABSTRACTION###")
-		new := []byte(abstraction)
-		snippet = bytes.Replace(snippet, old, new, -1)
-
-		old = []byte("###SLOT_SECURITY_TAGS###")
-		new = slotAppLabelExpr(slot)
-		snippet = bytes.Replace(snippet, old, new, -1)
-
-		return snippet, nil
+	// abstraction policy
+	abstraction, err := getAppArmorAbstraction(bus)
+	if err != nil {
+		return err
 	}
-	return nil, nil
+
+	old := "###DBUS_ABSTRACTION###"
+	new := abstraction
+	snippet = strings.Replace(snippet, old, new, -1)
+
+	old = "###SLOT_SECURITY_TAGS###"
+	new = slotAppLabelExpr(slot)
+	snippet = strings.Replace(snippet, old, new, -1)
+
+	spec.AddSnippet(snippet)
+	return nil
 }
 
-func (iface *DbusInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		bus, name, err := iface.getAttribs(slot.Attrs)
-		if err != nil {
-			return nil, err
-		}
-
-		snippets := bytes.NewBufferString("")
-
-		// well-known DBus name-specific permanent slot policy
-		snippet := getAppArmorSnippet([]byte(dbusPermanentSlotAppArmor), bus, name)
-
-		// abstraction policy
-		abstraction, err := getAppArmorAbstraction(bus)
-		if err != nil {
-			return nil, err
-		}
-
-		old := []byte("###DBUS_ABSTRACTION###")
-		new := []byte(abstraction)
-		snippet = bytes.Replace(snippet, old, new, -1)
-
-		snippets.Write(snippet)
-
-		if release.OnClassic {
-			// classic-only policy
-			snippets.Write(getAppArmorSnippet([]byte(dbusPermanentSlotAppArmorClassic), bus, name))
-		}
-
-		return snippets.Bytes(), nil
-	case interfaces.SecurityDBus:
-		bus, name, err := iface.getAttribs(slot.Attrs)
-		if err != nil {
-			return nil, err
-
-		}
-
-		// only system services need bus policy
-		if bus != "system" {
-			return nil, nil
-		}
-
-		old := []byte("###DBUS_NAME###")
-		new := []byte(name)
-		snippet := bytes.Replace([]byte(dbusPermanentSlotDBus), old, new, -1)
-
-		return []byte(snippet), nil
+func (iface *dbusInterface) DBusPermanentSlot(spec *dbus.Specification, slot *snap.SlotInfo) error {
+	bus, name, err := iface.getAttribs(slot)
+	if err != nil {
+		return err
 	}
-	return nil, nil
+
+	// only system services need bus policy
+	if bus != "system" {
+		return nil
+	}
+
+	old := "###DBUS_NAME###"
+	new := name
+	spec.AddSnippet(strings.Replace(dbusPermanentSlotDBus, old, new, -1))
+	return nil
 }
 
-func (iface *DbusInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	bus, name, err := iface.getAttribs(slot.Attrs)
+func (iface *dbusInterface) AppArmorPermanentSlot(spec *apparmor.Specification, slot *snap.SlotInfo) error {
+	bus, name, err := iface.getAttribs(slot)
 	if err != nil {
-		return nil, err
+		return err
 	}
 
-	busPlug, namePlug, err := iface.getAttribs(plug.Attrs)
+	// well-known DBus name-specific permanent slot policy
+	snippet := getAppArmorSnippet(dbusPermanentSlotAppArmor, bus, name)
+
+	// abstraction policy
+	abstraction, err := getAppArmorAbstraction(bus)
 	if err != nil {
-		return nil, err
+		return err
+	}
+
+	old := "###DBUS_ABSTRACTION###"
+	new := abstraction
+	snippet = strings.Replace(snippet, old, new, -1)
+	spec.AddSnippet(snippet)
+
+	if release.OnClassic {
+		// classic-only policy
+		spec.AddSnippet(getAppArmorSnippet(dbusPermanentSlotAppArmorClassic, bus, name))
+	}
+	return nil
+}
+
+func (iface *dbusInterface) SecCompPermanentSlot(spec *seccomp.Specification, slot *snap.SlotInfo) error {
+	spec.AddSnippet(dbusPermanentSlotSecComp)
+	return nil
+}
+
+func (iface *dbusInterface) AppArmorConnectedSlot(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	bus, name, err := iface.getAttribs(slot)
+	if err != nil {
+		return err
+	}
+
+	busPlug, namePlug, err := iface.getAttribs(plug)
+	if err != nil {
+		return err
 	}
 
 	// ensure that we only connect to slot with matching attributes. This
@@ -376,42 +404,34 @@ func (iface *DbusInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *in
 	// TODO: we can fix the 'snap interfaces' issue when interface/policy
 	// checkers when they are available
 	if bus != busPlug || name != namePlug {
-		return nil, nil
+		return nil
 	}
 
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		// well-known DBus name-specific connected slot policy
-		snippet := getAppArmorSnippet([]byte(dbusConnectedSlotAppArmor), bus, name)
+	// well-known DBus name-specific connected slot policy
+	snippet := getAppArmorSnippet(dbusConnectedSlotAppArmor, bus, name)
 
-		old := []byte("###PLUG_SECURITY_TAGS###")
-		new := plugAppLabelExpr(plug)
-		snippet = bytes.Replace(snippet, old, new, -1)
-
-		return snippet, nil
-	}
-	return nil, nil
+	old := "###PLUG_SECURITY_TAGS###"
+	new := plugAppLabelExpr(plug)
+	snippet = strings.Replace(snippet, old, new, -1)
+	spec.AddSnippet(snippet)
+	return nil
 }
 
-func (iface *DbusInterface) SanitizePlug(plug *interfaces.Plug) error {
-	if iface.Name() != plug.Interface {
-		panic(fmt.Sprintf("plug is not of interface %q", iface))
-	}
-
-	_, _, err := iface.getAttribs(plug.Attrs)
+func (iface *dbusInterface) BeforePreparePlug(plug *snap.PlugInfo) error {
+	_, _, err := iface.getAttribs(plug)
 	return err
 }
 
-func (iface *DbusInterface) SanitizeSlot(slot *interfaces.Slot) error {
-	if iface.Name() != slot.Interface {
-		panic(fmt.Sprintf("slot is not of interface %q", iface))
-	}
-
-	_, _, err := iface.getAttribs(slot.Attrs)
+func (iface *dbusInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
+	_, _, err := iface.getAttribs(slot)
 	return err
 }
 
-func (iface *DbusInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
+func (iface *dbusInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
 	// allow what declarations allowed
 	return true
 }
+
+func init() {
+	registerIface(&dbusInterface{})
+}
