@@ -23,8 +23,27 @@ import (
 	"fmt"
 
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/interfaces/apparmor"
+	"github.com/snapcore/snapd/interfaces/seccomp"
+	"github.com/snapcore/snapd/snap"
 )
 
+const dockerSupportSummary = `allows operating as the Docker daemon`
+
+const dockerSupportBaseDeclarationPlugs = `
+  docker-support:
+    allow-installation: false
+    deny-auto-connection: true
+`
+
+const dockerSupportBaseDeclarationSlots = `
+  docker-support:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
+
 const dockerSupportConnectedPlugAppArmor = `
 # Description: allow operating as the Docker daemon. This policy is
 # intentionally not restrictive and is here to help guard against programming
@@ -99,7 +118,7 @@ signal (send) peer=docker-default,
 ptrace (read, trace) peer=docker-default,
 
 # Graph (storage) driver bits
-/{dev,run}/shm/aufs.xino rw,
+/{dev,run}/shm/aufs.xino mrw,
 /proc/fs/aufs/plink_maint w,
 /sys/fs/aufs/** r,
 
@@ -518,54 +537,44 @@ const dockerSupportPrivilegedSecComp = `
 @unrestricted
 `
 
-type DockerSupportInterface struct{}
+type dockerSupportInterface struct{}
 
-func (iface *DockerSupportInterface) Name() string {
+func (iface *dockerSupportInterface) Name() string {
 	return "docker-support"
 }
 
-func (iface *DockerSupportInterface) PermanentPlugSnippet(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *DockerSupportInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	privileged, _ := plug.Attrs["privileged-containers"].(bool)
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		snippet := []byte(dockerSupportConnectedPlugAppArmor)
-		if privileged {
-			snippet = append(snippet, dockerSupportPrivilegedAppArmor...)
-		}
-		return snippet, nil
-	case interfaces.SecuritySecComp:
-		snippet := []byte(dockerSupportConnectedPlugSecComp)
-		if privileged {
-			snippet = append(snippet, dockerSupportPrivilegedSecComp...)
-		}
-		return snippet, nil
+func (iface *dockerSupportInterface) StaticInfo() interfaces.StaticInfo {
+	return interfaces.StaticInfo{
+		Summary:              dockerSupportSummary,
+		ImplicitOnCore:       true,
+		ImplicitOnClassic:    true,
+		BaseDeclarationPlugs: dockerSupportBaseDeclarationPlugs,
+		BaseDeclarationSlots: dockerSupportBaseDeclarationSlots,
 	}
-	return nil, nil
 }
 
-func (iface *DockerSupportInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *DockerSupportInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *DockerSupportInterface) SanitizeSlot(slot *interfaces.Slot) error {
-	if iface.Name() != slot.Interface {
-		panic(fmt.Sprintf("slot is not of interface %q", iface.Name()))
+func (iface *dockerSupportInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	var privileged bool
+	_ = plug.Attr("privileged-containers", &privileged)
+	spec.AddSnippet(dockerSupportConnectedPlugAppArmor)
+	if privileged {
+		spec.AddSnippet(dockerSupportPrivilegedAppArmor)
 	}
 	return nil
 }
 
-func (iface *DockerSupportInterface) SanitizePlug(plug *interfaces.Plug) error {
-	if iface.Name() != plug.Interface {
-		panic(fmt.Sprintf("plug is not of interface %q", iface.Name()))
+func (iface *dockerSupportInterface) SecCompConnectedPlug(spec *seccomp.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	var privileged bool
+	_ = plug.Attr("privileged-containers", &privileged)
+	snippet := dockerSupportConnectedPlugSecComp
+	if privileged {
+		snippet += dockerSupportPrivilegedSecComp
 	}
+	spec.AddSnippet(snippet)
+	return nil
+}
+
+func (iface *dockerSupportInterface) BeforePreparePlug(plug *snap.PlugInfo) error {
 	if v, ok := plug.Attrs["privileged-containers"]; ok {
 		if _, ok = v.(bool); !ok {
 			return fmt.Errorf("docker-support plug requires bool with 'privileged-containers'")
@@ -574,7 +583,11 @@ func (iface *DockerSupportInterface) SanitizePlug(plug *interfaces.Plug) error {
 	return nil
 }
 
-func (iface *DockerSupportInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
+func (iface *dockerSupportInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
 	// allow what declarations allowed
 	return true
 }
+
+func init() {
+	registerIface(&dockerSupportInterface{})
+}
