@@ -20,11 +20,26 @@
 package builtin
 
 import (
-	"bytes"
+	"strings"
 
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/interfaces/apparmor"
+	"github.com/snapcore/snapd/interfaces/dbus"
+	"github.com/snapcore/snapd/interfaces/seccomp"
+	"github.com/snapcore/snapd/snap"
 )
 
+const fwupdSummary = `allows operating as the fwupd service`
+
+const fwupdBaseDeclarationSlots = `
+  fwupd:
+    allow-installation:
+      slot-snap-type:
+        - app
+    deny-connection: true
+    deny-auto-connection: true
+`
+
 const fwupdPermanentSlotAppArmor = `
 # Description: Allow operating as the fwupd service. This gives privileged
 # access to the system.
@@ -92,10 +107,29 @@ const fwupdConnectedPlugAppArmor = `
   #Can access the network
   #include <abstractions/nameservice>
   #include <abstractions/ssl_certs>
+  /run/systemd/resolve/stub-resolv.conf r,
 
   # DBus accesses
   #include <abstractions/dbus-strict>
 
+  # systemd-resolved (not yet included in nameservice abstraction)
+  #
+  # Allow access to the safe members of the systemd-resolved D-Bus API:
+  #
+  #   https://www.freedesktop.org/wiki/Software/systemd/resolved/
+  #
+  # This API may be used directly over the D-Bus system bus or it may be used
+  # indirectly via the nss-resolve plugin:
+  #
+  #   https://www.freedesktop.org/software/systemd/man/nss-resolve.html
+  #
+  dbus send
+       bus=system
+       path="/org/freedesktop/resolve1"
+       interface="org.freedesktop.resolve1.Manager"
+       member="Resolve{Address,Hostname,Record,Service}"
+       peer=(name="org.freedesktop.resolve1"),
+
   # Allow access to fwupd service
   dbus (receive, send)
       bus=system
@@ -108,6 +142,15 @@ const fwupdConnectedPlugAppArmor = `
       path=/
       interface=org.freedesktop.DBus.Properties
       peer=(label=###SLOT_SECURITY_TAGS###),
+
+  # Allow clients to introspect the service on non-classic (due to the path,
+  # allowing on classic would reveal too much for unconfined)
+  dbus (send)
+      bus=system
+      path=/
+      interface=org.freedesktop.DBus.Introspectable
+      member=Introspect
+      peer=(label=###SLOT_SECURITY_TAGS###),
 `
 
 const fwupdConnectedSlotAppArmor = `
@@ -160,77 +203,72 @@ const fwupdPermanentSlotSecComp = `
 # access to the system.
 # Can communicate with DBus system service
 bind
+# for udev
+socket AF_NETLINK - NETLINK_KOBJECT_UEVENT
 `
-
 const fwupdConnectedPlugSecComp = `
 # Description: Allow using fwupd service. Reserved because this gives
 # privileged access to the fwupd service.
 bind
 `
 
-// FwupdInterface type
-type FwupdInterface struct{}
+// fwupdInterface type
+type fwupdInterface struct{}
 
-// Name of the FwupdInterface
-func (iface *FwupdInterface) Name() string {
+// Name of the fwupdInterface
+func (iface *fwupdInterface) Name() string {
 	return "fwupd"
 }
 
-// PermanentPlugSnippet - no slot snippets provided
-func (iface *FwupdInterface) PermanentPlugSnippet(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-// ConnectedPlugSnippet returns security snippets for plug at connection
-func (iface *FwupdInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		old := []byte("###SLOT_SECURITY_TAGS###")
-		new := slotAppLabelExpr(slot)
-		snippet := bytes.Replace([]byte(fwupdConnectedPlugAppArmor), old, new, -1)
-		return snippet, nil
-	case interfaces.SecuritySecComp:
-		return []byte(fwupdConnectedPlugSecComp), nil
+func (iface *fwupdInterface) StaticInfo() interfaces.StaticInfo {
+	return interfaces.StaticInfo{
+		Summary:              fwupdSummary,
+		BaseDeclarationSlots: fwupdBaseDeclarationSlots,
 	}
-	return nil, nil
 }
 
-// PermanentSlotSnippet returns security snippets for slot at install
-func (iface *FwupdInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		return []byte(fwupdPermanentSlotAppArmor), nil
-	case interfaces.SecurityDBus:
-		return []byte(fwupdPermanentSlotDBus), nil
-	case interfaces.SecuritySecComp:
-		return []byte(fwupdPermanentSlotSecComp), nil
-	}
-	return nil, nil
-}
-
-// ConnectedSlotSnippet returns security snippets for slot at connection
-func (iface *FwupdInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		old := []byte("###PLUG_SECURITY_TAGS###")
-		new := plugAppLabelExpr(plug)
-		snippet := bytes.Replace([]byte(fwupdConnectedSlotAppArmor), old, new, -1)
-		return snippet, nil
-	}
-	return nil, nil
-}
-
-// SanitizePlug checks the plug definition is valid
-func (iface *FwupdInterface) SanitizePlug(plug *interfaces.Plug) error {
+func (iface *fwupdInterface) DBusPermanentSlot(spec *dbus.Specification, slot *snap.SlotInfo) error {
+	spec.AddSnippet(fwupdPermanentSlotDBus)
 	return nil
 }
 
-// SanitizeSlot checks the slot definition is valid
-func (iface *FwupdInterface) SanitizeSlot(slot *interfaces.Slot) error {
+func (iface *fwupdInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	old := "###SLOT_SECURITY_TAGS###"
+	new := slotAppLabelExpr(slot)
+	snippet := strings.Replace(fwupdConnectedPlugAppArmor, old, new, -1)
+	spec.AddSnippet(snippet)
 	return nil
 }
 
-func (iface *FwupdInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
+func (iface *fwupdInterface) AppArmorPermanentSlot(spec *apparmor.Specification, slot *snap.SlotInfo) error {
+	spec.AddSnippet(fwupdPermanentSlotAppArmor)
+	return nil
+
+}
+
+func (iface *fwupdInterface) AppArmorConnectedSlot(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	old := "###PLUG_SECURITY_TAGS###"
+	new := plugAppLabelExpr(plug)
+	snippet := strings.Replace(fwupdConnectedSlotAppArmor, old, new, -1)
+	spec.AddSnippet(snippet)
+	return nil
+}
+
+func (iface *fwupdInterface) SecCompConnectedPlug(spec *seccomp.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	spec.AddSnippet(fwupdConnectedPlugSecComp)
+	return nil
+}
+
+func (iface *fwupdInterface) SecCompPermanentSlot(spec *seccomp.Specification, slot *snap.SlotInfo) error {
+	spec.AddSnippet(fwupdPermanentSlotSecComp)
+	return nil
+}
+
+func (iface *fwupdInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
 	// allow what declarations allowed
 	return true
 }
+
+func init() {
+	registerIface(&fwupdInterface{})
+}
