@@ -19,9 +19,16 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const homeSummary = `allows access to non-hidden files in the home directory`
+
+const homeBaseDeclarationSlots = `
+  home:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection:
+      on-classic: false
+`
 
 // http://bazaar.launchpad.net/~ubuntu-security/ubuntu-core-security/trunk/view/head:/data/apparmor/policygroups/ubuntu-core/16.04/home
 const homeConnectedPlugAppArmor = `
@@ -34,13 +41,21 @@ const homeConnectedPlugAppArmor = `
 owner @{HOME}/ r,
 
 # Allow read/write access to all files in @{HOME}, except snap application
-# data in @{HOME}/snaps and toplevel hidden directories in @{HOME}.
-owner @{HOME}/[^s.]**             rwk,
-owner @{HOME}/s[^n]**             rwk,
-owner @{HOME}/sn[^a]**            rwk,
-owner @{HOME}/sna[^p]**           rwk,
+# data in @{HOME}/snap and toplevel hidden directories in @{HOME}.
+owner @{HOME}/[^s.]**             rwklix,
+owner @{HOME}/s[^n]**             rwklix,
+owner @{HOME}/sn[^a]**            rwklix,
+owner @{HOME}/sna[^p]**           rwklix,
+owner @{HOME}/snap[^/]**          rwklix,
+
 # Allow creating a few files not caught above
-owner @{HOME}/{s,sn,sna}{,/} rwk,
+owner @{HOME}/{s,sn,sna}{,/} rwklix,
+
+# Allow access to @{HOME}/snap/ to allow directory traversals from
+# @{HOME}/snap/@{SNAP_NAME} through @{HOME}/snap to @{HOME}. While this leaks
+# snap names, it fixes usability issues for snaps that require this
+# transitional interface.
+owner @{HOME}/snap/ r,
 
 # Allow access to gvfs mounts for files owned by the user (including hidden
 # files; only allow writes to files, not the mount point).
@@ -48,11 +63,14 @@ owner /run/user/[0-9]*/gvfs/{,**} r,
 owner /run/user/[0-9]*/gvfs/*/**  w,
 `
 
-// NewHomeInterface returns a new "home" interface.
-func NewHomeInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "home",
+func init() {
+	registerIface(&commonInterface{
+		name:                  "home",
+		summary:               homeSummary,
+		implicitOnCore:        true,
+		implicitOnClassic:     true,
+		baseDeclarationSlots:  homeBaseDeclarationSlots,
 		connectedPlugAppArmor: homeConnectedPlugAppArmor,
 		reservedForOS:         true,
-	}
+	})
 }
