@@ -28,25 +28,46 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/udev"
+	"github.com/snapcore/snapd/snap"
 )
 
+const iioSummary = `allows access to a specific IIO device`
+
+const iioBaseDeclarationSlots = `
+  iio:
+    allow-installation:
+      slot-snap-type:
+        - gadget
+        - core
+    deny-auto-connection: true
+`
+
 const iioConnectedPlugAppArmor = `
 # Description: Give access to a specific IIO device on the system.
 
 ###IIO_DEVICE_PATH### rw,
 /sys/bus/iio/devices/###IIO_DEVICE_NAME###/ r,
 /sys/bus/iio/devices/###IIO_DEVICE_NAME###/** rwk,
+/sys/devices/**/###IIO_DEVICE_NAME###/ r,
+/sys/devices/**/###IIO_DEVICE_NAME###/** rwk,
 `
 
 // The type for iio interface
-type IioInterface struct{}
+type iioInterface struct{}
 
 // Getter for the name of the iio interface
-func (iface *IioInterface) Name() string {
+func (iface *iioInterface) Name() string {
 	return "iio"
 }
 
-func (iface *IioInterface) String() string {
+func (iface *iioInterface) StaticInfo() interfaces.StaticInfo {
+	return interfaces.StaticInfo{
+		Summary:              iioSummary,
+		BaseDeclarationSlots: iioBaseDeclarationSlots,
+	}
+}
+
+func (iface *iioInterface) String() string {
 	return iface.Name()
 }
 
@@ -56,16 +77,9 @@ func (iface *IioInterface) String() string {
 var iioControlDeviceNodePattern = regexp.MustCompile("^/dev/iio:device[0-9]+$")
 
 // Check validity of the defined slot
-func (iface *IioInterface) SanitizeSlot(slot *interfaces.Slot) error {
-	// Does it have right type?
-	if iface.Name() != slot.Interface {
-		panic(fmt.Sprintf("slot is not of interface %q", iface))
-	}
-
-	// Creation of the slot of this type
-	// is allowed only by a gadget or os snap
-	if !(slot.Snap.Type == "gadget" || slot.Snap.Type == "os") {
-		return fmt.Errorf("%s slots only allowed on gadget or core snaps", iface.Name())
+func (iface *iioInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
+	if err := sanitizeSlotReservedForOSOrGadget(iface, slot); err != nil {
+		return err
 	}
 
 	// Validate the path
@@ -83,18 +97,9 @@ func (iface *IioInterface) SanitizeSlot(slot *interfaces.Slot) error {
 	return nil
 }
 
-// Checks and possibly modifies a plug
-func (iface *IioInterface) SanitizePlug(plug *interfaces.Plug) error {
-	if iface.Name() != plug.Interface {
-		panic(fmt.Sprintf("plug is not of interface %q", iface))
-	}
-	// Currently nothing is checked on the plug side
-	return nil
-}
-
-func (iface *IioInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.Plug, slot *interfaces.Slot) error {
-	path, pathOk := slot.Attrs["path"].(string)
-	if !pathOk {
+func (iface *iioInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	var path string
+	if err := slot.Attr("path", &path); err != nil {
 		return nil
 	}
 
@@ -102,7 +107,7 @@ func (iface *IioInterface) AppArmorConnectedPlug(spec *apparmor.Specification, p
 	snippet := strings.Replace(iioConnectedPlugAppArmor, "###IIO_DEVICE_PATH###", cleanedPath, -1)
 
 	// The path is already verified against a regular expression
-	// in SanitizeSlot so we can rely on its structure here and
+	// in BeforePrepareSlot so we can rely on its structure here and
 	// safely strip the '/dev/' prefix to get the actual name of
 	// the IIO device.
 	deviceName := strings.TrimPrefix(path, "/dev/")
@@ -112,21 +117,20 @@ func (iface *IioInterface) AppArmorConnectedPlug(spec *apparmor.Specification, p
 	return nil
 }
 
-func (iface *IioInterface) UDevConnectedPlug(spec *udev.Specification, plug *interfaces.Plug, slot *interfaces.Slot) error {
-	path, pathOk := slot.Attrs["path"].(string)
-	if !pathOk {
+func (iface *iioInterface) UDevConnectedPlug(spec *udev.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	var path string
+	if err := slot.Attr("path", &path); err != nil {
 		return nil
 	}
-	const pathPrefix = "/dev/"
-	const udevRule = `KERNEL=="%s", TAG+="%s"`
-	for appName := range plug.Apps {
-		tag := udevSnapSecurityName(plug.Snap.Name(), appName)
-		spec.AddSnippet(fmt.Sprintf(udevRule, strings.TrimPrefix(path, pathPrefix), tag))
-	}
+	spec.TagDevice(fmt.Sprintf(`KERNEL=="%s"`, strings.TrimPrefix(path, "/dev/")))
 	return nil
 }
 
-func (iface *IioInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
+func (iface *iioInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
 	// Allow what is allowed in the declarations
 	return true
 }
+
+func init() {
+	registerIface(&iioInterface{})
+}
