@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -19,9 +19,21 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const kernelModuleControlSummary = `allows insertion, removal and querying of kernel modules`
+
+const kernelModuleControlBaseDeclarationPlugs = `
+  kernel-module-control:
+    allow-installation: false
+    deny-auto-connection: true
+`
+
+const kernelModuleControlBaseDeclarationSlots = `
+  kernel-module-control:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
 
 const kernelModuleControlConnectedPlugAppArmor = `
 # Description: Allow insertion, removal and querying of modules.
@@ -36,6 +48,9 @@ const kernelModuleControlConnectedPlugAppArmor = `
   # /proc/sys/kernel/dmesg_restrict is '1' (syslog(2)). These operations are
   # required to verify kernel modules that are loaded.
   capability syslog,
+
+  # Allow plug side to read information about loaded kernel modules
+  /sys/module/{,**} r,
 `
 
 const kernelModuleControlConnectedPlugSecComp = `
@@ -46,12 +61,19 @@ finit_module
 delete_module
 `
 
-// NewKernelModuleControlInterface returns a new "kernel-module" interface.
-func NewKernelModuleControlInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "kernel-module-control",
+var kernelModuleControlConnectedPlugUDev = []string{`KERNEL=="mem"`}
+
+func init() {
+	registerIface(&commonInterface{
+		name:                  "kernel-module-control",
+		summary:               kernelModuleControlSummary,
+		implicitOnCore:        true,
+		implicitOnClassic:     true,
+		baseDeclarationPlugs:  kernelModuleControlBaseDeclarationPlugs,
+		baseDeclarationSlots:  kernelModuleControlBaseDeclarationSlots,
 		connectedPlugAppArmor: kernelModuleControlConnectedPlugAppArmor,
 		connectedPlugSecComp:  kernelModuleControlConnectedPlugSecComp,
+		connectedPlugUDev:     kernelModuleControlConnectedPlugUDev,
 		reservedForOS:         true,
-	}
+	})
 }
