@@ -19,9 +19,15 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const localeControlSummary = `allows control over system locale`
+
+const localeControlBaseDeclarationSlots = `
+  locale-control:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
 
 // http://bazaar.launchpad.net/~ubuntu-security/ubuntu-core-security/trunk/view/head:/data/apparmor/policygroups/ubuntu-core/16.04/locale-control
 const localeControlConnectedPlugAppArmor = `
@@ -31,11 +37,13 @@ const localeControlConnectedPlugAppArmor = `
 /etc/default/locale rw,
 `
 
-// NewLocaleControlInterface returns a new "locale-control" interface.
-func NewLocaleControlInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "locale-control",
+func init() {
+	registerIface(&commonInterface{
+		name:                  "locale-control",
+		summary:               localeControlSummary,
+		implicitOnClassic:     true,
+		baseDeclarationSlots:  localeControlBaseDeclarationSlots,
 		connectedPlugAppArmor: localeControlConnectedPlugAppArmor,
 		reservedForOS:         true,
-	}
+	})
 }
