@@ -20,14 +20,29 @@
 package builtin
 
 import (
-	"bytes"
 	"fmt"
 	"regexp"
+	"strings"
 
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snap"
 )
 
+const mprisSummary = `allows operating as an MPRIS player`
+
+const mprisBaseDeclarationSlots = `
+  mpris:
+    allow-installation:
+      slot-snap-type:
+        - app
+    deny-connection:
+      slot-attributes:
+        name: .+
+    deny-auto-connection: true
+`
+
 const mprisPermanentSlotAppArmor = `
 # Description: Allow operating as an MPRIS player.
 
@@ -137,60 +152,51 @@ dbus (send)
     peer=(label=###SLOT_SECURITY_TAGS###),
 `
 
-type MprisInterface struct{}
+type mprisInterface struct{}
 
-func (iface *MprisInterface) Name() string {
+func (iface *mprisInterface) Name() string {
 	return "mpris"
 }
 
-func (iface *MprisInterface) PermanentPlugSnippet(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *MprisInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		old := []byte("###SLOT_SECURITY_TAGS###")
-		new := slotAppLabelExpr(slot)
-		snippet := bytes.Replace([]byte(mprisConnectedPlugAppArmor), old, new, -1)
-		return snippet, nil
+func (iface *mprisInterface) StaticInfo() interfaces.StaticInfo {
+	return interfaces.StaticInfo{
+		Summary:              mprisSummary,
+		BaseDeclarationSlots: mprisBaseDeclarationSlots,
 	}
-	return nil, nil
 }
 
-func (iface *MprisInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		name, err := iface.getName(slot.Attrs)
-		if err != nil {
-			return nil, err
-		}
+func (iface *mprisInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	old := "###SLOT_SECURITY_TAGS###"
+	new := slotAppLabelExpr(slot)
+	spec.AddSnippet(strings.Replace(mprisConnectedPlugAppArmor, old, new, -1))
+	return nil
+}
 
-		old := []byte("###MPRIS_NAME###")
-		new := []byte(name)
-		snippet := bytes.Replace([]byte(mprisPermanentSlotAppArmor), old, new, -1)
-		// on classic, allow unconfined remotes to control the player
-		// (eg, indicator-sound)
-		if release.OnClassic {
-			snippet = append(snippet, mprisConnectedSlotAppArmorClassic...)
-		}
-		return snippet, nil
+func (iface *mprisInterface) AppArmorPermanentSlot(spec *apparmor.Specification, slot *snap.SlotInfo) error {
+	name, err := iface.getName(slot.Attrs)
+	if err != nil {
+		return err
 	}
-	return nil, nil
-}
 
-func (iface *MprisInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		old := []byte("###PLUG_SECURITY_TAGS###")
-		new := plugAppLabelExpr(plug)
-		snippet := bytes.Replace([]byte(mprisConnectedSlotAppArmor), old, new, -1)
-		return snippet, nil
+	old := "###MPRIS_NAME###"
+	new := name
+	spec.AddSnippet(strings.Replace(mprisPermanentSlotAppArmor, old, new, -1))
+	// on classic, allow unconfined remotes to control the player
+	// (eg, indicator-sound)
+	if release.OnClassic {
+		spec.AddSnippet(mprisConnectedSlotAppArmorClassic)
 	}
-	return nil, nil
+	return nil
 }
 
-func (iface *MprisInterface) getName(attribs map[string]interface{}) (string, error) {
+func (iface *mprisInterface) AppArmorConnectedSlot(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	old := "###PLUG_SECURITY_TAGS###"
+	new := plugAppLabelExpr(plug)
+	spec.AddSnippet(strings.Replace(mprisConnectedSlotAppArmor, old, new, -1))
+	return nil
+}
+
+func (iface *mprisInterface) getName(attribs map[string]interface{}) (string, error) {
 	// default to snap name if 'name' attribute not set
 	mprisName := "@{SNAP_NAME}"
 	for attr := range attribs {
@@ -215,20 +221,16 @@ func (iface *MprisInterface) getName(attribs map[string]interface{}) (string, er
 	return mprisName, nil
 }
 
-func (iface *MprisInterface) SanitizePlug(slot *interfaces.Plug) error {
-	return nil
-}
-
-func (iface *MprisInterface) SanitizeSlot(slot *interfaces.Slot) error {
-	if iface.Name() != slot.Interface {
-		panic(fmt.Sprintf("slot is not of interface %q", iface))
-	}
-
+func (iface *mprisInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
 	_, err := iface.getName(slot.Attrs)
 	return err
 }
 
-func (iface *MprisInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
+func (iface *mprisInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
 	// allow what declarations allowed
 	return true
 }
+
+func init() {
+	registerIface(&mprisInterface{})
+}
