@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,24 +20,39 @@
 package builtin
 
 import (
-	"bytes"
+	"strings"
 
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/interfaces/apparmor"
+	"github.com/snapcore/snapd/interfaces/dbus"
+	"github.com/snapcore/snapd/interfaces/seccomp"
+	"github.com/snapcore/snapd/interfaces/udev"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snap"
 )
 
-var networkManagerPermanentSlotAppArmor = []byte(`
-# Description: Allow operating as the NetworkManager service. Reserved because this
-#  gives privileged access to the system.
-# Usage: reserved
+const networkManagerSummary = `allows operating as the NetworkManager service`
+
+const networkManagerBaseDeclarationSlots = `
+  network-manager:
+    allow-installation:
+      slot-snap-type:
+        - app
+        - core
+    deny-auto-connection: true
+    deny-connection:
+      on-classic: false
+`
+
+const networkManagerPermanentSlotAppArmor = `
+# Description: Allow operating as the NetworkManager service. This gives
+# privileged access to the system.
 
 capability net_admin,
 capability net_bind_service,
 capability net_raw,
 
 network netlink,
-network netlink raw,
-network netlink dgram,
 network bridge,
 network inet,
 network inet6,
@@ -69,6 +84,12 @@ network packet,
 
 /run/udev/data/* r,
 
+# Allow read and write access for all netplan configuration files
+# as NetworkManager will start using them to store the network
+# configuration instead of using its own internal keyfile based
+# format.
+/etc/netplan/{,**} rw,
+
 # Allow access to configuration files generated on the fly
 # from netplan and let NetworkManager store its DHCP leases
 # in the dhcp subdirectory so that console-conf can access
@@ -94,16 +115,40 @@ network packet,
 /etc/resolvconf/update.d/* ix,
 
 #include <abstractions/nameservice>
+/run/systemd/resolve/stub-resolv.conf r,
+
+# Explicitly deny plugging snaps from ptracing the slot to silence noisy
+# denials. Neither the NetworkManager service nor nmcli require ptrace
+# trace for full functionality.
+deny ptrace (trace) peer=###PLUG_SECURITY_TAGS###,
 
 # DBus accesses
 #include <abstractions/dbus-strict>
 
+# systemd-resolved (not yet included in nameservice abstraction)
+#
+# Allow access to the safe members of the systemd-resolved D-Bus API:
+#
+#   https://www.freedesktop.org/wiki/Software/systemd/resolved/
+#
+# This API may be used directly over the D-Bus system bus or it may be used
+# indirectly via the nss-resolve plugin:
+#
+#   https://www.freedesktop.org/software/systemd/man/nss-resolve.html
+#
+dbus send
+     bus=system
+     path="/org/freedesktop/resolve1"
+     interface="org.freedesktop.resolve1.Manager"
+     member="Resolve{Address,Hostname,Record,Service}"
+     peer=(name="org.freedesktop.resolve1"),
+
 dbus (send)
    bus=system
    path=/org/freedesktop/DBus
    interface=org.freedesktop.DBus
    member={Request,Release}Name
-   peer=(name=org.freedesktop.DBus),
+   peer=(name=org.freedesktop.DBus, label=unconfined),
 
 dbus (receive, send)
    bus=system
@@ -124,17 +169,20 @@ dbus (bind)
     bus=system
     name="org.freedesktop.NetworkManager",
 
-# Allow traffic to/from our path and interface with any method
+# Allow traffic to/from our path and interface with any method for unconfined
+# clients to talk to our service.
 dbus (receive, send)
     bus=system
     path=/org/freedesktop/NetworkManager{,/**}
-    interface=org.freedesktop.NetworkManager*,
+    interface=org.freedesktop.NetworkManager*
+    peer=(label=unconfined),
 
 # Allow traffic to/from org.freedesktop.DBus for NetworkManager service
 dbus (receive, send)
     bus=system
     path=/org/freedesktop/NetworkManager{,/**}
-    interface=org.freedesktop.DBus.*,
+    interface=org.freedesktop.DBus.*
+    peer=(label=unconfined),
 
 # Allow access to hostname system service
 dbus (receive, send)
@@ -162,6 +210,12 @@ dbus (receive)
     member=PrepareForSleep
     interface=org.freedesktop.login1.Manager
     peer=(label=unconfined),
+dbus (receive)
+    bus=system
+    path=/org/freedesktop/login1
+    interface=org.freedesktop.login1.Manager
+    member=Session{New,Removed}
+    peer=(label=unconfined),
 
 # Allow access to wpa-supplicant for managing WiFi networks
 dbus (receive, send)
@@ -174,12 +228,21 @@ dbus (receive, send)
     path=/fi/w1/wpa_supplicant1{,/**}
     interface=org.freedesktop.DBus.*
     peer=(label=unconfined),
-`)
+`
 
-var networkManagerConnectedPlugAppArmor = []byte(`
-# Description: Allow using NetworkManager service. Reserved because this gives
-#  privileged access to the NetworkManager service.
-# Usage: reserved
+const networkManagerConnectedSlotAppArmor = `
+# Allow connected clients to interact with the service
+
+# Allow traffic to/from our DBus path
+dbus (receive, send)
+    bus=system
+    path=/org/freedesktop/NetworkManager{,/**}
+    peer=(label=###PLUG_SECURITY_TAGS###),
+`
+
+const networkManagerConnectedPlugAppArmor = `
+# Description: Allow using NetworkManager service. This gives privileged access
+# to the NetworkManager service.
 
 #include <abstractions/dbus-strict>
 
@@ -188,69 +251,26 @@ dbus (receive, send)
     bus=system
     path=/org/freedesktop/NetworkManager{,/**}
     peer=(label=###SLOT_SECURITY_TAGS###),
-`)
+`
 
-var networkManagerPermanentSlotSecComp = []byte(`
-# Description: Allow operating as the NetworkManager service. Reserved because this
-#  gives privileged access to the system.
-# Usage: reserved
+const networkManagerConnectedPlugSecComp = `
+# Description: This is needed to talk to the network-manager service
+socket AF_NETLINK - NETLINK_KOBJECT_UEVENT
+`
+
+const networkManagerPermanentSlotSecComp = `
+# Description: Allow operating as the NetworkManager service. This gives
+# privileged access to the system.
 accept
 accept4
 bind
-connect
-getpeername
-getsockname
-getsockopt
 listen
-recv
-recvfrom
-recvmmsg
-recvmsg
-send
-sendmmsg
-sendmsg
-sendto
-setsockopt
 sethostname
-shutdown
-socketpair
-socket
-# Needed for keyfile settings plugin to allow adding settings
-# for different users. This is currently at runtime only used
-# to make new created network settings files only editable by
-# root:root. The existence of this chown call is only that its
-# used for some tests where a different user:group combination
-# will be supplied.
-# FIXME: adjust after seccomp argument filtering lands so that
-# we only allow chown and its variant to be called for root:root
-# and nothign else (LP: #1446748)
-chown
-chown32
-fchown
-fchown32
-fchownat
-lchown
-lchown32
-`)
+# netlink
+socket AF_NETLINK - -
+`
 
-var networkManagerConnectedPlugSecComp = []byte(`
-# Description: Allow using NetworkManager service. Reserved because this gives
-#  privileged access to the NetworkManager service.
-# Usage: reserved
-
-# Can communicate with DBus system service
-connect
-getsockname
-recv
-recvmsg
-recvfrom
-send
-sendto
-sendmsg
-socket
-`)
-
-var networkManagerPermanentSlotDBus = []byte(`
+const networkManagerPermanentSlotDBus = `
 <!-- DBus policy for NetworkManager (upstream version 1.2.2) -->
 <policy user="root">
     <allow own="org.freedesktop.NetworkManager"/>
@@ -392,65 +412,75 @@ var networkManagerPermanentSlotDBus = []byte(`
 
 <limit name="max_replies_per_connection">1024</limit>
 <limit name="max_match_rules_per_connection">2048</limit>
-`)
+`
 
-type NetworkManagerInterface struct{}
+type networkManagerInterface struct{}
 
-func (iface *NetworkManagerInterface) Name() string {
+func (iface *networkManagerInterface) Name() string {
 	return "network-manager"
 }
 
-func (iface *NetworkManagerInterface) PermanentPlugSnippet(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *NetworkManagerInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityDBus:
-		return nil, nil
-	case interfaces.SecurityAppArmor:
-		old := []byte("###SLOT_SECURITY_TAGS###")
-		var new []byte
-		if release.OnClassic {
-			// If we're running on classic NetworkManager will be part
-			// of the OS snap and will run unconfined.
-			new = []byte("unconfined")
-		} else {
-			new = slotAppLabelExpr(slot)
-		}
-		snippet := bytes.Replace(networkManagerConnectedPlugAppArmor, old, new, -1)
-		return snippet, nil
-	case interfaces.SecuritySecComp:
-		return networkManagerConnectedPlugSecComp, nil
+func (iface *networkManagerInterface) StaticInfo() interfaces.StaticInfo {
+	return interfaces.StaticInfo{
+		Summary:              networkManagerSummary,
+		ImplicitOnClassic:    true,
+		BaseDeclarationSlots: networkManagerBaseDeclarationSlots,
 	}
-	return nil, nil
 }
 
-func (iface *NetworkManagerInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		return networkManagerPermanentSlotAppArmor, nil
-	case interfaces.SecuritySecComp:
-		return networkManagerPermanentSlotSecComp, nil
-	case interfaces.SecurityDBus:
-		return networkManagerPermanentSlotDBus, nil
+func (iface *networkManagerInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	old := "###SLOT_SECURITY_TAGS###"
+	var new string
+	if release.OnClassic {
+		// If we're running on classic NetworkManager will be part
+		// of the OS snap and will run unconfined.
+		new = "unconfined"
+	} else {
+		new = slotAppLabelExpr(slot)
 	}
-	return nil, nil
-}
-
-func (iface *NetworkManagerInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *NetworkManagerInterface) SanitizePlug(plug *interfaces.Plug) error {
+	snippet := strings.Replace(networkManagerConnectedPlugAppArmor, old, new, -1)
+	spec.AddSnippet(snippet)
 	return nil
 }
 
-func (iface *NetworkManagerInterface) SanitizeSlot(slot *interfaces.Slot) error {
+func (iface *networkManagerInterface) AppArmorConnectedSlot(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	old := "###PLUG_SECURITY_TAGS###"
+	new := plugAppLabelExpr(plug)
+	snippet := strings.Replace(networkManagerConnectedSlotAppArmor, old, new, -1)
+	spec.AddSnippet(snippet)
 	return nil
 }
 
-func (iface *NetworkManagerInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
+func (iface *networkManagerInterface) AppArmorPermanentSlot(spec *apparmor.Specification, slot *snap.SlotInfo) error {
+	spec.AddSnippet(networkManagerPermanentSlotAppArmor)
+	return nil
+}
+
+func (iface *networkManagerInterface) DBusPermanentSlot(spec *dbus.Specification, slot *snap.SlotInfo) error {
+	spec.AddSnippet(networkManagerPermanentSlotDBus)
+	return nil
+}
+
+func (iface *networkManagerInterface) SecCompPermanentSlot(spec *seccomp.Specification, slot *snap.SlotInfo) error {
+	spec.AddSnippet(networkManagerPermanentSlotSecComp)
+	return nil
+}
+
+func (iface *networkManagerInterface) UDevPermanentSlot(spec *udev.Specification, slot *snap.SlotInfo) error {
+	spec.TagDevice(`KERNEL=="rfkill"`)
+	return nil
+}
+
+func (iface *networkManagerInterface) SecCompConnectedPlug(spec *seccomp.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	spec.AddSnippet(networkManagerConnectedPlugSecComp)
+	return nil
+}
+
+func (iface *networkManagerInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
 	// allow what declarations allowed
 	return true
 }
+
+func init() {
+	registerIface(&networkManagerInterface{})
+}
