@@ -19,17 +19,31 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const openvswitchSupportSummary = `allows operating as the openvswitch service`
+
+const openvswitchSupportBaseDeclarationSlots = `
+  openvswitch-support:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
+
+const openvswitchSupportConnectedPlugAppArmor = `
+/run/uuidd/request rw,
+`
 
 var openvswitchSupportConnectedPlugKmod = []string{`openvswitch`}
 
-// NewOpenvSwitchSupportInterface returns a new "openvswitch-support" interface.
-func NewOpenvSwitchSupportInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "openvswitch-support",
+func init() {
+	registerIface(&commonInterface{
+		name:                     "openvswitch-support",
+		summary:                  openvswitchSupportSummary,
+		implicitOnCore:           true,
+		implicitOnClassic:        true,
+		baseDeclarationSlots:     openvswitchSupportBaseDeclarationSlots,
 		connectedPlugKModModules: openvswitchSupportConnectedPlugKmod,
+		connectedPlugAppArmor:    openvswitchSupportConnectedPlugAppArmor,
 		reservedForOS:            true,
-	}
+	})
 }
