@@ -19,9 +19,15 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const pppSummary = `allows operating as the ppp service`
+
+const pppBaseDeclarationSlots = `
+  ppp:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
 
 const pppConnectedPlugAppArmor = `
 # Description: Allow operating ppp daemon. This gives privileged access to the
@@ -45,47 +51,25 @@ capability setuid,
 // ppp_generic creates /dev/ppp. Other ppp modules will be automatically loaded
 // by the kernel on different ioctl calls for this device. Note also that
 // in many cases ppp_generic is statically linked into the kernel (CONFIG_PPP=y)
-var pppConnectedPlugKmod = []byte(`
-ppp_generic
-`)
-
-type PppInterface struct{}
-
-func (iface *PppInterface) Name() string {
-	return "ppp"
+var pppConnectedPlugKmod = []string{
+	"ppp_generic",
 }
 
-func (iface *PppInterface) PermanentPlugSnippet(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
+var pppConnectedPlugUDev = []string{
+	`KERNEL=="ppp"`,
+	`KERNEL=="tty[A-Z]*[0-9]*"`,
 }
 
-func (iface *PppInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	switch securitySystem {
-	case interfaces.SecurityAppArmor:
-		return []byte(pppConnectedPlugAppArmor), nil
-	case interfaces.SecurityKMod:
-		return pppConnectedPlugKmod, nil
-	}
-	return nil, nil
-}
-
-func (iface *PppInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *PppInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-	return nil, nil
-}
-
-func (iface *PppInterface) SanitizePlug(plug *interfaces.Plug) error {
-	return nil
-}
-
-func (iface *PppInterface) SanitizeSlot(slot *interfaces.Slot) error {
-	return nil
-}
-
-func (iface *PppInterface) AutoConnect(*interfaces.Plug, *interfaces.Slot) bool {
-	// allow what declarations allowed
-	return true
+func init() {
+	registerIface(&commonInterface{
+		name:                     "ppp",
+		summary:                  pppSummary,
+		implicitOnCore:           true,
+		implicitOnClassic:        true,
+		baseDeclarationSlots:     pppBaseDeclarationSlots,
+		connectedPlugAppArmor:    pppConnectedPlugAppArmor,
+		connectedPlugKModModules: pppConnectedPlugKmod,
+		connectedPlugUDev:        pppConnectedPlugUDev,
+		reservedForOS:            true,
+	})
 }
