@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2018 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -19,17 +19,25 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const processControlSummary = `allows controlling other processes`
+
+const processControlBaseDeclarationSlots = `
+  process-control:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
 
 const processControlConnectedPlugAppArmor = `
 # Description: This interface allows for controlling other processes via
-# signals and nice. This is reserved because it grants privileged access to
-# all processes under root or processes running under the same UID otherwise.
-# Usage: reserved
+# signals, cpu affinity and nice. This is reserved because it grants privileged
+# access to all processes under root or processes running under the same UID
+# otherwise.
 
-/{,usr/}bin/nice ixr,
+# /{,usr/}bin/nice is already in default policy
+/{,usr/}bin/renice ixr,
+/{,usr/}bin/taskset ixr,
 
 capability sys_resource,
 capability sys_nice,
@@ -39,20 +47,27 @@ signal,
 
 const processControlConnectedPlugSecComp = `
 # Description: This interface allows for controlling other processes via
-# signals and nice. This is reserved because it grants privileged access to
-# all processes under root or processes running under the same UID otherwise.
-# Usage: reserved
+# signals, cpu affinity and nice. This is reserved because it grants privileged
+# access to all processes under root or processes running under the same UID
+# otherwise.
 
+# Allow setting the nice value/priority for any process
+nice
 setpriority
 sched_setaffinity
+sched_setparam
+sched_setscheduler
 `
 
-// NewProcessControlInterface returns a new "process-control" interface.
-func NewProcessControlInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "process-control",
+func init() {
+	registerIface(&commonInterface{
+		name:                  "process-control",
+		summary:               processControlSummary,
+		implicitOnCore:        true,
+		implicitOnClassic:     true,
+		baseDeclarationSlots:  processControlBaseDeclarationSlots,
 		connectedPlugAppArmor: processControlConnectedPlugAppArmor,
 		connectedPlugSecComp:  processControlConnectedPlugSecComp,
 		reservedForOS:         true,
-	}
+	})
 }
