@@ -19,23 +19,39 @@
 
 package builtin
 
-import (
-	"github.com/snapcore/snapd/interfaces"
-)
+const removableMediaSummary = `allows access to mounted removable storage`
+
+const removableMediaBaseDeclarationSlots = `
+  removable-media:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
 
 const removableMediaConnectedPlugAppArmor = `
 # Description: Can access removable storage filesystems
 
+# Allow read-access to /run/ for navigating to removable media.
+/run/ r,
+
+# Allow read on /run/media/ for navigating to the mount points. While this
+# allows enumerating users, this is already allowed via /etc/passwd and getent.
+/{,run/}media/ r,
+
 # Mount points could be in /run/media/<user>/* or /media/<user>/*
 /{,run/}media/*/ r,
-/{,run/}media/*/** rw,
+/{,run/}media/*/** rwk,
 `
 
-// NewRemovableMediaInterface returns a new "removable-media" interface.
-func NewRemovableMediaInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "removable-media",
+func init() {
+	registerIface(&commonInterface{
+		name:                  "removable-media",
+		summary:               removableMediaSummary,
+		implicitOnCore:        true,
+		implicitOnClassic:     true,
+		baseDeclarationSlots:  removableMediaBaseDeclarationSlots,
 		connectedPlugAppArmor: removableMediaConnectedPlugAppArmor,
 		reservedForOS:         true,
-	}
+	})
 }
