@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -19,7 +19,15 @@
 
 package builtin
 
-import "github.com/snapcore/snapd/interfaces"
+const tpmSummary = `allows access to the Trusted Platform Module device`
+
+const tpmBaseDeclarationSlots = `
+  tpm:
+    allow-installation:
+      slot-snap-type:
+        - core
+    deny-auto-connection: true
+`
 
 const tpmConnectedPlugAppArmor = `
 # Description: for those who need to talk to the system TPM chip over /dev/tpm0
@@ -27,10 +35,17 @@ const tpmConnectedPlugAppArmor = `
 /dev/tpm0 rw,
 `
 
-func NewTpmInterface() interfaces.Interface {
-	return &commonInterface{
-		name: "tpm",
+var tpmConnectedPlugUDev = []string{`KERNEL=="tpm[0-9]*"`}
+
+func init() {
+	registerIface(&commonInterface{
+		name:                  "tpm",
+		summary:               tpmSummary,
+		implicitOnCore:        true,
+		implicitOnClassic:     true,
+		baseDeclarationSlots:  tpmBaseDeclarationSlots,
 		connectedPlugAppArmor: tpmConnectedPlugAppArmor,
+		connectedPlugUDev:     tpmConnectedPlugUDev,
 		reservedForOS:         true,
-	}
+	})
 }
