@@ -31,33 +31,69 @@ import (
 	"bytes"
 	"fmt"
 	"os"
+	"path/filepath"
 
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 )
 
 // Backend is responsible for maintaining DBus policy files.
 type Backend struct{}
 
+// Initialize does nothing.
+func (b *Backend) Initialize() error {
+	return nil
+}
+
 // Name returns the name of the backend.
 func (b *Backend) Name() interfaces.SecuritySystem {
 	return "dbus"
 }
 
+// setupDbusServiceForUserd will setup the service file for the new
+// `snap userd` instance on re-exec
+func setupDbusServiceForUserd(snapInfo *snap.Info) error {
+	coreRoot := snapInfo.MountDir()
+
+	for _, srv := range []string{
+		"io.snapcraft.Launcher.service",
+		"io.snapcraft.Settings.service",
+	} {
+		dst := filepath.Join("/usr/share/dbus-1/services/", srv)
+		src := filepath.Join(coreRoot, dst)
+		if !osutil.FilesAreEqual(src, dst) {
+			if err := osutil.CopyFile(src, dst, osutil.CopyFlagPreserveAll); err != nil {
+				return err
+			}
+		}
+	}
+	return nil
+}
+
 // Setup creates dbus configuration files specific to a given snap.
 //
 // DBus has no concept of a complain mode so confinment type is ignored.
 func (b *Backend) Setup(snapInfo *snap.Info, opts interfaces.ConfinementOptions, repo *interfaces.Repository) error {
 	snapName := snapInfo.Name()
 	// Get the snippets that apply to this snap
-	snippets, err := repo.SecuritySnippetsForSnap(snapInfo.Name(), interfaces.SecurityDBus)
+	spec, err := repo.SnapSpecification(b.Name(), snapName)
 	if err != nil {
-		return fmt.Errorf("cannot obtain DBus security snippets for snap %q: %s", snapName, err)
+		return fmt.Errorf("cannot obtain dbus specification for snap %q: %s", snapName, err)
 	}
+
+	// core on classic is special
+	if snapName == "core" && release.OnClassic {
+		if err := setupDbusServiceForUserd(snapInfo); err != nil {
+			logger.Noticef("cannot create host `snap userd` dbus service file: %s", err)
+		}
+	}
+
 	// Get the files that this snap should have
-	content, err := b.combineSnippets(snapInfo, snippets)
+	content, err := b.deriveContent(spec.(*Specification), snapInfo)
 	if err != nil {
 		return fmt.Errorf("cannot obtain expected DBus configuration files for snap %q: %s", snapName, err)
 	}
@@ -85,13 +121,13 @@ func (b *Backend) Remove(snapName string) error {
 	return nil
 }
 
-// combineSnippets combines security snippets collected from all the interfaces
+// deriveContent combines security snippets collected from all the interfaces
 // affecting a given snap into a content map applicable to EnsureDirState.
-func (b *Backend) combineSnippets(snapInfo *snap.Info, snippets map[string][][]byte) (content map[string]*osutil.FileState, err error) {
+func (b *Backend) deriveContent(spec *Specification, snapInfo *snap.Info) (content map[string]*osutil.FileState, err error) {
 	for _, appInfo := range snapInfo.Apps {
 		securityTag := appInfo.SecurityTag()
-		appSnippets := snippets[securityTag]
-		if len(appSnippets) == 0 {
+		appSnippets := spec.SnippetForTag(securityTag)
+		if appSnippets == "" {
 			continue
 		}
 		if content == nil {
@@ -103,8 +139,8 @@ func (b *Backend) combineSnippets(snapInfo *snap.Info, snippets map[string][][]b
 
 	for _, hookInfo := range snapInfo.Hooks {
 		securityTag := hookInfo.SecurityTag()
-		hookSnippets := snippets[securityTag]
-		if len(hookSnippets) == 0 {
+		hookSnippets := spec.SnippetForTag(securityTag)
+		if hookSnippets == "" {
 			continue
 		}
 		if content == nil {
@@ -117,13 +153,10 @@ func (b *Backend) combineSnippets(snapInfo *snap.Info, snippets map[string][][]b
 	return content, nil
 }
 
-func addContent(securityTag string, executableSnippets [][]byte, content map[string]*osutil.FileState) {
+func addContent(securityTag string, snippet string, content map[string]*osutil.FileState) {
 	var buffer bytes.Buffer
 	buffer.Write(xmlHeader)
-	for _, snippet := range executableSnippets {
-		buffer.Write(snippet)
-		buffer.WriteRune('\n')
-	}
+	buffer.WriteString(snippet)
 	buffer.Write(xmlFooter)
 
 	content[fmt.Sprintf("%s.conf", securityTag)] = &osutil.FileState{
@@ -133,5 +166,5 @@ func addContent(securityTag string, executableSnippets [][]byte, content map[str
 }
 
 func (b *Backend) NewSpecification() interfaces.Specification {
-	panic(fmt.Errorf("%s is not using specifications yet", b.Name()))
+	return &Specification{}
 }
