@@ -29,10 +29,11 @@ import (
 )
 
 type BackendSuite struct {
-	Backend interfaces.SecurityBackend
-	Repo    *interfaces.Repository
-	Iface   *TestInterface
-	RootDir string
+	Backend         interfaces.SecurityBackend
+	Repo            *interfaces.Repository
+	Iface           *TestInterface
+	RootDir         string
+	restoreSanitize func()
 }
 
 func (s *BackendSuite) SetUpTest(c *C) {
@@ -44,10 +45,13 @@ func (s *BackendSuite) SetUpTest(c *C) {
 	s.Iface = &TestInterface{InterfaceName: "iface"}
 	err := s.Repo.AddInterface(s.Iface)
 	c.Assert(err, IsNil)
+
+	s.restoreSanitize = snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {})
 }
 
 func (s *BackendSuite) TearDownTest(c *C) {
 	dirs.SetRootDir("/")
+	s.restoreSanitize()
 }
 
 // Tests for Setup() and Remove()
@@ -99,6 +103,7 @@ slots:
 `
 const SambaYamlWithHook = `
 name: samba
+version: 0
 apps:
     smbd:
     nmbd:
@@ -174,13 +179,11 @@ func (s *BackendSuite) RemoveSnap(c *C, snapInfo *snap.Info) {
 
 func (s *BackendSuite) addPlugsSlots(c *C, snapInfo *snap.Info) {
 	for _, plugInfo := range snapInfo.Plugs {
-		plug := &interfaces.Plug{PlugInfo: plugInfo}
-		err := s.Repo.AddPlug(plug)
+		err := s.Repo.AddPlug(plugInfo)
 		c.Assert(err, IsNil)
 	}
 	for _, slotInfo := range snapInfo.Slots {
-		slot := &interfaces.Slot{SlotInfo: slotInfo}
-		err := s.Repo.AddSlot(slot)
+		err := s.Repo.AddSlot(slotInfo)
 		c.Assert(err, IsNil)
 	}
 }
