@@ -51,6 +51,11 @@ import (
 // Backend is responsible for maintaining kernel modules
 type Backend struct{}
 
+// Initialize does nothing.
+func (b *Backend) Initialize() error {
+	return nil
+}
+
 // Name returns the name of the backend.
 func (b *Backend) Name() interfaces.SecuritySystem {
 	return "kmod"
@@ -64,18 +69,14 @@ func (b *Backend) Name() interfaces.SecuritySystem {
 func (b *Backend) Setup(snapInfo *snap.Info, confinement interfaces.ConfinementOptions, repo *interfaces.Repository) error {
 	snapName := snapInfo.Name()
 	// Get the snippets that apply to this snap
-	snippets, err := repo.SecuritySnippetsForSnap(snapInfo.Name(), interfaces.SecurityKMod)
+	spec, err := repo.SnapSpecification(b.Name(), snapName)
 	if err != nil {
-		return fmt.Errorf("cannot obtain kmod security snippets for snap %q: %s", snapName, err)
+		return fmt.Errorf("cannot obtain kmod specification for snap %q: %s", snapName, err)
 	}
 
-	// Get the files that this snap should have
+	content, modules := deriveContent(spec.(*Specification), snapInfo)
+	// synchronize the content with the filesystem
 	glob := interfaces.SecurityTagGlob(snapName)
-	content, modules, err := b.combineSnippets(snapInfo, snippets)
-	if err != nil {
-		return fmt.Errorf("cannot obtain expected security files for snap %q: %s", snapName, err)
-	}
-
 	dir := dirs.SnapKModModulesDir
 	if err := os.MkdirAll(dir, 0755); err != nil {
 		return fmt.Errorf("cannot create directory for kmod files %q: %s", dir, err)
@@ -87,7 +88,7 @@ func (b *Backend) Setup(snapInfo *snap.Info, confinement interfaces.ConfinementO
 	}
 
 	if len(changed) > 0 {
-		return loadModules(modules)
+		loadModules(modules)
 	}
 	return nil
 }
@@ -103,54 +104,30 @@ func (b *Backend) Remove(snapName string) error {
 	return err
 }
 
-// combineSnippets combines security snippets collected from all the interfaces
-// affecting a given snap into a de-duplicated list of kernel modules.
-func (b *Backend) combineSnippets(snapInfo *snap.Info, snippets map[string][][]byte) (content map[string]*osutil.FileState, modules []string, err error) {
-	content = make(map[string]*osutil.FileState)
-
-	for _, appInfo := range snapInfo.Apps {
-		for _, snippet := range snippets[appInfo.SecurityTag()] {
-			// split snippet by newline to get the list of modules
-			for _, line := range bytes.Split(snippet, []byte{'\n'}) {
-				l := bytes.TrimSpace(line)
-				// ignore empty lines and comments
-				if len(l) > 0 && l[0] != '#' {
-					modules = append(modules, string(l))
-				}
-			}
-		}
+func deriveContent(spec *Specification, snapInfo *snap.Info) (map[string]*osutil.FileState, []string) {
+	if len(spec.modules) == 0 {
+		return nil, nil
+	}
+	content := make(map[string]*osutil.FileState)
+	var modules []string
+	for k := range spec.modules {
+		modules = append(modules, k)
 	}
-
 	sort.Strings(modules)
-	modules = uniqueLines(modules)
-	if len(modules) > 0 {
-		var buffer bytes.Buffer
-		buffer.WriteString("# This file is automatically generated.\n")
-		for _, module := range modules {
-			buffer.WriteString(module)
-			buffer.WriteByte('\n')
-		}
 
-		content[fmt.Sprintf("%s.conf", snap.SecurityTag(snapInfo.Name()))] = &osutil.FileState{
-			Content: buffer.Bytes(),
-			Mode:    0644,
-		}
+	var buffer bytes.Buffer
+	buffer.WriteString("# This file is automatically generated.\n")
+	for _, module := range modules {
+		buffer.WriteString(module)
+		buffer.WriteRune('\n')
 	}
-
-	return content, modules, nil
-}
-
-func uniqueLines(lines []string) (deduplicated []string) {
-	dedup := make(map[string]bool)
-	for _, line := range lines {
-		if !dedup[line] {
-			dedup[line] = true
-			deduplicated = append(deduplicated, line)
-		}
+	content[fmt.Sprintf("%s.conf", snap.SecurityTag(snapInfo.Name()))] = &osutil.FileState{
+		Content: buffer.Bytes(),
+		Mode:    0644,
 	}
-	return deduplicated
+	return content, modules
 }
 
 func (b *Backend) NewSpecification() interfaces.Specification {
-	panic(fmt.Errorf("%s is not using specifications yet", b.Name()))
+	return &Specification{}
 }
