@@ -35,6 +35,8 @@ import (
 	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/testutil"
 )
 
 func Test(t *testing.T) {
@@ -53,6 +55,8 @@ func (s *backendSuite) SetUpTest(c *C) {
 	s.Backend = &mount.Backend{}
 	s.BackendSuite.SetUpTest(c)
 
+	c.Assert(s.Repo.AddBackend(s.Backend), IsNil)
+
 	err := os.MkdirAll(dirs.SnapMountPolicyDir, 0700)
 	c.Assert(err, IsNil)
 
@@ -67,7 +71,7 @@ func (s *backendSuite) TearDownTest(c *C) {
 }
 
 func (s *backendSuite) TestName(c *C) {
-	c.Check(s.Backend.Name(), Equals, "mount")
+	c.Check(s.Backend.Name(), Equals, interfaces.SecurityMount)
 }
 
 func (s *backendSuite) TestRemove(c *C) {
@@ -79,18 +83,26 @@ func (s *backendSuite) TestRemove(c *C) {
 	err = ioutil.WriteFile(hookCanaryToGo, []byte("ni! ni! ni!"), 0644)
 	c.Assert(err, IsNil)
 
-	canaryToStay := filepath.Join(dirs.SnapMountPolicyDir, "snap.i-stay.really.fstab")
-	err = ioutil.WriteFile(canaryToStay, []byte("stay!"), 0644)
+	snapCanaryToGo := filepath.Join(dirs.SnapMountPolicyDir, "snap.hello-world.fstab")
+	err = ioutil.WriteFile(snapCanaryToGo, []byte("ni! ni! ni!"), 0644)
+	c.Assert(err, IsNil)
+
+	appCanaryToStay := filepath.Join(dirs.SnapMountPolicyDir, "snap.i-stay.really.fstab")
+	err = ioutil.WriteFile(appCanaryToStay, []byte("stay!"), 0644)
+	c.Assert(err, IsNil)
+
+	snapCanaryToStay := filepath.Join(dirs.SnapMountPolicyDir, "snap.i-stay.fstab")
+	err = ioutil.WriteFile(snapCanaryToStay, []byte("stay!"), 0644)
 	c.Assert(err, IsNil)
 
 	err = s.Backend.Remove("hello-world")
 	c.Assert(err, IsNil)
 
+	c.Assert(osutil.FileExists(snapCanaryToGo), Equals, false)
 	c.Assert(osutil.FileExists(appCanaryToGo), Equals, false)
 	c.Assert(osutil.FileExists(hookCanaryToGo), Equals, false)
-	content, err := ioutil.ReadFile(canaryToStay)
-	c.Assert(err, IsNil)
-	c.Assert(string(content), Equals, "stay!")
+	c.Assert(appCanaryToStay, testutil.FileEquals, "stay!")
+	c.Assert(snapCanaryToStay, testutil.FileEquals, "stay!")
 }
 
 var mockSnapYaml = `name: snap-name
@@ -100,67 +112,60 @@ apps:
     app2:
 hooks:
     configure:
-        plugs: [iface-plug, iface2-plug]
+        plugs: [iface-plug]
 plugs:
     iface-plug:
         interface: iface
-    iface2-plug:
-        interface: iface2
 slots:
     iface-slot:
-        interface: iface
-    iface2-slot:
         interface: iface2
 `
 
 func (s *backendSuite) TestSetupSetsupSimple(c *C) {
-	fsEntryIF1 := "/src-1 /dst-1 none bind,ro 0 0"
-	fsEntryIF2 := "/src-2 /dst-2 none bind,ro 0 0"
+	fsEntry1 := osutil.MountEntry{Name: "/src-1", Dir: "/dst-1", Type: "none", Options: []string{"bind", "ro"}, DumpFrequency: 0, CheckPassNumber: 0}
+	fsEntry2 := osutil.MountEntry{Name: "/src-2", Dir: "/dst-2", Type: "none", Options: []string{"bind", "ro"}, DumpFrequency: 0, CheckPassNumber: 0}
+	fsEntry3 := osutil.MountEntry{Name: "/src-3", Dir: "/dst-3", Type: "none", Options: []string{"bind", "ro"}, DumpFrequency: 0, CheckPassNumber: 0}
 
-	s.Iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(fsEntryIF1), nil
+	// Give the plug a permanent effect
+	s.Iface.MountPermanentPlugCallback = func(spec *mount.Specification, plug *snap.PlugInfo) error {
+		if err := spec.AddMountEntry(fsEntry1); err != nil {
+			return err
+		}
+		return spec.AddUserMountEntry(fsEntry3)
 	}
-	s.Iface.PermanentPlugSnippetCallback = func(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(fsEntryIF1), nil
-	}
-	s.iface2.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(fsEntryIF2), nil
-	}
-	s.iface2.PermanentPlugSnippetCallback = func(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(fsEntryIF2), nil
+	// Give the slot a permanent effect
+	s.iface2.MountPermanentSlotCallback = func(spec *mount.Specification, slot *snap.SlotInfo) error {
+		return spec.AddMountEntry(fsEntry2)
 	}
 
 	// confinement options are irrelevant to this security backend
 	s.InstallSnap(c, interfaces.ConfinementOptions{}, mockSnapYaml, 0)
 
-	// ensure both security snippets for iface/iface2 are combined
-	expected := strings.Split(fmt.Sprintf("%s\n%s\n", fsEntryIF1, fsEntryIF2), "\n")
+	// ensure both security effects from iface/iface2 are combined
+	// (because mount profiles are global in the whole snap)
+	expected := strings.Split(fmt.Sprintf("%s\n%s\n", fsEntry1, fsEntry2), "\n")
 	sort.Strings(expected)
-	// and we have them both for both apps and the hook
-	for _, binary := range []string{"app1", "app2", "hook.configure"} {
-		fn1 := filepath.Join(dirs.SnapMountPolicyDir, fmt.Sprintf("snap.snap-name.%s.fstab", binary))
-		content, err := ioutil.ReadFile(fn1)
-		c.Assert(err, IsNil, Commentf("Expected mount file for %q", binary))
-		got := strings.Split(string(content), "\n")
-		sort.Strings(got)
-		c.Check(got, DeepEquals, expected)
-	}
+	// and that we have the modern fstab file (global for snap)
+	fn := filepath.Join(dirs.SnapMountPolicyDir, "snap.snap-name.fstab")
+	content, err := ioutil.ReadFile(fn)
+	c.Assert(err, IsNil, Commentf("Expected mount profile for the whole snap"))
+	got := strings.Split(string(content), "\n")
+	sort.Strings(got)
+	c.Check(got, DeepEquals, expected)
+
+	// Check that the user-fstab file was written with the user mount
+	fn = filepath.Join(dirs.SnapMountPolicyDir, "snap.snap-name.user-fstab")
+	content, err = ioutil.ReadFile(fn)
+	c.Assert(err, IsNil, Commentf("Expected user mount profile for the whole snap"))
+	c.Check(string(content), Equals, fsEntry3.String()+"\n")
 }
 
 func (s *backendSuite) TestSetupSetsupWithoutDir(c *C) {
-	s.Iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte("xxx"), nil
-	}
-	s.Iface.PermanentPlugSnippetCallback = func(plug *interfaces.Plug, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte("xxx"), nil
+	s.Iface.MountPermanentPlugCallback = func(spec *mount.Specification, plug *snap.PlugInfo) error {
+		return spec.AddMountEntry(osutil.MountEntry{})
 	}
 
 	// Ensure that backend.Setup() creates the required dir on demand
 	os.Remove(dirs.SnapMountPolicyDir)
 	s.InstallSnap(c, interfaces.ConfinementOptions{}, mockSnapYaml, 0)
-
-	for _, binary := range []string{"app1", "app2", "hook.configure"} {
-		fn := filepath.Join(dirs.SnapMountPolicyDir, fmt.Sprintf("snap.snap-name.%s.fstab", binary))
-		c.Assert(osutil.FileExists(fn), Equals, true, Commentf("Expected mount file for %q", binary))
-	}
 }
