@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -45,11 +45,14 @@ type policySuite struct {
 
 	randomSnap *snap.Info
 	randomDecl *asserts.SnapDeclaration
+
+	restoreSanitize func()
 }
 
 var _ = Suite(&policySuite{})
 
 func (s *policySuite) SetUpSuite(c *C) {
+	s.restoreSanitize = snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {})
 	a, err := asserts.Decode([]byte(`type: base-declaration
 authority-id: canonical
 series: 16
@@ -71,6 +74,14 @@ plugs:
     allow-connection:
       slot-publisher-id:
         - $PLUG_PUBLISHER_ID
+  plug-plug-attr:
+    allow-connection:
+      slot-attributes:
+        c: $PLUG(c)
+  plug-slot-attr:
+    allow-connection:
+      plug-attributes:
+        c: $SLOT(c)
   plug-or:
     allow-connection:
       -
@@ -171,6 +182,19 @@ slots:
     allow-connection:
       plug-publisher-id:
         - $SLOT_PUBLISHER_ID
+  slot-slot-attr:
+    allow-connection:
+      plug-attributes:
+        a:
+          b: $SLOT(a.b)
+  slot-plug-attr:
+    allow-connection:
+      slot-attributes:
+        c: $PLUG(c)
+  slot-plug-missing:
+    allow-connection:
+      plug-attributes:
+        x: $MISSING
   slot-or:
     allow-connection:
       -
@@ -262,6 +286,7 @@ AXNpZw==`))
 
 	s.plugSnap = snaptest.MockInfo(c, `
 name: plug-snap
+version: 0
 plugs:
    random:
    mismatchy:
@@ -330,6 +355,39 @@ plugs:
 
    same-plug-publisher-id:
 
+   slot-slot-attr-mismatch:
+     interface: slot-slot-attr
+     a:
+       b: []
+
+   slot-slot-attr-match:
+     interface: slot-slot-attr
+     a:
+       b: ["x", "y"]
+
+   slot-plug-attr-mismatch:
+     interface: slot-plug-attr
+     c: "Z"
+
+   slot-plug-attr-match:
+     interface: slot-plug-attr
+     c: "C"
+
+   slot-plug-missing-mismatch:
+     interface: slot-plug-missing
+     x: 1
+     z: 2
+
+   slot-plug-missing-match:
+     interface: slot-plug-missing
+     z: 2
+
+   plug-plug-attr:
+     c: "C"
+
+   plug-slot-attr:
+     c: "C"
+
    plug-or-p1-s1:
      interface: plug-or
      p: P1
@@ -389,6 +447,7 @@ plugs:
 
 	s.slotSnap = snaptest.MockInfo(c, `
 name: slot-snap
+version: 0
 slots:
    random:
    mismatchy:
@@ -456,6 +515,31 @@ slots:
 
    same-slot-publisher-id:
 
+   slot-slot-attr:
+     a:
+       b: ["x", "y"]
+
+   slot-plug-attr:
+     c: "C"
+
+   slot-plug-missing:
+
+   plug-plug-attr-mismatch:
+     interface: plug-plug-attr
+     c: "Z"
+
+   plug-plug-attr-match:
+     interface: plug-plug-attr
+     c: "C"
+
+   plug-slot-attr-mismatch:
+     interface: plug-slot-attr
+     c: "Z"
+
+   plug-slot-attr-match:
+     interface: plug-slot-attr
+     c: "C"
+
    plug-or-p1-s1:
      interface: plug-or
      s: S1
@@ -597,10 +681,12 @@ AXNpZw==`))
 
 	s.randomSnap = snaptest.MockInfo(c, `
 name: random-snap
+version: 0
 plugs:
   precise-plug-snap-id:
   checked-plug-publisher-id:
   same-slot-publisher-id:
+  slot-slot-attr:
 slots:
   precise-slot-snap-id:
   checked-slot-publisher-id:
@@ -621,6 +707,10 @@ AXNpZw==`))
 	s.randomDecl = a.(*asserts.SnapDeclaration)
 }
 
+func (s *policySuite) TearDownSuite(c *C) {
+	s.restoreSanitize()
+}
+
 func (s *policySuite) TestBaselineDefaultIsAllow(c *C) {
 	cand := policy.ConnectCandidate{
 		Plug:            s.plugSnap.Plugs["random"],
@@ -795,6 +885,7 @@ func (s *policySuite) TestSnapDeclAllowDenyAutoConnection(c *C) {
 func (s *policySuite) TestSnapTypeCheckConnection(c *C) {
 	gadgetSnap := snaptest.MockInfo(c, `
 name: gadget
+version: 0
 type: gadget
 plugs:
    gadgethelp:
@@ -804,6 +895,7 @@ slots:
 
 	coreSnap := snaptest.MockInfo(c, `
 name: core
+version: 0
 type: os
 slots:
    gadgethelp:
@@ -998,6 +1090,7 @@ func (s *policySuite) TestDollarPlugPublisherIDCheckConnection(c *C) {
 	// slot publisher id == plug publisher id
 	samePubSlotSnap := snaptest.MockInfo(c, `
 name: same-pub-slot-snap
+version: 0
 slots:
   same-plug-publisher-id:
 `, nil)
@@ -1056,6 +1149,7 @@ func (s *policySuite) TestDollarSlotPublisherIDCheckConnection(c *C) {
 	// plug publisher id == slot publisher id
 	samePubPlugSnap := snaptest.MockInfo(c, `
 name: same-pub-plug-snap
+version: 0
 plugs:
   same-slot-publisher-id:
 `, nil)
@@ -1086,6 +1180,7 @@ AXNpZw==`))
 func (s *policySuite) TestBaselineDefaultIsAllowInstallation(c *C) {
 	installSnap := snaptest.MockInfo(c, `
 name: install-slot-snap
+version: 0
 slots:
   random1:
 plugs:
@@ -1107,62 +1202,75 @@ func (s *policySuite) TestBaseDeclAllowDenyInstallation(c *C) {
 		expected    string // "" => no error
 	}{
 		{`name: install-snap
+version: 0
 slots:
   innocuous:
   install-slot-coreonly:
 `, `installation not allowed by "install-slot-coreonly" slot rule of interface "install-slot-coreonly"`},
 		{`name: install-snap
+version: 0
 slots:
   install-slot-attr-ok:
     attr: ok
 `, ""},
 		{`name: install-snap
+version: 0
 slots:
   install-slot-attr-deny:
     trust: trusted
 `, `installation denied by "install-slot-attr-deny" slot rule of interface "install-slot-attr-deny"`},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-attr-ok:
     attr: ok
 `, ""},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-attr-ok:
     attr: not-ok
 `, `installation not allowed by "install-plug-attr-ok" plug rule of interface "install-plug-attr-ok"`},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-gadget-only:
 `, `installation not allowed by "install-plug-gadget-only" plug rule of interface "install-plug-gadget-only"`},
 		{`name: install-gadget
+version: 0
 type: gadget
 plugs:
   install-plug-gadget-only:
 `, ""},
 		{`name: install-gadget
+version: 0
 type: gadget
 plugs:
   install-plug-or:
      p: P2`, `installation denied by "install-plug-or" plug rule.*`},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-or:
      p: P1`, `installation denied by "install-plug-or" plug rule.*`},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-or:
      p: P3`, ""},
 		{`name: install-gadget
+version: 0
 type: gadget
 slots:
   install-slot-or:
      p: P2`, `installation denied by "install-slot-or" slot rule.*`},
 		{`name: install-snap
+version: 0
 slots:
   install-slot-or:
      p: P1`, `installation denied by "install-slot-or" slot rule.*`},
 		{`name: install-snap
+version: 0
 slots:
   install-slot-or:
      p: P3`, ""},
@@ -1193,6 +1301,7 @@ func (s *policySuite) TestSnapDeclAllowDenyInstallation(c *C) {
 		expected    string // "" => no error
 	}{
 		{`name: install-snap
+version: 0
 slots:
   install-slot-base-allow-snap-deny:
     have: yes # bool
@@ -1203,6 +1312,7 @@ slots:
         have: true
 `, `installation denied by "install-slot-base-allow-snap-deny" slot rule of interface "install-slot-base-allow-snap-deny" for "install-snap" snap`},
 		{`name: install-snap
+version: 0
 slots:
   install-slot-base-allow-snap-not-allow:
     have: yes # bool
@@ -1213,6 +1323,7 @@ slots:
         have: false
 `, `installation not allowed by "install-slot-base-allow-snap-not-allow" slot rule of interface "install-slot-base-allow-snap-not-allow" for "install-snap" snap`},
 		{`name: install-snap
+version: 0
 slots:
   install-slot-base-deny-snap-allow:
     have: yes
@@ -1221,6 +1332,7 @@ slots:
     allow-installation: true
 `, ""},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-base-allow-snap-deny:
     attr: give-me
@@ -1231,6 +1343,7 @@ plugs:
         attr: .*
 `, `installation denied by "install-plug-base-allow-snap-deny" plug rule of interface "install-plug-base-allow-snap-deny" for "install-snap" snap`},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-base-allow-snap-not-allow:
     attr: give-me
@@ -1241,6 +1354,7 @@ plugs:
         attr: minimal
 `, `installation not allowed by "install-plug-base-allow-snap-not-allow" plug rule of interface "install-plug-base-allow-snap-not-allow" for "install-snap" snap`},
 		{`name: install-snap
+version: 0
 plugs:
   install-plug-base-deny-snap-allow:
     attr: attrvalue
@@ -1384,15 +1498,19 @@ func (s *policySuite) TestOnClassicInstallation(c *C) {
 		err         string // "" => no error
 	}{
 		{"", `name: install-snap
+version: 0
 slots:
   install-slot-on-classic-distros:`, `installation not allowed by "install-slot-on-classic-distros" slot rule.*`},
 		{"debian", `name: install-snap
+version: 0
 slots:
   install-slot-on-classic-distros:`, ""},
 		{"", `name: install-snap
+version: 0
 plugs:
   install-plug-on-classic-distros:`, `installation not allowed by "install-plug-on-classic-distros" plug rule.*`},
 		{"debian", `name: install-snap
+version: 0
 plugs:
   install-plug-on-classic-distros:`, ""},
 	}
@@ -1421,3 +1539,101 @@ plugs:
 		}
 	}
 }
+
+func (s *policySuite) TestSlotDollarSlotAttrConnection(c *C) {
+	// no corresponding attr
+	cand := policy.ConnectCandidate{
+		Plug:            s.randomSnap.Plugs["slot-slot-attr"],
+		Slot:            s.slotSnap.Slots["slot-slot-attr"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), ErrorMatches, "connection not allowed.*")
+
+	// different attr values
+	cand = policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["slot-slot-attr-mismatch"],
+		Slot:            s.slotSnap.Slots["slot-slot-attr"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), ErrorMatches, "connection not allowed.*")
+
+	// plug attr == slot attr
+	cand = policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["slot-slot-attr-match"],
+		Slot:            s.slotSnap.Slots["slot-slot-attr"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), IsNil)
+}
+
+func (s *policySuite) TestSlotDollarPlugAttrConnection(c *C) {
+	// different attr values
+	cand := policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["slot-plug-attr-mismatch"],
+		Slot:            s.slotSnap.Slots["slot-plug-attr"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), ErrorMatches, "connection not allowed.*")
+
+	// plug attr == slot attr
+	cand = policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["slot-plug-attr-match"],
+		Slot:            s.slotSnap.Slots["slot-plug-attr"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), IsNil)
+}
+
+func (s *policySuite) TestPlugDollarPlugAttrConnection(c *C) {
+	// different attr values
+	cand := policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["plug-plug-attr"],
+		Slot:            s.slotSnap.Slots["plug-plug-attr-mismatch"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), ErrorMatches, "connection not allowed.*")
+
+	// plug attr == slot attr
+	cand = policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["plug-plug-attr"],
+		Slot:            s.slotSnap.Slots["plug-plug-attr-match"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), IsNil)
+}
+
+func (s *policySuite) TestPlugDollarSlotAttrConnection(c *C) {
+	// different attr values
+	cand := policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["plug-slot-attr"],
+		Slot:            s.slotSnap.Slots["plug-slot-attr-mismatch"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), ErrorMatches, "connection not allowed.*")
+
+	// plug attr == slot attr
+	cand = policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["plug-slot-attr"],
+		Slot:            s.slotSnap.Slots["plug-slot-attr-match"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), IsNil)
+}
+
+func (s *policySuite) TestDollarMissingConnection(c *C) {
+	// not missing
+	cand := policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["slot-plug-missing-mismatch"],
+		Slot:            s.slotSnap.Slots["slot-plug-missing"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), ErrorMatches, "connection not allowed.*")
+
+	// missing
+	cand = policy.ConnectCandidate{
+		Plug:            s.plugSnap.Plugs["slot-plug-missing-match"],
+		Slot:            s.slotSnap.Slots["slot-plug-missing"],
+		BaseDeclaration: s.baseDecl,
+	}
+	c.Check(cand.Check(), IsNil)
+}
