@@ -20,7 +20,6 @@
 package interfaces
 
 import (
-	"bytes"
 	"fmt"
 	"sort"
 	"strings"
@@ -35,25 +34,27 @@ type Repository struct {
 	m      sync.Mutex
 	ifaces map[string]Interface
 	// Indexed by [snapName][plugName]
-	plugs map[string]map[string]*Plug
-	slots map[string]map[string]*Slot
+	plugs map[string]map[string]*snap.PlugInfo
+	slots map[string]map[string]*snap.SlotInfo
 	// given a slot and a plug, are they connected?
-	slotPlugs map[*Slot]map[*Plug]bool
+	slotPlugs map[*snap.SlotInfo]map[*snap.PlugInfo]*Connection
 	// given a plug and a slot, are they connected?
-	plugSlots map[*Plug]map[*Slot]bool
+	plugSlots map[*snap.PlugInfo]map[*snap.SlotInfo]*Connection
 	backends  map[SecuritySystem]SecurityBackend
 }
 
 // NewRepository creates an empty plug repository.
 func NewRepository() *Repository {
-	return &Repository{
+	repo := &Repository{
 		ifaces:    make(map[string]Interface),
-		plugs:     make(map[string]map[string]*Plug),
-		slots:     make(map[string]map[string]*Slot),
-		slotPlugs: make(map[*Slot]map[*Plug]bool),
-		plugSlots: make(map[*Plug]map[*Slot]bool),
+		plugs:     make(map[string]map[string]*snap.PlugInfo),
+		slots:     make(map[string]map[string]*snap.SlotInfo),
+		slotPlugs: make(map[*snap.SlotInfo]map[*snap.PlugInfo]*Connection),
+		plugSlots: make(map[*snap.PlugInfo]map[*snap.SlotInfo]*Connection),
 		backends:  make(map[SecuritySystem]SecurityBackend),
 	}
+
+	return repo
 }
 
 // Interface returns an interface with a given name.
@@ -80,6 +81,115 @@ func (r *Repository) AddInterface(i Interface) error {
 	return nil
 }
 
+// InfoOptions describes options for Info.
+//
+// Names: return just this subset if non-empty.
+// Doc: return documentation.
+// Plugs: return information about plugs.
+// Slots: return information about slots.
+// Connected: only consider interfaces with at least one connection.
+type InfoOptions struct {
+	Names     []string
+	Doc       bool
+	Plugs     bool
+	Slots     bool
+	Connected bool
+}
+
+func (r *Repository) interfaceInfo(iface Interface, opts *InfoOptions) *Info {
+	// NOTE: InfoOptions.Connected is handled by Info
+	si := StaticInfoOf(iface)
+	ifaceName := iface.Name()
+	ii := &Info{
+		Name:    ifaceName,
+		Summary: si.Summary,
+	}
+	if opts != nil && opts.Doc {
+		// Collect documentation URL
+		ii.DocURL = si.DocURL
+	}
+	if opts != nil && opts.Plugs {
+		// Collect all plugs of this interface type.
+		for _, snapName := range sortedSnapNamesWithPlugs(r.plugs) {
+			for _, plugName := range sortedPlugNames(r.plugs[snapName]) {
+				plugInfo := r.plugs[snapName][plugName]
+				if plugInfo.Interface == ifaceName {
+					ii.Plugs = append(ii.Plugs, plugInfo)
+				}
+			}
+		}
+	}
+	if opts != nil && opts.Slots {
+		// Collect all slots of this interface type.
+		for _, snapName := range sortedSnapNamesWithSlots(r.slots) {
+			for _, slotName := range sortedSlotNames(r.slots[snapName]) {
+				slotInfo := r.slots[snapName][slotName]
+				if slotInfo.Interface == ifaceName {
+					ii.Slots = append(ii.Slots, slotInfo)
+				}
+			}
+		}
+	}
+	return ii
+}
+
+// Info returns information about interfaces in the system.
+//
+// If names is empty then all interfaces are considered. Query options decide
+// which data to return but can also skip interfaces without connections. See
+// the documentation of InfoOptions for details.
+func (r *Repository) Info(opts *InfoOptions) []*Info {
+	r.m.Lock()
+	defer r.m.Unlock()
+
+	// If necessary compute the set of interfaces with any connections.
+	var connected map[string]bool
+	if opts != nil && opts.Connected {
+		connected = make(map[string]bool)
+		for _, plugMap := range r.slotPlugs {
+			for plug, conn := range plugMap {
+				if conn != nil {
+					connected[plug.Interface] = true
+				}
+			}
+		}
+		for _, slotMap := range r.plugSlots {
+			for slot, conn := range slotMap {
+				if conn != nil {
+					connected[slot.Interface] = true
+				}
+			}
+		}
+	}
+
+	// If weren't asked about specific interfaces then query every interface.
+	var names []string
+	if opts == nil || len(opts.Names) == 0 {
+		for _, iface := range r.ifaces {
+			name := iface.Name()
+			if connected == nil || connected[name] {
+				// Optionally filter out interfaces without connections.
+				names = append(names, name)
+			}
+		}
+	} else {
+		names = make([]string, len(opts.Names))
+		copy(names, opts.Names)
+	}
+	sort.Strings(names)
+
+	// Query each interface we are interested in.
+	infos := make([]*Info, 0, len(names))
+	for _, name := range names {
+		if iface, ok := r.ifaces[name]; ok {
+			if connected == nil || connected[name] {
+				infos = append(infos, r.interfaceInfo(iface, opts))
+			}
+		}
+	}
+	return infos
+}
+
 // AddBackend adds the provided security backend to the repository.
 func (r *Repository) AddBackend(backend SecurityBackend) error {
 	r.m.Lock()
@@ -95,11 +205,11 @@ func (r *Repository) AddBackend(backend SecurityBackend) error {
 
 // AllPlugs returns all plugs of the given interface.
 // If interfaceName is the empty string, all plugs are returned.
-func (r *Repository) AllPlugs(interfaceName string) []*Plug {
+func (r *Repository) AllPlugs(interfaceName string) []*snap.PlugInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	var result []*Plug
+	var result []*snap.PlugInfo
 	for _, plugsForSnap := range r.plugs {
 		for _, plug := range plugsForSnap {
 			if interfaceName == "" || plug.Interface == interfaceName {
@@ -112,11 +222,11 @@ func (r *Repository) AllPlugs(interfaceName string) []*Plug {
 }
 
 // Plugs returns the plugs offered by the named snap.
-func (r *Repository) Plugs(snapName string) []*Plug {
+func (r *Repository) Plugs(snapName string) []*snap.PlugInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	var result []*Plug
+	var result []*snap.PlugInfo
 	for _, plug := range r.plugs[snapName] {
 		result = append(result, plug)
 	}
@@ -125,7 +235,7 @@ func (r *Repository) Plugs(snapName string) []*Plug {
 }
 
 // Plug returns the specified plug from the named snap.
-func (r *Repository) Plug(snapName, plugName string) *Plug {
+func (r *Repository) Plug(snapName, plugName string) *snap.PlugInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
@@ -135,12 +245,14 @@ func (r *Repository) Plug(snapName, plugName string) *Plug {
 // AddPlug adds a plug to the repository.
 // Plug names must be valid snap names, as defined by ValidateName.
 // Plug name must be unique within a particular snap.
-func (r *Repository) AddPlug(plug *Plug) error {
+func (r *Repository) AddPlug(plug *snap.PlugInfo) error {
 	r.m.Lock()
 	defer r.m.Unlock()
 
+	snapName := plug.Snap.Name()
+
 	// Reject snaps with invalid names
-	if err := snap.ValidateName(plug.Snap.Name()); err != nil {
+	if err := snap.ValidateName(snapName); err != nil {
 		return err
 	}
 	// Reject plug with invalid names
@@ -151,17 +263,16 @@ func (r *Repository) AddPlug(plug *Plug) error {
 	if i == nil {
 		return fmt.Errorf("cannot add plug, interface %q is not known", plug.Interface)
 	}
-	// Reject plug that don't pass interface-specific sanitization
-	if err := i.SanitizePlug(plug); err != nil {
-		return fmt.Errorf("cannot add plug: %v", err)
+	if _, ok := r.plugs[snapName][plug.Name]; ok {
+		return fmt.Errorf("snap %q has plugs conflicting on name %q", snapName, plug.Name)
 	}
-	if _, ok := r.plugs[plug.Snap.Name()][plug.Name]; ok {
-		return fmt.Errorf("cannot add plug, snap %q already has plug %q", plug.Snap.Name(), plug.Name)
+	if _, ok := r.slots[snapName][plug.Name]; ok {
+		return fmt.Errorf("snap %q has plug and slot conflicting on name %q", snapName, plug.Name)
 	}
-	if r.plugs[plug.Snap.Name()] == nil {
-		r.plugs[plug.Snap.Name()] = make(map[string]*Plug)
+	if r.plugs[snapName] == nil {
+		r.plugs[snapName] = make(map[string]*snap.PlugInfo)
 	}
-	r.plugs[plug.Snap.Name()][plug.Name] = plug
+	r.plugs[snapName][plug.Name] = plug
 	return nil
 }
 
@@ -189,11 +300,11 @@ func (r *Repository) RemovePlug(snapName, plugName string) error {
 
 // AllSlots returns all slots of the given interface.
 // If interfaceName is the empty string, all slots are returned.
-func (r *Repository) AllSlots(interfaceName string) []*Slot {
+func (r *Repository) AllSlots(interfaceName string) []*snap.SlotInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	var result []*Slot
+	var result []*snap.SlotInfo
 	for _, slotsForSnap := range r.slots {
 		for _, slot := range slotsForSnap {
 			if interfaceName == "" || slot.Interface == interfaceName {
@@ -206,11 +317,11 @@ func (r *Repository) AllSlots(interfaceName string) []*Slot {
 }
 
 // Slots returns the slots offered by the named snap.
-func (r *Repository) Slots(snapName string) []*Slot {
+func (r *Repository) Slots(snapName string) []*snap.SlotInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	var result []*Slot
+	var result []*snap.SlotInfo
 	for _, slot := range r.slots[snapName] {
 		result = append(result, slot)
 	}
@@ -219,7 +330,7 @@ func (r *Repository) Slots(snapName string) []*Slot {
 }
 
 // Slot returns the specified slot from the named snap.
-func (r *Repository) Slot(snapName, slotName string) *Slot {
+func (r *Repository) Slot(snapName, slotName string) *snap.SlotInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
@@ -229,12 +340,14 @@ func (r *Repository) Slot(snapName, slotName string) *Slot {
 // AddSlot adds a new slot to the repository.
 // Adding a slot with invalid name returns an error.
 // Adding a slot that has the same name and snap name as another slot returns an error.
-func (r *Repository) AddSlot(slot *Slot) error {
+func (r *Repository) AddSlot(slot *snap.SlotInfo) error {
 	r.m.Lock()
 	defer r.m.Unlock()
 
+	snapName := slot.Snap.Name()
+
 	// Reject snaps with invalid names
-	if err := snap.ValidateName(slot.Snap.Name()); err != nil {
+	if err := snap.ValidateName(snapName); err != nil {
 		return err
 	}
 	// Reject plug with invalid names
@@ -246,16 +359,16 @@ func (r *Repository) AddSlot(slot *Slot) error {
 	if i == nil {
 		return fmt.Errorf("cannot add slot, interface %q is not known", slot.Interface)
 	}
-	if err := i.SanitizeSlot(slot); err != nil {
-		return fmt.Errorf("cannot add slot: %v", err)
+	if _, ok := r.slots[snapName][slot.Name]; ok {
+		return fmt.Errorf("snap %q has slots conflicting on name %q", snapName, slot.Name)
 	}
-	if _, ok := r.slots[slot.Snap.Name()][slot.Name]; ok {
-		return fmt.Errorf("cannot add slot, snap %q already has slot %q", slot.Snap.Name(), slot.Name)
+	if _, ok := r.plugs[snapName][slot.Name]; ok {
+		return fmt.Errorf("snap %q has plug and slot conflicting on name %q", snapName, slot.Name)
 	}
-	if r.slots[slot.Snap.Name()] == nil {
-		r.slots[slot.Snap.Name()] = make(map[string]*Slot)
+	if r.slots[snapName] == nil {
+		r.slots[snapName] = make(map[string]*snap.SlotInfo)
 	}
-	r.slots[slot.Snap.Name()][slot.Name] = slot
+	r.slots[snapName][slot.Name] = slot
 	return nil
 }
 
@@ -345,7 +458,7 @@ func (r *Repository) ResolveConnect(plugSnapName, plugName, slotSnapName, slotNa
 		return ref, fmt.Errorf("cannot connect %s:%s (%q interface) to %s:%s (%q interface)",
 			plugSnapName, plugName, plug.Interface, slotSnapName, slotName, slot.Interface)
 	}
-	ref = ConnRef{PlugRef: plug.Ref(), SlotRef: slot.Ref()}
+	ref = *NewConnRef(plug, slot)
 	return ref, nil
 }
 
@@ -400,11 +513,11 @@ func (r *Repository) ResolveDisconnect(plugSnapName, plugName, slotSnapName, slo
 			return nil, fmt.Errorf("snap %q has no slot named %q", slotSnapName, slotName)
 		}
 		// Ensure that slot and plug are connected
-		if !r.slotPlugs[slot][plug] {
+		if r.slotPlugs[slot][plug] == nil {
 			return nil, fmt.Errorf("cannot disconnect %s:%s from %s:%s, it is not connected",
 				plugSnapName, plugName, slotSnapName, slotName)
 		}
-		return []ConnRef{{PlugRef: plug.Ref(), SlotRef: slot.Ref()}}, nil
+		return []ConnRef{*NewConnRef(plug, slot)}, nil
 	// 2: <snap>:<plug or slot> (through 1st pair)
 	// Return a list of connections involving specified plug or slot.
 	case plugName != "" && slotName == "" && slotSnapName == "":
@@ -453,21 +566,25 @@ func (r *Repository) Connect(ref ConnRef) error {
 			plugSnapName, plugName, plug.Interface, slotSnapName, slotName, slot.Interface)
 	}
 	// Ensure that slot and plug are not connected yet
-	if r.slotPlugs[slot][plug] {
+	if r.slotPlugs[slot][plug] != nil {
 		// But if they are don't treat this as an error.
 		return nil
 	}
 	// Connect the plug
 	if r.slotPlugs[slot] == nil {
-		r.slotPlugs[slot] = make(map[*Plug]bool)
+		r.slotPlugs[slot] = make(map[*snap.PlugInfo]*Connection)
 	}
 	if r.plugSlots[plug] == nil {
-		r.plugSlots[plug] = make(map[*Slot]bool)
+		r.plugSlots[plug] = make(map[*snap.SlotInfo]*Connection)
 	}
-	r.slotPlugs[slot][plug] = true
-	r.plugSlots[plug][slot] = true
-	slot.Connections = append(slot.Connections, PlugRef{plug.Snap.Name(), plug.Name})
-	plug.Connections = append(plug.Connections, SlotRef{slot.Snap.Name(), slot.Name})
+
+	// TODO: store copy of attributes from hooks
+	cplug := NewConnectedPlug(plug, nil)
+	cslot := NewConnectedSlot(slot, nil)
+
+	conn := &Connection{plug: cplug, slot: cslot}
+	r.slotPlugs[slot][plug] = conn
+	r.plugSlots[plug][slot] = conn
 	return nil
 }
 
@@ -505,7 +622,7 @@ func (r *Repository) Disconnect(plugSnapName, plugName, slotSnapName, slotName s
 		return fmt.Errorf("snap %q has no slot named %q", slotSnapName, slotName)
 	}
 	// Ensure that slot and plug are connected
-	if !r.slotPlugs[slot][plug] {
+	if r.slotPlugs[slot][plug] == nil {
 		return fmt.Errorf("cannot disconnect %s:%s from %s:%s, it is not connected",
 			plugSnapName, plugName, slotSnapName, slotName)
 	}
@@ -538,18 +655,21 @@ func (r *Repository) connected(snapName, plugOrSlotName string) ([]ConnRef, erro
 		return nil, fmt.Errorf("snap %q has no plug or slot named %q", snapName, plugOrSlotName)
 	}
 	// Collect all the relevant connections
+
 	if plug, ok := r.plugs[snapName][plugOrSlotName]; ok {
-		for _, slotRef := range plug.Connections {
-			connRef := ConnRef{PlugRef: plug.Ref(), SlotRef: slotRef}
+		for slotInfo := range r.plugSlots[plug] {
+			connRef := *NewConnRef(plug, slotInfo)
 			conns = append(conns, connRef)
 		}
 	}
+
 	if slot, ok := r.slots[snapName][plugOrSlotName]; ok {
-		for _, plugRef := range slot.Connections {
-			connRef := ConnRef{PlugRef: plugRef, SlotRef: slot.Ref()}
+		for plugInfo := range r.slotPlugs[slot] {
+			connRef := *NewConnRef(plugInfo, slot)
 			conns = append(conns, connRef)
 		}
 	}
+
 	return conns, nil
 }
 
@@ -580,7 +700,7 @@ func (r *Repository) DisconnectAll(conns []ConnRef) {
 }
 
 // disconnect disconnects a plug from a slot.
-func (r *Repository) disconnect(plug *Plug, slot *Slot) {
+func (r *Repository) disconnect(plug *snap.PlugInfo, slot *snap.SlotInfo) {
 	delete(r.slotPlugs[slot], plug)
 	if len(r.slotPlugs[slot]) == 0 {
 		delete(r.slotPlugs, slot)
@@ -589,26 +709,6 @@ func (r *Repository) disconnect(plug *Plug, slot *Slot) {
 	if len(r.plugSlots[plug]) == 0 {
 		delete(r.plugSlots, plug)
 	}
-	for i, plugRef := range slot.Connections {
-		if plugRef.Snap == plug.Snap.Name() && plugRef.Name == plug.Name {
-			slot.Connections[i] = slot.Connections[len(slot.Connections)-1]
-			slot.Connections = slot.Connections[:len(slot.Connections)-1]
-			if len(slot.Connections) == 0 {
-				slot.Connections = nil
-			}
-			break
-		}
-	}
-	for i, slotRef := range plug.Connections {
-		if slotRef.Snap == slot.Snap.Name() && slotRef.Name == slot.Name {
-			plug.Connections[i] = plug.Connections[len(plug.Connections)-1]
-			plug.Connections = plug.Connections[:len(plug.Connections)-1]
-			if len(plug.Connections) == 0 {
-				plug.Connections = nil
-			}
-			break
-		}
-	}
 }
 
 // Backends returns all the security backends.
@@ -630,29 +730,28 @@ func (r *Repository) Interfaces() *Interfaces {
 	defer r.m.Unlock()
 
 	ifaces := &Interfaces{}
+
 	// Copy and flatten plugs and slots
 	for _, plugs := range r.plugs {
-		for _, plug := range plugs {
-			p := &Plug{
-				PlugInfo:    plug.PlugInfo,
-				Connections: append([]SlotRef(nil), plug.Connections...),
-			}
-			sort.Sort(bySlotRef(p.Connections))
-			ifaces.Plugs = append(ifaces.Plugs, p)
+		for _, plugInfo := range plugs {
+			ifaces.Plugs = append(ifaces.Plugs, plugInfo)
 		}
 	}
 	for _, slots := range r.slots {
-		for _, slot := range slots {
-			s := &Slot{
-				SlotInfo:    slot.SlotInfo,
-				Connections: append([]PlugRef(nil), slot.Connections...),
-			}
-			sort.Sort(byPlugRef(s.Connections))
-			ifaces.Slots = append(ifaces.Slots, s)
+		for _, slotInfo := range slots {
+			ifaces.Slots = append(ifaces.Slots, slotInfo)
 		}
 	}
+
+	for plug, slots := range r.plugSlots {
+		for slot := range slots {
+			ifaces.Connections = append(ifaces.Connections, NewConnRef(plug, slot))
+		}
+	}
+
 	sort.Sort(byPlugSnapAndName(ifaces.Plugs))
 	sort.Sort(bySlotSnapAndName(ifaces.Slots))
+	sort.Sort(byConnRef(ifaces.Connections))
 	return ifaces
 }
 
@@ -669,25 +768,25 @@ func (r *Repository) SnapSpecification(securitySystem SecuritySystem, snapName s
 	spec := backend.NewSpecification()
 
 	// slot side
-	for _, slot := range r.slots[snapName] {
-		iface := r.ifaces[slot.Interface]
-		if err := spec.AddPermanentSlot(iface, slot); err != nil {
+	for _, slotInfo := range r.slots[snapName] {
+		iface := r.ifaces[slotInfo.Interface]
+		if err := spec.AddPermanentSlot(iface, slotInfo); err != nil {
 			return nil, err
 		}
-		for plug := range r.slotPlugs[slot] {
-			if err := spec.AddConnectedSlot(iface, plug, slot); err != nil {
+		for _, conn := range r.slotPlugs[slotInfo] {
+			if err := spec.AddConnectedSlot(iface, conn.plug, conn.slot); err != nil {
 				return nil, err
 			}
 		}
 	}
 	// plug side
-	for _, plug := range r.plugs[snapName] {
-		iface := r.ifaces[plug.Interface]
-		if err := spec.AddPermanentPlug(iface, plug); err != nil {
+	for _, plugInfo := range r.plugs[snapName] {
+		iface := r.ifaces[plugInfo.Interface]
+		if err := spec.AddPermanentPlug(iface, plugInfo); err != nil {
 			return nil, err
 		}
-		for slot := range r.plugSlots[plug] {
-			if err := spec.AddConnectedPlug(iface, plug, slot); err != nil {
+		for _, conn := range r.plugSlots[plugInfo] {
+			if err := spec.AddConnectedPlug(iface, conn.plug, conn.slot); err != nil {
 				return nil, err
 			}
 		}
@@ -695,39 +794,6 @@ func (r *Repository) SnapSpecification(securitySystem SecuritySystem, snapName s
 	return spec, nil
 }
 
-// BadInterfacesError is returned when some snap interfaces could not be registered.
-// Those interfaces not mentioned in the error were successfully registered.
-type BadInterfacesError struct {
-	snap   string
-	issues map[string]string // slot or plug name => message
-}
-
-func (e *BadInterfacesError) Error() string {
-	inverted := make(map[string][]string)
-	for name, reason := range e.issues {
-		inverted[reason] = append(inverted[reason], name)
-	}
-	var buf bytes.Buffer
-	fmt.Fprintf(&buf, "snap %q has bad plugs or slots: ", e.snap)
-	reasons := make([]string, 0, len(inverted))
-	for reason := range inverted {
-		reasons = append(reasons, reason)
-	}
-	sort.Strings(reasons)
-	for _, reason := range reasons {
-		names := inverted[reason]
-		sort.Strings(names)
-		for i, name := range names {
-			if i > 0 {
-				buf.WriteString(", ")
-			}
-			buf.WriteString(name)
-		}
-		fmt.Fprintf(&buf, " (%s); ", reason)
-	}
-	return strings.TrimSuffix(buf.String(), "; ")
-}
-
 // AddSnap adds plugs and slots declared by the given snap to the repository.
 //
 // This function can be used to implement snap install or, when used along with
@@ -742,6 +808,14 @@ func (e *BadInterfacesError) Error() string {
 // Unknown interfaces and plugs/slots that don't validate are not added.
 // Information about those failures are returned to the caller.
 func (r *Repository) AddSnap(snapInfo *snap.Info) error {
+	if snapInfo.Broken != "" {
+		return fmt.Errorf("snap is broken: %s", snapInfo.Broken)
+	}
+	err := snap.Validate(snapInfo)
+	if err != nil {
+		return err
+	}
+
 	r.m.Lock()
 	defer r.m.Unlock()
 
@@ -751,57 +825,24 @@ func (r *Repository) AddSnap(snapInfo *snap.Info) error {
 		return fmt.Errorf("cannot register interfaces for snap %q more than once", snapName)
 	}
 
-	bad := BadInterfacesError{
-		snap:   snapName,
-		issues: make(map[string]string),
-	}
-
 	for plugName, plugInfo := range snapInfo.Plugs {
-		iface, ok := r.ifaces[plugInfo.Interface]
-		if !ok {
-			bad.issues[plugName] = "unknown interface"
-			continue
-		}
-		// Reject plug with invalid name
-		if err := ValidateName(plugName); err != nil {
-			bad.issues[plugName] = err.Error()
-			continue
-		}
-		plug := &Plug{PlugInfo: plugInfo}
-		if err := iface.SanitizePlug(plug); err != nil {
-			bad.issues[plugName] = err.Error()
+		if _, ok := r.ifaces[plugInfo.Interface]; !ok {
 			continue
 		}
 		if r.plugs[snapName] == nil {
-			r.plugs[snapName] = make(map[string]*Plug)
+			r.plugs[snapName] = make(map[string]*snap.PlugInfo)
 		}
-		r.plugs[snapName][plugName] = plug
+		r.plugs[snapName][plugName] = plugInfo
 	}
 
 	for slotName, slotInfo := range snapInfo.Slots {
-		iface, ok := r.ifaces[slotInfo.Interface]
-		if !ok {
-			bad.issues[slotName] = "unknown interface"
-			continue
-		}
-		// Reject slot with invalid name
-		if err := ValidateName(slotName); err != nil {
-			bad.issues[slotName] = err.Error()
-			continue
-		}
-		slot := &Slot{SlotInfo: slotInfo}
-		if err := iface.SanitizeSlot(slot); err != nil {
-			bad.issues[slotName] = err.Error()
+		if _, ok := r.ifaces[slotInfo.Interface]; !ok {
 			continue
 		}
 		if r.slots[snapName] == nil {
-			r.slots[snapName] = make(map[string]*Slot)
+			r.slots[snapName] = make(map[string]*snap.SlotInfo)
 		}
-		r.slots[snapName][slotName] = slot
-	}
-
-	if len(bad.issues) > 0 {
-		return &bad
+		r.slots[snapName][slotName] = slotInfo
 	}
 	return nil
 }
@@ -819,12 +860,12 @@ func (r *Repository) RemoveSnap(snapName string) error {
 	defer r.m.Unlock()
 
 	for plugName, plug := range r.plugs[snapName] {
-		if len(plug.Connections) > 0 {
+		if len(r.plugSlots[plug]) > 0 {
 			return fmt.Errorf("cannot remove connected plug %s.%s", snapName, plugName)
 		}
 	}
 	for slotName, slot := range r.slots[snapName] {
-		if len(slot.Connections) > 0 {
+		if len(r.slotPlugs[slot]) > 0 {
 			return fmt.Errorf("cannot remove connected slot %s.%s", snapName, slotName)
 		}
 	}
@@ -876,30 +917,33 @@ func (r *Repository) DisconnectSnap(snapName string) ([]string, error) {
 
 // AutoConnectCandidateSlots finds and returns viable auto-connection candidates
 // for a given plug.
-func (r *Repository) AutoConnectCandidateSlots(plugSnapName, plugName string, policyCheck func(*Plug, *Slot) bool) []*Slot {
+func (r *Repository) AutoConnectCandidateSlots(plugSnapName, plugName string, policyCheck func(*Plug, *Slot) bool) []*snap.SlotInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	plug := r.plugs[plugSnapName][plugName]
-	if plug == nil {
+	plugInfo := r.plugs[plugSnapName][plugName]
+	if plugInfo == nil {
 		return nil
 	}
 
-	var candidates []*Slot
+	var candidates []*snap.SlotInfo
 	for _, slotsForSnap := range r.slots {
-		for _, slot := range slotsForSnap {
-			if slot.Interface != plug.Interface {
+		for _, slotInfo := range slotsForSnap {
+			if slotInfo.Interface != plugInfo.Interface {
 				continue
 			}
-			iface := slot.Interface
+			iface := slotInfo.Interface
 
+			// FIXME: use ConnectedPlug/Slot for AutoConnect (once it's refactored to use tasks).
+			plug := &Plug{PlugInfo: plugInfo}
+			slot := &Slot{SlotInfo: slotInfo}
 			// declaration based checks disallow
 			if !policyCheck(plug, slot) {
 				continue
 			}
 
 			if r.ifaces[iface].AutoConnect(plug, slot) {
-				candidates = append(candidates, slot)
+				candidates = append(candidates, slotInfo)
 			}
 		}
 	}
@@ -908,30 +952,33 @@ func (r *Repository) AutoConnectCandidateSlots(plugSnapName, plugName string, po
 
 // AutoConnectCandidatePlugs finds and returns viable auto-connection candidates
 // for a given slot.
-func (r *Repository) AutoConnectCandidatePlugs(slotSnapName, slotName string, policyCheck func(*Plug, *Slot) bool) []*Plug {
+func (r *Repository) AutoConnectCandidatePlugs(slotSnapName, slotName string, policyCheck func(*Plug, *Slot) bool) []*snap.PlugInfo {
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	slot := r.slots[slotSnapName][slotName]
-	if slot == nil {
+	slotInfo := r.slots[slotSnapName][slotName]
+	if slotInfo == nil {
 		return nil
 	}
 
-	var candidates []*Plug
+	var candidates []*snap.PlugInfo
 	for _, plugsForSnap := range r.plugs {
-		for _, plug := range plugsForSnap {
-			if slot.Interface != plug.Interface {
+		for _, plugInfo := range plugsForSnap {
+			if slotInfo.Interface != plugInfo.Interface {
 				continue
 			}
-			iface := slot.Interface
+			iface := slotInfo.Interface
 
+			// FIXME: use ConnectedPlug/Slot for AutoConnect (once it's refactored to use tasks).
+			plug := &Plug{PlugInfo: plugInfo}
+			slot := &Slot{SlotInfo: slotInfo}
 			// declaration based checks disallow
 			if !policyCheck(plug, slot) {
 				continue
 			}
 
 			if r.ifaces[iface].AutoConnect(plug, slot) {
-				candidates = append(candidates, plug)
+				candidates = append(candidates, plugInfo)
 			}
 		}
 	}
