@@ -32,9 +32,10 @@ import (
 )
 
 type RepositorySuite struct {
+	testutil.BaseTest
 	iface     Interface
-	plug      *Plug
-	slot      *Slot
+	plug      *snap.PlugInfo
+	slot      *snap.SlotInfo
 	coreSnap  *snap.Info
 	emptyRepo *Repository
 	// Repository pre-populated with s.iface
@@ -48,8 +49,12 @@ var _ = Suite(&RepositorySuite{
 })
 
 func (s *RepositorySuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+
 	consumer := snaptest.MockInfo(c, `
 name: consumer
+version: 0
 apps:
     app:
 hooks:
@@ -60,9 +65,10 @@ plugs:
         label: label
         attr: value
 `, nil)
-	s.plug = &Plug{PlugInfo: consumer.Plugs["plug"]}
+	s.plug = consumer.Plugs["plug"]
 	producer := snaptest.MockInfo(c, `
 name: producer
+version: 0
 apps:
     app:
 hooks:
@@ -73,12 +79,13 @@ slots:
         label: label
         attr: value
 `, nil)
-	s.slot = &Slot{SlotInfo: producer.Slots["slot"]}
+	s.slot = producer.Slots["slot"]
 	// NOTE: The core snap has a slot so that it shows up in the
 	// repository. The repository doesn't record snaps unless they
 	// have at least one interface.
 	s.coreSnap = snaptest.MockInfo(c, `
 name: core
+version: 0
 type: os
 slots:
     network:
@@ -90,17 +97,21 @@ slots:
 	c.Assert(err, IsNil)
 }
 
+func (s *RepositorySuite) TearDownTest(c *C) {
+	s.BaseTest.TearDownTest(c)
+}
+
 func addPlugsSlots(c *C, repo *Repository, yamls ...string) []*snap.Info {
 	result := make([]*snap.Info, len(yamls))
 	for i, yaml := range yamls {
 		info := snaptest.MockInfo(c, yaml, nil)
 		result[i] = info
 		for _, plugInfo := range info.Plugs {
-			err := repo.AddPlug(&Plug{PlugInfo: plugInfo})
+			err := repo.AddPlug(plugInfo)
 			c.Assert(err, IsNil)
 		}
 		for _, slotInfo := range info.Slots {
-			err := repo.AddSlot(&Slot{SlotInfo: slotInfo})
+			err := repo.AddSlot(slotInfo)
 			c.Assert(err, IsNil)
 		}
 	}
@@ -190,22 +201,40 @@ func (s *RepositorySuite) TestAddPlug(c *C) {
 	c.Assert(s.testRepo.Plug(s.plug.Snap.Name(), s.plug.Name), DeepEquals, s.plug)
 }
 
-func (s *RepositorySuite) TestAddPlugClash(c *C) {
+func (s *RepositorySuite) TestAddPlugClashingPlug(c *C) {
 	err := s.testRepo.AddPlug(s.plug)
 	c.Assert(err, IsNil)
 	err = s.testRepo.AddPlug(s.plug)
-	c.Assert(err, ErrorMatches, `cannot add plug, snap "consumer" already has plug "plug"`)
+	c.Assert(err, ErrorMatches, `snap "consumer" has plugs conflicting on name "plug"`)
 	c.Assert(s.testRepo.AllPlugs(""), HasLen, 1)
 	c.Assert(s.testRepo.Plug(s.plug.Snap.Name(), s.plug.Name), DeepEquals, s.plug)
 }
 
+func (s *RepositorySuite) TestAddPlugClashingSlot(c *C) {
+	snapInfo := &snap.Info{SuggestedName: "snap"}
+	plug := &snap.PlugInfo{
+		Snap:      snapInfo,
+		Name:      "clashing",
+		Interface: "interface",
+	}
+	slot := &snap.SlotInfo{
+		Snap:      snapInfo,
+		Name:      "clashing",
+		Interface: "interface",
+	}
+	err := s.testRepo.AddSlot(slot)
+	c.Assert(err, IsNil)
+	err = s.testRepo.AddPlug(plug)
+	c.Assert(err, ErrorMatches, `snap "snap" has plug and slot conflicting on name "clashing"`)
+	c.Assert(s.testRepo.AllSlots(""), HasLen, 1)
+	c.Assert(s.testRepo.Slot(slot.Snap.Name(), slot.Name), DeepEquals, slot)
+}
+
 func (s *RepositorySuite) TestAddPlugFailsWithInvalidSnapName(c *C) {
-	plug := &Plug{
-		PlugInfo: &snap.PlugInfo{
-			Snap:      &snap.Info{SuggestedName: "bad-snap-"},
-			Name:      "interface",
-			Interface: "interface",
-		},
+	plug := &snap.PlugInfo{
+		Snap:      &snap.Info{SuggestedName: "bad-snap-"},
+		Name:      "interface",
+		Interface: "interface",
 	}
 	err := s.testRepo.AddPlug(plug)
 	c.Assert(err, ErrorMatches, `invalid snap name: "bad-snap-"`)
@@ -213,12 +242,10 @@ func (s *RepositorySuite) TestAddPlugFailsWithInvalidSnapName(c *C) {
 }
 
 func (s *RepositorySuite) TestAddPlugFailsWithInvalidPlugName(c *C) {
-	plug := &Plug{
-		PlugInfo: &snap.PlugInfo{
-			Snap:      &snap.Info{SuggestedName: "snap"},
-			Name:      "bad-name-",
-			Interface: "interface",
-		},
+	plug := &snap.PlugInfo{
+		Snap:      &snap.Info{SuggestedName: "snap"},
+		Name:      "bad-name-",
+		Interface: "interface",
 	}
 	err := s.testRepo.AddPlug(plug)
 	c.Assert(err, ErrorMatches, `invalid interface name: "bad-name-"`)
@@ -231,20 +258,6 @@ func (s *RepositorySuite) TestAddPlugFailsWithUnknownInterface(c *C) {
 	c.Assert(s.emptyRepo.AllPlugs(""), HasLen, 0)
 }
 
-func (s *RepositorySuite) TestAddPlugFailsWithUnsanitizedPlug(c *C) {
-	iface := &ifacetest.TestInterface{
-		InterfaceName: "interface",
-		SanitizePlugCallback: func(plug *Plug) error {
-			return fmt.Errorf("plug is dirty")
-		},
-	}
-	err := s.emptyRepo.AddInterface(iface)
-	c.Assert(err, IsNil)
-	err = s.emptyRepo.AddPlug(s.plug)
-	c.Assert(err, ErrorMatches, "cannot add plug: plug is dirty")
-	c.Assert(s.emptyRepo.AllPlugs(""), HasLen, 0)
-}
-
 // Tests for Repository.Plug()
 
 func (s *RepositorySuite) TestPlug(c *C) {
@@ -257,12 +270,14 @@ func (s *RepositorySuite) TestPlug(c *C) {
 func (s *RepositorySuite) TestPlugSearch(c *C) {
 	addPlugsSlots(c, s.testRepo, `
 name: x
+version: 0
 plugs:
     a: interface
     b: interface
     c: interface
 `, `
 name: y
+version: 0
 plugs:
     a: interface
     b: interface
@@ -297,8 +312,8 @@ func (s *RepositorySuite) TestRemovePlugFailsWhenPlugIsConnected(c *C) {
 	c.Assert(err, IsNil)
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, IsNil)
 	// Removing a plug used by a slot returns an appropriate error
 	err = s.testRepo.RemovePlug(s.plug.Snap.Name(), s.plug.Name)
@@ -313,21 +328,23 @@ func (s *RepositorySuite) TestRemovePlugFailsWhenPlugIsConnected(c *C) {
 func (s *RepositorySuite) TestAllPlugsWithoutInterfaceName(c *C) {
 	snaps := addPlugsSlots(c, s.testRepo, `
 name: snap-a
+version: 0
 plugs:
     name-a: interface
 `, `
 name: snap-b
+version: 0
 plugs:
     name-a: interface
     name-b: interface
     name-c: interface
 `)
 	// The result is sorted by snap and name
-	c.Assert(s.testRepo.AllPlugs(""), DeepEquals, []*Plug{
-		{PlugInfo: snaps[0].Plugs["name-a"]},
-		{PlugInfo: snaps[1].Plugs["name-a"]},
-		{PlugInfo: snaps[1].Plugs["name-b"]},
-		{PlugInfo: snaps[1].Plugs["name-c"]},
+	c.Assert(s.testRepo.AllPlugs(""), DeepEquals, []*snap.PlugInfo{
+		snaps[0].Plugs["name-a"],
+		snaps[1].Plugs["name-a"],
+		snaps[1].Plugs["name-b"],
+		snaps[1].Plugs["name-c"],
 	})
 }
 
@@ -337,18 +354,18 @@ func (s *RepositorySuite) TestAllPlugsWithInterfaceName(c *C) {
 	c.Assert(err, IsNil)
 	snaps := addPlugsSlots(c, s.testRepo, `
 name: snap-a
+version: 0
 plugs:
     name-a: interface
 `, `
 name: snap-b
+version: 0
 plugs:
     name-a: interface
     name-b: other-interface
     name-c: interface
 `)
-	c.Assert(s.testRepo.AllPlugs("other-interface"), DeepEquals, []*Plug{
-		{PlugInfo: snaps[1].Plugs["name-b"]},
-	})
+	c.Assert(s.testRepo.AllPlugs("other-interface"), DeepEquals, []*snap.PlugInfo{snaps[1].Plugs["name-b"]})
 }
 
 // Tests for Repository.Plugs()
@@ -356,20 +373,22 @@ plugs:
 func (s *RepositorySuite) TestPlugs(c *C) {
 	snaps := addPlugsSlots(c, s.testRepo, `
 name: snap-a
+version: 0
 plugs:
     name-a: interface
 `, `
 name: snap-b
+version: 0
 plugs:
     name-a: interface
     name-b: interface
     name-c: interface
 `)
 	// The result is sorted by snap and name
-	c.Assert(s.testRepo.Plugs("snap-b"), DeepEquals, []*Plug{
-		{PlugInfo: snaps[1].Plugs["name-a"]},
-		{PlugInfo: snaps[1].Plugs["name-b"]},
-		{PlugInfo: snaps[1].Plugs["name-c"]},
+	c.Assert(s.testRepo.Plugs("snap-b"), DeepEquals, []*snap.PlugInfo{
+		snaps[1].Plugs["name-a"],
+		snaps[1].Plugs["name-b"],
+		snaps[1].Plugs["name-c"],
 	})
 	// The result is empty if the snap is not known
 	c.Assert(s.testRepo.Plugs("snap-x"), HasLen, 0)
@@ -382,23 +401,25 @@ func (s *RepositorySuite) TestAllSlots(c *C) {
 	c.Assert(err, IsNil)
 	snaps := addPlugsSlots(c, s.testRepo, `
 name: snap-a
+version: 0
 slots:
     name-a: interface
     name-b: interface
 `, `
 name: snap-b
+version: 0
 slots:
     name-a: other-interface
 `)
 	// AllSlots("") returns all slots, sorted by snap and slot name
-	c.Assert(s.testRepo.AllSlots(""), DeepEquals, []*Slot{
-		{SlotInfo: snaps[0].Slots["name-a"]},
-		{SlotInfo: snaps[0].Slots["name-b"]},
-		{SlotInfo: snaps[1].Slots["name-a"]},
+	c.Assert(s.testRepo.AllSlots(""), DeepEquals, []*snap.SlotInfo{
+		snaps[0].Slots["name-a"],
+		snaps[0].Slots["name-b"],
+		snaps[1].Slots["name-a"],
 	})
 	// AllSlots("") returns all slots, sorted by snap and slot name
-	c.Assert(s.testRepo.AllSlots("other-interface"), DeepEquals, []*Slot{
-		{SlotInfo: snaps[1].Slots["name-a"]},
+	c.Assert(s.testRepo.AllSlots("other-interface"), DeepEquals, []*snap.SlotInfo{
+		snaps[1].Slots["name-a"],
 	})
 }
 
@@ -407,22 +428,24 @@ slots:
 func (s *RepositorySuite) TestSlots(c *C) {
 	snaps := addPlugsSlots(c, s.testRepo, `
 name: snap-a
+version: 0
 slots:
     name-a: interface
     name-b: interface
 `, `
 name: snap-b
+version: 0
 slots:
     name-a: interface
 `)
 	// Slots("snap-a") returns slots present in that snap
-	c.Assert(s.testRepo.Slots("snap-a"), DeepEquals, []*Slot{
-		{SlotInfo: snaps[0].Slots["name-a"]},
-		{SlotInfo: snaps[0].Slots["name-b"]},
+	c.Assert(s.testRepo.Slots("snap-a"), DeepEquals, []*snap.SlotInfo{
+		snaps[0].Slots["name-a"],
+		snaps[0].Slots["name-b"],
 	})
 	// Slots("snap-b") returns slots present in that snap
-	c.Assert(s.testRepo.Slots("snap-b"), DeepEquals, []*Slot{
-		{SlotInfo: snaps[1].Slots["name-a"]},
+	c.Assert(s.testRepo.Slots("snap-b"), DeepEquals, []*snap.SlotInfo{
+		snaps[1].Slots["name-a"],
 	})
 	// Slots("snap-c") returns no slots (because that snap doesn't exist)
 	c.Assert(s.testRepo.Slots("snap-c"), HasLen, 0)
@@ -452,12 +475,10 @@ func (s *RepositorySuite) TestAddSlotFailsWhenInterfaceIsUnknown(c *C) {
 }
 
 func (s *RepositorySuite) TestAddSlotFailsWhenSlotNameIsInvalid(c *C) {
-	slot := &Slot{
-		SlotInfo: &snap.SlotInfo{
-			Snap:      &snap.Info{SuggestedName: "snap"},
-			Name:      "bad-name-",
-			Interface: "interface",
-		},
+	slot := &snap.SlotInfo{
+		Snap:      &snap.Info{SuggestedName: "snap"},
+		Name:      "bad-name-",
+		Interface: "interface",
 	}
 	err := s.emptyRepo.AddSlot(slot)
 	c.Assert(err, ErrorMatches, `invalid interface name: "bad-name-"`)
@@ -465,39 +486,43 @@ func (s *RepositorySuite) TestAddSlotFailsWhenSlotNameIsInvalid(c *C) {
 }
 
 func (s *RepositorySuite) TestAddSlotFailsWithInvalidSnapName(c *C) {
-	slot := &Slot{
-		SlotInfo: &snap.SlotInfo{
-			Snap:      &snap.Info{SuggestedName: "bad-snap-"},
-			Name:      "slot",
-			Interface: "interface",
-		},
+	slot := &snap.SlotInfo{
+		Snap:      &snap.Info{SuggestedName: "bad-snap-"},
+		Name:      "slot",
+		Interface: "interface",
 	}
 	err := s.emptyRepo.AddSlot(slot)
 	c.Assert(err, ErrorMatches, `invalid snap name: "bad-snap-"`)
 	c.Assert(s.emptyRepo.AllSlots(""), HasLen, 0)
 }
 
-func (s *RepositorySuite) TestAddSlotFailsForDuplicates(c *C) {
+func (s *RepositorySuite) TestAddSlotClashingSlot(c *C) {
 	// Adding the first slot succeeds
 	err := s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	// Adding the slot again fails with appropriate error
 	err = s.testRepo.AddSlot(s.slot)
-	c.Assert(err, ErrorMatches, `cannot add slot, snap "producer" already has slot "slot"`)
+	c.Assert(err, ErrorMatches, `snap "producer" has slots conflicting on name "slot"`)
 }
 
-func (s *RepositorySuite) TestAddSlotFailsWithUnsanitizedSlot(c *C) {
-	iface := &ifacetest.TestInterface{
-		InterfaceName: "interface",
-		SanitizeSlotCallback: func(slot *Slot) error {
-			return fmt.Errorf("slot is dirty")
-		},
+func (s *RepositorySuite) TestAddSlotClashingPlug(c *C) {
+	snapInfo := &snap.Info{SuggestedName: "snap"}
+	plug := &snap.PlugInfo{
+		Snap:      snapInfo,
+		Name:      "clashing",
+		Interface: "interface",
 	}
-	err := s.emptyRepo.AddInterface(iface)
+	slot := &snap.SlotInfo{
+		Snap:      snapInfo,
+		Name:      "clashing",
+		Interface: "interface",
+	}
+	err := s.testRepo.AddPlug(plug)
 	c.Assert(err, IsNil)
-	err = s.emptyRepo.AddSlot(s.slot)
-	c.Assert(err, ErrorMatches, "cannot add slot: slot is dirty")
-	c.Assert(s.emptyRepo.AllSlots(""), HasLen, 0)
+	err = s.testRepo.AddSlot(slot)
+	c.Assert(err, ErrorMatches, `snap "snap" has plug and slot conflicting on name "clashing"`)
+	c.Assert(s.testRepo.AllPlugs(""), HasLen, 1)
+	c.Assert(s.testRepo.Plug(plug.Snap.Name(), plug.Name), DeepEquals, plug)
 }
 
 func (s *RepositorySuite) TestAddSlotStoresCorrectData(c *C) {
@@ -533,8 +558,8 @@ func (s *RepositorySuite) TestRemoveSlotFailsWhenSlotIsConnected(c *C) {
 	c.Assert(err, IsNil)
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, IsNil)
 	// Removing a slot occupied by a plug returns an appropriate error
 	err = s.testRepo.RemoveSlot(s.slot.Snap.Name(), s.slot.Name)
@@ -561,6 +586,7 @@ func (s *RepositorySuite) TestResolveConnectExplicit(c *C) {
 func (s *RepositorySuite) TestResolveConnectImplicitCoreSlot(c *C) {
 	coreSnap := snaptest.MockInfo(c, `
 name: core
+version: 0
 type: os
 slots:
     slot:
@@ -580,6 +606,7 @@ slots:
 func (s *RepositorySuite) TestResolveConnectImplicitUbuntuCoreSlot(c *C) {
 	ubuntuCoreSnap := snaptest.MockInfo(c, `
 name: ubuntu-core
+version: 0
 type: os
 slots:
     slot:
@@ -599,6 +626,7 @@ slots:
 func (s *RepositorySuite) TestResolveConnectImplicitSlotPrefersCore(c *C) {
 	coreSnap := snaptest.MockInfo(c, `
 name: core
+version: 0
 type: os
 slots:
     slot:
@@ -606,6 +634,7 @@ slots:
 `, nil)
 	ubuntuCoreSnap := snaptest.MockInfo(c, `
 name: ubuntu-core
+version: 0
 type: os
 slots:
     slot:
@@ -625,6 +654,7 @@ func (s *RepositorySuite) TestResolveConnectNoImplicitCandidates(c *C) {
 	c.Assert(err, IsNil)
 	coreSnap := snaptest.MockInfo(c, `
 name: core
+version: 0
 type: os
 slots:
     slot:
@@ -641,6 +671,7 @@ slots:
 func (s *RepositorySuite) TestResolveConnectAmbiguity(c *C) {
 	coreSnap := snaptest.MockInfo(c, `
 name: core
+version: 0
 type: os
 slots:
     slot-a:
@@ -704,12 +735,10 @@ func (s *RepositorySuite) TestResolveNoSuchSlot(c *C) {
 // Plug and slot must have matching types
 func (s *RepositorySuite) TestResolveIncompatibleTypes(c *C) {
 	c.Assert(s.testRepo.AddInterface(&ifacetest.TestInterface{InterfaceName: "other-interface"}), IsNil)
-	plug := &Plug{
-		PlugInfo: &snap.PlugInfo{
-			Snap:      &snap.Info{SuggestedName: "consumer"},
-			Name:      "plug",
-			Interface: "other-interface",
-		},
+	plug := &snap.PlugInfo{
+		Snap:      &snap.Info{SuggestedName: "consumer"},
+		Name:      "plug",
+		Interface: "other-interface",
 	}
 	c.Assert(s.testRepo.AddPlug(plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
@@ -947,8 +976,8 @@ func (s *RepositorySuite) TestResolveDisconnectMatrixTypical(c *C) {
 	c.Assert(s.testRepo.AddSnap(s.coreSnap), IsNil)
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	connRef := ConnRef{s.plug.Ref(), s.slot.Ref()}
-	c.Assert(s.testRepo.Connect(connRef), IsNil)
+	connRef := NewConnRef(s.plug, s.slot)
+	c.Assert(s.testRepo.Connect(*connRef), IsNil)
 
 	scenarios := []struct {
 		plugSnapName, plugName, slotSnapName, slotName string
@@ -1017,7 +1046,7 @@ func (s *RepositorySuite) TestResolveDisconnectMatrixTypical(c *C) {
 			c.Check(connRefList, HasLen, 0)
 		} else {
 			c.Check(err, IsNil)
-			c.Check(connRefList, DeepEquals, []ConnRef{connRef})
+			c.Check(connRefList, DeepEquals, []ConnRef{*connRef})
 		}
 	}
 }
@@ -1028,8 +1057,8 @@ func (s *RepositorySuite) TestConnectFailsWhenPlugDoesNotExist(c *C) {
 	err := s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	// Connecting an unknown plug returns an appropriate error
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, ErrorMatches, `cannot connect plug "plug" from snap "consumer", no such plug`)
 }
 
@@ -1037,8 +1066,8 @@ func (s *RepositorySuite) TestConnectFailsWhenSlotDoesNotExist(c *C) {
 	err := s.testRepo.AddPlug(s.plug)
 	c.Assert(err, IsNil)
 	// Connecting to an unknown slot returns an error
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, ErrorMatches, `cannot connect plug to slot "slot" from snap "producer", no such slot`)
 }
 
@@ -1047,34 +1076,27 @@ func (s *RepositorySuite) TestConnectSucceedsWhenIdenticalConnectExists(c *C) {
 	c.Assert(err, IsNil)
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, IsNil)
 	// Connecting exactly the same thing twice succeeds without an error but does nothing.
-	err = s.testRepo.Connect(connRef)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, IsNil)
 	// Only one connection is actually present.
 	c.Assert(s.testRepo.Interfaces(), DeepEquals, &Interfaces{
-		Plugs: []*Plug{{
-			PlugInfo:    s.plug.PlugInfo,
-			Connections: []SlotRef{{Snap: s.slot.Snap.Name(), Name: s.slot.Name}},
-		}},
-		Slots: []*Slot{{
-			SlotInfo:    s.slot.SlotInfo,
-			Connections: []PlugRef{{Snap: s.plug.Snap.Name(), Name: s.plug.Name}},
-		}},
+		Plugs:       []*snap.PlugInfo{s.plug},
+		Slots:       []*snap.SlotInfo{s.slot},
+		Connections: []*ConnRef{NewConnRef(s.plug, s.slot)},
 	})
 }
 
 func (s *RepositorySuite) TestConnectFailsWhenSlotAndPlugAreIncompatible(c *C) {
 	otherInterface := &ifacetest.TestInterface{InterfaceName: "other-interface"}
 	err := s.testRepo.AddInterface(otherInterface)
-	plug := &Plug{
-		PlugInfo: &snap.PlugInfo{
-			Snap:      &snap.Info{SuggestedName: "consumer"},
-			Name:      "plug",
-			Interface: "other-interface",
-		},
+	plug := &snap.PlugInfo{
+		Snap:      &snap.Info{SuggestedName: "consumer"},
+		Name:      "plug",
+		Interface: "other-interface",
 	}
 	c.Assert(err, IsNil)
 	err = s.testRepo.AddPlug(plug)
@@ -1082,8 +1104,8 @@ func (s *RepositorySuite) TestConnectFailsWhenSlotAndPlugAreIncompatible(c *C) {
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	// Connecting a plug to an incompatible slot fails with an appropriate error
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, ErrorMatches, `cannot connect plug "consumer:plug" \(interface "other-interface"\) to "producer:slot" \(interface "interface"\)`)
 }
 
@@ -1093,8 +1115,8 @@ func (s *RepositorySuite) TestConnectSucceeds(c *C) {
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	// Connecting a plug works okay
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, IsNil)
 }
 
@@ -1138,12 +1160,12 @@ func (s *RepositorySuite) TestDisconnectFailsWhenNotConnected(c *C) {
 func (s *RepositorySuite) TestDisconnectSucceeds(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	c.Assert(s.testRepo.Connect(ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}), IsNil)
+	c.Assert(s.testRepo.Connect(*NewConnRef(s.plug, s.slot)), IsNil)
 	err := s.testRepo.Disconnect(s.plug.Snap.Name(), s.plug.Name, s.slot.Snap.Name(), s.slot.Name)
 	c.Assert(err, IsNil)
 	c.Assert(s.testRepo.Interfaces(), DeepEquals, &Interfaces{
-		Plugs: []*Plug{{PlugInfo: s.plug.PlugInfo}},
-		Slots: []*Slot{{SlotInfo: s.slot.SlotInfo}},
+		Plugs: []*snap.PlugInfo{s.plug},
+		Slots: []*snap.SlotInfo{s.slot},
 	})
 }
 
@@ -1173,39 +1195,31 @@ func (s *RepositorySuite) TestConnectedFailsWithoutPlugOrSlot(c *C) {
 func (s *RepositorySuite) TestConnectedFindsConnections(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	c.Assert(s.testRepo.Connect(ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}), IsNil)
+	c.Assert(s.testRepo.Connect(*NewConnRef(s.plug, s.slot)), IsNil)
 
 	conns, err := s.testRepo.Connected(s.plug.Snap.Name(), s.plug.Name)
 	c.Assert(err, IsNil)
-	c.Check(conns, DeepEquals, []ConnRef{
-		{s.plug.Ref(), s.slot.Ref()},
-	})
+	c.Check(conns, DeepEquals, []ConnRef{*NewConnRef(s.plug, s.slot)})
 
 	conns, err = s.testRepo.Connected(s.slot.Snap.Name(), s.slot.Name)
 	c.Assert(err, IsNil)
-	c.Check(conns, DeepEquals, []ConnRef{
-		{s.plug.Ref(), s.slot.Ref()},
-	})
+	c.Check(conns, DeepEquals, []ConnRef{*NewConnRef(s.plug, s.slot)})
 }
 
 // Connected uses the core snap if snap name is empty
 func (s *RepositorySuite) TestConnectedFindsCoreSnap(c *C) {
-	slot := &Slot{
-		SlotInfo: &snap.SlotInfo{
-			Snap:      &snap.Info{SuggestedName: "core", Type: snap.TypeOS},
-			Name:      "slot",
-			Interface: "interface",
-		},
+	slot := &snap.SlotInfo{
+		Snap:      &snap.Info{SuggestedName: "core", Type: snap.TypeOS},
+		Name:      "slot",
+		Interface: "interface",
 	}
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(slot), IsNil)
-	c.Assert(s.testRepo.Connect(ConnRef{PlugRef: s.plug.Ref(), SlotRef: slot.Ref()}), IsNil)
+	c.Assert(s.testRepo.Connect(*NewConnRef(s.plug, slot)), IsNil)
 
 	conns, err := s.testRepo.Connected("", s.slot.Name)
 	c.Assert(err, IsNil)
-	c.Check(conns, DeepEquals, []ConnRef{
-		{s.plug.Ref(), slot.Ref()},
-	})
+	c.Check(conns, DeepEquals, []ConnRef{*NewConnRef(s.plug, slot)})
 }
 
 // Tests for Repository.DisconnectAll()
@@ -1213,13 +1227,13 @@ func (s *RepositorySuite) TestConnectedFindsCoreSnap(c *C) {
 func (s *RepositorySuite) TestDisconnectAll(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	c.Assert(s.testRepo.Connect(ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}), IsNil)
+	c.Assert(s.testRepo.Connect(*NewConnRef(s.plug, s.slot)), IsNil)
 
-	conns := []ConnRef{{s.plug.Ref(), s.slot.Ref()}}
+	conns := []ConnRef{*NewConnRef(s.plug, s.slot)}
 	s.testRepo.DisconnectAll(conns)
 	c.Assert(s.testRepo.Interfaces(), DeepEquals, &Interfaces{
-		Plugs: []*Plug{{PlugInfo: s.plug.PlugInfo}},
-		Slots: []*Slot{{SlotInfo: s.slot.SlotInfo}},
+		Plugs: []*snap.PlugInfo{s.plug},
+		Slots: []*snap.SlotInfo{s.slot},
 	})
 }
 
@@ -1231,27 +1245,22 @@ func (s *RepositorySuite) TestInterfacesSmokeTest(c *C) {
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	// After connecting the result is as expected
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = s.testRepo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = s.testRepo.Connect(*connRef)
 	c.Assert(err, IsNil)
 	ifaces := s.testRepo.Interfaces()
 	c.Assert(ifaces, DeepEquals, &Interfaces{
-		Plugs: []*Plug{{
-			PlugInfo:    s.plug.PlugInfo,
-			Connections: []SlotRef{{Snap: s.slot.Snap.Name(), Name: s.slot.Name}},
-		}},
-		Slots: []*Slot{{
-			SlotInfo:    s.slot.SlotInfo,
-			Connections: []PlugRef{{Snap: s.plug.Snap.Name(), Name: s.plug.Name}},
-		}},
+		Plugs:       []*snap.PlugInfo{s.plug},
+		Slots:       []*snap.SlotInfo{s.slot},
+		Connections: []*ConnRef{NewConnRef(s.plug, s.slot)},
 	})
 	// After disconnecting the connections become empty
 	err = s.testRepo.Disconnect(s.plug.Snap.Name(), s.plug.Name, s.slot.Snap.Name(), s.slot.Name)
 	c.Assert(err, IsNil)
 	ifaces = s.testRepo.Interfaces()
 	c.Assert(ifaces, DeepEquals, &Interfaces{
-		Plugs: []*Plug{{PlugInfo: s.plug.PlugInfo}},
-		Slots: []*Slot{{SlotInfo: s.slot.SlotInfo}},
+		Plugs: []*snap.PlugInfo{s.plug},
+		Slots: []*snap.SlotInfo{s.slot},
 	})
 }
 
@@ -1261,19 +1270,19 @@ const testSecurity SecuritySystem = "test"
 
 var testInterface = &ifacetest.TestInterface{
 	InterfaceName: "interface",
-	TestPermanentPlugCallback: func(spec *ifacetest.Specification, plug *Plug) error {
+	TestPermanentPlugCallback: func(spec *ifacetest.Specification, plug *snap.PlugInfo) error {
 		spec.AddSnippet("static plug snippet")
 		return nil
 	},
-	TestConnectedPlugCallback: func(spec *ifacetest.Specification, plug *Plug, slot *Slot) error {
+	TestConnectedPlugCallback: func(spec *ifacetest.Specification, plug *ConnectedPlug, slot *ConnectedSlot) error {
 		spec.AddSnippet("connection-specific plug snippet")
 		return nil
 	},
-	TestPermanentSlotCallback: func(spec *ifacetest.Specification, slot *Slot) error {
+	TestPermanentSlotCallback: func(spec *ifacetest.Specification, slot *snap.SlotInfo) error {
 		spec.AddSnippet("static slot snippet")
 		return nil
 	},
-	TestConnectedSlotCallback: func(spec *ifacetest.Specification, plug *Plug, slot *Slot) error {
+	TestConnectedSlotCallback: func(spec *ifacetest.Specification, plug *ConnectedPlug, slot *ConnectedSlot) error {
 		spec.AddSnippet("connection-specific slot snippet")
 		return nil
 	},
@@ -1297,8 +1306,8 @@ func (s *RepositorySuite) TestSnapSpecification(c *C) {
 	c.Check(spec.(*ifacetest.Specification).Snippets, DeepEquals, []string{"static slot snippet"})
 
 	// Establish connection between plug and slot
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	err = repo.Connect(connRef)
+	connRef := NewConnRef(s.plug, s.slot)
+	err = repo.Connect(*connRef)
 	c.Assert(err, IsNil)
 
 	// Snaps should get static and connection-specific security now
@@ -1322,10 +1331,10 @@ func (s *RepositorySuite) TestSnapSpecificationFailureWithConnectionSnippets(c *
 	backend := &ifacetest.TestSecurityBackend{BackendName: testSecurity}
 	iface := &ifacetest.TestInterface{
 		InterfaceName: "interface",
-		TestConnectedSlotCallback: func(spec *ifacetest.Specification, plug *Plug, slot *Slot) error {
+		TestConnectedSlotCallback: func(spec *ifacetest.Specification, plug *ConnectedPlug, slot *ConnectedSlot) error {
 			return fmt.Errorf("cannot compute snippet for provider")
 		},
-		TestConnectedPlugCallback: func(spec *ifacetest.Specification, plug *Plug, slot *Slot) error {
+		TestConnectedPlugCallback: func(spec *ifacetest.Specification, plug *ConnectedPlug, slot *ConnectedSlot) error {
 			return fmt.Errorf("cannot compute snippet for consumer")
 		},
 	}
@@ -1335,8 +1344,8 @@ func (s *RepositorySuite) TestSnapSpecificationFailureWithConnectionSnippets(c *
 	c.Assert(repo.AddInterface(iface), IsNil)
 	c.Assert(repo.AddPlug(s.plug), IsNil)
 	c.Assert(repo.AddSlot(s.slot), IsNil)
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	c.Assert(repo.Connect(connRef), IsNil)
+	connRef := NewConnRef(s.plug, s.slot)
+	c.Assert(repo.Connect(*connRef), IsNil)
 
 	spec, err := repo.SnapSpecification(testSecurity, s.plug.Snap.Name())
 	c.Assert(err, ErrorMatches, "cannot compute snippet for consumer")
@@ -1351,10 +1360,10 @@ func (s *RepositorySuite) TestSnapSpecificationFailureWithPermanentSnippets(c *C
 	var testSecurity SecuritySystem = "security"
 	iface := &ifacetest.TestInterface{
 		InterfaceName: "interface",
-		TestPermanentSlotCallback: func(spec *ifacetest.Specification, slot *Slot) error {
+		TestPermanentSlotCallback: func(spec *ifacetest.Specification, slot *snap.SlotInfo) error {
 			return fmt.Errorf("cannot compute snippet for provider")
 		},
-		TestPermanentPlugCallback: func(spec *ifacetest.Specification, plug *Plug) error {
+		TestPermanentPlugCallback: func(spec *ifacetest.Specification, plug *snap.PlugInfo) error {
 			return fmt.Errorf("cannot compute snippet for consumer")
 		},
 	}
@@ -1364,8 +1373,8 @@ func (s *RepositorySuite) TestSnapSpecificationFailureWithPermanentSnippets(c *C
 	c.Assert(repo.AddInterface(iface), IsNil)
 	c.Assert(repo.AddPlug(s.plug), IsNil)
 	c.Assert(repo.AddSlot(s.slot), IsNil)
-	connRef := ConnRef{PlugRef: s.plug.Ref(), SlotRef: s.slot.Ref()}
-	c.Assert(repo.Connect(connRef), IsNil)
+	connRef := NewConnRef(s.plug, s.slot)
+	c.Assert(repo.Connect(*connRef), IsNil)
 
 	spec, err := repo.SnapSpecification(testSecurity, s.plug.Snap.Name())
 	c.Assert(err, ErrorMatches, "cannot compute snippet for consumer")
@@ -1391,12 +1400,14 @@ func (s *RepositorySuite) TestAutoConnectCandidatePlugsAndSlots(c *C) {
 	// Add a pair of snaps with plugs/slots using those two interfaces
 	consumer := snaptest.MockInfo(c, `
 name: consumer
+version: 0
 plugs:
     auto:
     manual:
 `, nil)
 	producer := snaptest.MockInfo(c, `
 name: producer
+version: 0
 type: os
 slots:
     auto:
@@ -1420,66 +1431,106 @@ slots:
 	c.Check(candidatePlugs[0].Name, Equals, "auto")
 }
 
+func (s *RepositorySuite) TestAutoConnectCandidatePlugsAndSlotsSymmetry(c *C) {
+	repo := s.emptyRepo
+	// Add a "auto" interface
+	err := repo.AddInterface(&ifacetest.TestInterface{InterfaceName: "auto"})
+	c.Assert(err, IsNil)
+
+	policyCheck := func(plug *Plug, slot *Slot) bool {
+		return slot.Interface == "auto"
+	}
+
+	// Add a producer snap for "auto"
+	producer := snaptest.MockInfo(c, `
+name: producer
+version: 0
+type: os
+slots:
+    auto:
+`, nil)
+	err = repo.AddSnap(producer)
+	c.Assert(err, IsNil)
+
+	// Add two consumers snaps for "auto"
+	consumer1 := snaptest.MockInfo(c, `
+name: consumer1
+version: 0
+plugs:
+    auto:
+`, nil)
+
+	err = repo.AddSnap(consumer1)
+	c.Assert(err, IsNil)
+
+	// Add two consumers snaps for "auto"
+	consumer2 := snaptest.MockInfo(c, `
+name: consumer2
+version: 0
+plugs:
+    auto:
+`, nil)
+
+	err = repo.AddSnap(consumer2)
+	c.Assert(err, IsNil)
+
+	// Both can auto-connect
+	candidateSlots := repo.AutoConnectCandidateSlots("consumer1", "auto", policyCheck)
+	c.Assert(candidateSlots, HasLen, 1)
+	c.Check(candidateSlots[0].Snap.Name(), Equals, "producer")
+	c.Check(candidateSlots[0].Interface, Equals, "auto")
+	c.Check(candidateSlots[0].Name, Equals, "auto")
+
+	candidateSlots = repo.AutoConnectCandidateSlots("consumer2", "auto", policyCheck)
+	c.Assert(candidateSlots, HasLen, 1)
+	c.Check(candidateSlots[0].Snap.Name(), Equals, "producer")
+	c.Check(candidateSlots[0].Interface, Equals, "auto")
+	c.Check(candidateSlots[0].Name, Equals, "auto")
+
+	// Plugs candidates seen from the producer (for example if
+	// it's installed after) should be the same
+	candidatePlugs := repo.AutoConnectCandidatePlugs("producer", "auto", policyCheck)
+	c.Assert(candidatePlugs, HasLen, 2)
+}
+
 // Tests for AddSnap and RemoveSnap
 
 type AddRemoveSuite struct {
+	testutil.BaseTest
 	repo *Repository
 }
 
 var _ = Suite(&AddRemoveSuite{})
 
 func (s *AddRemoveSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+
 	s.repo = NewRepository()
 	err := s.repo.AddInterface(&ifacetest.TestInterface{InterfaceName: "iface"})
 	c.Assert(err, IsNil)
 	err = s.repo.AddInterface(&ifacetest.TestInterface{
-		InterfaceName:        "invalid",
-		SanitizePlugCallback: func(plug *Plug) error { return fmt.Errorf("plug is invalid") },
-		SanitizeSlotCallback: func(slot *Slot) error { return fmt.Errorf("slot is invalid") },
+		InterfaceName:             "invalid",
+		BeforePreparePlugCallback: func(plug *snap.PlugInfo) error { return fmt.Errorf("plug is invalid") },
+		BeforePrepareSlotCallback: func(slot *snap.SlotInfo) error { return fmt.Errorf("slot is invalid") },
 	})
 	c.Assert(err, IsNil)
 }
 
-func (s *AddRemoveSuite) TestAddSnapComplexErrorHandling(c *C) {
-	err := s.repo.AddInterface(&ifacetest.TestInterface{
-		InterfaceName:        "invalid-plug-iface",
-		SanitizePlugCallback: func(plug *Plug) error { return fmt.Errorf("plug is invalid") },
-		SanitizeSlotCallback: func(slot *Slot) error { return fmt.Errorf("slot is invalid") },
-	})
-	c.Assert(err, IsNil)
-	err = s.repo.AddInterface(&ifacetest.TestInterface{
-		InterfaceName:        "invalid-slot-iface",
-		SanitizePlugCallback: func(plug *Plug) error { return fmt.Errorf("plug is invalid") },
-		SanitizeSlotCallback: func(slot *Slot) error { return fmt.Errorf("slot is invalid") },
-	})
-	c.Assert(err, IsNil)
-	snapInfo := snaptest.MockInfo(c, `
-name: complex
-plugs:
-    invalid-plug-iface:
-    unknown-plug-iface:
-slots:
-    invalid-slot-iface:
-    unknown-slot-iface:
-`, nil)
-	err = s.repo.AddSnap(snapInfo)
-	c.Check(err, ErrorMatches,
-		`snap "complex" has bad plugs or slots: invalid-plug-iface \(plug is invalid\); invalid-slot-iface \(slot is invalid\); unknown-plug-iface, unknown-slot-iface \(unknown interface\)`)
-	// Nothing was added
-	c.Check(s.repo.Plug("complex", "invalid-plug-iface"), IsNil)
-	c.Check(s.repo.Plug("complex", "unknown-plug-iface"), IsNil)
-	c.Check(s.repo.Slot("complex", "invalid-slot-iface"), IsNil)
-	c.Check(s.repo.Slot("complex", "unknown-slot-iface"), IsNil)
+func (s *AddRemoveSuite) TearDownTest(c *C) {
+	s.BaseTest.TearDownTest(c)
 }
 
 const testConsumerYaml = `
 name: consumer
+version: 0
 apps:
     app:
         plugs: [iface]
 `
 const testProducerYaml = `
 name: producer
+version: 0
 apps:
     app:
         slots: [iface]
@@ -1487,6 +1538,7 @@ apps:
 
 const testConsumerInvalidSlotNameYaml = `
 name: consumer
+version: 0
 slots:
  ttyS5:
   interface: iface
@@ -1497,6 +1549,7 @@ apps:
 
 const testConsumerInvalidPlugNameYaml = `
 name: consumer
+version: 0
 plugs:
  ttyS3:
   interface: iface
@@ -1517,18 +1570,6 @@ func (s *AddRemoveSuite) TestAddSnapAddsPlugs(c *C) {
 	c.Assert(s.repo.Plug("consumer", "iface"), Not(IsNil))
 }
 
-func (s *AddRemoveSuite) TestAddSnapErrorsOnInvalidSlotNames(c *C) {
-	_, err := s.addSnap(c, testConsumerInvalidSlotNameYaml)
-	c.Assert(err, NotNil)
-	c.Check(err, ErrorMatches, `snap "consumer" has bad plugs or slots: ttyS5 \(invalid interface name: "ttyS5"\)`)
-}
-
-func (s *AddRemoveSuite) TestAddSnapErrorsOnInvalidPlugNames(c *C) {
-	_, err := s.addSnap(c, testConsumerInvalidPlugNameYaml)
-	c.Assert(err, NotNil)
-	c.Check(err, ErrorMatches, `snap "consumer" has bad plugs or slots: ttyS3 \(invalid interface name: "ttyS3"\)`)
-}
-
 func (s *AddRemoveSuite) TestAddSnapErrorsOnExistingSnapPlugs(c *C) {
 	_, err := s.addSnap(c, testConsumerYaml)
 	c.Assert(err, IsNil)
@@ -1550,6 +1591,24 @@ func (s *AddRemoveSuite) TestAddSnapErrorsOnExistingSnapSlots(c *C) {
 	c.Assert(err, ErrorMatches, `cannot register interfaces for snap "producer" more than once`)
 }
 
+func (s *AddRemoveSuite) TestAddSnapSkipsUnknownInterfaces(c *C) {
+	info, err := s.addSnap(c, `
+name: bogus
+version: 0
+plugs:
+  bogus-plug:
+slots:
+  bogus-slot:
+`)
+	c.Assert(err, IsNil)
+	// the snap knowns about the bogus plug and slot
+	c.Assert(info.Plugs["bogus-plug"], NotNil)
+	c.Assert(info.Slots["bogus-slot"], NotNil)
+	// but the repository ignores them
+	c.Assert(s.repo.Plug("bogus", "bogus-plug"), IsNil)
+	c.Assert(s.repo.Slot("bogus", "bogus-slot"), IsNil)
+}
+
 func (s AddRemoveSuite) TestRemoveRemovesPlugs(c *C) {
 	_, err := s.addSnap(c, testConsumerYaml)
 	c.Assert(err, IsNil)
@@ -1589,6 +1648,7 @@ func (s *AddRemoveSuite) TestRemoveSnapErrorsOnStillConnectedSlot(c *C) {
 }
 
 type DisconnectSnapSuite struct {
+	testutil.BaseTest
 	repo   *Repository
 	s1, s2 *snap.Info
 }
@@ -1596,6 +1656,9 @@ type DisconnectSnapSuite struct {
 var _ = Suite(&DisconnectSnapSuite{})
 
 func (s *DisconnectSnapSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+
 	s.repo = NewRepository()
 
 	err := s.repo.AddInterface(&ifacetest.TestInterface{InterfaceName: "iface-a"})
@@ -1605,6 +1668,7 @@ func (s *DisconnectSnapSuite) SetUpTest(c *C) {
 
 	s.s1 = snaptest.MockInfo(c, `
 name: s1
+version: 0
 plugs:
     iface-a:
 slots:
@@ -1615,6 +1679,7 @@ slots:
 
 	s.s2 = snaptest.MockInfo(c, `
 name: s2
+version: 0
 plugs:
     iface-b:
 slots:
@@ -1625,6 +1690,10 @@ slots:
 	c.Assert(err, IsNil)
 }
 
+func (s *DisconnectSnapSuite) TearDownTest(c *C) {
+	s.BaseTest.TearDownTest(c)
+}
+
 func (s *DisconnectSnapSuite) TestNotConnected(c *C) {
 	affected, err := s.repo.DisconnectSnap("s1")
 	c.Assert(err, IsNil)
@@ -1686,6 +1755,7 @@ func makeContentConnectionTestSnaps(c *C, plugContentToken, slotContentToken str
 
 	plugSnap := snaptest.MockInfo(c, fmt.Sprintf(`
 name: content-plug-snap
+version: 0
 plugs:
   imported-content:
     interface: content
@@ -1693,6 +1763,7 @@ plugs:
 `, plugContentToken), nil)
 	slotSnap := snaptest.MockInfo(c, fmt.Sprintf(`
 name: content-slot-snap
+version: 0
 slots:
   exported-content:
     interface: content
@@ -1746,3 +1817,86 @@ func (s *RepositorySuite) TestAutoConnectContentInterfaceNoMatchingDeveloper(c *
 	candidatePlugs := repo.AutoConnectCandidatePlugs("content-slot-snap", "exported-content", contentPolicyCheck)
 	c.Assert(candidatePlugs, HasLen, 0)
 }
+
+func (s *RepositorySuite) TestInfo(c *C) {
+	r := s.emptyRepo
+
+	// Add some test interfaces.
+	i1 := &ifacetest.TestInterface{InterfaceName: "i1", InterfaceStaticInfo: StaticInfo{Summary: "i1 summary", DocURL: "http://example.com/i1"}}
+	i2 := &ifacetest.TestInterface{InterfaceName: "i2", InterfaceStaticInfo: StaticInfo{Summary: "i2 summary", DocURL: "http://example.com/i2"}}
+	i3 := &ifacetest.TestInterface{InterfaceName: "i3", InterfaceStaticInfo: StaticInfo{Summary: "i3 summary", DocURL: "http://example.com/i3"}}
+	c.Assert(r.AddInterface(i1), IsNil)
+	c.Assert(r.AddInterface(i2), IsNil)
+	c.Assert(r.AddInterface(i3), IsNil)
+
+	// Add some test snaps.
+	s1 := snaptest.MockInfo(c, fmt.Sprintf(`
+name: s1
+version: 0
+apps:
+  s1:
+    plugs: [i1, i2]
+`), nil)
+	c.Assert(r.AddSnap(s1), IsNil)
+
+	s2 := snaptest.MockInfo(c, fmt.Sprintf(`
+name: s2
+version: 0
+apps:
+  s2:
+    slots: [i1, i3]
+`), nil)
+	c.Assert(r.AddSnap(s2), IsNil)
+
+	s3 := snaptest.MockInfo(c, fmt.Sprintf(`
+name: s3
+version: 0
+type: os
+slots:
+  i2:
+`), nil)
+	c.Assert(r.AddSnap(s3), IsNil)
+
+	// Connect a few things for the tests below.
+	c.Assert(r.Connect(ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i1"}, SlotRef: SlotRef{Snap: "s2", Name: "i1"}}), IsNil)
+	c.Assert(r.Connect(ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i2"}, SlotRef: SlotRef{Snap: "s3", Name: "i2"}}), IsNil)
+
+	// Without any names or options we get the summary of all the interfaces.
+	infos := r.Info(nil)
+	c.Assert(infos, DeepEquals, []*Info{
+		{Name: "i1", Summary: "i1 summary"},
+		{Name: "i2", Summary: "i2 summary"},
+		{Name: "i3", Summary: "i3 summary"},
+	})
+
+	// We can choose specific interfaces, unknown names are just skipped.
+	infos = r.Info(&InfoOptions{Names: []string{"i2", "i4"}})
+	c.Assert(infos, DeepEquals, []*Info{
+		{Name: "i2", Summary: "i2 summary"},
+	})
+
+	// We can ask for documentation.
+	infos = r.Info(&InfoOptions{Names: []string{"i2"}, Doc: true})
+	c.Assert(infos, DeepEquals, []*Info{
+		{Name: "i2", Summary: "i2 summary", DocURL: "http://example.com/i2"},
+	})
+
+	// We can ask for a list of plugs.
+	infos = r.Info(&InfoOptions{Names: []string{"i2"}, Plugs: true})
+	c.Assert(infos, DeepEquals, []*Info{
+		{Name: "i2", Summary: "i2 summary", Plugs: []*snap.PlugInfo{s1.Plugs["i2"]}},
+	})
+
+	// We can ask for a list of slots too.
+	infos = r.Info(&InfoOptions{Names: []string{"i2"}, Slots: true})
+	c.Assert(infos, DeepEquals, []*Info{
+		{Name: "i2", Summary: "i2 summary", Slots: []*snap.SlotInfo{s3.Slots["i2"]}},
+	})
+
+	// We can also ask for only those interfaces that have connected plugs or slots.
+	infos = r.Info(&InfoOptions{Connected: true})
+	c.Assert(infos, DeepEquals, []*Info{
+		{Name: "i1", Summary: "i1 summary"},
+		{Name: "i2", Summary: "i2 summary"},
+	})
+}
