@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -29,15 +29,16 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/interfaces/systemd"
-	"github.com/snapcore/snapd/osutil"
-	"github.com/snapcore/snapd/testutil"
+	"github.com/snapcore/snapd/snap"
 
 	sysd "github.com/snapcore/snapd/systemd"
 )
 
 type backendSuite struct {
 	ifacetest.BackendSuite
-	systemctlCmd *testutil.MockCmd
+
+	systemctlArgs     [][]string
+	systemctlRestorer func()
 }
 
 var _ = Suite(&backendSuite{})
@@ -50,122 +51,38 @@ var testedConfinementOpts = []interfaces.ConfinementOptions{
 }
 
 func (s *backendSuite) SetUpTest(c *C) {
-	s.BackendSuite.SetUpTest(c)
 	s.Backend = &systemd.Backend{}
-	s.systemctlCmd = testutil.MockCommand(c, "systemctl", "echo ActiveState=inactive")
+	s.BackendSuite.SetUpTest(c)
+	c.Assert(s.Repo.AddBackend(s.Backend), IsNil)
+	s.systemctlRestorer = sysd.MockSystemctl(func(args ...string) ([]byte, error) {
+		s.systemctlArgs = append(s.systemctlArgs, append([]string{"systemctl"}, args...))
+		return []byte("ActiveState=inactive"), nil
+	})
 }
 
 func (s *backendSuite) TearDownTest(c *C) {
-	s.systemctlCmd.Restore()
+	s.systemctlRestorer()
 	s.BackendSuite.TearDownTest(c)
 }
 
 func (s *backendSuite) TestName(c *C) {
-	c.Check(s.Backend.Name(), Equals, "systemd")
-}
-
-func (s *backendSuite) TestUnmarshalRawSnippetMap(c *C) {
-	rawSnippetMap := map[string][][]byte{
-		"security-tag": {
-			[]byte(`{"services": {"foo.service": {"exec-start": "/bin/true"}}}`),
-			[]byte(`{"services": {"bar.service": {"exec-start": "/bin/false"}}}`),
-		},
-	}
-	richSnippetMap, err := systemd.UnmarshalRawSnippetMap(rawSnippetMap)
-	c.Assert(err, IsNil)
-	c.Assert(richSnippetMap, DeepEquals, map[string][]*systemd.Snippet{
-		"security-tag": {
-			{
-				Services: map[string]systemd.Service{
-					"foo.service": {ExecStart: "/bin/true"},
-				},
-			},
-			{
-				Services: map[string]systemd.Service{
-					"bar.service": {ExecStart: "/bin/false"},
-				},
-			},
-		},
-	})
-}
-
-func (s *backendSuite) TestMergeSnippetMapOK(c *C) {
-	snippetMap := map[string][]*systemd.Snippet{
-		"security-tag": {
-			{
-				Services: map[string]systemd.Service{
-					"foo.service": {ExecStart: "/bin/true"},
-				},
-			},
-		},
-		"another-tag": {
-			{
-				Services: map[string]systemd.Service{
-					"bar.service": {ExecStart: "/bin/false"},
-				},
-			},
-		},
-	}
-	snippet, err := systemd.MergeSnippetMap(snippetMap)
-	c.Assert(err, IsNil)
-	c.Assert(snippet, DeepEquals, &systemd.Snippet{
-		Services: map[string]systemd.Service{
-			"foo.service": {ExecStart: "/bin/true"},
-			"bar.service": {ExecStart: "/bin/false"},
-		},
-	})
-}
-
-func (s *backendSuite) TestMergeSnippetMapClashing(c *C) {
-	snippetMap := map[string][]*systemd.Snippet{
-		"security-tag": {
-			{
-				Services: map[string]systemd.Service{
-					"foo.service": {ExecStart: "/bin/true"},
-				},
-			},
-		},
-		"another-tag": {
-			{
-				Services: map[string]systemd.Service{
-					"foo.service": {ExecStart: "/bin/evil"},
-				},
-			},
-		},
-	}
-	snippet, err := systemd.MergeSnippetMap(snippetMap)
-	c.Assert(err, ErrorMatches, `interface require conflicting system needs`)
-	c.Assert(snippet, IsNil)
-}
-
-func (s *backendSuite) TestRenderSnippet(c *C) {
-	snippet := &systemd.Snippet{
-		Services: map[string]systemd.Service{
-			"foo.service": {ExecStart: "/bin/true"},
-		},
-	}
-	content, err := systemd.RenderSnippet(snippet)
-	c.Assert(err, IsNil)
-	c.Assert(content, DeepEquals, map[string]*osutil.FileState{
-		"foo.service": {
-			Content: []byte("[Service]\nExecStart=/bin/true\n\n[Install]\nWantedBy=multi-user.target\n"),
-			Mode:    0644,
-		},
-	})
+	c.Check(s.Backend.Name(), Equals, interfaces.SecuritySystemd)
 }
 
 func (s *backendSuite) TestInstallingSnapWritesStartsServices(c *C) {
-	prevctlCmd := sysd.SystemctlCmd
 	var sysdLog [][]string
-	sysd.SystemctlCmd = func(cmd ...string) ([]byte, error) {
+
+	r := sysd.MockSystemctl(func(cmd ...string) ([]byte, error) {
 		sysdLog = append(sysdLog, cmd)
 		if cmd[0] == "show" {
 			return []byte("ActiveState=inactive\n"), nil
 		}
 		return []byte{}, nil
-	}
-	s.Iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(`{"services": {"snap.samba.interface.foo.service": {"exec-start": "/bin/true"}}}`), nil
+	})
+	defer r()
+
+	s.Iface.SystemdPermanentSlotCallback = func(spec *systemd.Specification, slot *snap.SlotInfo) error {
+		return spec.AddService("snap.samba.interface.foo.service", &systemd.Service{ExecStart: "/bin/true"})
 	}
 	s.InstallSnap(c, interfaces.ConfinementOptions{}, ifacetest.SambaYamlV1, 1)
 	service := filepath.Join(dirs.SnapServicesDir, "snap.samba.interface.foo.service")
@@ -180,23 +97,22 @@ func (s *backendSuite) TestInstallingSnapWritesStartsServices(c *C) {
 		{"show", "--property=ActiveState", "snap.samba.interface.foo.service"},
 		{"start", "snap.samba.interface.foo.service"},
 	})
-	sysd.SystemctlCmd = prevctlCmd
 }
 
 func (s *backendSuite) TestRemovingSnapRemovesAndStopsServices(c *C) {
-	s.Iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(`{"services": {"snap.samba.interface.foo.service": {"exec-start": "/bin/true"}}}`), nil
+	s.Iface.SystemdPermanentSlotCallback = func(spec *systemd.Specification, slot *snap.SlotInfo) error {
+		return spec.AddService("snap.samba.interface.foo.service", &systemd.Service{ExecStart: "/bin/true"})
 	}
 	for _, opts := range testedConfinementOpts {
 		snapInfo := s.InstallSnap(c, opts, ifacetest.SambaYamlV1, 1)
-		s.systemctlCmd.ForgetCalls()
+		s.systemctlArgs = nil
 		s.RemoveSnap(c, snapInfo)
 		service := filepath.Join(dirs.SnapServicesDir, "snap.samba.interface.foo.service")
 		// the service file was removed
 		_, err := os.Stat(service)
 		c.Check(os.IsNotExist(err), Equals, true)
 		// the service was stopped
-		c.Check(s.systemctlCmd.Calls(), DeepEquals, [][]string{
+		c.Check(s.systemctlArgs, DeepEquals, [][]string{
 			{"systemctl", "--root", dirs.GlobalRootDir, "disable", "snap.samba.interface.foo.service"},
 			{"systemctl", "stop", "snap.samba.interface.foo.service"},
 			{"systemctl", "show", "--property=ActiveState", "snap.samba.interface.foo.service"},
@@ -206,11 +122,15 @@ func (s *backendSuite) TestRemovingSnapRemovesAndStopsServices(c *C) {
 }
 
 func (s *backendSuite) TestSettingUpSecurityWithFewerServices(c *C) {
-	s.Iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(`{"services": {"snap.samba.interface.foo.service": {"exec-start": "/bin/true"}, "snap.samba.interface.bar.service": {"exec-start": "/bin/false"}}}`), nil
+	s.Iface.SystemdPermanentSlotCallback = func(spec *systemd.Specification, slot *snap.SlotInfo) error {
+		err := spec.AddService("snap.samba.interface.foo.service", &systemd.Service{ExecStart: "/bin/true"})
+		if err != nil {
+			return err
+		}
+		return spec.AddService("snap.samba.interface.bar.service", &systemd.Service{ExecStart: "/bin/false"})
 	}
 	snapInfo := s.InstallSnap(c, interfaces.ConfinementOptions{}, ifacetest.SambaYamlV1, 1)
-	s.systemctlCmd.ForgetCalls()
+	s.systemctlArgs = nil
 	serviceFoo := filepath.Join(dirs.SnapServicesDir, "snap.samba.interface.foo.service")
 	serviceBar := filepath.Join(dirs.SnapServicesDir, "snap.samba.interface.bar.service")
 	// the services were created
@@ -220,13 +140,13 @@ func (s *backendSuite) TestSettingUpSecurityWithFewerServices(c *C) {
 	c.Check(err, IsNil)
 
 	// Change what the interface returns to simulate some useful change
-	s.Iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
-		return []byte(`{"services": {"snap.samba.interface.foo.service": {"exec-start": "/bin/true"}}}`), nil
+	s.Iface.SystemdPermanentSlotCallback = func(spec *systemd.Specification, slot *snap.SlotInfo) error {
+		return spec.AddService("snap.samba.interface.foo.service", &systemd.Service{ExecStart: "/bin/true"})
 	}
 	// Update over to the same snap to regenerate security
 	s.UpdateSnap(c, snapInfo, interfaces.ConfinementOptions{}, ifacetest.SambaYamlV1, 0)
 	// The bar service should have been stopped
-	c.Check(s.systemctlCmd.Calls(), DeepEquals, [][]string{
+	c.Check(s.systemctlArgs, DeepEquals, [][]string{
 		{"systemctl", "--root", dirs.GlobalRootDir, "disable", "snap.samba.interface.bar.service"},
 		{"systemctl", "stop", "snap.samba.interface.bar.service"},
 		{"systemctl", "show", "--property=ActiveState", "snap.samba.interface.bar.service"},
