@@ -20,25 +20,29 @@
 package hookstate
 
 import (
-	"crypto/rand"
-	"encoding/base64"
+	"bytes"
 	"encoding/json"
 	"fmt"
 	"sync"
 	"sync/atomic"
 	"time"
 
+	"github.com/snapcore/snapd/jsonutil"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/strutil"
 )
 
-// Context represents the context under which a given hook is running.
+// Context represents the context under which the snap is calling back into snapd.
+// It is associated with a task when the callback is happening from within a hook,
+// or otherwise considered an ephemeral context in that its associated data will
+// be discarded once that individual call is finished.
 type Context struct {
 	task    *state.Task
+	state   *state.State
 	setup   *HookSetup
 	id      string
 	handler Handler
-	timeout time.Duration
 
 	cache  map[interface{}]interface{}
 	onDone []func() error
@@ -47,19 +51,20 @@ type Context struct {
 	mutexChecker int32
 }
 
-// NewContext returns a new Context.
-func NewContext(task *state.Task, setup *HookSetup, handler Handler) (*Context, error) {
-	// Generate a secure, random ID for this context
-	idBytes := make([]byte, 32)
-	_, err := rand.Read(idBytes)
-	if err != nil {
-		return nil, fmt.Errorf("cannot generate context ID: %s", err)
+// NewContext returns a new context associated with the provided task or
+// an ephemeral context if task is nil.
+//
+// A random ID is generated if contextID is empty.
+func NewContext(task *state.Task, state *state.State, setup *HookSetup, handler Handler, contextID string) (*Context, error) {
+	if contextID == "" {
+		contextID = strutil.MakeRandomString(44)
 	}
 
 	return &Context{
 		task:    task,
+		state:   state,
 		setup:   setup,
-		id:      base64.URLEncoding.EncodeToString(idBytes),
+		id:      contextID,
 		handler: handler,
 		cache:   make(map[interface{}]interface{}),
 	}, nil
@@ -75,6 +80,12 @@ func (c *Context) SnapRevision() snap.Revision {
 	return c.setup.Revision
 }
 
+// Task returns the task associated with the hook or (nil, false) if the context is ephemeral
+// and task is not available.
+func (c *Context) Task() (*state.Task, bool) {
+	return c.task, c.task != nil
+}
+
 // HookName returns the name of the hook in this context.
 func (c *Context) HookName() string {
 	return c.setup.Hook
@@ -99,14 +110,14 @@ func (c *Context) Handler() Handler {
 // and OnDone/Done).
 func (c *Context) Lock() {
 	c.mutex.Lock()
-	c.task.State().Lock()
+	c.state.Lock()
 	atomic.AddInt32(&c.mutexChecker, 1)
 }
 
 // Unlock releases the lock for this context.
 func (c *Context) Unlock() {
 	atomic.AddInt32(&c.mutexChecker, -1)
-	c.task.State().Unlock()
+	c.state.Unlock()
 	c.mutex.Unlock()
 }
 
@@ -129,8 +140,12 @@ func (c *Context) Set(key string, value interface{}) {
 	c.writing()
 
 	var data map[string]*json.RawMessage
-	if err := c.task.Get("hook-context", &data); err != nil && err != state.ErrNoState {
-		panic(fmt.Sprintf("internal error: cannot unmarshal context: %v", err))
+	if c.IsEphemeral() {
+		data, _ = c.cache["ephemeral-context"].(map[string]*json.RawMessage)
+	} else {
+		if err := c.task.Get("hook-context", &data); err != nil && err != state.ErrNoState {
+			panic(fmt.Sprintf("internal error: cannot unmarshal context: %v", err))
+		}
 	}
 	if data == nil {
 		data = make(map[string]*json.RawMessage)
@@ -143,7 +158,11 @@ func (c *Context) Set(key string, value interface{}) {
 	raw := json.RawMessage(marshalledValue)
 	data[key] = &raw
 
-	c.task.Set("hook-context", data)
+	if c.IsEphemeral() {
+		c.cache["ephemeral-context"] = data
+	} else {
+		c.task.Set("hook-context", data)
+	}
 }
 
 // Get unmarshals the stored value associated with the provided key into the
@@ -153,8 +172,15 @@ func (c *Context) Get(key string, value interface{}) error {
 	c.reading()
 
 	var data map[string]*json.RawMessage
-	if err := c.task.Get("hook-context", &data); err != nil {
-		return err
+	if c.IsEphemeral() {
+		data, _ = c.cache["ephemeral-context"].(map[string]*json.RawMessage)
+		if data == nil {
+			return state.ErrNoState
+		}
+	} else {
+		if err := c.task.Get("hook-context", &data); err != nil {
+			return err
+		}
 	}
 
 	raw, ok := data[key]
@@ -162,7 +188,7 @@ func (c *Context) Get(key string, value interface{}) error {
 		return state.ErrNoState
 	}
 
-	err := json.Unmarshal([]byte(*raw), &value)
+	err := jsonutil.DecodeWithNumber(bytes.NewReader(*raw), &value)
 	if err != nil {
 		return fmt.Errorf("cannot unmarshal context value for %q: %s", key, err)
 	}
@@ -172,7 +198,7 @@ func (c *Context) Get(key string, value interface{}) error {
 
 // State returns the state contained within the context
 func (c *Context) State() *state.State {
-	return c.task.State()
+	return c.state
 }
 
 // Cached returns the cached value associated with the provided key. It returns
@@ -218,3 +244,7 @@ func (c *Context) Done() error {
 
 	return firstErr
 }
+
+func (c *Context) IsEphemeral() bool {
+	return c.task == nil
+}
