@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -17,20 +17,16 @@
  *
  */
 
-// Package hookstate implements the manager and state aspects responsible for
-// the running of hooks.
 package hookstate
 
 import (
-	"bytes"
 	"fmt"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"regexp"
 	"strings"
 	"sync"
-	"syscall"
+	"sync/atomic"
 	"time"
 
 	"gopkg.in/tomb.v2"
@@ -44,6 +40,9 @@ import (
 	"github.com/snapcore/snapd/snap"
 )
 
+type hijackFunc func(ctx *Context) error
+type hijackKey struct{ hook, snap string }
+
 // HookManager is responsible for the maintenance of hooks in the system state.
 // It runs hooks when they're requested, assuming they're present in the given
 // snap. Otherwise they're skipped with no error.
@@ -54,6 +53,10 @@ type HookManager struct {
 
 	contextsMutex sync.RWMutex
 	contexts      map[string]*Context
+
+	hijackMap map[hijackKey]hijackFunc
+
+	runningHooks int32
 }
 
 // Handler is the interface a client must satify to handle hooks.
@@ -86,37 +89,62 @@ type HookSetup struct {
 // Manager returns a new HookManager.
 func Manager(s *state.State) (*HookManager, error) {
 	runner := state.NewTaskRunner(s)
+
+	// Make sure we only run 1 hook task for given snap at a time
+	runner.SetBlocked(func(thisTask *state.Task, running []*state.Task) bool {
+		// check if we're a hook task, probably not needed but let's take extra care
+		if thisTask.Kind() != "run-hook" {
+			return false
+		}
+		var hooksup HookSetup
+		if thisTask.Get("hook-setup", &hooksup) != nil {
+			return false
+		}
+		thisSnapName := hooksup.Snap
+		// examine all hook tasks, block thisTask if we find any other hook task affecting same snap
+		for _, t := range running {
+			if t.Kind() != "run-hook" || t.Get("hook-setup", &hooksup) != nil {
+				continue // ignore errors and continue checking remaining tasks
+			}
+			if hooksup.Snap == thisSnapName {
+				// found hook task affecting same snap, block thisTask.
+				return true
+			}
+		}
+		return false
+	})
+
 	manager := &HookManager{
 		state:      s,
 		runner:     runner,
 		repository: newRepository(),
 		contexts:   make(map[string]*Context),
+		hijackMap:  make(map[hijackKey]hijackFunc),
 	}
 
 	runner.AddHandler("run-hook", manager.doRunHook, nil)
+	// Compatibility with snapd between 2.29 and 2.30 in edge only.
+	// We generated a configure-snapd task on core refreshes and
+	// for compatibility we need to handle those.
+	runner.AddHandler("configure-snapd", func(*state.Task, *tomb.Tomb) error {
+		return nil
+	}, nil)
+
+	setupHooks(manager)
 
 	return manager, nil
 }
 
-// HookTask returns a task that will run the specified hook. Note that the
-// initial context must properly marshal and unmarshal with encoding/json.
-func HookTask(st *state.State, summary string, setup *HookSetup, contextData map[string]interface{}) *state.Task {
-	task := st.NewTask("run-hook", summary)
-	task.Set("hook-setup", setup)
-
-	// Initial data for Context.Get/Set.
-	if len(contextData) > 0 {
-		task.Set("hook-context", contextData)
-	}
-	return task
-}
-
 // Register registers a function to create Handler values whenever hooks
 // matching the provided pattern are run.
 func (m *HookManager) Register(pattern *regexp.Regexp, generator HandlerGenerator) {
 	m.repository.addHandlerGenerator(pattern, generator)
 }
 
+func (m *HookManager) KnownTaskKinds() []string {
+	return m.runner.KnownTaskKinds()
+}
+
 // Ensure implements StateManager.Ensure.
 func (m *HookManager) Ensure() error {
 	m.runner.Ensure()
@@ -133,14 +161,45 @@ func (m *HookManager) Stop() {
 	m.runner.Stop()
 }
 
-// Context obtains the context for the given context ID.
-func (m *HookManager) Context(contextID string) (*Context, error) {
+func (m *HookManager) hijacked(hookName, snapName string) hijackFunc {
+	return m.hijackMap[hijackKey{hookName, snapName}]
+}
+
+func (m *HookManager) RegisterHijack(hookName, snapName string, f hijackFunc) {
+	if _, ok := m.hijackMap[hijackKey{hookName, snapName}]; ok {
+		panic(fmt.Sprintf("hook %s for snap %s already hijacked", hookName, snapName))
+	}
+	m.hijackMap[hijackKey{hookName, snapName}] = f
+}
+
+func (m *HookManager) ephemeralContext(cookieID string) (context *Context, err error) {
+	var contexts map[string]string
+	m.state.Lock()
+	defer m.state.Unlock()
+	err = m.state.Get("snap-cookies", &contexts)
+	if err != nil {
+		return nil, fmt.Errorf("cannot get snap cookies: %v", err)
+	}
+	if snapName, ok := contexts[cookieID]; ok {
+		// create new ephemeral cookie
+		context, err = NewContext(nil, m.state, &HookSetup{Snap: snapName}, nil, cookieID)
+		return context, err
+	}
+	return nil, fmt.Errorf("invalid snap cookie requested")
+}
+
+// Context obtains the context for the given cookie ID.
+func (m *HookManager) Context(cookieID string) (*Context, error) {
 	m.contextsMutex.RLock()
 	defer m.contextsMutex.RUnlock()
 
-	context, ok := m.contexts[contextID]
+	var err error
+	context, ok := m.contexts[cookieID]
 	if !ok {
-		return nil, fmt.Errorf("no context for ID: %q", contextID)
+		context, err = m.ephemeralContext(cookieID)
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	return context, nil
@@ -155,16 +214,32 @@ func hookSetup(task *state.Task) (*HookSetup, *snapstate.SnapState, error) {
 
 	var snapst snapstate.SnapState
 	err = snapstate.Get(task.State(), hooksup.Snap, &snapst)
-	if err == state.ErrNoState {
-		return nil, nil, fmt.Errorf("cannot find %q snap", hooksup.Snap)
-	}
-	if err != nil {
+	if err != nil && err != state.ErrNoState {
 		return nil, nil, fmt.Errorf("cannot handle %q snap: %v", hooksup.Snap, err)
 	}
 
 	return &hooksup, &snapst, nil
 }
 
+// NumRunningHooks returns the number of hooks running at the moment.
+func (m *HookManager) NumRunningHooks() int {
+	return int(atomic.LoadInt32(&m.runningHooks))
+}
+
+// GracefullyWaitRunningHooks waits for currently running hooks to finish up to the default hook timeout. Returns true if there are no more running hooks on exit.
+func (m *HookManager) GracefullyWaitRunningHooks() bool {
+	toutC := time.After(defaultHookTimeout)
+	doWait := true
+	for m.NumRunningHooks() > 0 && doWait {
+		select {
+		case <-time.After(1 * time.Second):
+		case <-toutC:
+			doWait = false
+		}
+	}
+	return m.NumRunningHooks() == 0
+}
+
 // doRunHook actually runs the hook that was requested.
 //
 // Note that this method is synchronous, as the task is already running in a
@@ -177,17 +252,38 @@ func (m *HookManager) doRunHook(task *state.Task, tomb *tomb.Tomb) error {
 		return err
 	}
 
-	info, err := snapst.CurrentInfo()
-	if err != nil {
-		return fmt.Errorf("cannot read %q snap details: %v", hooksup.Snap, err)
+	mustHijack := m.hijacked(hooksup.Hook, hooksup.Snap) != nil
+	hookExists := false
+	if !mustHijack {
+		// not hijacked, snap must be installed
+		if !snapst.IsInstalled() {
+			return fmt.Errorf("cannot find %q snap", hooksup.Snap)
+		}
+
+		info, err := snapst.CurrentInfo()
+		if err != nil {
+			return fmt.Errorf("cannot read %q snap details: %v", hooksup.Snap, err)
+		}
+
+		hookExists = info.Hooks[hooksup.Hook] != nil
+		if !hookExists && !hooksup.Optional {
+			return fmt.Errorf("snap %q has no %q hook", hooksup.Snap, hooksup.Hook)
+		}
 	}
 
-	hookExists := info.Hooks[hooksup.Hook] != nil
-	if !hookExists && !hooksup.Optional {
-		return fmt.Errorf("snap %q has no %q hook", hooksup.Snap, hooksup.Hook)
+	if hookExists || mustHijack {
+		// we will run something, not a noop
+		if task.State().Restarting() {
+			// don't start running a hook if we are restarting
+			return &state.Retry{}
+		}
+
+		// keep count of running hooks
+		atomic.AddInt32(&m.runningHooks, 1)
+		defer atomic.AddInt32(&m.runningHooks, -1)
 	}
 
-	context, err := NewContext(task, hooksup, nil)
+	context, err := NewContext(task, task.State(), hooksup, nil, "")
 	if err != nil {
 		return err
 	}
@@ -198,12 +294,17 @@ func (m *HookManager) doRunHook(task *state.Task, tomb *tomb.Tomb) error {
 	handlers := m.repository.generateHandlers(context)
 	handlersCount := len(handlers)
 	if handlersCount == 0 {
+		// Do not report error if hook handler doesn't exist as long as the hook is optional.
+		// This is to avoid issues when downgrading to an old core snap that doesn't know about
+		// particular hook type and a task for it exists (e.g. "post-refresh" hook).
+		if hooksup.Optional {
+			return nil
+		}
 		return fmt.Errorf("internal error: no registered handlers for hook %q", hooksup.Hook)
 	}
 	if handlersCount > 1 {
 		return fmt.Errorf("internal error: %d handlers registered for hook %q, expected 1", handlersCount, hooksup.Hook)
 	}
-
 	context.handler = handlers[0]
 
 	contextID := context.ID()
@@ -221,24 +322,28 @@ func (m *HookManager) doRunHook(task *state.Task, tomb *tomb.Tomb) error {
 		return err
 	}
 
-	if hookExists {
-		output, err := runHook(context, tomb)
-		if err != nil {
-			if hooksup.TrackError {
-				trackHookError(context, output, err)
+	// some hooks get hijacked, e.g. the core configuration
+	var output []byte
+	if f := m.hijacked(hooksup.Hook, hooksup.Snap); f != nil {
+		err = f(context)
+	} else if hookExists {
+		output, err = runHook(context, tomb)
+	}
+	if err != nil {
+		if hooksup.TrackError {
+			trackHookError(context, output, err)
+		}
+		err = osutil.OutputErr(output, err)
+		if hooksup.IgnoreError {
+			task.State().Lock()
+			task.Errorf("ignoring failure in hook %q: %v", hooksup.Hook, err)
+			task.State().Unlock()
+		} else {
+			if handlerErr := context.Handler().Error(err); handlerErr != nil {
+				return handlerErr
 			}
-			err = osutil.OutputErr(output, err)
-			if hooksup.IgnoreError {
-				task.State().Lock()
-				task.Errorf("ignoring failure in hook %q: %v", hooksup.Hook, err)
-				task.State().Unlock()
-			} else {
-				if handlerErr := context.Handler().Error(err); handlerErr != nil {
-					return handlerErr
-				}
 
-				return fmt.Errorf("run hook %q: %v", hooksup.Hook, err)
-			}
+			return fmt.Errorf("run hook %q: %v", hooksup.Hook, err)
 		}
 	}
 
@@ -281,6 +386,7 @@ func snapCmd() string {
 
 	exe, err := osReadlink("/proc/self/exe")
 	if err != nil {
+		logger.Noticef("cannot read /proc/self/exe: %v, using default snap command", err)
 		return snapCmd
 	}
 	if !strings.HasPrefix(exe, dirs.SnapMountDir) {
@@ -292,88 +398,25 @@ func snapCmd() string {
 	return filepath.Join(filepath.Dir(exe), "../../bin/snap")
 }
 
-var syscallKill = syscall.Kill
-var cmdWaitTimeout = 5 * time.Second
-
-func killemAll(cmd *exec.Cmd) error {
-	pgid, err := syscall.Getpgid(cmd.Process.Pid)
-	if err != nil {
-		return err
-	}
-	if pgid == 1 {
-		return fmt.Errorf("cannot kill pgid 1")
-	}
-	return syscallKill(-pgid, 9)
-}
+var defaultHookTimeout = 10 * time.Minute
 
 func runHookAndWait(snapName string, revision snap.Revision, hookName, hookContext string, timeout time.Duration, tomb *tomb.Tomb) ([]byte, error) {
-	command := exec.Command(snapCmd(), "run", "--hook", hookName, "-r", revision.String(), snapName)
-
-	// setup a process group for the command so that we can kill parent
-	// and children on e.g. timeout
-	command.SysProcAttr = &syscall.SysProcAttr{Setpgid: true}
-
-	// Make sure the hook has its context defined so it can communicate via the
-	// REST API.
-	command.Env = append(os.Environ(), fmt.Sprintf("SNAP_CONTEXT=%s", hookContext))
-
-	// Make sure we can obtain stdout and stderror. Same buffer so they're
-	// combined.
-	buffer := bytes.NewBuffer(nil)
-	command.Stdout = buffer
-	command.Stderr = buffer
-
-	// Actually run the hook.
-	if err := command.Start(); err != nil {
-		return nil, err
+	argv := []string{snapCmd(), "run", "--hook", hookName, "-r", revision.String(), snapName}
+	if timeout == 0 {
+		timeout = defaultHookTimeout
 	}
 
-	// add timeout handling
-	var killTimerCh <-chan time.Time
-	if timeout > 0 {
-		killTimerCh = time.After(timeout)
+	env := []string{
+		// Make sure the hook has its context defined so it can
+		// communicate via the REST API.
+		fmt.Sprintf("SNAP_COOKIE=%s", hookContext),
+		// Set SNAP_CONTEXT too for compatibility with old snapctl
+		// binary when transitioning to a new core - otherwise configure
+		// hook would fail during transition.
+		fmt.Sprintf("SNAP_CONTEXT=%s", hookContext),
 	}
 
-	hookCompleted := make(chan struct{})
-	var hookError error
-	go func() {
-		// Wait for hook to complete
-		hookError = command.Wait()
-		close(hookCompleted)
-	}()
-
-	var abortOrTimeoutError error
-	select {
-	case <-hookCompleted:
-		// Hook completed; it may or may not have been successful.
-		return buffer.Bytes(), hookError
-	case <-tomb.Dying():
-		// Hook was aborted, process will get killed below
-		abortOrTimeoutError = fmt.Errorf("hook aborted")
-	case <-killTimerCh:
-		// Max timeout reached, process will get killed below
-		abortOrTimeoutError = fmt.Errorf("exceeded maximum runtime of %s", timeout)
-	}
-
-	// select above exited which means that aborted or killTimeout
-	// was reached. Kill the command and wait for command.Wait()
-	// to clean it up (but limit the wait with the cmdWaitTimer)
-	if err := killemAll(command); err != nil {
-		return nil, fmt.Errorf("cannot abort hook: %s", err)
-	}
-	select {
-	case <-time.After(cmdWaitTimeout):
-		// cmdWaitTimeout was reached, i.e. command.Wait() did not
-		// finish in a reasonable amount of time, we can not use
-		// buffer in this case so return without it.
-		return nil, fmt.Errorf("%v, but did not stop", abortOrTimeoutError)
-	case <-hookCompleted:
-		// cmd.Wait came back from waiting the killed process
-		break
-	}
-	fmt.Fprintf(buffer, "\n<%s>", abortOrTimeoutError)
-
-	return buffer.Bytes(), abortOrTimeoutError
+	return osutil.RunAndWait(argv, env, timeout, tomb)
 }
 
 var errtrackerReport = errtracker.Report
