@@ -21,16 +21,19 @@ package hookstate_test
 
 import (
 	"encoding/json"
+	"fmt"
 	"os"
 	"path/filepath"
 	"regexp"
-	"syscall"
+	"sort"
+	"sync/atomic"
 	"testing"
 	"time"
 
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/hookstate"
 	"github.com/snapcore/snapd/overlord/hookstate/hooktest"
 	"github.com/snapcore/snapd/overlord/snapstate"
@@ -45,6 +48,7 @@ func TestHookManager(t *testing.T) { TestingT(t) }
 type hookManagerSuite struct {
 	testutil.BaseTest
 
+	o           *overlord.Overlord
 	state       *state.State
 	manager     *hookstate.HookManager
 	context     *hookstate.Context
@@ -63,16 +67,33 @@ hooks:
     configure:
     prepare-device:
 `
-var snapContents = ""
+
+var snapYaml1 = `
+name: test-snap-1
+version: 1.0
+hooks:
+    prepare-device:
+`
+
+var snapYaml2 = `
+name: test-snap-2
+version: 1.0
+hooks:
+    prepare-device:
+`
 
 func (s *hookManagerSuite) SetUpTest(c *C) {
 	s.BaseTest.SetUpTest(c)
 
 	dirs.SetRootDir(c.MkDir())
-	s.state = state.New(nil)
+	s.o = overlord.Mock()
+	s.state = s.o.State()
 	manager, err := hookstate.Manager(s.state)
 	c.Assert(err, IsNil)
 	s.manager = manager
+	s.o.AddManager(s.manager)
+
+	s.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
 
 	hooksup := &hookstate.HookSetup{
 		Snap:     "test-snap",
@@ -92,7 +113,7 @@ func (s *hookManagerSuite) SetUpTest(c *C) {
 	s.change.AddTask(s.task)
 
 	sideInfo := &snap.SideInfo{RealName: "test-snap", SnapID: "some-snap-id", Revision: snap.R(1)}
-	snaptest.MockSnap(c, snapYaml, snapContents, sideInfo)
+	snaptest.MockSnap(c, snapYaml, sideInfo)
 	snapstate.Set(s.state, "test-snap", &snapstate.SnapState{
 		Active:   true,
 		Sequence: []*snap.SideInfo{sideInfo},
@@ -101,6 +122,7 @@ func (s *hookManagerSuite) SetUpTest(c *C) {
 	s.state.Unlock()
 
 	s.command = testutil.MockCommand(c, "snap", "")
+	s.AddCleanup(s.command.Restore)
 
 	s.context = nil
 	s.mockHandler = hooktest.NewMockHandler()
@@ -120,11 +142,22 @@ func (s *hookManagerSuite) TearDownTest(c *C) {
 	dirs.SetRootDir("")
 }
 
+func (s *hookManagerSuite) settle(c *C) {
+	err := s.o.Settle(5 * time.Second)
+	c.Assert(err, IsNil)
+}
+
 func (s *hookManagerSuite) TestSmoke(c *C) {
 	s.manager.Ensure()
 	s.manager.Wait()
 }
 
+func (s *hookManagerSuite) TestKnownTaskKinds(c *C) {
+	kinds := s.manager.KnownTaskKinds()
+	sort.Strings(kinds)
+	c.Assert(kinds, DeepEquals, []string{"configure-snapd", "run-hook"})
+}
+
 func (s *hookManagerSuite) TestHookSetupJsonMarshal(c *C) {
 	hookSetup := &hookstate.HookSetup{Snap: "snap-name", Revision: snap.R(1), Hook: "hook-name"}
 	out, err := json.Marshal(hookSetup)
@@ -166,7 +199,20 @@ func (s *hookManagerSuite) TestHookTask(c *C) {
 }
 
 func (s *hookManagerSuite) TestHookTaskEnsure(c *C) {
+	didRun := make(chan bool)
+	s.mockHandler.BeforeCallback = func() {
+		c.Check(s.manager.NumRunningHooks(), Equals, 1)
+		go func() {
+			didRun <- s.manager.GracefullyWaitRunningHooks()
+		}()
+	}
 	s.manager.Ensure()
+	select {
+	case ok := <-didRun:
+		c.Check(ok, Equals, true)
+	case <-time.After(5 * time.Second):
+		c.Fatal("hook run should have been done by now")
+	}
 	s.manager.Wait()
 
 	s.state.Lock()
@@ -188,6 +234,116 @@ func (s *hookManagerSuite) TestHookTaskEnsure(c *C) {
 	c.Check(s.task.Kind(), Equals, "run-hook")
 	c.Check(s.task.Status(), Equals, state.DoneStatus)
 	c.Check(s.change.Status(), Equals, state.DoneStatus)
+
+	c.Check(s.manager.NumRunningHooks(), Equals, 0)
+}
+
+func (s *hookManagerSuite) TestHookTaskEnsureRestarting(c *C) {
+	// we do no start new hooks runs if we are restarting
+	s.state.RequestRestart(state.RestartDaemon)
+
+	s.manager.Ensure()
+	s.manager.Wait()
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Assert(s.context, IsNil)
+
+	c.Check(s.command.Calls(), HasLen, 0)
+
+	c.Check(s.mockHandler.BeforeCalled, Equals, false)
+	c.Check(s.mockHandler.DoneCalled, Equals, false)
+	c.Check(s.mockHandler.ErrorCalled, Equals, false)
+
+	c.Check(s.task.Status(), Equals, state.DoingStatus)
+	c.Check(s.change.Status(), Equals, state.DoingStatus)
+
+	c.Check(s.manager.NumRunningHooks(), Equals, 0)
+}
+
+func (s *hookManagerSuite) TestHookSnapMissing(c *C) {
+	s.state.Lock()
+	snapstate.Set(s.state, "test-snap", nil)
+	s.state.Unlock()
+
+	s.manager.Ensure()
+	s.manager.Wait()
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Check(s.change.Err(), ErrorMatches, `(?s).*cannot find "test-snap" snap.*`)
+}
+
+func (s *hookManagerSuite) TestHookHijackingHappy(c *C) {
+	// this works even if test-snap is not present
+	s.state.Lock()
+	snapstate.Set(s.state, "test-snap", nil)
+	s.state.Unlock()
+
+	var hijackedContext *hookstate.Context
+	s.manager.RegisterHijack("configure", "test-snap", func(ctx *hookstate.Context) error {
+		hijackedContext = ctx
+		return nil
+	})
+
+	s.manager.Ensure()
+	s.manager.Wait()
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Check(hijackedContext, DeepEquals, s.context)
+	c.Check(s.command.Calls(), HasLen, 0)
+
+	c.Assert(s.context, NotNil)
+	c.Check(s.context.SnapName(), Equals, "test-snap")
+	c.Check(s.context.SnapRevision(), Equals, snap.R(1))
+	c.Check(s.context.HookName(), Equals, "configure")
+
+	c.Check(s.mockHandler.BeforeCalled, Equals, true)
+	c.Check(s.mockHandler.DoneCalled, Equals, true)
+	c.Check(s.mockHandler.ErrorCalled, Equals, false)
+
+	c.Check(s.task.Kind(), Equals, "run-hook")
+	c.Check(s.task.Status(), Equals, state.DoneStatus)
+	c.Check(s.change.Status(), Equals, state.DoneStatus)
+}
+
+func (s *hookManagerSuite) TestHookHijackingUnHappy(c *C) {
+	s.manager.RegisterHijack("configure", "test-snap", func(ctx *hookstate.Context) error {
+		return fmt.Errorf("not-happy-at-all")
+	})
+
+	s.manager.Ensure()
+	s.manager.Wait()
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Check(s.command.Calls(), HasLen, 0)
+
+	c.Assert(s.context, NotNil)
+	c.Check(s.context.SnapName(), Equals, "test-snap")
+	c.Check(s.context.SnapRevision(), Equals, snap.R(1))
+	c.Check(s.context.HookName(), Equals, "configure")
+
+	c.Check(s.mockHandler.BeforeCalled, Equals, true)
+	c.Check(s.mockHandler.DoneCalled, Equals, false)
+	c.Check(s.mockHandler.ErrorCalled, Equals, true)
+
+	c.Check(s.task.Kind(), Equals, "run-hook")
+	c.Check(s.task.Status(), Equals, state.ErrorStatus)
+	c.Check(s.change.Status(), Equals, state.ErrorStatus)
+}
+
+func (s *hookManagerSuite) TestHookHijackingVeryUnHappy(c *C) {
+	f := func(ctx *hookstate.Context) error {
+		return nil
+	}
+	s.manager.RegisterHijack("configure", "test-snap", f)
+	c.Check(func() { s.manager.RegisterHijack("configure", "test-snap", f) }, PanicMatches, "hook configure for snap test-snap already hijacked")
 }
 
 func (s *hookManagerSuite) TestHookTaskInitializesContext(c *C) {
@@ -204,8 +360,9 @@ func (s *hookManagerSuite) TestHookTaskInitializesContext(c *C) {
 
 func (s *hookManagerSuite) TestHookTaskHandlesHookError(c *C) {
 	// Force the snap command to exit 1, and print something to stderr
-	s.command = testutil.MockCommand(
+	cmd := testutil.MockCommand(
 		c, "snap", ">&2 echo 'hook failed at user request'; exit 1")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	s.manager.Wait()
@@ -221,6 +378,8 @@ func (s *hookManagerSuite) TestHookTaskHandlesHookError(c *C) {
 	c.Check(s.task.Status(), Equals, state.ErrorStatus)
 	c.Check(s.change.Status(), Equals, state.ErrorStatus)
 	checkTaskLogContains(c, s.task, ".*failed at user request.*")
+
+	c.Check(s.manager.NumRunningHooks(), Equals, 0)
 }
 
 func (s *hookManagerSuite) TestHookTaskHandleIgnoreErrorWorks(c *C) {
@@ -232,8 +391,9 @@ func (s *hookManagerSuite) TestHookTaskHandleIgnoreErrorWorks(c *C) {
 	s.state.Unlock()
 
 	// Force the snap command to exit 1, and print something to stderr
-	s.command = testutil.MockCommand(
+	cmd := testutil.MockCommand(
 		c, "snap", ">&2 echo 'hook failed at user request'; exit 1")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	s.manager.Wait()
@@ -261,7 +421,8 @@ func (s *hookManagerSuite) TestHookTaskEnforcesTimeout(c *C) {
 	s.state.Unlock()
 
 	// Force the snap command to hang
-	s.command = testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
+	cmd := testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	completed := make(chan struct{})
@@ -289,38 +450,16 @@ func (s *hookManagerSuite) TestHookTaskEnforcesTimeout(c *C) {
 	checkTaskLogContains(c, s.task, `.*exceeded maximum runtime of 200ms`)
 }
 
-func (s *hookManagerSuite) TestHookTaskEnforcesMaxWaitTime(c *C) {
-	var hooksup hookstate.HookSetup
-
-	s.state.Lock()
-	s.task.Get("hook-setup", &hooksup)
-	hooksup.Timeout = time.Duration(200 * time.Millisecond)
-	s.task.Set("hook-setup", &hooksup)
-	s.state.Unlock()
+func (s *hookManagerSuite) TestHookTaskEnforcesDefaultTimeout(c *C) {
+	restore := hookstate.MockDefaultHookTimeout(150 * time.Millisecond)
+	defer restore()
 
 	// Force the snap command to hang
-	s.command = testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
-
-	// simulate a process that can not be killed
-	restore := hookstate.MockSyscallKill(func(int, syscall.Signal) error {
-		return nil
-	})
-	defer restore()
-
-	restore = hookstate.MockCmdWaitTimeout(100 * time.Millisecond)
-	defer restore()
+	cmd := testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
-	completed := make(chan struct{})
-	go func() {
-		s.manager.Wait()
-		close(completed)
-	}()
-
-	s.state.Lock()
-	s.state.Unlock()
-	s.manager.Ensure()
-	<-completed
+	s.manager.Wait()
 
 	s.state.Lock()
 	defer s.state.Unlock()
@@ -328,12 +467,12 @@ func (s *hookManagerSuite) TestHookTaskEnforcesMaxWaitTime(c *C) {
 	c.Check(s.mockHandler.BeforeCalled, Equals, true)
 	c.Check(s.mockHandler.DoneCalled, Equals, false)
 	c.Check(s.mockHandler.ErrorCalled, Equals, true)
-	c.Check(s.mockHandler.Err, ErrorMatches, `.*exceeded maximum runtime of 200ms, but did not stop`)
+	c.Check(s.mockHandler.Err, ErrorMatches, `.*exceeded maximum runtime of 150ms.*`)
 
 	c.Check(s.task.Kind(), Equals, "run-hook")
 	c.Check(s.task.Status(), Equals, state.ErrorStatus)
 	c.Check(s.change.Status(), Equals, state.ErrorStatus)
-	checkTaskLogContains(c, s.task, `.*exceeded maximum runtime of 200ms, but did not stop`)
+	checkTaskLogContains(c, s.task, `.*exceeded maximum runtime of 150ms`)
 }
 
 func (s *hookManagerSuite) TestHookTaskEnforcedTimeoutWithIgnoreError(c *C) {
@@ -347,7 +486,8 @@ func (s *hookManagerSuite) TestHookTaskEnforcedTimeoutWithIgnoreError(c *C) {
 	s.state.Unlock()
 
 	// Force the snap command to hang
-	s.command = testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
+	cmd := testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	completed := make(chan struct{})
@@ -377,7 +517,8 @@ func (s *hookManagerSuite) TestHookTaskEnforcedTimeoutWithIgnoreError(c *C) {
 
 func (s *hookManagerSuite) TestHookTaskCanKillHook(c *C) {
 	// Force the snap command to hang
-	s.command = testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
+	cmd := testutil.MockCommand(c, "snap", "while true; do sleep 1; done")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	completed := make(chan struct{})
@@ -400,19 +541,22 @@ func (s *hookManagerSuite) TestHookTaskCanKillHook(c *C) {
 	c.Check(s.mockHandler.BeforeCalled, Equals, true)
 	c.Check(s.mockHandler.DoneCalled, Equals, false)
 	c.Check(s.mockHandler.ErrorCalled, Equals, true)
-	c.Check(s.mockHandler.Err, ErrorMatches, ".*hook aborted.*")
+	c.Check(s.mockHandler.Err, ErrorMatches, "<aborted>")
 
 	c.Check(s.task.Kind(), Equals, "run-hook")
 	c.Check(s.task.Status(), Equals, state.ErrorStatus)
 	c.Check(s.change.Status(), Equals, state.ErrorStatus)
-	checkTaskLogContains(c, s.task, `.*hook aborted.*`)
+	checkTaskLogContains(c, s.task, `run hook "[^"]*": <aborted>`)
+
+	c.Check(s.manager.NumRunningHooks(), Equals, 0)
 }
 
 func (s *hookManagerSuite) TestHookTaskCorrectlyIncludesContext(c *C) {
 	// Force the snap command to exit with a failure and print to stderr so we
 	// can catch and verify it.
-	s.command = testutil.MockCommand(
-		c, "snap", ">&2 echo \"SNAP_CONTEXT=$SNAP_CONTEXT\"; exit 1")
+	cmd := testutil.MockCommand(
+		c, "snap", ">&2 echo \"SNAP_COOKIE=$SNAP_COOKIE\"; exit 1")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	s.manager.Wait()
@@ -427,7 +571,7 @@ func (s *hookManagerSuite) TestHookTaskCorrectlyIncludesContext(c *C) {
 	c.Check(s.task.Kind(), Equals, "run-hook")
 	c.Check(s.task.Status(), Equals, state.ErrorStatus)
 	c.Check(s.change.Status(), Equals, state.ErrorStatus)
-	checkTaskLogContains(c, s.task, `.*SNAP_CONTEXT=\S+`)
+	checkTaskLogContains(c, s.task, `.*SNAP_COOKIE=\S+`)
 }
 
 func (s *hookManagerSuite) TestHookTaskHandlerBeforeError(c *C) {
@@ -472,7 +616,8 @@ func (s *hookManagerSuite) TestHookTaskHandlerErrorError(c *C) {
 	s.mockHandler.ErrorError = true
 
 	// Force the snap command to simply exit 1, so the handler Error() runs
-	s.command = testutil.MockCommand(c, "snap", "exit 1")
+	cmd := testutil.MockCommand(c, "snap", "exit 1")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	s.manager.Wait()
@@ -585,6 +730,31 @@ func (s *hookManagerSuite) TestHookWithoutHookOptional(c *C) {
 	c.Logf("Task log:\n%s\n", s.task.Log())
 }
 
+func (s *hookManagerSuite) TestOptionalHookWithMissingHandler(c *C) {
+	hooksup := &hookstate.HookSetup{
+		Snap:     "test-snap",
+		Hook:     "missing-hook-and-no-handler",
+		Optional: true,
+	}
+	s.state.Lock()
+	s.task.Set("hook-setup", hooksup)
+	s.state.Unlock()
+
+	s.manager.Ensure()
+	s.manager.Wait()
+
+	c.Check(s.command.Calls(), IsNil)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Check(s.task.Kind(), Equals, "run-hook")
+	c.Check(s.task.Status(), Equals, state.DoneStatus)
+	c.Check(s.change.Status(), Equals, state.DoneStatus)
+
+	c.Logf("Task log:\n%s\n", s.task.Log())
+}
+
 func checkTaskLogContains(c *C, task *state.Task, pattern string) {
 	exp := regexp.MustCompile(pattern)
 	found := false
@@ -601,7 +771,8 @@ func (s *hookManagerSuite) TestHookTaskRunsRightSnapCmd(c *C) {
 	coreSnapCmdPath := filepath.Join(dirs.SnapMountDir, "core/12/usr/bin/snap")
 	err := os.MkdirAll(filepath.Dir(coreSnapCmdPath), 0755)
 	c.Assert(err, IsNil)
-	s.command = testutil.MockCommand(c, coreSnapCmdPath, "")
+	cmd := testutil.MockCommand(c, coreSnapCmdPath, "")
+	defer cmd.Restore()
 
 	r := hookstate.MockReadlink(func(p string) (string, error) {
 		c.Assert(p, Equals, "/proc/self/exe")
@@ -616,7 +787,7 @@ func (s *hookManagerSuite) TestHookTaskRunsRightSnapCmd(c *C) {
 	defer s.state.Unlock()
 
 	c.Assert(s.context, NotNil, Commentf("Expected handler generator to be called with a valid context"))
-	c.Check(s.command.Calls(), DeepEquals, [][]string{{
+	c.Check(cmd.Calls(), DeepEquals, [][]string{{
 		"snap", "run", "--hook", "configure", "-r", "1", "test-snap",
 	}})
 
@@ -641,8 +812,9 @@ func (s *hookManagerSuite) TestHookTaskHandlerReportsErrorIfRequested(c *C) {
 	})
 
 	// Force the snap command to exit 1, and print something to stderr
-	s.command = testutil.MockCommand(
+	cmd := testutil.MockCommand(
 		c, "snap", ">&2 echo 'hook failed at user request'; exit 1")
+	defer cmd.Restore()
 
 	s.manager.Ensure()
 	s.manager.Wait()
@@ -652,3 +824,209 @@ func (s *hookManagerSuite) TestHookTaskHandlerReportsErrorIfRequested(c *C) {
 
 	c.Check(errtrackerCalled, Equals, true)
 }
+
+func (s *hookManagerSuite) TestHookTasksForSameSnapAreSerialized(c *C) {
+	var Executing int32
+	var TotalExecutions int32
+
+	s.mockHandler.BeforeCallback = func() {
+		executing := atomic.AddInt32(&Executing, 1)
+		if executing != 1 {
+			panic(fmt.Sprintf("More than one handler executed: %d", executing))
+		}
+	}
+
+	s.mockHandler.DoneCallback = func() {
+		executing := atomic.AddInt32(&Executing, -1)
+		if executing != 0 {
+			panic(fmt.Sprintf("More than one handler executed: %d", executing))
+		}
+		atomic.AddInt32(&TotalExecutions, 1)
+	}
+
+	hooksup := &hookstate.HookSetup{
+		Snap:     "test-snap",
+		Hook:     "configure",
+		Revision: snap.R(1),
+	}
+
+	s.state.Lock()
+
+	var tasks []*state.Task
+	for i := 0; i < 20; i++ {
+		task := hookstate.HookTask(s.state, "test summary", hooksup, nil)
+		c.Assert(s.task, NotNil)
+		change := s.state.NewChange("kind", "summary")
+		change.AddTask(task)
+		tasks = append(tasks, task)
+	}
+	s.state.Unlock()
+
+	s.settle(c)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Check(s.task.Kind(), Equals, "run-hook")
+	c.Check(s.task.Status(), Equals, state.DoneStatus)
+	c.Check(s.change.Status(), Equals, state.DoneStatus)
+
+	for i := 0; i < len(tasks); i++ {
+		c.Check(tasks[i].Kind(), Equals, "run-hook")
+		c.Check(tasks[i].Status(), Equals, state.DoneStatus)
+	}
+	c.Assert(atomic.LoadInt32(&TotalExecutions), Equals, int32(1+len(tasks)))
+	c.Assert(atomic.LoadInt32(&Executing), Equals, int32(0))
+}
+
+type MockConcurrentHandler struct {
+	onDone func()
+}
+
+func (h *MockConcurrentHandler) Before() error {
+	return nil
+}
+
+func (h *MockConcurrentHandler) Done() error {
+	h.onDone()
+	return nil
+}
+
+func (h *MockConcurrentHandler) Error(err error) error {
+	return nil
+}
+
+func NewMockConcurrentHandler(onDone func()) *MockConcurrentHandler {
+	return &MockConcurrentHandler{onDone: onDone}
+}
+
+func (s *hookManagerSuite) TestHookTasksForDifferentSnapsRunConcurrently(c *C) {
+	hooksup1 := &hookstate.HookSetup{
+		Snap:     "test-snap-1",
+		Hook:     "prepare-device",
+		Revision: snap.R(1),
+	}
+	hooksup2 := &hookstate.HookSetup{
+		Snap:     "test-snap-2",
+		Hook:     "prepare-device",
+		Revision: snap.R(1),
+	}
+
+	s.state.Lock()
+
+	sideInfo := &snap.SideInfo{RealName: "test-snap-1", SnapID: "some-snap-id1", Revision: snap.R(1)}
+	info := snaptest.MockSnap(c, snapYaml1, sideInfo)
+	c.Assert(info.Hooks, HasLen, 1)
+	snapstate.Set(s.state, "test-snap-1", &snapstate.SnapState{
+		Active:   true,
+		Sequence: []*snap.SideInfo{sideInfo},
+		Current:  snap.R(1),
+	})
+
+	sideInfo = &snap.SideInfo{RealName: "test-snap-2", SnapID: "some-snap-id2", Revision: snap.R(1)}
+	snaptest.MockSnap(c, snapYaml2, sideInfo)
+	snapstate.Set(s.state, "test-snap-2", &snapstate.SnapState{
+		Active:   true,
+		Sequence: []*snap.SideInfo{sideInfo},
+		Current:  snap.R(1),
+	})
+
+	var testSnap1HookCalls, testSnap2HookCalls int
+	ch := make(chan struct{})
+	mockHandler1 := NewMockConcurrentHandler(func() {
+		ch <- struct{}{}
+		testSnap1HookCalls++
+	})
+	mockHandler2 := NewMockConcurrentHandler(func() {
+		<-ch
+		testSnap2HookCalls++
+	})
+	s.manager.Register(regexp.MustCompile("prepare-device"), func(context *hookstate.Context) hookstate.Handler {
+		if context.SnapName() == "test-snap-1" {
+			return mockHandler1
+		}
+		if context.SnapName() == "test-snap-2" {
+			return mockHandler2
+		}
+		c.Fatalf("unknown snap: %s", context.SnapName())
+		return nil
+	})
+
+	task1 := hookstate.HookTask(s.state, "test summary", hooksup1, nil)
+	c.Assert(task1, NotNil)
+	change1 := s.state.NewChange("kind", "summary")
+	change1.AddTask(task1)
+
+	task2 := hookstate.HookTask(s.state, "test summary", hooksup2, nil)
+	c.Assert(task2, NotNil)
+	change2 := s.state.NewChange("kind", "summary")
+	change2.AddTask(task2)
+
+	s.state.Unlock()
+
+	s.settle(c)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Check(task1.Status(), Equals, state.DoneStatus)
+	c.Check(change1.Status(), Equals, state.DoneStatus)
+	c.Check(task2.Status(), Equals, state.DoneStatus)
+	c.Check(change2.Status(), Equals, state.DoneStatus)
+	c.Assert(testSnap1HookCalls, Equals, 1)
+	c.Assert(testSnap2HookCalls, Equals, 1)
+}
+
+func (s *hookManagerSuite) TestCompatForConfigureSnapd(c *C) {
+	st := s.state
+
+	st.Lock()
+	defer st.Unlock()
+
+	task := st.NewTask("configure-snapd", "Snapd between 2.29 and 2.30 in edge insertd those tasks")
+	chg := st.NewChange("configure", "configure snapd")
+	chg.AddTask(task)
+
+	st.Unlock()
+	s.manager.Ensure()
+	s.manager.Wait()
+	st.Lock()
+
+	c.Check(chg.Status(), Equals, state.DoneStatus)
+	c.Check(task.Status(), Equals, state.DoneStatus)
+}
+
+func (s *hookManagerSuite) TestGracefullyWaitRunningHooksTimeout(c *C) {
+	restore := hookstate.MockDefaultHookTimeout(100 * time.Millisecond)
+	defer restore()
+
+	// this works even if test-snap is not present
+	s.state.Lock()
+	snapstate.Set(s.state, "test-snap", nil)
+	s.state.Unlock()
+
+	quit := make(chan struct{})
+	defer func() {
+		quit <- struct{}{}
+	}()
+	didRun := make(chan bool)
+	s.mockHandler.BeforeCallback = func() {
+		c.Check(s.manager.NumRunningHooks(), Equals, 1)
+		go func() {
+			didRun <- s.manager.GracefullyWaitRunningHooks()
+		}()
+	}
+
+	s.manager.RegisterHijack("configure", "test-snap", func(ctx *hookstate.Context) error {
+		<-quit
+		return nil
+	})
+
+	s.manager.Ensure()
+	select {
+	case noPending := <-didRun:
+		c.Check(noPending, Equals, false)
+	case <-time.After(2 * time.Second):
+		c.Fatal("timeout should have expired")
+	}
+}
