@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2018 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -19,18 +19,30 @@ package ifacestate
 
 import (
 	"errors"
+	"time"
 
 	"gopkg.in/tomb.v2"
 
 	"github.com/snapcore/snapd/overlord/state"
 )
 
+var (
+	AddImplicitSlots          = addImplicitSlots
+	SnapsWithSecurityProfiles = snapsWithSecurityProfiles
+)
+
 // AddForeignTaskHandlers registers handlers for tasks handled outside of the
 // InterfaceManager.
-func (m *InterfaceManager) AddForeignTaskHandlers() {
+func AddForeignTaskHandlers(m *InterfaceManager) {
 	// Add handler to test full aborting of changes
 	erroringHandler := func(task *state.Task, _ *tomb.Tomb) error {
 		return errors.New("error out")
 	}
 	m.runner.AddHandler("error-trigger", erroringHandler, nil)
 }
+
+func MockContentLinkRetryTimeout(d time.Duration) (restore func()) {
+	old := contentLinkRetryTimeout
+	contentLinkRetryTimeout = d
+	return func() { contentLinkRetryTimeout = old }
+}
