@@ -22,6 +22,7 @@ package ifacestate
 import (
 	"fmt"
 	"sort"
+	"strings"
 	"time"
 
 	"gopkg.in/tomb.v2"
@@ -63,7 +64,7 @@ func (m *InterfaceManager) setupAffectedSnaps(task *state.Task, affectingSnap st
 		if err != nil {
 			return err
 		}
-		snap.AddImplicitSlots(affectedSnapInfo)
+		addImplicitSlots(affectedSnapInfo)
 		opts := confinementOptions(snapst.Flags)
 		if err := m.setupSnapSecurity(task, affectedSnapInfo, opts); err != nil {
 			return err
@@ -118,6 +119,27 @@ func (m *InterfaceManager) doSetupProfiles(task *state.Task, tomb *tomb.Tomb) er
 				return fmt.Errorf("cannot finish core installation, there was a rollback across reboot")
 			}
 		}
+
+		// Compatibility with old snapd: check if we have auto-connect task and if not, inject it after self (setup-profiles).
+		// Inject it for core after the 2nd setup-profiles - same placement as done in doInstall.
+		// In the older snapd versions interfaces were auto-connected as part of setupProfilesForSnap.
+		var hasAutoConnect bool
+		for _, t := range task.Change().Tasks() {
+			if t.Kind() == "auto-connect" {
+				otherSnapsup, err := snapstate.TaskSnapSetup(t)
+				if err != nil {
+					return err
+				}
+				// Check if this is auto-connect task for same snap
+				if snapsup.Name() == otherSnapsup.Name() {
+					hasAutoConnect = true
+					break
+				}
+			}
+		}
+		if !hasAutoConnect {
+			snapstate.InjectAutoConnect(task, snapsup)
+		}
 	}
 
 	opts := confinementOptions(snapsup.Flags)
@@ -125,7 +147,7 @@ func (m *InterfaceManager) doSetupProfiles(task *state.Task, tomb *tomb.Tomb) er
 }
 
 func (m *InterfaceManager) setupProfilesForSnap(task *state.Task, _ *tomb.Tomb, snapInfo *snap.Info, opts interfaces.ConfinementOptions) error {
-	snap.AddImplicitSlots(snapInfo)
+	addImplicitSlots(snapInfo)
 	snapName := snapInfo.Name()
 
 	// The snap may have been updated so perform the following operation to
@@ -148,18 +170,13 @@ func (m *InterfaceManager) setupProfilesForSnap(task *state.Task, _ *tomb.Tomb,
 		return err
 	}
 	if err := m.repo.AddSnap(snapInfo); err != nil {
-		if _, ok := err.(*interfaces.BadInterfacesError); ok {
-			task.Logf("%s", err)
-		} else {
-			return err
-		}
-	}
-	if err := m.reloadConnections(snapName); err != nil {
 		return err
 	}
-	// FIXME: here we should not reconnect auto-connect plug/slot
-	// pairs that were explicitly disconnected by the user
-	connectedSnaps, err := m.autoConnect(task, snapName, nil)
+	if len(snapInfo.BadInterfaces) > 0 {
+		task.Logf("%s", snap.BadInterfacesSummary(snapInfo))
+	}
+
+	reconnectedSnaps, err := m.reloadConnections(snapName)
 	if err != nil {
 		return err
 	}
@@ -170,7 +187,7 @@ func (m *InterfaceManager) setupProfilesForSnap(task *state.Task, _ *tomb.Tomb,
 	for _, name := range disconnectedSnaps {
 		affectedSet[name] = true
 	}
-	for _, name := range connectedSnaps {
+	for _, name := range reconnectedSnaps {
 		affectedSet[name] = true
 	}
 	// The principal snap was already handled above.
@@ -342,6 +359,11 @@ func (m *InterfaceManager) doConnect(task *state.Task, _ *tomb.Tomb) error {
 		return err
 	}
 
+	var autoConnect bool
+	if err := task.Get("auto", &autoConnect); err != nil && err != state.ErrNoState {
+		return err
+	}
+
 	conns, err := getConns(st)
 	if err != nil {
 		return err
@@ -349,10 +371,36 @@ func (m *InterfaceManager) doConnect(task *state.Task, _ *tomb.Tomb) error {
 
 	connRef := interfaces.ConnRef{PlugRef: plugRef, SlotRef: slotRef}
 
+	var plugSnapst snapstate.SnapState
+	if err := snapstate.Get(st, plugRef.Snap, &plugSnapst); err != nil {
+		if autoConnect && err == state.ErrNoState {
+			// ignore the error if auto-connecting
+			task.Logf("snap %q is no longer available for auto-connecting", plugRef.Snap)
+			return nil
+		}
+		return err
+	}
+
+	var slotSnapst snapstate.SnapState
+	if err := snapstate.Get(st, slotRef.Snap, &slotSnapst); err != nil {
+		if autoConnect && err == state.ErrNoState {
+			// ignore the error if auto-connecting
+			task.Logf("snap %q is no longer available for auto-connecting", slotRef.Snap)
+			return nil
+		}
+		return err
+	}
+
 	plug := m.repo.Plug(connRef.PlugRef.Snap, connRef.PlugRef.Name)
 	if plug == nil {
+		if autoConnect {
+			// ignore the error if auto-connecting
+			task.Logf("snap %q no longer has %q plug", connRef.PlugRef.Snap, connRef.PlugRef.Name)
+			return nil
+		}
 		return fmt.Errorf("snap %q has no %q plug", connRef.PlugRef.Snap, connRef.PlugRef.Name)
 	}
+
 	var plugDecl *asserts.SnapDeclaration
 	if plug.Snap.SnapID != "" {
 		var err error
@@ -364,56 +412,56 @@ func (m *InterfaceManager) doConnect(task *state.Task, _ *tomb.Tomb) error {
 
 	slot := m.repo.Slot(connRef.SlotRef.Snap, connRef.SlotRef.Name)
 	if slot == nil {
+		if autoConnect {
+			// ignore the error if auto-connecting
+			task.Logf("snap %q no longer has %q slot", connRef.SlotRef.Snap, connRef.SlotRef.Name)
+			return nil
+		}
 		return fmt.Errorf("snap %q has no %q slot", connRef.SlotRef.Snap, connRef.SlotRef.Name)
 	}
-	var slotDecl *asserts.SnapDeclaration
-	if slot.Snap.SnapID != "" {
-		var err error
-		slotDecl, err = assertstate.SnapDeclaration(st, slot.Snap.SnapID)
+
+	// FIXME: check auto connect policy when interface hooks land.
+	if !autoConnect {
+		var slotDecl *asserts.SnapDeclaration
+		if slot.Snap.SnapID != "" {
+			var err error
+			slotDecl, err = assertstate.SnapDeclaration(st, slot.Snap.SnapID)
+			if err != nil {
+				return fmt.Errorf("cannot find snap declaration for %q: %v", slot.Snap.Name(), err)
+			}
+		}
+
+		baseDecl, err := assertstate.BaseDeclaration(st)
 		if err != nil {
-			return fmt.Errorf("cannot find snap declaration for %q: %v", slot.Snap.Name(), err)
+			return fmt.Errorf("internal error: cannot find base declaration: %v", err)
 		}
-	}
 
-	baseDecl, err := assertstate.BaseDeclaration(st)
-	if err != nil {
-		return fmt.Errorf("internal error: cannot find base declaration: %v", err)
-	}
-
-	// check the connection against the declarations' rules
-	ic := policy.ConnectCandidate{
-		Plug:                plug.PlugInfo,
-		PlugSnapDeclaration: plugDecl,
-		Slot:                slot.SlotInfo,
-		SlotSnapDeclaration: slotDecl,
-		BaseDeclaration:     baseDecl,
-	}
+		// check the connection against the declarations' rules
+		ic := policy.ConnectCandidate{
+			Plug:                plug,
+			PlugSnapDeclaration: plugDecl,
+			Slot:                slot,
+			SlotSnapDeclaration: slotDecl,
+			BaseDeclaration:     baseDecl,
+		}
 
-	// if either of plug or slot snaps don't have a declaration it
-	// means they were installed with "dangerous", so the security
-	// check should be skipped at this point.
-	if plugDecl != nil && slotDecl != nil {
-		err = ic.Check()
-		if err != nil {
-			return err
+		// if either of plug or slot snaps don't have a declaration it
+		// means they were installed with "dangerous", so the security
+		// check should be skipped at this point.
+		if plugDecl != nil && slotDecl != nil {
+			err = ic.Check()
+			if err != nil {
+				return err
+			}
 		}
 	}
 
+	// TODO: pass dynamic attributes from hooks
 	err = m.repo.Connect(connRef)
 	if err != nil {
 		return err
 	}
 
-	var plugSnapst snapstate.SnapState
-	if err := snapstate.Get(st, connRef.PlugRef.Snap, &plugSnapst); err != nil {
-		return err
-	}
-
-	var slotSnapst snapstate.SnapState
-	if err := snapstate.Get(st, connRef.SlotRef.Snap, &slotSnapst); err != nil {
-		return err
-	}
-
 	slotOpts := confinementOptions(slotSnapst.Flags)
 	if err := m.setupSnapSecurity(task, slot.Snap, slotOpts); err != nil {
 		return err
@@ -423,26 +471,12 @@ func (m *InterfaceManager) doConnect(task *state.Task, _ *tomb.Tomb) error {
 		return err
 	}
 
-	conns[connRef.ID()] = connState{Interface: plug.Interface}
+	conns[connRef.ID()] = connState{Interface: plug.Interface, Auto: autoConnect}
 	setConns(st, conns)
 
 	return nil
 }
 
-func snapNamesFromConns(conns []interfaces.ConnRef) []string {
-	m := make(map[string]bool)
-	for _, conn := range conns {
-		m[conn.PlugRef.Snap] = true
-		m[conn.SlotRef.Snap] = true
-	}
-	l := make([]string, 0, len(m))
-	for name := range m {
-		l = append(l, name)
-	}
-	sort.Strings(l)
-	return l
-}
-
 func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 	st := task.State()
 	st.Lock()
@@ -458,18 +492,25 @@ func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 		return err
 	}
 
-	affectedConns, err := m.repo.ResolveDisconnect(plugRef.Snap, plugRef.Name, slotRef.Snap, slotRef.Name)
-	if err != nil {
-		return err
-	}
-	m.repo.DisconnectAll(affectedConns)
-	affectedSnaps := snapNamesFromConns(affectedConns)
-	for _, snapName := range affectedSnaps {
+	var snapStates []snapstate.SnapState
+	for _, snapName := range []string{plugRef.Snap, slotRef.Snap} {
 		var snapst snapstate.SnapState
 		if err := snapstate.Get(st, snapName, &snapst); err != nil {
+			if err == state.ErrNoState {
+				task.Logf("skipping disconnect operation for connection %s %s, snap %q doesn't exist", plugRef, slotRef, snapName)
+				return nil
+			}
 			task.Errorf("skipping security profiles setup for snap %q when disconnecting %s from %s: %v", snapName, plugRef, slotRef, err)
-			continue
+		} else {
+			snapStates = append(snapStates, snapst)
 		}
+	}
+
+	err = m.repo.Disconnect(plugRef.Snap, plugRef.Name, slotRef.Snap, slotRef.Name)
+	if err != nil {
+		return fmt.Errorf("snapd changed, please retry the operation: %v", err)
+	}
+	for _, snapst := range snapStates {
 		snapInfo, err := snapst.CurrentInfo()
 		if err != nil {
 			return err
@@ -479,14 +520,199 @@ func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 			return err
 		}
 	}
-	for _, conn := range affectedConns {
-		delete(conns, conn.ID())
-	}
+
+	conn := interfaces.ConnRef{PlugRef: plugRef, SlotRef: slotRef}
+	delete(conns, conn.ID())
 
 	setConns(st, conns)
 	return nil
 }
 
+// timeout for shared content retry
+var contentLinkRetryTimeout = 30 * time.Second
+
+// defaultContentProviders returns a dict of the default-providers for the
+// content plugs for the given snapName
+func (m *InterfaceManager) defaultContentProviders(snapName string) map[string]bool {
+	plugs := m.repo.Plugs(snapName)
+	defaultProviders := make(map[string]bool, len(plugs))
+	for _, plug := range plugs {
+		if plug.Interface == "content" {
+			var s string
+			if err := plug.Attr("content", &s); err == nil && s != "" {
+				var dprovider string
+				if err := plug.Attr("default-provider", &dprovider); err == nil && dprovider != "" {
+					defaultProviders[dprovider] = true
+				}
+			}
+		}
+	}
+	return defaultProviders
+}
+
+// doAutoConnect creates task(s) to connect the given snap to viable candidates.
+func (m *InterfaceManager) doAutoConnect(task *state.Task, _ *tomb.Tomb) error {
+	// FIXME: here we should not reconnect auto-connect plug/slot
+	// pairs that were explicitly disconnected by the user
+
+	st := task.State()
+	st.Lock()
+	defer st.Unlock()
+
+	var conns map[string]connState
+	err := st.Get("conns", &conns)
+	if err != nil && err != state.ErrNoState {
+		return err
+	}
+	if conns == nil {
+		conns = make(map[string]connState)
+	}
+
+	snapsup, err := snapstate.TaskSnapSetup(task)
+	if err != nil {
+		return err
+	}
+
+	snapName := snapsup.Name()
+
+	autots := state.NewTaskSet()
+	autochecker, err := newAutoConnectChecker(st)
+	if err != nil {
+		return err
+	}
+
+	// wait for auto-install, started by prerequisites code, for
+	// the default-providers of content ifaces so we can
+	// auto-connect to them
+	defaultProviders := m.defaultContentProviders(snapName)
+	for _, chg := range st.Changes() {
+		if chg.Status().Ready() {
+			continue
+		}
+		for _, t := range chg.Tasks() {
+			if t.Status().Ready() {
+				continue
+			}
+			if t.Kind() != "link-snap" && t.Kind() != "setup-profiles" {
+				continue
+			}
+			if snapsup, err := snapstate.TaskSnapSetup(t); err == nil {
+				if defaultProviders[snapsup.Name()] {
+					return &state.Retry{After: contentLinkRetryTimeout}
+				}
+			}
+		}
+	}
+
+	chg := task.Change()
+	// Auto-connect all the plugs
+	for _, plug := range m.repo.Plugs(snapName) {
+		candidates := m.repo.AutoConnectCandidateSlots(snapName, plug.Name, autochecker.check)
+		if len(candidates) == 0 {
+			continue
+		}
+		// If we are in a core transition we may have both the old ubuntu-core
+		// snap and the new core snap providing the same interface. In that
+		// situation we want to ignore any candidates in ubuntu-core and simply
+		// go with those from the new core snap.
+		if len(candidates) == 2 {
+			switch {
+			case candidates[0].Snap.Name() == "ubuntu-core" && candidates[1].Snap.Name() == "core":
+				candidates = candidates[1:2]
+			case candidates[1].Snap.Name() == "ubuntu-core" && candidates[0].Snap.Name() == "core":
+				candidates = candidates[0:1]
+			}
+		}
+		if len(candidates) != 1 {
+			crefs := make([]string, len(candidates))
+			for i, candidate := range candidates {
+				crefs[i] = candidate.String()
+			}
+			task.Logf("cannot auto-connect plug %s, candidates found: %s", plug, strings.Join(crefs, ", "))
+			continue
+		}
+		slot := candidates[0]
+		connRef := interfaces.NewConnRef(plug, slot)
+		key := connRef.ID()
+		if _, ok := conns[key]; ok {
+			// Suggested connection already exist so don't clobber it.
+			// NOTE: we don't log anything here as this is a normal and common condition.
+			continue
+		}
+
+		ts, err := AutoConnect(st, chg, task, plug.Snap.Name(), plug.Name, slot.Snap.Name(), slot.Name)
+		if err != nil {
+			task.Logf("cannot auto-connect plug %s to %s: %s", connRef.PlugRef, connRef.SlotRef, err)
+			continue
+		}
+		autots.AddAll(ts)
+	}
+	// Auto-connect all the slots
+	for _, slot := range m.repo.Slots(snapName) {
+		candidates := m.repo.AutoConnectCandidatePlugs(snapName, slot.Name, autochecker.check)
+		if len(candidates) == 0 {
+			continue
+		}
+
+		for _, plug := range candidates {
+			// make sure slot is the only viable
+			// connection for plug, same check as if we were
+			// considering auto-connections from plug
+			candSlots := m.repo.AutoConnectCandidateSlots(plug.Snap.Name(), plug.Name, autochecker.check)
+
+			if len(candSlots) != 1 || candSlots[0].String() != slot.String() {
+				crefs := make([]string, len(candSlots))
+				for i, candidate := range candSlots {
+					crefs[i] = candidate.String()
+				}
+				task.Logf("cannot auto-connect slot %s to %s, candidates found: %s", slot, plug, strings.Join(crefs, ", "))
+				continue
+			}
+
+			connRef := interfaces.NewConnRef(plug, slot)
+			key := connRef.ID()
+			if _, ok := conns[key]; ok {
+				// Suggested connection already exist so don't clobber it.
+				// NOTE: we don't log anything here as this is a normal and common condition.
+				continue
+			}
+			ts, err := AutoConnect(st, chg, task, plug.Snap.Name(), plug.Name, slot.Snap.Name(), slot.Name)
+			if err != nil {
+				task.Logf("cannot auto-connect slot %s to %s: %s", connRef.SlotRef, connRef.PlugRef, err)
+				continue
+			}
+			autots.AddAll(ts)
+		}
+	}
+
+	task.SetStatus(state.DoneStatus)
+
+	lanes := task.Lanes()
+	if len(lanes) == 1 && lanes[0] == 0 {
+		lanes = nil
+	}
+	ht := task.HaltTasks()
+
+	// add all connect tasks to the change of main "auto-connect" task and to the same lane.
+	for _, l := range lanes {
+		autots.JoinLane(l)
+	}
+	chg.AddAll(autots)
+	// make all halt tasks of the main 'auto-connect' task wait on connect tasks
+	for _, t := range ht {
+		t.WaitAll(autots)
+	}
+
+	st.EnsureBefore(0)
+	return nil
+}
+
+func (m *InterfaceManager) undoAutoConnect(task *state.Task, _ *tomb.Tomb) error {
+	// TODO Introduce disconnection hooks, and run them here as well to give a chance
+	// for the snap to undo whatever it did when the connection was established.
+	return nil
+}
+
 // transitionConnectionsCoreMigration will transition all connections
 // from oldName to newName. Note that this is only useful when you
 // know that newName supports everything that oldName supports,
@@ -516,10 +742,10 @@ func (m *InterfaceManager) transitionConnectionsCoreMigration(st *state.State, o
 	// on disk are rewriten. This is ok because core/ubuntu-core have
 	// exactly the same profiles and nothing in the generated policies
 	// has the slot-name encoded.
-	if err := m.reloadConnections(oldName); err != nil {
+	if _, err := m.reloadConnections(oldName); err != nil {
 		return err
 	}
-	if err := m.reloadConnections(newName); err != nil {
+	if _, err := m.reloadConnections(newName); err != nil {
 		return err
 	}
 
