@@ -47,12 +47,17 @@ func (m *InterfaceManager) initialize(extraInterfaces []interfaces.Interface, ex
 	if err := m.addSnaps(); err != nil {
 		return err
 	}
-	if err := m.reloadConnections(""); err != nil {
+	if err := m.renameCorePlugConnection(); err != nil {
 		return err
 	}
-	if err := m.regenerateAllSecurityProfiles(); err != nil {
+	if _, err := m.reloadConnections(""); err != nil {
 		return err
 	}
+	if m.profilesNeedRegeneration() {
+		if err := m.regenerateAllSecurityProfiles(); err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
@@ -72,11 +77,17 @@ func (m *InterfaceManager) addInterfaces(extra []interfaces.Interface) error {
 
 func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error {
 	for _, backend := range backends.All {
+		if err := backend.Initialize(); err != nil {
+			return err
+		}
 		if err := m.repo.AddBackend(backend); err != nil {
 			return err
 		}
 	}
 	for _, backend := range extra {
+		if err := backend.Initialize(); err != nil {
+			return err
+		}
 		if err := m.repo.AddBackend(backend); err != nil {
 			return err
 		}
@@ -85,39 +96,41 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 }
 
 func (m *InterfaceManager) addSnaps() error {
-	snaps, err := snapstate.ActiveInfos(m.state)
+	snaps, err := snapsWithSecurityProfiles(m.state)
 	if err != nil {
 		return err
 	}
 	for _, snapInfo := range snaps {
-		snap.AddImplicitSlots(snapInfo)
+		addImplicitSlots(snapInfo)
 		if err := m.repo.AddSnap(snapInfo); err != nil {
-			logger.Noticef("%s", err)
+			logger.Noticef("cannot add snap %q to interface repository: %s", snapInfo.Name(), err)
 		}
 	}
 	return nil
 }
 
-// regenerateAllSecurityProfiles will regenerate the security profiles
-// for apparmor and seccomp. This is needed because:
-// - for seccomp we may have "terms" on disk that the current snap-confine
-//   does not understand (e.g. in a rollback scenario). a refresh ensures
-//   we have a profile that matches what snap-confine understand
-// - for apparmor the kernel 4.4.0-65.86 has an incompatible apparmor
-//   change that breaks existing profiles for installed snaps. With a
-//   refresh those get fixed.
+func (m *InterfaceManager) profilesNeedRegeneration() bool {
+	mismatch, err := interfaces.SystemKeyMismatch()
+	if err != nil {
+		logger.Noticef("error trying to compare the snap system key: %v", err)
+		return true
+	}
+	return mismatch
+}
+
+// regenerateAllSecurityProfiles will regenerate all security profiles.
 func (m *InterfaceManager) regenerateAllSecurityProfiles() error {
 	// Get all the security backends
 	securityBackends := m.repo.Backends()
 
 	// Get all the snap infos
-	snaps, err := snapstate.ActiveInfos(m.state)
+	snaps, err := snapsWithSecurityProfiles(m.state)
 	if err != nil {
 		return err
 	}
 	// Add implicit slots to all snaps
 	for _, snapInfo := range snaps {
-		snap.AddImplicitSlots(snapInfo)
+		addImplicitSlots(snapInfo)
 	}
 
 	// For each snap:
@@ -134,12 +147,8 @@ func (m *InterfaceManager) regenerateAllSecurityProfiles() error {
 
 		// For each backend:
 		for _, backend := range securityBackends {
-			// The issue this is attempting to fix is only
-			// affecting seccomp/apparmor so limit the work just to
-			// this backend.
-			shouldRefresh := (backend.Name() == interfaces.SecuritySecComp || backend.Name() == interfaces.SecurityAppArmor)
-			if !shouldRefresh {
-				continue
+			if backend.Name() == "" {
+				continue // Test backends have no name, skip them to simplify testing.
 			}
 			// Refresh security of this snap and backend
 			if err := backend.Setup(snapInfo, opts, m.repo); err != nil {
@@ -150,21 +159,56 @@ func (m *InterfaceManager) regenerateAllSecurityProfiles() error {
 		}
 	}
 
+	if err := interfaces.WriteSystemKey(); err != nil {
+		logger.Noticef("cannot write system key: %v", err)
+	}
+	return nil
+}
+
+// renameCorePlugConnection renames one connection from "core-support" plug to
+// slot so that the plug name is "core-support-plug" while the slot is
+// unchanged. This matches a change introduced in 2.24, where the core snap no
+// longer has the "core-support" plug as that was clashing with the slot with
+// the same name.
+func (m *InterfaceManager) renameCorePlugConnection() error {
+	conns, err := getConns(m.state)
+	if err != nil {
+		return err
+	}
+	const oldPlugName = "core-support"
+	const newPlugName = "core-support-plug"
+	// old connection, note that slotRef is the same in both
+	slotRef := interfaces.SlotRef{Snap: "core", Name: oldPlugName}
+	oldPlugRef := interfaces.PlugRef{Snap: "core", Name: oldPlugName}
+	oldConnRef := interfaces.ConnRef{PlugRef: oldPlugRef, SlotRef: slotRef}
+	oldID := oldConnRef.ID()
+	// if the old connection is saved, replace it with the new connection
+	if cState, ok := conns[oldID]; ok {
+		newPlugRef := interfaces.PlugRef{Snap: "core", Name: newPlugName}
+		newConnRef := interfaces.ConnRef{PlugRef: newPlugRef, SlotRef: slotRef}
+		newID := newConnRef.ID()
+		delete(conns, oldID)
+		conns[newID] = cState
+		setConns(m.state, conns)
+	}
 	return nil
 }
 
 // reloadConnections reloads connections stored in the state in the repository.
 // Using non-empty snapName the operation can be scoped to connections
 // affecting a given snap.
-func (m *InterfaceManager) reloadConnections(snapName string) error {
+//
+// The return value is the list of affected snap names.
+func (m *InterfaceManager) reloadConnections(snapName string) ([]string, error) {
 	conns, err := getConns(m.state)
 	if err != nil {
-		return err
+		return nil, err
 	}
+	affected := make(map[string]bool)
 	for id := range conns {
 		connRef, err := interfaces.ParseConnRef(id)
 		if err != nil {
-			return err
+			return nil, err
 		}
 		if snapName != "" && connRef.PlugRef.Snap != snapName && connRef.SlotRef.Snap != snapName {
 			continue
@@ -172,8 +216,14 @@ func (m *InterfaceManager) reloadConnections(snapName string) error {
 		if err := m.repo.Connect(connRef); err != nil {
 			logger.Noticef("%s", err)
 		}
+		affected[connRef.PlugRef.Snap] = true
+		affected[connRef.SlotRef.Snap] = true
 	}
-	return nil
+	result := make([]string, 0, len(affected))
+	for name := range affected {
+		result = append(result, name)
+	}
+	return result, nil
 }
 
 func (m *InterfaceManager) setupSnapSecurity(task *state.Task, snapInfo *snap.Info, opts interfaces.ConfinementOptions) error {
@@ -275,90 +325,6 @@ func (c *autoConnectChecker) check(plug *interfaces.Plug, slot *interfaces.Slot)
 	return ic.CheckAutoConnect() == nil
 }
 
-// autoConnect connects the given snap to viable candidates returning the list
-// of connected snap names.  The blacklist can prevent auto-connection to
-// specific interfaces (blacklist entries are plug or slot names).
-func (m *InterfaceManager) autoConnect(task *state.Task, snapName string, blacklist map[string]bool) ([]string, error) {
-	var conns map[string]connState
-	var affectedSnapNames []string
-	err := task.State().Get("conns", &conns)
-	if err != nil && err != state.ErrNoState {
-		return nil, err
-	}
-	if conns == nil {
-		conns = make(map[string]connState)
-	}
-
-	autochecker, err := newAutoConnectChecker(task.State())
-	if err != nil {
-		return nil, err
-	}
-
-	// Auto-connect all the plugs
-	for _, plug := range m.repo.Plugs(snapName) {
-		if blacklist[plug.Name] {
-			continue
-		}
-		candidates := m.repo.AutoConnectCandidateSlots(snapName, plug.Name, autochecker.check)
-		// If we are in a core transition we may have both the old ubuntu-core
-		// snap and the new core snap providing the same interface. In that
-		// situation we want to ignore any candidates in ubuntu-core and simply
-		// go with those from the new core snap.
-		if len(candidates) == 2 {
-			switch {
-			case candidates[0].Snap.Name() == "ubuntu-core" && candidates[1].Snap.Name() == "core":
-				candidates = candidates[1:2]
-			case candidates[1].Snap.Name() == "ubuntu-core" && candidates[0].Snap.Name() == "core":
-				candidates = candidates[0:1]
-			}
-		}
-		if len(candidates) != 1 {
-			continue
-		}
-		slot := candidates[0]
-		connRef := interfaces.ConnRef{PlugRef: plug.Ref(), SlotRef: slot.Ref()}
-		key := connRef.ID()
-		if _, ok := conns[key]; ok {
-			// Suggested connection already exist so don't clobber it.
-			continue
-		}
-		if err := m.repo.Connect(connRef); err != nil {
-			task.Logf("cannot auto connect %s to %s: %s (plug auto-connection)", connRef.PlugRef, connRef.SlotRef, err)
-			continue
-		}
-		affectedSnapNames = append(affectedSnapNames, connRef.PlugRef.Snap)
-		affectedSnapNames = append(affectedSnapNames, connRef.SlotRef.Snap)
-		conns[key] = connState{Interface: plug.Interface, Auto: true}
-	}
-	// Auto-connect all the slots
-	for _, slot := range m.repo.Slots(snapName) {
-		if blacklist[slot.Name] {
-			continue
-		}
-		candidates := m.repo.AutoConnectCandidatePlugs(snapName, slot.Name, autochecker.check)
-		if len(candidates) != 1 {
-			continue
-		}
-		plug := candidates[0]
-		connRef := interfaces.ConnRef{PlugRef: plug.Ref(), SlotRef: slot.Ref()}
-		key := connRef.ID()
-		if _, ok := conns[key]; ok {
-			// Suggested connection already exist so don't clobber it.
-			continue
-		}
-		if err := m.repo.Connect(connRef); err != nil {
-			task.Logf("cannot auto connect %s to %s: %s (slot auto-connection)", connRef.PlugRef, connRef.SlotRef, err)
-			continue
-		}
-		affectedSnapNames = append(affectedSnapNames, connRef.PlugRef.Snap)
-		affectedSnapNames = append(affectedSnapNames, connRef.SlotRef.Snap)
-		conns[key] = connState{Interface: plug.Interface, Auto: true}
-	}
-
-	task.State().Set("conns", conns)
-	return affectedSnapNames, nil
-}
-
 func getPlugAndSlotRefs(task *state.Task) (interfaces.PlugRef, interfaces.SlotRef, error) {
 	var plugRef interfaces.PlugRef
 	var slotRef interfaces.SlotRef
@@ -387,3 +353,57 @@ func getConns(st *state.State) (map[string]connState, error) {
 func setConns(st *state.State, conns map[string]connState) {
 	st.Set("conns", conns)
 }
+
+// snapsWithSecurityProfiles returns all snaps that have active
+// security profiles: these are either snaps that are active, or about
+// to be active (pending link-snap) with a done setup-profiles
+func snapsWithSecurityProfiles(st *state.State) ([]*snap.Info, error) {
+	infos, err := snapstate.ActiveInfos(st)
+	if err != nil {
+		return nil, err
+	}
+	seen := make(map[string]bool, len(infos))
+	for _, info := range infos {
+		seen[info.Name()] = true
+	}
+	for _, t := range st.Tasks() {
+		if t.Kind() != "link-snap" || t.Status().Ready() {
+			continue
+		}
+		snapsup, err := snapstate.TaskSnapSetup(t)
+		if err != nil {
+			return nil, err
+		}
+		snapName := snapsup.Name()
+		if seen[snapName] {
+			continue
+		}
+
+		doneProfiles := false
+		for _, t1 := range t.WaitTasks() {
+			if t1.Kind() == "setup-profiles" && t1.Status() == state.DoneStatus {
+				snapsup1, err := snapstate.TaskSnapSetup(t)
+				if err != nil {
+					return nil, err
+				}
+				if snapsup1.Name() == snapName {
+					doneProfiles = true
+					break
+				}
+			}
+		}
+		if !doneProfiles {
+			continue
+		}
+
+		seen[snapName] = true
+		snapInfo, err := snap.ReadInfo(snapName, snapsup.SideInfo)
+		if err != nil {
+			logger.Noticef("cannot retrieve info for snap %q: %s", snapName, err)
+			continue
+		}
+		infos = append(infos, snapInfo)
+	}
+
+	return infos, nil
+}
