@@ -20,6 +20,9 @@
 package backend_test
 
 import (
+	"errors"
+	"io/ioutil"
+	"os"
 	"path/filepath"
 
 	. "gopkg.in/check.v1"
@@ -35,9 +38,9 @@ import (
 )
 
 type linkSuite struct {
-	be           backend.Backend
-	nullProgress progress.NullProgress
-	prevctlCmd   func(...string) ([]byte, error)
+	be backend.Backend
+
+	systemctlRestorer func()
 }
 
 var _ = Suite(&linkSuite{})
@@ -45,15 +48,14 @@ var _ = Suite(&linkSuite{})
 func (s *linkSuite) SetUpTest(c *C) {
 	dirs.SetRootDir(c.MkDir())
 
-	s.prevctlCmd = systemd.SystemctlCmd
-	systemd.SystemctlCmd = func(cmd ...string) ([]byte, error) {
+	s.systemctlRestorer = systemd.MockSystemctl(func(cmd ...string) ([]byte, error) {
 		return []byte("ActiveState=inactive\n"), nil
-	}
+	})
 }
 
 func (s *linkSuite) TearDownTest(c *C) {
 	dirs.SetRootDir("")
-	systemd.SystemctlCmd = s.prevctlCmd
+	s.systemctlRestorer()
 }
 
 func (s *linkSuite) TestLinkDoUndoGenerateWrappers(c *C) {
@@ -69,9 +71,7 @@ apps:
    command: svc
    daemon: simple
 `
-	const contents = ""
-
-	info := snaptest.MockSnap(c, yaml, contents, &snap.SideInfo{Revision: snap.R(11)})
+	info := snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
 
 	err := s.be.LinkSnap(info)
 	c.Assert(err, IsNil)
@@ -84,7 +84,7 @@ apps:
 	c.Assert(l, HasLen, 1)
 
 	// undo will remove
-	err = s.be.UnlinkSnap(info, &s.nullProgress)
+	err = s.be.UnlinkSnap(info, progress.Null)
 	c.Assert(err, IsNil)
 
 	l, err = filepath.Glob(filepath.Join(dirs.SnapBinariesDir, "*"))
@@ -101,7 +101,7 @@ version: 1.0
 `
 	const contents = ""
 
-	info := snaptest.MockSnap(c, yaml, contents, &snap.SideInfo{Revision: snap.R(11)})
+	info := snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
 
 	err := s.be.LinkSnap(info)
 	c.Assert(err, IsNil)
@@ -119,7 +119,7 @@ version: 1.0
 	c.Assert(currentDataDir, Equals, dataDir)
 
 	// undo will remove the symlinks
-	err = s.be.UnlinkSnap(info, &s.nullProgress)
+	err = s.be.UnlinkSnap(info, progress.Null)
 	c.Assert(err, IsNil)
 
 	c.Check(osutil.FileExists(currentActiveSymlink), Equals, false)
@@ -143,7 +143,7 @@ apps:
 `
 	const contents = ""
 
-	info := snaptest.MockSnap(c, yaml, contents, &snap.SideInfo{Revision: snap.R(11)})
+	info := snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
 
 	err := s.be.LinkSnap(info)
 	c.Assert(err, IsNil)
@@ -185,15 +185,15 @@ apps:
 `
 	const contents = ""
 
-	info := snaptest.MockSnap(c, yaml, contents, &snap.SideInfo{Revision: snap.R(11)})
+	info := snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
 
 	err := s.be.LinkSnap(info)
 	c.Assert(err, IsNil)
 
-	err = s.be.UnlinkSnap(info, &s.nullProgress)
+	err = s.be.UnlinkSnap(info, progress.Null)
 	c.Assert(err, IsNil)
 
-	err = s.be.UnlinkSnap(info, &s.nullProgress)
+	err = s.be.UnlinkSnap(info, progress.Null)
 	c.Assert(err, IsNil)
 
 	// no wrappers
@@ -210,3 +210,106 @@ apps:
 	c.Check(osutil.FileExists(currentActiveSymlink), Equals, false)
 	c.Check(osutil.FileExists(currentDataSymlink), Equals, false)
 }
+
+func (s *linkSuite) TestLinkFailsForUnsetRevision(c *C) {
+	info := &snap.Info{
+		SuggestedName: "foo",
+	}
+	err := s.be.LinkSnap(info)
+	c.Assert(err, ErrorMatches, `cannot link snap "foo" with unset revision`)
+}
+
+type linkCleanupSuite struct {
+	linkSuite
+	info *snap.Info
+}
+
+var _ = Suite(&linkCleanupSuite{})
+
+func (s *linkCleanupSuite) SetUpTest(c *C) {
+	s.linkSuite.SetUpTest(c)
+
+	const yaml = `name: hello
+version: 1.0
+environment:
+ KEY: value
+
+apps:
+ foo:
+   command: foo
+ bar:
+   command: bar
+ svc:
+   command: svc
+   daemon: simple
+`
+	s.info = snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
+
+	guiDir := filepath.Join(s.info.MountDir(), "meta", "gui")
+	c.Assert(os.MkdirAll(guiDir, 0755), IsNil)
+	c.Assert(ioutil.WriteFile(filepath.Join(guiDir, "bin.desktop"), []byte(`
+[Desktop Entry]
+Name=bin
+Icon=${SNAP}/bin.png
+Exec=bin
+`), 0644), IsNil)
+
+	r := systemd.MockSystemctl(func(...string) ([]byte, error) {
+		return nil, nil
+	})
+	defer r()
+
+	// sanity checks
+	for _, d := range []string{dirs.SnapBinariesDir, dirs.SnapDesktopFilesDir, dirs.SnapServicesDir} {
+		os.MkdirAll(d, 0755)
+		l, err := filepath.Glob(filepath.Join(d, "*"))
+		c.Assert(err, IsNil, Commentf(d))
+		c.Assert(l, HasLen, 0, Commentf(d))
+	}
+}
+
+func (s *linkCleanupSuite) testLinkCleanupDirOnFail(c *C, dir string) {
+	c.Assert(os.Chmod(dir, 0), IsNil)
+	defer os.Chmod(dir, 0755)
+
+	err := s.be.LinkSnap(s.info)
+	c.Assert(err, NotNil)
+	c.Assert(err, FitsTypeOf, &os.PathError{})
+
+	for _, d := range []string{dirs.SnapBinariesDir, dirs.SnapDesktopFilesDir, dirs.SnapServicesDir} {
+		l, err := filepath.Glob(filepath.Join(d, "*"))
+		c.Check(err, IsNil, Commentf(d))
+		c.Check(l, HasLen, 0, Commentf(d))
+	}
+}
+
+func (s *linkCleanupSuite) TestLinkCleanupOnDesktopFail(c *C) {
+	s.testLinkCleanupDirOnFail(c, dirs.SnapDesktopFilesDir)
+}
+
+func (s *linkCleanupSuite) TestLinkCleanupOnBinariesFail(c *C) {
+	// this one is the trivial case _as the code stands today_,
+	// but nothing guarantees that ordering.
+	s.testLinkCleanupDirOnFail(c, dirs.SnapBinariesDir)
+}
+
+func (s *linkCleanupSuite) TestLinkCleanupOnServicesFail(c *C) {
+	s.testLinkCleanupDirOnFail(c, dirs.SnapServicesDir)
+}
+
+func (s *linkCleanupSuite) TestLinkCleanupOnSystemctlFail(c *C) {
+	r := systemd.MockSystemctl(func(...string) ([]byte, error) {
+		return nil, errors.New("ouchie")
+	})
+	defer r()
+
+	err := s.be.LinkSnap(s.info)
+	c.Assert(err, ErrorMatches, "ouchie")
+
+	for _, d := range []string{dirs.SnapBinariesDir, dirs.SnapDesktopFilesDir, dirs.SnapServicesDir} {
+		l, err := filepath.Glob(filepath.Join(d, "*"))
+		c.Check(err, IsNil, Commentf(d))
+		c.Check(l, HasLen, 0, Commentf(d))
+	}
+
+}
