@@ -24,34 +24,44 @@ package snapstate_test
 import (
 	"os"
 	"path/filepath"
+	"time"
 
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/boot/boottest"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/partition"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
+	"github.com/snapcore/snapd/testutil"
 )
 
 type bootedSuite struct {
+	testutil.BaseTest
 	bootloader *boottest.MockBootloader
 
+	o           *overlord.Overlord
 	state       *state.State
 	snapmgr     *snapstate.SnapManager
 	fakeBackend *fakeSnappyBackend
+	restore     func()
 }
 
 var _ = Suite(&bootedSuite{})
 
 func (bs *bootedSuite) SetUpTest(c *C) {
+	bs.BaseTest.SetUpTest(c)
+
 	dirs.SetRootDir(c.MkDir())
 	err := os.MkdirAll(filepath.Dir(dirs.SnapStateFile), 0755)
 	c.Assert(err, IsNil)
 
+	bs.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+
 	// booted is not running on classic
 	release.MockOnClassic(false)
 
@@ -61,18 +71,22 @@ func (bs *bootedSuite) SetUpTest(c *C) {
 	partition.ForceBootloader(bs.bootloader)
 
 	bs.fakeBackend = &fakeSnappyBackend{}
-	bs.state = state.New(nil)
+	bs.o = overlord.Mock()
+	bs.state = bs.o.State()
 	bs.snapmgr, err = snapstate.Manager(bs.state)
 	c.Assert(err, IsNil)
 	bs.snapmgr.AddForeignTaskHandlers(bs.fakeBackend)
 
+	bs.o.AddManager(bs.snapmgr)
+
 	snapstate.SetSnapManagerBackend(bs.snapmgr, bs.fakeBackend)
-	snapstate.AutoAliases = func(*state.State, *snap.Info) ([]string, error) {
+	snapstate.AutoAliases = func(*state.State, *snap.Info) (map[string]string, error) {
 		return nil, nil
 	}
 }
 
 func (bs *bootedSuite) TearDownTest(c *C) {
+	bs.BaseTest.TearDownTest(c)
 	snapstate.AutoAliases = nil
 	release.MockOnClassic(true)
 	dirs.SetRootDir("")
@@ -85,15 +99,12 @@ var kernelSI1 = &snap.SideInfo{RealName: "canonical-pc-linux", Revision: snap.R(
 var kernelSI2 = &snap.SideInfo{RealName: "canonical-pc-linux", Revision: snap.R(2)}
 
 func (bs *bootedSuite) settle() {
-	for i := 0; i < 50; i++ {
-		bs.snapmgr.Ensure()
-		bs.snapmgr.Wait()
-	}
+	bs.o.Settle(5 * time.Second)
 }
 
 func (bs *bootedSuite) makeInstalledKernelOS(c *C, st *state.State) {
-	snaptest.MockSnap(c, "name: core\ntype: os\nversion: 1", "", osSI1)
-	snaptest.MockSnap(c, "name: core\ntype: os\nversion: 2", "", osSI2)
+	snaptest.MockSnap(c, "name: core\ntype: os\nversion: 1", osSI1)
+	snaptest.MockSnap(c, "name: core\ntype: os\nversion: 2", osSI2)
 	snapstate.Set(st, "core", &snapstate.SnapState{
 		SnapType: "os",
 		Active:   true,
@@ -101,8 +112,8 @@ func (bs *bootedSuite) makeInstalledKernelOS(c *C, st *state.State) {
 		Current:  snap.R(2),
 	})
 
-	snaptest.MockSnap(c, "name: canonical-pc-linux\ntype: os\nversion: 1", "", kernelSI1)
-	snaptest.MockSnap(c, "name: canonical-pc-linux\ntype: os\nversion: 2", "", kernelSI2)
+	snaptest.MockSnap(c, "name: canonical-pc-linux\ntype: os\nversion: 1", kernelSI1)
+	snaptest.MockSnap(c, "name: canonical-pc-linux\ntype: os\nversion: 2", kernelSI2)
 	snapstate.Set(st, "canonical-pc-linux", &snapstate.SnapState{
 		SnapType: "kernel",
 		Active:   true,
@@ -210,7 +221,7 @@ func (bs *bootedSuite) TestUpdateBootRevisionsOSErrorsLate(c *C) {
 	defer st.Unlock()
 
 	// have a kernel
-	snaptest.MockSnap(c, "name: canonical-pc-linux\ntype: os\nversion: 2", "", kernelSI2)
+	snaptest.MockSnap(c, "name: canonical-pc-linux\ntype: os\nversion: 2", kernelSI2)
 	snapstate.Set(st, "canonical-pc-linux", &snapstate.SnapState{
 		SnapType: "kernel",
 		Active:   true,
