@@ -27,6 +27,7 @@ import (
 
 	"github.com/snapcore/snapd/arch"
 	"github.com/snapcore/snapd/cmd"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/snapstate/backend"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
@@ -99,20 +100,27 @@ func checkVersion(version string) bool {
 	return true
 }
 
-type ErrSnapNeedsMode struct {
-	Mode snap.ConfinementType
+type SnapNeedsDevModeError struct {
 	Snap string
 }
 
-func (e *ErrSnapNeedsMode) Error() string {
-	return fmt.Sprintf("snap %q requires %s or confinement override", e.Snap, e.Mode)
+func (e *SnapNeedsDevModeError) Error() string {
+	return fmt.Sprintf("snap %q requires devmode or confinement override", e.Snap)
 }
 
-type ErrSnapNeedsClassicSystem struct {
+type SnapNeedsClassicError struct {
 	Snap string
 }
 
-func (e *ErrSnapNeedsClassicSystem) Error() string {
+func (e *SnapNeedsClassicError) Error() string {
+	return fmt.Sprintf("snap %q requires classic confinement", e.Snap)
+}
+
+type SnapNeedsClassicSystemError struct {
+	Snap string
+}
+
+func (e *SnapNeedsClassicSystemError) Error() string {
 	return fmt.Sprintf("snap %q requires classic confinement which is only available on classic systems", e.Snap)
 }
 
@@ -128,13 +136,12 @@ func validateFlagsForInfo(info *snap.Info, snapst *SnapState, flags Flags) error
 		if flags.DevModeAllowed() {
 			return nil
 		}
-		return &ErrSnapNeedsMode{
-			Mode: c,
+		return &SnapNeedsDevModeError{
 			Snap: info.Name(),
 		}
 	case snap.ClassicConfinement:
 		if !release.OnClassic {
-			return &ErrSnapNeedsClassicSystem{Snap: info.Name()}
+			return &SnapNeedsClassicSystemError{Snap: info.Name()}
 		}
 
 		if flags.Classic {
@@ -145,8 +152,7 @@ func validateFlagsForInfo(info *snap.Info, snapst *SnapState, flags Flags) error
 			return nil
 		}
 
-		return &ErrSnapNeedsMode{
-			Mode: c,
+		return &SnapNeedsClassicError{
 			Snap: info.Name(),
 		}
 	default:
@@ -177,11 +183,19 @@ func validateInfoAndFlags(info *snap.Info, snapst *SnapState, flags Flags) error
 
 var openSnapFile = backend.OpenSnapFile
 
+func validateContainer(c snap.Container, s *snap.Info, logf func(format string, v ...interface{})) error {
+	err := snap.ValidateContainer(c, s, logf)
+	if err == nil {
+		return nil
+	}
+	return fmt.Errorf("%v; contact developer", err)
+}
+
 // checkSnap ensures that the snap can be installed.
 func checkSnap(st *state.State, snapFilePath string, si *snap.SideInfo, curInfo *snap.Info, flags Flags) error {
 	// This assumes that the snap was already verified or --dangerous was used.
 
-	s, _, err := openSnapFile(snapFilePath, si)
+	s, c, err := openSnapFile(snapFilePath, si)
 	if err != nil {
 		return err
 	}
@@ -190,6 +204,10 @@ func checkSnap(st *state.State, snapFilePath string, si *snap.SideInfo, curInfo
 		return err
 	}
 
+	if err := validateContainer(c, s, logger.Noticef); err != nil {
+		return err
+	}
+
 	st.Lock()
 	defer st.Unlock()
 
@@ -301,7 +319,34 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, flags Fl
 	return nil
 }
 
+func checkBases(st *state.State, snapInfo, curInfo *snap.Info, flags Flags) error {
+	// check if this is relevant
+	if snapInfo.Type != snap.TypeApp && snapInfo.Type != snap.TypeGadget {
+		return nil
+	}
+	if snapInfo.Base == "" {
+		return nil
+	}
+
+	snapStates, err := All(st)
+	if err != nil {
+		return err
+	}
+	for otherSnap, snapst := range snapStates {
+		typ, err := snapst.Type()
+		if err != nil {
+			return err
+		}
+		if typ == snap.TypeBase && otherSnap == snapInfo.Base {
+			return nil
+		}
+	}
+
+	return fmt.Errorf("cannot find required base %q", snapInfo.Base)
+}
+
 func init() {
 	AddCheckSnapCallback(checkCoreName)
 	AddCheckSnapCallback(checkGadgetOrKernel)
+	AddCheckSnapCallback(checkBases)
 }
