@@ -45,6 +45,10 @@ type fakeManager struct {
 	ensureError, stopError error
 }
 
+func (m *fakeManager) KnownTaskKinds() []string {
+	return []string{}
+}
+
 func (fm *fakeManager) Ensure() error {
 	*fm.calls = append(*fm.calls, "ensure:"+fm.name)
 	return fm.ensureError
