@@ -31,3 +31,16 @@ func MockOSReleasePath(filename string) (restore func()) {
 		fallbackOsReleasePath = oldFallback
 	}
 }
+
+func MockAppArmorFeaturesSysPath(path string) (restorer func()) {
+	old := appArmorFeaturesSysPath
+	appArmorFeaturesSysPath = path
+	return func() {
+		appArmorFeaturesSysPath = old
+	}
+}
+
+var (
+	ProbeAppArmor            = probeAppArmor
+	RequiredAppArmorFeatures = requiredAppArmorFeatures
+)
