@@ -33,45 +33,24 @@ var Series = "16"
 
 // OS contains information about the system extracted from /etc/os-release.
 type OS struct {
-	ID        string `json:"id"`
-	VersionID string `json:"version-id,omitempty"`
+	ID        string   `json:"id"`
+	IDLike    []string `json:"-"`
+	VersionID string   `json:"version-id,omitempty"`
 }
 
 // ForceDevMode returns true if the distribution doesn't implement required
 // security features for confinement and devmode is forced.
-func (os *OS) ForceDevMode() bool {
-	supported := []struct {
-		ID        string
-		VersionID string
-	}{
-		{"neon", ""},
-		{"ubuntu", ""},
-		{"ubuntu-core", ""},
-		// NOTE: elementary is "Elementary OS" but normalization
-		// applies
-		{"elementary", "0.4"},
-		// NOTE: mint uses "LinuxMint" (mixed capitalization)
-		// but this is normalized by readOSRelease.
-		{"linuxmint", "18"},
-		{"galliumos", "2.0"},
-		{"peppermint", "7.0"},
-		// NOTE: zorin is "Zorin OS" but normalization applies
-		{"zorin", "12"},
-	}
+func (o *OS) ForceDevMode() bool {
+	return AppArmorLevel() != FullAppArmor
+}
 
-	for _, s := range supported {
-		if os.ID == s.ID {
-			res, err := strutil.VersionCompare(os.VersionID, s.VersionID)
-			if err == nil && res >= 0 {
-				return false
-			}
+func DistroLike(distros ...string) bool {
+	for _, distro := range distros {
+		if ReleaseInfo.ID == distro || strutil.ListContains(ReleaseInfo.IDLike, distro) {
+			return true
 		}
 	}
-
-	// NOTE: Other distributions can move out of devmode by
-	// integrating with the interface security backends. This will
-	// be documented separately in the porting guide.
-	return true
+	return false
 }
 
 var (
@@ -83,7 +62,6 @@ var (
 func readOSRelease() OS {
 	// TODO: separate this out into its own thing maybe (if made more general)
 	osRelease := OS{
-		VersionID: "unknown",
 		// from os-release(5): If not set, defaults to "ID=linux".
 		ID: "linux",
 	}
@@ -119,6 +97,9 @@ func readOSRelease() OS {
 			// not being too good at reading comprehension.
 			// Works around e.g. lp:1602317
 			osRelease.ID = strings.Fields(strings.ToLower(v))[0]
+		case "ID_LIKE":
+			// This is like ID, except it's a space separated list... hooray?
+			osRelease.IDLike = strings.Fields(strings.ToLower(v))
 		case "VERSION_ID":
 			osRelease.VersionID = v
 		}
@@ -155,3 +136,13 @@ func MockReleaseInfo(osRelease *OS) (restore func()) {
 	ReleaseInfo = *osRelease
 	return func() { ReleaseInfo = old }
 }
+
+// MockForcedDevmode fake the system to believe its in a distro
+// that is in ForcedDevmode
+func MockForcedDevmode(isDevmode bool) (restore func()) {
+	level := FullAppArmor
+	if isDevmode {
+		level = NoAppArmor
+	}
+	return MockAppArmorLevel(level)
+}
