@@ -20,9 +20,9 @@
 package snap
 
 var (
-	ImplicitSlotsForTests        = implicitSlots
-	ImplicitClassicSlotsForTests = implicitClassicSlots
 	NewHookType                  = newHookType
+	ValidateSocketName           = validateSocketName
+	InfoFromSnapYamlWithSideInfo = infoFromSnapYamlWithSideInfo
 )
 
 func MockSupportedHookTypes(hookTypes []*HookType) (restore func()) {
@@ -30,3 +30,7 @@ func MockSupportedHookTypes(hookTypes []*HookType) (restore func()) {
 	supportedHooks = hookTypes
 	return func() { supportedHooks = old }
 }
+
+func (info *Info) RenamePlug(oldName, newName string) {
+	info.renamePlug(oldName, newName)
+}
