@@ -47,7 +47,7 @@ type GadgetVolume struct {
 // type when we actually handle these.
 
 type VolumeStructure struct {
-	Label       string          `yaml:"label"`
+	Label       string          `yaml:"filesystem-label"`
 	Offset      string          `yaml:"offset"`
 	OffsetWrite string          `yaml:"offset-write"`
 	Size        string          `yaml:"size"`
@@ -95,6 +95,14 @@ func ReadGadgetInfo(info *Info, classic bool) (*GadgetInfo, error) {
 		return nil, fmt.Errorf(errorFormat, err)
 	}
 
+	for k, v := range gi.Defaults {
+		dflt, err := normalizeYamlValue(v)
+		if err != nil {
+			return nil, fmt.Errorf("default value %q of %q: %v", v, k, err)
+		}
+		gi.Defaults[k] = dflt.(map[string]interface{})
+	}
+
 	if classic && len(gi.Volumes) == 0 {
 		// volumes can be left out on classic
 		// can still specify defaults though
@@ -102,22 +110,22 @@ func ReadGadgetInfo(info *Info, classic bool) (*GadgetInfo, error) {
 	}
 
 	// basic validation
-	foundBootloader := false
+	var bootloadersFound int
 	for _, v := range gi.Volumes {
-		if foundBootloader {
-			return nil, fmt.Errorf(errorFormat, "bootloader already declared")
-		}
 		switch v.Bootloader {
 		case "":
-			return nil, fmt.Errorf(errorFormat, "bootloader cannot be empty")
-		case "grub", "u-boot":
-			foundBootloader = true
+			// pass
+		case "grub", "u-boot", "android-boot":
+			bootloadersFound += 1
 		default:
-			return nil, fmt.Errorf(errorFormat, "bootloader must be either grub or u-boot")
+			return nil, fmt.Errorf(errorFormat, "bootloader must be one of grub, u-boot or android-boot")
 		}
 	}
-	if !foundBootloader {
+	switch {
+	case bootloadersFound == 0:
 		return nil, fmt.Errorf(errorFormat, "bootloader not declared in any volume")
+	case bootloadersFound > 1:
+		return nil, fmt.Errorf(errorFormat, fmt.Sprintf("too many (%d) bootloaders declared", bootloadersFound))
 	}
 
 	return &gi, nil
