@@ -51,7 +51,7 @@ volumes:
     bootloader: u-boot
     id:     id,guid
     structure:
-      - label: system-boot
+      - filesystem-label: system-boot
         offset: 12345
         offset-write: 777
         size: 88888
@@ -69,14 +69,48 @@ volumes:
             unpack: false
 `)
 
+var mockMultiVolumeGadgetYaml = []byte(`
+device-tree: frobinator-3000.dtb
+device-tree-origin: kernel
+volumes:
+  frobinator-3000-image:
+    bootloader: u-boot
+    schema: mbr
+    structure:
+      - name: system-boot
+        type: 0C
+        filesystem: vfat
+        filesystem-label: system-boot
+        size: 128M
+        role: system-boot
+        content:
+          - source: splash.bmp
+            target: .
+      - name: writable
+        type: 83
+        filesystem: ext4
+        filesystem-label: writable
+        size: 380M
+        role: system-data
+  u-boot-frobinator-3000:
+    structure:
+      - name: u-boot
+        type: bare
+        size: 623000
+        offset: 0
+        content:
+          - image: u-boot.imz
+`)
+
 var mockClassicGadgetYaml = []byte(`
 defaults:
   core:
     something: true
+  other:
+    foo:
+      bar: baz
 `)
 
-var mockGadgetSnapContents = "SNAP"
-
 func (s *gadgetYamlTestSuite) SetUpTest(c *C) {
 	dirs.SetRootDir(c.MkDir())
 }
@@ -88,26 +122,27 @@ func (s *gadgetYamlTestSuite) TearDownTest(c *C) {
 func (s *gadgetYamlTestSuite) TestReadGadgetNotAGadget(c *C) {
 	info := snaptest.MockInfo(c, `
 name: other
+version: 0
 `, &snap.SideInfo{Revision: snap.R(42)})
 	_, err := snap.ReadGadgetInfo(info, false)
 	c.Assert(err, ErrorMatches, "cannot read gadget snap details: not a gadget snap")
 }
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlMissing(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
 	_, err := snap.ReadGadgetInfo(info, false)
 	c.Assert(err, ErrorMatches, ".*meta/gadget.yaml: no such file or directory")
 }
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlOnClassicOptional(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
 	gi, err := snap.ReadGadgetInfo(info, true)
 	c.Assert(err, IsNil)
 	c.Check(gi, NotNil)
 }
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlOnClassicEmptyIsValid(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
 	err := ioutil.WriteFile(filepath.Join(info.MountDir(), "meta", "gadget.yaml"), nil, 0644)
 	c.Assert(err, IsNil)
 
@@ -117,7 +152,7 @@ func (s *gadgetYamlTestSuite) TestReadGadgetYamlOnClassicEmptyIsValid(c *C) {
 }
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlOnClassicOnylDefaultsIsValid(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
 	err := ioutil.WriteFile(filepath.Join(info.MountDir(), "meta", "gadget.yaml"), mockClassicGadgetYaml, 0644)
 	c.Assert(err, IsNil)
 
@@ -125,13 +160,14 @@ func (s *gadgetYamlTestSuite) TestReadGadgetYamlOnClassicOnylDefaultsIsValid(c *
 	c.Assert(err, IsNil)
 	c.Assert(ginfo, DeepEquals, &snap.GadgetInfo{
 		Defaults: map[string]map[string]interface{}{
-			"core": {"something": true},
+			"core":  {"something": true},
+			"other": {"foo": map[string]interface{}{"bar": "baz"}},
 		},
 	})
 }
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlValid(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
 	err := ioutil.WriteFile(filepath.Join(info.MountDir(), "meta", "gadget.yaml"), mockGadgetYaml, 0644)
 	c.Assert(err, IsNil)
 
@@ -176,23 +212,60 @@ func (s *gadgetYamlTestSuite) TestReadGadgetYamlValid(c *C) {
 	})
 }
 
-func (s *gadgetYamlTestSuite) TestReadGadgetYamlEmptydBootloader(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
-	mockGadgetYamlBroken := []byte(`
-volumes:
- name:
-  bootloader: 
-`)
-
-	err := ioutil.WriteFile(filepath.Join(info.MountDir(), "meta", "gadget.yaml"), mockGadgetYamlBroken, 0644)
+func (s *gadgetYamlTestSuite) TestReadMultiVolumeGadgetYamlValid(c *C) {
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
+	err := ioutil.WriteFile(filepath.Join(info.MountDir(), "meta", "gadget.yaml"), mockMultiVolumeGadgetYaml, 0644)
 	c.Assert(err, IsNil)
 
-	_, err = snap.ReadGadgetInfo(info, false)
-	c.Assert(err, ErrorMatches, "cannot read gadget snap details: bootloader cannot be empty")
+	ginfo, err := snap.ReadGadgetInfo(info, false)
+	c.Assert(err, IsNil)
+	c.Check(ginfo.Volumes, HasLen, 2)
+	c.Assert(ginfo, DeepEquals, &snap.GadgetInfo{
+		Volumes: map[string]snap.GadgetVolume{
+			"frobinator-3000-image": {
+				Schema:     "mbr",
+				Bootloader: "u-boot",
+				Structure: []snap.VolumeStructure{
+					{
+						Label:      "system-boot",
+						Size:       "128M",
+						Filesystem: "vfat",
+						Type:       "0C",
+						Content: []snap.VolumeContent{
+							{
+								Source: "splash.bmp",
+								Target: ".",
+							},
+						},
+					},
+					{
+						Label:      "writable",
+						Type:       "83",
+						Filesystem: "ext4",
+						Size:       "380M",
+					},
+				},
+			},
+			"u-boot-frobinator-3000": {
+				Structure: []snap.VolumeStructure{
+					{
+						Type:   "bare",
+						Size:   "623000",
+						Offset: "0",
+						Content: []snap.VolumeContent{
+							{
+								Image: "u-boot.imz",
+							},
+						},
+					},
+				},
+			},
+		},
+	})
 }
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlInvalidBootloader(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
 	mockGadgetYamlBroken := []byte(`
 volumes:
  name:
@@ -203,11 +276,26 @@ volumes:
 	c.Assert(err, IsNil)
 
 	_, err = snap.ReadGadgetInfo(info, false)
-	c.Assert(err, ErrorMatches, "cannot read gadget snap details: bootloader must be either grub or u-boot")
+	c.Assert(err, ErrorMatches, "cannot read gadget snap details: bootloader must be one of grub, u-boot or android-boot")
+}
+
+func (s *gadgetYamlTestSuite) TestReadGadgetYamlEmptydBootloader(c *C) {
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
+	mockGadgetYamlBroken := []byte(`
+volumes:
+ name:
+  bootloader:
+`)
+
+	err := ioutil.WriteFile(filepath.Join(info.MountDir(), "meta", "gadget.yaml"), mockGadgetYamlBroken, 0644)
+	c.Assert(err, IsNil)
+
+	_, err = snap.ReadGadgetInfo(info, false)
+	c.Assert(err, ErrorMatches, "cannot read gadget snap details: bootloader not declared in any volume")
 }
 
 func (s *gadgetYamlTestSuite) TestReadGadgetYamlMissingBootloader(c *C) {
-	info := snaptest.MockSnap(c, mockGadgetSnapYaml, mockGadgetSnapContents, &snap.SideInfo{Revision: snap.R(42)})
+	info := snaptest.MockSnap(c, mockGadgetSnapYaml, &snap.SideInfo{Revision: snap.R(42)})
 
 	err := ioutil.WriteFile(filepath.Join(info.MountDir(), "meta", "gadget.yaml"), nil, 0644)
 	c.Assert(err, IsNil)
