@@ -26,6 +26,10 @@ import (
 var supportedHooks = []*HookType{
 	newHookType(regexp.MustCompile("^prepare-device$")),
 	newHookType(regexp.MustCompile("^configure$")),
+	newHookType(regexp.MustCompile("^install$")),
+	newHookType(regexp.MustCompile("^pre-refresh$")),
+	newHookType(regexp.MustCompile("^post-refresh$")),
+	newHookType(regexp.MustCompile("^remove$")),
 	newHookType(regexp.MustCompile("^prepare-(?:plug|slot)-[-a-z0-9]+$")),
 	newHookType(regexp.MustCompile("^connect-(?:plug|slot)-[-a-z0-9]+$")),
 }
