@@ -68,6 +68,7 @@ type appYaml struct {
 	StopTimeout     timeout.Timeout `yaml:"stop-timeout,omitempty"`
 	Completer       string          `yaml:"completer,omitempty"`
 	RefreshMode     string          `yaml:"refresh-mode,omitempty"`
+	StopMode        StopModeType    `yaml:"stop-mode,omitempty"`
 
 	RestartCond RestartCondition `yaml:"restart-condition,omitempty"`
 	SlotNames   []string         `yaml:"slots,omitempty"`
@@ -83,6 +84,8 @@ type appYaml struct {
 	Before []string `yaml:"before,omitempty"`
 
 	Timer string `yaml:"timer,omitempty"`
+
+	Autostart string `yaml:"autostart,omitempty"`
 }
 
 type hookYaml struct {
@@ -298,8 +301,10 @@ func setAppsFromSnapYaml(y snapYaml, snap *Info) error {
 			Environment:     yApp.Environment,
 			Completer:       yApp.Completer,
 			RefreshMode:     yApp.RefreshMode,
+			StopMode:        yApp.StopMode,
 			Before:          yApp.Before,
 			After:           yApp.After,
+			Autostart:       yApp.Autostart,
 		}
 		if len(y.Plugs) > 0 || len(yApp.PlugNames) > 0 {
 			app.Plugs = make(map[string]*PlugInfo)
