@@ -195,6 +195,23 @@ func (s *infoSuite) TestReadInfo(c *C) {
 	c.Check(snapInfo2, DeepEquals, snapInfo1)
 }
 
+func (s *infoSuite) TestReadCurrentInfo(c *C) {
+	si := &snap.SideInfo{Revision: snap.R(42)}
+
+	snapInfo1 := snaptest.MockSnapCurrent(c, sampleYaml, si)
+
+	snapInfo2, err := snap.ReadCurrentInfo("sample")
+	c.Assert(err, IsNil)
+
+	c.Check(snapInfo2.Name(), Equals, "sample")
+	c.Check(snapInfo2.Revision, Equals, snap.R(42))
+	c.Check(snapInfo2, DeepEquals, snapInfo1)
+
+	snapInfo3, err := snap.ReadCurrentInfo("not-sample")
+	c.Check(snapInfo3, IsNil)
+	c.Assert(err, ErrorMatches, `cannot find current revision for snap not-sample:.*`)
+}
+
 func (s *infoSuite) TestInstallDate(c *C) {
 	si := &snap.SideInfo{Revision: snap.R(1)}
 	info := snaptest.MockSnap(c, sampleYaml, si)
@@ -910,3 +927,41 @@ func (s *infoSuite) TestExpandSnapVariables(c *C) {
 	c.Assert(info.ExpandSnapVariables("$SNAP_COMMON/stuff"), Equals, "/var/snap/foo/common/stuff")
 	c.Assert(info.ExpandSnapVariables("$GARBAGE/rocks"), Equals, "/rocks")
 }
+
+func (s *infoSuite) TestStopModeTypeKillMode(c *C) {
+	for _, t := range []struct {
+		stopMode string
+		killall  bool
+	}{
+		{"", true},
+		{"sigterm", false},
+		{"sigterm-all", true},
+		{"sighup", false},
+		{"sighup-all", true},
+		{"sigusr1", false},
+		{"sigusr1-all", true},
+		{"sigusr2", false},
+		{"sigusr2-all", true},
+	} {
+		c.Check(snap.StopModeType(t.stopMode).KillAll(), Equals, t.killall, Commentf("wrong KillAll for %v", t.stopMode))
+	}
+}
+
+func (s *infoSuite) TestStopModeTypeKillSignal(c *C) {
+	for _, t := range []struct {
+		stopMode string
+		killSig  string
+	}{
+		{"", ""},
+		{"sigterm", "SIGTERM"},
+		{"sigterm-all", "SIGTERM"},
+		{"sighup", "SIGHUP"},
+		{"sighup-all", "SIGHUP"},
+		{"sigusr1", "SIGUSR1"},
+		{"sigusr1-all", "SIGUSR1"},
+		{"sigusr2", "SIGUSR2"},
+		{"sigusr2-all", "SIGUSR2"},
+	} {
+		c.Check(snap.StopModeType(t.stopMode).KillSignal(), Equals, t.killSig)
+	}
+}
