@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2016 Canonical Ltd
+ * Copyright (C) 2014-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,16 +20,23 @@
 package store
 
 import (
+	"encoding/json"
 	"io"
 	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
+	"time"
 
 	. "gopkg.in/check.v1"
 	"gopkg.in/macaroon.v1"
+	"gopkg.in/retry.v1"
+
+	"github.com/snapcore/snapd/testutil"
 )
 
-type authTestSuite struct{}
+type authTestSuite struct {
+	testutil.BaseTest
+}
 
 var _ = Suite(&authTestSuite{})
 
@@ -70,12 +77,21 @@ const mockStoreReturnNonce = `{"nonce": "the-nonce"}`
 
 const mockStoreReturnNoNonce = `{}`
 
+func (s *authTestSuite) SetUpTest(c *C) {
+	MockDefaultRetryStrategy(&s.BaseTest, retry.LimitCount(5, retry.LimitTime(1*time.Second,
+		retry.Exponential{
+			Initial: 1 * time.Millisecond,
+			Factor:  1.1,
+		},
+	)))
+}
+
 func (s *authTestSuite) TestRequestStoreMacaroon(c *C) {
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		io.WriteString(w, mockStoreReturnMacaroon)
 	}))
 	defer mockServer.Close()
-	MyAppsMacaroonACLAPI = mockServer.URL + "/acl/"
+	MacaroonACLAPI = mockServer.URL + "/acl/"
 
 	macaroon, err := requestStoreMacaroon()
 	c.Assert(err, IsNil)
@@ -87,7 +103,7 @@ func (s *authTestSuite) TestRequestStoreMacaroonMissingData(c *C) {
 		io.WriteString(w, mockStoreReturnNoMacaroon)
 	}))
 	defer mockServer.Close()
-	MyAppsMacaroonACLAPI = mockServer.URL + "/acl/"
+	MacaroonACLAPI = mockServer.URL + "/acl/"
 
 	macaroon, err := requestStoreMacaroon()
 	c.Assert(err, ErrorMatches, "cannot get snap access permission from store: empty macaroon returned")
@@ -95,14 +111,17 @@ func (s *authTestSuite) TestRequestStoreMacaroonMissingData(c *C) {
 }
 
 func (s *authTestSuite) TestRequestStoreMacaroonError(c *C) {
+	n := 0
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		w.WriteHeader(500)
+		n++
 	}))
 	defer mockServer.Close()
-	MyAppsMacaroonACLAPI = mockServer.URL + "/acl/"
+	MacaroonACLAPI = mockServer.URL + "/acl/"
 
 	macaroon, err := requestStoreMacaroon()
 	c.Assert(err, ErrorMatches, "cannot get snap access permission from store: store server returned status 500")
+	c.Assert(n, Equals, 5)
 	c.Assert(macaroon, Equals, "")
 }
 
@@ -153,7 +172,7 @@ func (s *authTestSuite) TestDischargeAuthCaveatInvalidLogin(c *C) {
 	UbuntuoneDischargeAPI = mockServer.URL + "/tokens/discharge"
 
 	discharge, err := dischargeAuthCaveat("third-party-caveat", "foo@example.com", "passwd", "")
-	c.Assert(err, ErrorMatches, "cannot authenticate to snap store: Provided email/password is not correct.")
+	c.Assert(err, Equals, ErrInvalidCredentials)
 	c.Assert(discharge, Equals, "")
 }
 
@@ -202,7 +221,7 @@ func (s *authTestSuite) TestRefreshDischargeMacaroonInvalidLogin(c *C) {
 	UbuntuoneRefreshDischargeAPI = mockServer.URL + "/tokens/refresh"
 
 	discharge, err := refreshDischargeMacaroon("soft-expired-serialized-discharge-macaroon")
-	c.Assert(err, ErrorMatches, "cannot authenticate to snap store: Provided email/password is not correct.")
+	c.Assert(err, Equals, ErrInvalidCredentials)
 	c.Assert(discharge, Equals, "")
 }
 
@@ -219,14 +238,21 @@ func (s *authTestSuite) TestRefreshDischargeMacaroonMissingData(c *C) {
 }
 
 func (s *authTestSuite) TestRefreshDischargeMacaroonError(c *C) {
+	n := 0
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		data, err := ioutil.ReadAll(r.Body)
+		c.Assert(err, IsNil)
+		c.Assert(data, NotNil)
+		c.Assert(string(data), Equals, `{"discharge_macaroon":"soft-expired-serialized-discharge-macaroon"}`)
 		w.WriteHeader(500)
+		n++
 	}))
 	defer mockServer.Close()
 	UbuntuoneRefreshDischargeAPI = mockServer.URL + "/tokens/refresh"
 
 	discharge, err := refreshDischargeMacaroon("soft-expired-serialized-discharge-macaroon")
 	c.Assert(err, ErrorMatches, "cannot authenticate to snap store: server returned status 500")
+	c.Assert(n, Equals, 5)
 	c.Assert(discharge, Equals, "")
 }
 
@@ -257,50 +283,108 @@ func (s *authTestSuite) TestRequestStoreDeviceNonce(c *C) {
 		io.WriteString(w, mockStoreReturnNonce)
 	}))
 	defer mockServer.Close()
-	MyAppsDeviceNonceAPI = mockServer.URL + "/identity/api/v1/nonces"
 
-	nonce, err := requestStoreDeviceNonce()
+	deviceNonceAPI := mockServer.URL + "/api/v1/snaps/auth/nonces"
+	nonce, err := requestStoreDeviceNonce(deviceNonceAPI)
 	c.Assert(err, IsNil)
 	c.Assert(nonce, Equals, "the-nonce")
 }
 
+func (s *authTestSuite) TestRequestStoreDeviceNonceRetry500(c *C) {
+	n := 0
+	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		n++
+		if n < 4 {
+			w.WriteHeader(500)
+		} else {
+			io.WriteString(w, mockStoreReturnNonce)
+		}
+	}))
+	defer mockServer.Close()
+
+	deviceNonceAPI := mockServer.URL + "/api/v1/snaps/auth/nonces"
+	nonce, err := requestStoreDeviceNonce(deviceNonceAPI)
+	c.Assert(err, IsNil)
+	c.Assert(nonce, Equals, "the-nonce")
+	c.Assert(n, Equals, 4)
+}
+
+func (s *authTestSuite) TestRequestStoreDeviceNonce500(c *C) {
+	n := 0
+	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		n++
+		w.WriteHeader(500)
+	}))
+	defer mockServer.Close()
+
+	deviceNonceAPI := mockServer.URL + "/api/v1/snaps/auth/nonces"
+	_, err := requestStoreDeviceNonce(deviceNonceAPI)
+	c.Assert(err, NotNil)
+	c.Assert(err, ErrorMatches, `cannot get nonce from store: store server returned status 500`)
+	c.Assert(n, Equals, 5)
+}
+
+func (s *authTestSuite) TestRequestStoreDeviceNonceFailureOnDNS(c *C) {
+	deviceNonceAPI := "http://nonexistingserver121321.com/api/v1/snaps/auth/nonces"
+	_, err := requestStoreDeviceNonce(deviceNonceAPI)
+	c.Assert(err, NotNil)
+	c.Assert(err, ErrorMatches, `cannot get nonce from store.*`)
+}
+
 func (s *authTestSuite) TestRequestStoreDeviceNonceEmptyResponse(c *C) {
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		io.WriteString(w, mockStoreReturnNoNonce)
 	}))
 	defer mockServer.Close()
-	MyAppsDeviceNonceAPI = mockServer.URL + "/identity/api/v1/nonces"
 
-	nonce, err := requestStoreDeviceNonce()
+	deviceNonceAPI := mockServer.URL + "/api/v1/snaps/auth/nonces"
+	nonce, err := requestStoreDeviceNonce(deviceNonceAPI)
 	c.Assert(err, ErrorMatches, "cannot get nonce from store: empty nonce returned")
 	c.Assert(nonce, Equals, "")
 }
 
 func (s *authTestSuite) TestRequestStoreDeviceNonceError(c *C) {
+	n := 0
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		w.WriteHeader(500)
+		n++
 	}))
 	defer mockServer.Close()
-	MyAppsDeviceNonceAPI = mockServer.URL + "/identity/api/v1/nonces"
 
-	nonce, err := requestStoreDeviceNonce()
+	deviceNonceAPI := mockServer.URL + "/api/v1/snaps/auth/nonces"
+	nonce, err := requestStoreDeviceNonce(deviceNonceAPI)
 	c.Assert(err, ErrorMatches, "cannot get nonce from store: store server returned status 500")
+	c.Assert(n, Equals, 5)
 	c.Assert(nonce, Equals, "")
 }
 
+type testDeviceSessionRequestParamsEncoder struct{}
+
+func (pe *testDeviceSessionRequestParamsEncoder) EncodedRequest() string {
+	return "session-request"
+}
+
+func (pe *testDeviceSessionRequestParamsEncoder) EncodedSerial() string {
+	return "serial-assertion"
+}
+
+func (pe *testDeviceSessionRequestParamsEncoder) EncodedModel() string {
+	return "model-assertion"
+}
+
 func (s *authTestSuite) TestRequestDeviceSession(c *C) {
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		jsonReq, err := ioutil.ReadAll(r.Body)
 		c.Assert(err, IsNil)
-		c.Check(string(jsonReq), Equals, `{"device-session-request":"session-request","serial-assertion":"serial-assertion"}`)
+		c.Check(string(jsonReq), Equals, `{"device-session-request":"session-request","model-assertion":"model-assertion","serial-assertion":"serial-assertion"}`)
 		c.Check(r.Header.Get("X-Device-Authorization"), Equals, "")
 
 		io.WriteString(w, mockStoreReturnMacaroon)
 	}))
 	defer mockServer.Close()
-	MyAppsDeviceSessionAPI = mockServer.URL + "/identity/api/v1/sessions"
 
-	macaroon, err := requestDeviceSession("serial-assertion", "session-request", "")
+	deviceSessionAPI := mockServer.URL + "/api/v1/snaps/auth/sessions"
+	macaroon, err := requestDeviceSession(deviceSessionAPI, &testDeviceSessionRequestParamsEncoder{}, "")
 	c.Assert(err, IsNil)
 	c.Assert(macaroon, Equals, "the-root-macaroon-serialized-data")
 }
@@ -309,15 +393,15 @@ func (s *authTestSuite) TestRequestDeviceSessionWithPreviousSession(c *C) {
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		jsonReq, err := ioutil.ReadAll(r.Body)
 		c.Assert(err, IsNil)
-		c.Check(string(jsonReq), Equals, `{"device-session-request":"session-request","serial-assertion":"serial-assertion"}`)
+		c.Check(string(jsonReq), Equals, `{"device-session-request":"session-request","model-assertion":"model-assertion","serial-assertion":"serial-assertion"}`)
 		c.Check(r.Header.Get("X-Device-Authorization"), Equals, `Macaroon root="previous-session"`)
 
 		io.WriteString(w, mockStoreReturnMacaroon)
 	}))
 	defer mockServer.Close()
-	MyAppsDeviceSessionAPI = mockServer.URL + "/identity/api/v1/sessions"
 
-	macaroon, err := requestDeviceSession("serial-assertion", "session-request", "previous-session")
+	deviceSessionAPI := mockServer.URL + "/api/v1/snaps/auth/sessions"
+	macaroon, err := requestDeviceSession(deviceSessionAPI, &testDeviceSessionRequestParamsEncoder{}, "previous-session")
 	c.Assert(err, IsNil)
 	c.Assert(macaroon, Equals, "the-root-macaroon-serialized-data")
 }
@@ -327,22 +411,35 @@ func (s *authTestSuite) TestRequestDeviceSessionMissingData(c *C) {
 		io.WriteString(w, mockStoreReturnNoMacaroon)
 	}))
 	defer mockServer.Close()
-	MyAppsDeviceSessionAPI = mockServer.URL + "/identity/api/v1/sessions"
 
-	macaroon, err := requestDeviceSession("serial-assertion", "session-request", "")
+	deviceSessionAPI := mockServer.URL + "/api/v1/snaps/auth/sessions"
+	macaroon, err := requestDeviceSession(deviceSessionAPI, &testDeviceSessionRequestParamsEncoder{}, "")
 	c.Assert(err, ErrorMatches, "cannot get device session from store: empty session returned")
 	c.Assert(macaroon, Equals, "")
 }
 
 func (s *authTestSuite) TestRequestDeviceSessionError(c *C) {
+	n := 0
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		w.WriteHeader(500)
 		w.Write([]byte("error body"))
+		n++
 	}))
 	defer mockServer.Close()
-	MyAppsDeviceSessionAPI = mockServer.URL + "/identity/api/v1/sessions"
 
-	macaroon, err := requestDeviceSession("serial-assertion", "session-request", "")
+	deviceSessionAPI := mockServer.URL + "/api/v1/snaps/auth/sessions"
+	macaroon, err := requestDeviceSession(deviceSessionAPI, &testDeviceSessionRequestParamsEncoder{}, "")
 	c.Assert(err, ErrorMatches, `cannot get device session from store: store server returned status 500 and body "error body"`)
+	c.Assert(n, Equals, 5)
 	c.Assert(macaroon, Equals, "")
 }
+
+func (s *authTestSuite) TestStringish(c *C) {
+	var x stringList
+
+	c.Check(json.Unmarshal([]byte(`"hello"`), &x), IsNil)
+	c.Check(x, DeepEquals, stringList([]string{"hello"}))
+
+	c.Check(json.Unmarshal([]byte(`["hello", "world"]`), &x), IsNil)
+	c.Check(x, DeepEquals, stringList([]string{"hello", "world"}))
+}
