@@ -34,16 +34,21 @@ type snapDetails struct {
 	Deltas           []snapDeltaDetail  `json:"deltas,omitempty"`
 	DownloadSize     int64              `json:"binary_filesize,omitempty"`
 	DownloadURL      string             `json:"download_url,omitempty"`
-	Epoch            string             `json:"epoch"`
+	Epoch            snap.Epoch         `json:"epoch"`
 	IconURL          string             `json:"icon_url"`
 	LastUpdated      string             `json:"last_updated,omitempty"`
 	Name             string             `json:"package_name"`
 	Prices           map[string]float64 `json:"prices,omitempty"`
-	Publisher        string             `json:"publisher,omitempty"`
-	RatingsAverage   float64            `json:"ratings_average,omitempty"`
-	Revision         int                `json:"revision"` // store revisions are ints starting at 1
-	ScreenshotURLs   []string           `json:"screenshot_urls,omitempty"`
-	SnapID           string             `json:"snap_id"`
+	// Note that the publisher is really the "display name" of the
+	// publisher
+	Publisher      string   `json:"publisher,omitempty"`
+	RatingsAverage float64  `json:"ratings_average,omitempty"`
+	Revision       int      `json:"revision"` // store revisions are ints starting at 1
+	ScreenshotURLs []string `json:"screenshot_urls,omitempty"`
+	SnapID         string   `json:"snap_id"`
+	SnapYAML       string   `json:"snap_yaml_raw"`
+	License        string   `json:"license,omitempty"`
+	Base           string   `json:"base,omitempty"`
 
 	// FIXME: the store should send "contact" here, once it does we
 	//        can remove support_url
@@ -60,6 +65,14 @@ type snapDetails struct {
 
 	Private     bool   `json:"private"`
 	Confinement string `json:"confinement"`
+
+	ChannelMapList []channelMap `json:"channel_maps_list,omitempty"`
+}
+
+// channelMap contains
+type channelMap struct {
+	Track       string                   `json:"track"`
+	SnapDetails []channelSnapInfoDetails `json:"map,omitempty"`
 }
 
 type snapDeltaDetail struct {
@@ -75,10 +88,11 @@ type snapDeltaDetail struct {
 // channelSnapInfoDetails is the subset of snapDetails we need to get
 // information about the snaps in the various channels
 type channelSnapInfoDetails struct {
-	Revision     int    `json:"revision"` // store revisions are ints starting at 1
-	Confinement  string `json:"confinement"`
-	Version      string `json:"version"`
-	Channel      string `json:"channel"`
-	Epoch        string `json:"epoch"`
-	DownloadSize int64  `json:"binary_filesize"`
+	Revision     int        `json:"revision"` // store revisions are ints starting at 1
+	Confinement  string     `json:"confinement"`
+	Version      string     `json:"version"`
+	Channel      string     `json:"channel"`
+	Epoch        snap.Epoch `json:"epoch"`
+	DownloadSize int64      `json:"binary_filesize"`
+	Info         string     `json:"info"`
 }
