@@ -23,6 +23,7 @@ import (
 	"bytes"
 	"fmt"
 	"math/rand"
+	"sort"
 	"strconv"
 	"strings"
 	"time"
@@ -105,3 +106,44 @@ func WordWrap(s string, max int) []string {
 
 	return out
 }
+
+// ListContains determines whether the given string is contained in the
+// given list of strings.
+func ListContains(list []string, str string) bool {
+	for _, k := range list {
+		if k == str {
+			return true
+		}
+	}
+	return false
+}
+
+// SortedListContains determines whether the given string is contained
+// in the given list of strings, which must be sorted.
+func SortedListContains(list []string, str string) bool {
+	i := sort.SearchStrings(list, str)
+	if i >= len(list) {
+		return false
+	}
+	return list[i] == str
+}
+
+// TruncateOutput truncates input data by maxLines, imposing maxBytes limit (total) for them.
+// The maxLines may be 0 to avoid the constraint on number of lines.
+func TruncateOutput(data []byte, maxLines, maxBytes int) []byte {
+	if maxBytes > len(data) {
+		maxBytes = len(data)
+	}
+	lines := maxLines
+	bytes := maxBytes
+	for i := len(data) - 1; i >= 0; i-- {
+		if data[i] == '\n' {
+			lines--
+		}
+		if lines == 0 || bytes == 0 {
+			return data[i+1:]
+		}
+		bytes--
+	}
+	return data
+}
