@@ -20,12 +20,12 @@
 package systemd
 
 import (
+	"io"
 	"time"
 )
 
 var (
-	SystemdRun = run // NOTE: plain Run clashes with check.v1
-	Jctl       = jctl
+	Jctl = jctl
 )
 
 func MockStopDelays(checkDelay, notifyDelay time.Duration) func() {
@@ -38,3 +38,15 @@ func MockStopDelays(checkDelay, notifyDelay time.Duration) func() {
 		stopNotifyDelay = oldNotifyDelay
 	}
 }
+
+func MockOsGetenv(f func(string) string) func() {
+	oldOsGetenv := osGetenv
+	osGetenv = f
+	return func() { osGetenv = oldOsGetenv }
+}
+
+func MockOsutilStreamCommand(f func(string, ...string) (io.ReadCloser, error)) func() {
+	old := osutilStreamCommand
+	osutilStreamCommand = f
+	return func() { osutilStreamCommand = old }
+}
