@@ -24,6 +24,7 @@ import (
 	"io/ioutil"
 	"os"
 	"path"
+	"path/filepath"
 	"strings"
 
 	"gopkg.in/check.v1"
@@ -36,34 +37,54 @@ type MockCmd struct {
 	logFile string
 }
 
-var scriptTpl = `#!/bin/sh
-echo "$(basename "$0")" >> %[1]q
+// The top of the script generate the output to capture the
+// command that was run and the arguments used. To support
+// mocking commands that need "\n" in their args (like zenity)
+// we use the following convention:
+// - generate \0 to separate args
+// - generate \0\0 to separate commands
+var scriptTpl = `#!/bin/bash
+printf "%%s" "$(basename "$0")" >> %[1]q
+printf "\0" >> %[1]q
+
 for arg in "$@"; do
-    echo "$arg" >> %[1]q
+     printf "%%s" "$arg" >> %[1]q
+     printf "\0"  >> %[1]q
 done
-echo >> %[1]q
+
+printf "\0" >> %[1]q
 %s
 `
 
-// MockCommand adds a mocked command to PATH.
+// MockCommand adds a mocked command. If the basename argument is a command
+// it is added to PATH. If it is an absolute path it is just created there.
+// the caller is responsible for the cleanup in this case.
 //
 // The command logs all invocations to a dedicated log file. If script is
 // non-empty then it is used as is and the caller is responsible for how the
 // script behaves (exit code and any extra behavior). If script is empty then
 // the command exits successfully without any other side-effect.
 func MockCommand(c *check.C, basename, script string) *MockCmd {
-	binDir := c.MkDir()
-	exeFile := path.Join(binDir, basename)
-	logFile := path.Join(binDir, basename+".log")
+	var binDir, exeFile, logFile string
+	if filepath.IsAbs(basename) {
+		binDir = filepath.Dir(basename)
+		exeFile = basename
+		logFile = basename + ".log"
+	} else {
+		binDir = c.MkDir()
+		exeFile = path.Join(binDir, basename)
+		logFile = path.Join(binDir, basename+".log")
+		os.Setenv("PATH", binDir+":"+os.Getenv("PATH"))
+	}
 	err := ioutil.WriteFile(exeFile, []byte(fmt.Sprintf(scriptTpl, logFile, script)), 0700)
 	if err != nil {
 		panic(err)
 	}
-	os.Setenv("PATH", binDir+":"+os.Getenv("PATH"))
+
 	return &MockCmd{binDir: binDir, exeFile: exeFile, logFile: logFile}
 }
 
-// Also mock this command, using the same bindir and log
+// Also mock this command, using the same bindir and log.
 // Useful when you want to check the ordering of things.
 func (cmd *MockCmd) Also(basename, script string) *MockCmd {
 	exeFile := path.Join(cmd.binDir, basename)
@@ -100,13 +121,13 @@ func (cmd *MockCmd) Calls() [][]string {
 	if err != nil {
 		panic(err)
 	}
-	logContent := strings.TrimSuffix(string(raw), "\n")
+	logContent := strings.TrimSuffix(string(raw), "\000")
 
 	allCalls := [][]string{}
-	calls := strings.Split(logContent, "\n\n")
+	calls := strings.Split(logContent, "\000\000")
 	for _, call := range calls {
-		call = strings.TrimSuffix(call, "\n")
-		allCalls = append(allCalls, strings.Split(call, "\n"))
+		call = strings.TrimSuffix(call, "\000")
+		allCalls = append(allCalls, strings.Split(call, "\000"))
 	}
 	return allCalls
 }
