@@ -36,6 +36,7 @@ import (
 )
 
 type desktopSuite struct {
+	testutil.BaseTest
 	tempdir string
 
 	mockUpdateDesktopDatabase *testutil.MockCmd
@@ -44,6 +45,8 @@ type desktopSuite struct {
 var _ = Suite(&desktopSuite{})
 
 func (s *desktopSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
 	s.tempdir = c.MkDir()
 	dirs.SetRootDir(s.tempdir)
 
@@ -51,6 +54,8 @@ func (s *desktopSuite) SetUpTest(c *C) {
 }
 
 func (s *desktopSuite) TearDownTest(c *C) {
+	s.BaseTest.TearDownTest(c)
+	s.mockUpdateDesktopDatabase.Restore()
 	dirs.SetRootDir("")
 }
 
@@ -69,7 +74,7 @@ func (s *desktopSuite) TestAddPackageDesktopFiles(c *C) {
 	expectedDesktopFilePath := filepath.Join(dirs.SnapDesktopFilesDir, "foo_foobar.desktop")
 	c.Assert(osutil.FileExists(expectedDesktopFilePath), Equals, false)
 
-	info := snaptest.MockSnap(c, desktopAppYaml, desktopContents, &snap.SideInfo{Revision: snap.R(11)})
+	info := snaptest.MockSnap(c, desktopAppYaml, &snap.SideInfo{Revision: snap.R(11)})
 
 	// generate .desktop file in the package baseDir
 	baseDir := info.MountDir()
@@ -105,12 +110,48 @@ func (s *desktopSuite) TestRemovePackageDesktopFiles(c *C) {
 	})
 }
 
+func (s *desktopSuite) TestAddPackageDesktopFilesCleanup(c *C) {
+	mockDesktopFilePath := filepath.Join(dirs.SnapDesktopFilesDir, "foo_foobar1.desktop")
+	c.Assert(osutil.FileExists(mockDesktopFilePath), Equals, false)
+
+	err := os.MkdirAll(filepath.Join(dirs.SnapDesktopFilesDir, "foo_foobar2.desktop", "potato"), 0755)
+	c.Assert(err, IsNil)
+
+	info := snaptest.MockSnap(c, desktopAppYaml, &snap.SideInfo{Revision: snap.R(11)})
+
+	// generate .desktop file in the package baseDir
+	baseDir := info.MountDir()
+	err = os.MkdirAll(filepath.Join(baseDir, "meta", "gui"), 0755)
+	c.Assert(err, IsNil)
+
+	err = ioutil.WriteFile(filepath.Join(baseDir, "meta", "gui", "foobar1.desktop"), mockDesktopFile, 0644)
+	c.Assert(err, IsNil)
+	err = ioutil.WriteFile(filepath.Join(baseDir, "meta", "gui", "foobar2.desktop"), mockDesktopFile, 0644)
+	c.Assert(err, IsNil)
+
+	err = wrappers.AddSnapDesktopFiles(info)
+	c.Check(err, NotNil)
+	c.Check(osutil.FileExists(mockDesktopFilePath), Equals, false)
+	c.Check(s.mockUpdateDesktopDatabase.Calls(), HasLen, 0)
+}
+
 // sanitize
 
-type sanitizeDesktopFileSuite struct{}
+type sanitizeDesktopFileSuite struct {
+	testutil.BaseTest
+}
 
 var _ = Suite(&sanitizeDesktopFileSuite{})
 
+func (s *sanitizeDesktopFileSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+}
+
+func (s *sanitizeDesktopFileSuite) TearDownTest(c *C) {
+	s.BaseTest.TearDownTest(c)
+}
+
 func (s *sanitizeDesktopFileSuite) TestSanitizeIgnoreNotWhitelisted(c *C) {
 	snap := &snap.Info{SideInfo: snap.SideInfo{RealName: "foo", Revision: snap.R(12)}}
 	desktopContent := []byte(`[Desktop Entry]
@@ -126,7 +167,8 @@ Icon=${SNAP}/meep
 Name=foo
 Icon=%s/foo/12/meep
 
-# the empty line above is fine`, dirs.SnapMountDir))
+# the empty line above is fine
+`, dirs.SnapMountDir))
 }
 
 func (s *sanitizeDesktopFileSuite) TestSanitizeFiltersExec(c *C) {
@@ -145,7 +187,8 @@ Exec=baz
 
 	e := wrappers.SanitizeDesktopFile(snap, "foo.desktop", desktopContent)
 	c.Assert(string(e), Equals, `[Desktop Entry]
-Name=foo`)
+Name=foo
+`)
 }
 
 func (s *sanitizeDesktopFileSuite) TestSanitizeFiltersExecPrefix(c *C) {
@@ -164,7 +207,29 @@ Exec=snap.app.evil.evil
 
 	e := wrappers.SanitizeDesktopFile(snap, "foo.desktop", desktopContent)
 	c.Assert(string(e), Equals, `[Desktop Entry]
-Name=foo`)
+Name=foo
+`)
+}
+
+func (s *sanitizeDesktopFileSuite) TestSanitizeFiltersExecRewriteFromDesktop(c *C) {
+	snap, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+version: 1.0
+apps:
+ app:
+  command: cmd
+`))
+	c.Assert(err, IsNil)
+	desktopContent := []byte(`[Desktop Entry]
+Name=foo
+Exec=snap.app.evil.evil
+`)
+
+	e := wrappers.SanitizeDesktopFile(snap, "app.desktop", desktopContent)
+	c.Assert(string(e), Equals, fmt.Sprintf(`[Desktop Entry]
+Name=foo
+Exec=env BAMF_DESKTOP_FILE_HINT=app.desktop %s/bin/snap.app
+`, dirs.SnapMountDir))
 }
 
 func (s *sanitizeDesktopFileSuite) TestSanitizeFiltersExecOk(c *C) {
@@ -184,7 +249,8 @@ Exec=snap.app %U
 	e := wrappers.SanitizeDesktopFile(snap, "foo.desktop", desktopContent)
 	c.Assert(string(e), Equals, fmt.Sprintf(`[Desktop Entry]
 Name=foo
-Exec=env BAMF_DESKTOP_FILE_HINT=foo.desktop %s/bin/snap.app %%U`, dirs.SnapMountDir))
+Exec=env BAMF_DESKTOP_FILE_HINT=foo.desktop %s/bin/snap.app %%U
+`, dirs.SnapMountDir))
 }
 
 // we do not support TryExec (even if its a valid line), this test ensures
@@ -205,7 +271,8 @@ TryExec=snap.app %U
 
 	e := wrappers.SanitizeDesktopFile(snap, "foo.desktop", desktopContent)
 	c.Assert(string(e), Equals, `[Desktop Entry]
-Name=foo`)
+Name=foo
+`)
 }
 
 func (s *sanitizeDesktopFileSuite) TestSanitizeWorthWithI18n(c *C) {
@@ -226,15 +293,37 @@ Name=foo
 GenericName=bar
 GenericName[de]=einsehrlangeszusammengesetzteswort
 GenericName[tlh_TLH]=Qapla'
-GenericName[ca@valencia]=Hola!`)
+GenericName[ca@valencia]=Hola!
+`)
 }
 
 func (s *sanitizeDesktopFileSuite) TestSanitizeDesktopActionsOk(c *C) {
 	snap := &snap.Info{}
-	desktopContent := []byte(`[Desktop Action is-ok]`)
+	desktopContent := []byte("[Desktop Action is-ok]\n")
 
 	e := wrappers.SanitizeDesktopFile(snap, "foo.desktop", desktopContent)
-	c.Assert(string(e), Equals, `[Desktop Action is-ok]`)
+	c.Assert(string(e), Equals, string(desktopContent))
+}
+
+func (s *sanitizeDesktopFileSuite) TestSanitizeDesktopFileAyatana(c *C) {
+	snap := &snap.Info{}
+
+	desktopContent := []byte(`[Desktop Entry]
+Version=1.0
+Name=Firefox Web Browser
+X-Ayatana-Desktop-Shortcuts=NewWindow;Private
+
+[NewWindow Shortcut Group]
+Name=Open a New Window
+TargetEnvironment=Unity
+
+[Private Shortcut Group]
+Name=Private Mode
+TargetEnvironment=Unity
+`)
+
+	e := wrappers.SanitizeDesktopFile(snap, "foo.desktop", desktopContent)
+	c.Assert(string(e), Equals, string(desktopContent))
 }
 
 func (s *sanitizeDesktopFileSuite) TestRewriteExecLineInvalid(c *C) {
@@ -258,23 +347,30 @@ apps:
 	c.Assert(newl, Equals, fmt.Sprintf("Exec=env BAMF_DESKTOP_FILE_HINT=foo.desktop %s/bin/snap.app", dirs.SnapMountDir))
 }
 
-func (s *sanitizeDesktopFileSuite) TestTrimLang(c *C) {
+func (s *sanitizeDesktopFileSuite) TestLangLang(c *C) {
 	langs := []struct {
-		in  string
-		out string
+		line    string
+		isValid bool
 	}{
 		// langCodes
-		{"[lang_COUNTRY@MODIFIER]=foo", "=foo"},
-		{"[lang_COUNTRY]=bar", "=bar"},
-		{"[lang_COUNTRY]=baz", "=baz"},
-		{"[lang]=foobar", "=foobar"},
-		// non-langCodes, should be ignored
-		{"", ""},
-		{"Name=foobar", "Name=foobar"},
-		// corner case
-		{"[foo=bar", "[foo=bar"},
+		{"Name[lang]=lang-alone", true},
+		{"Name[_COUNTRY]=country-alone", false},
+		{"Name[.ENC-0DING]=encoding-alone", false},
+		{"Name[@modifier]=modifier-alone", false},
+		{"Name[lang_COUNTRY]=lang+country", true},
+		{"Name[lang.ENC-0DING]=lang+encoding", true},
+		{"Name[lang@modifier]=lang+modifier", true},
+		// could also test all bad combos of 2, and all combos of 3...
+		{"Name[lang_COUNTRY.ENC-0DING@modifier]=all", true},
+		// other localised entries
+		{"GenericName[xx]=a", true},
+		{"Comment[xx]=b", true},
+		{"Keywords[xx]=b", true},
+		// bad ones
+		{"Name[foo=bar", false},
+		{"Icon[xx]=bar", false},
 	}
 	for _, t := range langs {
-		c.Assert(wrappers.TrimLang(t.in), Equals, t.out)
+		c.Assert(wrappers.IsValidDesktopFileLine([]byte(t.line)), Equals, t.isValid)
 	}
 }
